/*!
********************************************************************************
**
** \file        ./boards/fme/mb86h61_dk01/mb86h61_dk01.c
**
** \version     $Id: mb86h61_dk01.c 5986 2011-03-31 18:35:29Z jrende $
**
** \brief       FME devkit specific board service function.
**
** This file contains the implementation of the FME devkit specific board
** services functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/

#define BOARDS_FMEDEF_DIAG_INFO__MACRO_ENABLED

#define NEW_TS_PATH_MODEL_4CI_PLUS  // NOTE: we use this 'model' for all SmartGoHd versions now
//#define JUST_REGISTER_PINS        // NOTE: will not work for 'low-level boards-test-applicatons'

//#define USE_LED_TEST_GPIOS                // only used for special HW-debug
//#define INSTALL_HW_TEST_ICC_DRIVER        // only used for 'hw-testing'
//#define TEST_PATA_INTERFACE_VIA_J900_ADAPTER  // only used for special HW-test

#include <fapi/reg_gpio.h>
#include <fapi/drv_gpio.h>
#include <fapi/board_ifc.h>
#ifndef __LINUX__
#include <fe/fe.h>
#endif

#include <boards/boards_common.h>
#include <boards/boards_com_dirx.h>
#include <boards/boards_com_fp.h>
#include <boards/mb86h61_dk01.h>

//#define PIN_REGISTER_PREINIT_FUNC BRDCOM_BC_DirectConfigureGpioPin    // direct (no-FREG-Lib)
//#define PIN_REGISTER_PREINIT_FUNC   BRDCOM_BC_RegisterGpioMode_PreInit  // will need FREG-Lib

/*lint -save -e923 -e778 */

//#define FBD_ID   BOARDS_FMEDEF_DIAG_INFO_DATA     // simple local re-define to save code-spase
#define FBD_ID   FAPI_BOARD_DIAG_BITS_JUST_INFO

#define GP_XSI 11
#define GP_XSO 38
#define GP_XLD 40
#define GP_XCK 39

#define LATCH_INIT_MASK (0xFF)          // init all bits
//#define LATCH_INIT_BITS (GPO_SFTRG_USB_CBU0|GPO_SFTRG_USB_CBU1|GPO_SFTRG_CI_IN_ENA_BIT)
#define LATCH_INIT_BITS (GPO_SFTRG_CI_IN_ENA_BIT)   // NOTE: by default, USB-VBUS is off

static int32_t BOARD_FME_H61DK01_BoardIfc_InitDetect(uint32_t initDetectFlags, void *optInitDataPtr);

#ifdef BOARDS_FMEDEF_DIAG_INFO__MACRO_ENABLED
  static const char_t DEBSTR_SysPwrActive[] = "SystemPower->ACTIVE";
  static const char_t DEBSTR_SysPwrOff[]    = "SystemPower->OFF";
  static const char_t DEBSTR_UsbPhyCtrl[]   = "USB-PhyCtrl";
#endif

static BOARDS_FMEDEF_SerialIoExtInfoT  H61DkSerIoExtInfos = {
// SIpin,  SOpin,  CLKpin, LDpin,  nShifts,     tDelCnt
   GP_XSI, GP_XSO, GP_XCK, GP_XLD, USE_N_SHIFT_CLOCKS, 20      };

//------------------------------------------------------------
//-- Platform Info/Configuration-Data for Application-SW    --
//------------------------------------------------------------

static const char_t Manufact[] = "Fujitsu(RSD)";
#ifdef NEW_TS_PATH_MODEL_4CI_PLUS
  const char_t BOARD_FME_H61DK01_ModelStr4CiPlus[]    = "MB86h61 Test-Board (Ci+ support)";
  #define ACTUAL_MODEL_STR  BOARD_FME_H61DK01_ModelStr4CiPlus
#else
  const char_t BOARD_FME_H61DK01_ModelStr_Initial[]   = "MB86h61 Test-Board (inc.CI-Logic)";
  #define ACTUAL_MODEL_STR  BOARD_FME_H61DK01_ModelStr_Initial
#endif

static const FAPI_BOARD_HwPlatformInfosT  FmeMb86H61Dk01_HwInfo =
{
    FAPI_BOARD_HWCONFIG_VERSION,//!< MagicAndVersion
    Manufact,                   //!< pHwManufactString
    ACTUAL_MODEL_STR,           //!< pHwModelString
    NULL,                       //!< pOptionalAdditionalInfo
    0,                          //!< flagUsbHostOnly
#ifdef NEW_TS_PATH_MODEL_4CI_PLUS
    FAPI_BOARD_BITFLAG_PFLASH,  //!< typeOfFlashAvailable
#else
    FAPI_BOARD_BITFLAG_SFLASH,  //!< typeOfFlashAvailable
#endif /* NEW_TS_PATH_MODEL_4CI_PLUS */
    0,                          //!< AudioConfigurationInfoA
    0,                          //!< AudioConfigurationInfoB
    2,                          //!< numberOfAvlFrontEnds
    1,                          //!< numberOfAvlCiSlots

    {
        // TS-A
#ifdef NEW_TS_PATH_MODEL_4CI_PLUS
        { BOARDS_FMEDEF_SATELLITE,      //!< defaultFeType
          FAPI_BOARD_RSRC_TSIN_FE1,     //!< inputOptionsTsPath
          FAPI_BOARD_RSRC_CIIN_NONE },  //!< inputOptionsCiPath
#else
        { BOARDS_FMEDEF_SATELLITE,      //!< defaultFeType
          FAPI_BOARD_RSRC_TSIN_FE1      //!< inputOptionsTsPath
        | FAPI_BOARD_RSRC_TSIN_CI,
          FAPI_BOARD_RSRC_CIIN_FE1 },   //!< inputOptionsCiPath
#endif
        // TS-B
        { BOARDS_FMEDEF_SATELLITE,      //!< defaultFeType
          FAPI_BOARD_RSRC_TSIN_FE2,     //!< inputOptionsTsPath
          FAPI_BOARD_RSRC_CIIN_NONE },  //!< inputOptionsCiPath

        // TS-C
#ifdef NEW_TS_PATH_MODEL_4CI_PLUS
        { BOARDS_FMEDEF_SATELLITE,      //!< defaultFeType
          FAPI_BOARD_RSRC_TSIN_CI,      //!< inputOptionsTsPath
          FAPI_BOARD_RSRC_CIIN_TSGEN }, //!< inputOptionsCiPath
#else
        { BOARDS_FMEDEF_SATELLITE,      //!< defaultFeType
          FAPI_BOARD_RSRC_TSIN_NONE,    //!< inputOptionsTsPath
          FAPI_BOARD_RSRC_CIIN_NONE },  //!< inputOptionsCiPath
#endif
        // TS-D
        { BOARDS_FMEDEF_SATELLITE,      //!< defaultFeType
          FAPI_BOARD_RSRC_TSIN_NONE,    //!< inputOptionsTsPath
          FAPI_BOARD_RSRC_CIIN_NONE }   //!< inputOptionsCiPath
    },
    FAPI_BOARD_HD1_P_ATA,           //!< flagHardDiskInterface
    0,                              //!< flgDdr2MemoryInfos
    FAPI_BOARD_VID_AMP_C0           //!< video/scaler optimize info
};

BOARDS_CommonSupportFunctionsT  *FME_H61TB01_CbsfPtr;   //

static uint8_t  BefPOff16State, BefPOffPin8State;
static uint8_t  ActPin16State, ActPin8State;

#ifdef BOARD_FME_H61DK01_WAKEUP_HARDWARE_WORKAROUND
/* The wakeup pin status must be retrieved before the GPIO is reconfigured. */ 
static uint32_t GpioStartupValue63_32 = 0;
#endif

/*!
********************************************************************************
** \brief Controls a LED.
**
** Controls a diagnostic LED and/or a ScopeTrigger-Pin on the hw-platform
**
** \param index   Index of the LED to be controlled.
** \param command LED value to drive (FAPI_BOARD_LedCommandT).
** \param optPtr  Optional parameter pointer depending on 'command'
**                in case of 'FAPI_BOARD_GET_TEST_KEY' points to uint32_t
**
** \sa FAPI_BOARD_LedCommandT
**
** \return
** - FAPI_ERR_FEATURE_NOT_SUPPORTED if there is no function registered
** - any other value is the result of the registered function call
********************************************************************************
*/
static int32_t BOARD_FME_H61DK01_ControlAlifeLed
                 ( int32_t index, FAPI_BOARD_LedCommandT command, void* optPtr )
{
#ifndef USE_LED_TEST_GPIOS
    index   = index;
    command = command;
    optPtr  = optPtr;
#else
    uint32_t  gpio_pin_num, pin_val;
    optPtr  = optPtr;
    gpio_pin_num = 0; pin_val = 0;                  // default: no action
    if (command == FAPI_BOARD_TEST_LED_ON) {
        if (index==0) {
//            gpio_pin_num = GPIO_UPI_BUF_DIR; pin_val = 1;
        }
//      else if (index==1) {                        // e.g. special SmartCard-Test-Pin
//          gpio_pin_num = PIN_TDA_ICC0_TEST;  pin_val = 1;
//      }
    }
    else if (command == FAPI_BOARD_TEST_LED_OFF) {
        if (index==0) {
//            gpio_pin_num = GPIO_UPI_BUF_DIR; pin_val = 0;
        }
//      else if (index==1) {                        // e.g. special SmartCard-Test-Pin
//          gpio_pin_num = PIN_TDA_ICC0_TEST;  pin_val = 0;
//      }
    }
    if (gpio_pin_num) {                             // if (pin != 0) to be changed
        FREG_GPIO_SetDataOut(gpio_pin_num, pin_val);
    }
#endif
    return  FAPI_OK;
}

/*!
********************************************************************************
** \brief Controls the reset-signal and power-supply of the frontend.
**
** This function controls the reset-signal and power-supply of the frontend.
**
** \param index   Index of the frontend to be controlled.
** \param command Type of reset signal (#FAPI_BOARD_ControlCommandT).
** \param optPtr  Optional parameters (not used yet).
**
** \return
** - #FAPI_ERR_FEATURE_NOT_SUPPORTED if there is no function registered
** - any other value is the result of the registered function call
********************************************************************************
*/
static int32_t BOARD_FME_H61DK01_ControlFeResetPower     // todo !
               ( int32_t index, FAPI_BOARD_ControlCommandT command, void* optPtr )
{
    uint32_t gpio_pin;
    uint32_t bit_valu;

    optPtr   = optPtr;
    bit_valu = 0;                       // default
    //-- check supported indexes --
    if      (index==0)
        gpio_pin = GPIO_OP_FE_RES_A;
    else if (index==1)
        gpio_pin = GPIO_OP_FE_RES_B;
    else
        return FAPI_ERR_FEATURE_NOT_SUPPORTED;

    //-- check supported controls --
    if      (command == FAPI_BOARD_RESET_INACTIVE)
        bit_valu = 1;
    else if (command == FAPI_BOARD_RESET_ACTIVE)
        bit_valu = 0;
    else if (command == FAPI_BOARD_RESET_PULSE)
    {
        (void) BOARD_FME_H61DK01_ControlFeResetPower(index, FAPI_BOARD_RESET_ACTIVE, optPtr);
        FME_H61TB01_CbsfPtr->SimpleTimeDelay(50);
        (void) BOARD_FME_H61DK01_ControlFeResetPower(index, FAPI_BOARD_RESET_INACTIVE, optPtr);
        FME_H61TB01_CbsfPtr->SimpleTimeDelay(50);
        return FAPI_OK;
    }
    else
        return FAPI_ERR_FEATURE_NOT_SUPPORTED;

    FREG_GPIO_SetDataOut(gpio_pin, bit_valu);

    return  FAPI_OK;
}

/*!
********************************************************************************
** \brief 'USB-Phy-Control', in this case control VBUS On/Off
**
** This function controls relevant modes of 'USB-Phy-Interface'
**
** \param index   Index
** \param command Type of control
** \param optPtr  Optional parameters (not used yet).
**
** \return
** - #FAPI_ERR_FEATURE_NOT_SUPPORTED if there is no function registered
** - FAPI_OK
********************************************************************************
*/
static int32_t BOARD_FME_H61DK01_CtrlUsbPhy
               ( int32_t index, uint32_t ctrl, void* optPtr )
{
    FAPI_BOARD_ControlCommandT command;
    uint32_t bit_mask;
    uint32_t bit_valu;

    optPtr  = optPtr;
    bit_mask = 0;
    bit_valu = 0;
    if (index >= 2) return FAPI_ERR_BAD_PARAMETER;

    command = (FAPI_BOARD_ControlCommandT) ctrl;
    FAPI_BOARD_DIAGNOSTIC_TS_PATH(FBD_ID, (void*) DEBSTR_UsbPhyCtrl, ctrl, 0);
    if     (command == FAPI_BOARD_POWER_ON)
    {
        bit_mask = GPO_SFTRG_USB_CBU0;
        bit_valu = GPO_SFTRG_USB_CBU0;
    }
    else if (command == FAPI_BOARD_POWER_OFF)
    {
        bit_mask = GPO_SFTRG_USB_CBU0;
//        bit_valu = 0;
    }

    if (index) {
        bit_mask = bit_mask << 1;
        bit_valu = bit_valu << 1;
    }
    if (bit_mask)
    {
        FME_H61TB01_CbsfPtr->WriteExtLatch(bit_mask, bit_valu);
    }
    return  FAPI_OK;
}

/*!
********************************************************************************
** \brief Controls the reset-signal and power-supply of the harddisk.
**
** This function controls the reset-signal and power-supply of the harddisk.
**
** \param index   Index of the harddisk to be controlled.
** \param Ctrl    Type of reset signal (#FAPI_BOARD_ControlCommandT).
** \param optPtr  Optional parameters, if command equals FAPI_BOARD_POLL_IRQ_STATUS
**                optPtr must point to an uint32_t variable where the resulting
**                status will be written into.
**
** \return
** - #FAPI_ERR_FEATURE_NOT_SUPPORTED if there is no function registered
** - any other value is the result of the registered function call
********************************************************************************
*/
static int32_t BOARD_FME_H61DK01_ControlHdResetPower
               ( int32_t index, FAPI_BOARD_ControlCommandT Ctrl, void* optPtr )
{
    index = index;
    Ctrl  = Ctrl;
    optPtr  = optPtr;
    return FAPI_ERR_FEATURE_NOT_SUPPORTED;
}

/*!
********************************************************************************
** \brief Controls a specific SCART pin.
**
** This function controls a specific SCART pin.
**
** \param index   Index of the Scart connector to be controlled.
** \param command Type of scart pin command (FAPI_BOARD_ScartCommandT).
** \param optPtr  Optional parameters (not used yet).
**
** \return
** - FAPI_ERR_FEATURE_NOT_SUPPORTED if there is no function registered
** - any other value is the result of the registered function call
********************************************************************************
*/

static int32_t BOARD_FME_H61DK01_ControlScartPin8A16
                ( int32_t index, FAPI_BOARD_ScartCommandT command, void* optPtr )
{
    uint32_t bit_mask;
    uint32_t bit_valu;
    index = index;

//        FAPI_BOARD_HW_DIAGNOSTIC(DEBSTR_ControlScart, FAPI_BOARD_DIAG_INFO);
    optPtr = optPtr;
    if (index>0)
        return FAPI_ERR_FEATURE_NOT_SUPPORTED;

    if      (command == FAPI_BOARD_SCART_MODE_OFF)
    {
//        BTLIB_IO_PrStrHexStr(0, (const char_t*)"{off}\n", 0, 0);
        ActPin8State = (uint8_t) command;
        bit_mask = GPO_SFTRG_SCART_PIN8_12V|GPO_SFTRG_SCART_PIN8_5V;  // involved bits
        bit_valu = GPO_SFTRG_SCART_PIN8_12V|GPO_SFTRG_SCART_PIN8_5V;  // set this one
    }
    else if (command == FAPI_BOARD_SCART_MODE_16_9)
    {
//        BTLIB_IO_PrStrHexStr(0, (const char_t*)"{16/9}\n", 0, 0);
        ActPin8State = (uint8_t) command;
        bit_mask = GPO_SFTRG_SCART_PIN8_12V|GPO_SFTRG_SCART_PIN8_5V;   // involved bits
        bit_valu = GPO_SFTRG_SCART_PIN8_5V;                            // set these
    }
    else if (command == FAPI_BOARD_SCART_MODE_4_3)
    {
//        BTLIB_IO_PrStrHexStr(0, (const char_t*)"{4/3}\n", 0, 0);
        ActPin8State = (uint8_t) command;
        bit_mask = GPO_SFTRG_SCART_PIN8_12V|GPO_SFTRG_SCART_PIN8_5V;  // involved bits
        bit_valu = 0;                                                   //
    }

    else if (command == FAPI_BOARD_SCART_VIDEO_OUTPUT_CVBS)
    {
        ActPin16State = (uint8_t) command;
        bit_mask = GPO_SFTRG_SCART_PIN16;                               // involved bits
        bit_valu = GPO_SFTRG_SCART_PIN16;                               // set to '1'
    }
    else if (command == FAPI_BOARD_SCART_VIDEO_OUTPUT_RGB)
    {
        ActPin16State = (uint8_t) command;
        bit_mask = GPO_SFTRG_SCART_PIN16;                               // involved bits
        bit_valu = 0;                                                    // set to '0'
    }
    else
        return FAPI_ERR_FEATURE_NOT_SUPPORTED;

    FME_H61TB01_CbsfPtr->WriteExtLatch(bit_mask, bit_valu);
    return FAPI_OK;
}

/*!
********************************************************************************
** \brief Controls the 'System Power Mode'
**
** HW-Platform specific controls depending on 'System Power-Mode'
**
** \param index   Optional parameter, not used here
** \param command Enumerate for specific power-mode
** \param optPtr  Optional parameters (not used yet).
**
** \return
** - FAPI_ERR_FEATURE_NOT_SUPPORTED if there is no function registered
** - any other value is the result of the registered function call
********************************************************************************
*/
static int32_t BOARD_FME_H61DK01_ControlSystemPowerMode
                ( int32_t index, FAPI_BOARD_PowerModeCmdT command, void* optPtr )
{
    FAPI_BOARD_ScartCommandT  befcmd;
    uint32_t *on_reason_ptr;
    optPtr = optPtr;

    if (index>0)
        return FAPI_ERR_FEATURE_NOT_SUPPORTED;

    if      (command == FAPI_BOARD_POWER_FULL_ACTIVE)               //== activ power-mode ==
    {
        FAPI_BOARD_DIAGNOSTIC_GENERALPURPOSE(FBD_ID, (void*) DEBSTR_SysPwrActive, ActPin8State, 0);
        //-- if we are comming from a 'fake-power-down-mode' (No-HW-Reset-happened)
        FREG_GPIO_SetDataOut(P55_WAKUP_TSO7,BDEF_GPIO_OUT_TS_DATA_7);   // we must restore this pin-function again

        befcmd = (FAPI_BOARD_ScartCommandT) BefPOffPin8State;
        (void) BOARD_FME_H61DK01_ControlScartPin8A16(0,befcmd,0);
        befcmd = (FAPI_BOARD_ScartCommandT) BefPOff16State;
        (void) BOARD_FME_H61DK01_ControlScartPin8A16(0,befcmd,0);
    }
    else if (  (command == FAPI_BOARD_POWER_MAX_OFF)                //== lowest power-mode ==
             ||(command == FAPI_BOARD_POWER_STANDBY))               //( or 'power-isle-standby )
    {
        BefPOffPin8State = ActPin8State;
        BefPOff16State   = ActPin16State;
        FAPI_BOARD_DIAGNOSTIC_GENERALPURPOSE(FBD_ID, (void*) DEBSTR_SysPwrOff, ActPin16State, 0);
        (void) BOARD_FME_H61DK01_ControlScartPin8A16(0,FAPI_BOARD_SCART_MODE_OFF,0);
        (void) BOARD_FME_H61DK01_ControlScartPin8A16(0,FAPI_BOARD_SCART_VIDEO_OUTPUT_CVBS,0);
        //-- we do something special to the 'GPIO' used for 'power-on reason-detection' --
        FREG_GPIO_SetDataOut(P55_WAKUP_TSO7,0);         // active 'pull-down'
        BRDCOM_BC_SimpleTimeDelay(100);
        FREG_GPIO_SetDataOut(P55_WAKUP_TSO7,(1<<8));    // active 'pull-up'
    }

    else if (command == FAPI_BOARD_GET_POWER_ON_REASON)             //== lowest power-mode ==
    {
#ifdef BOARD_FME_H61DK01_WAKEUP_HARDWARE_WORKAROUND
        // wakeup work around not implemented in hardware
        uint32_t inp_level;

        inp_level = GpioStartupValue63_32;
        inp_level = (inp_level >> (P55_WAKUP_TSO7-32)) & 1;
        if (inp_level) {
            inp_level = FAPI_BOARD_POWER_ONBY_STANDBY_BUTTON; }
        else {
            inp_level = FAPI_BOARD_POWER_ONBY_MAIN_POWER_ON; }
        on_reason_ptr = optPtr;
        if (on_reason_ptr)
            *on_reason_ptr = inp_level;
#else
        // assume cold start
        on_reason_ptr = optPtr;
        if (on_reason_ptr)
            *on_reason_ptr = FAPI_BOARD_POWER_ONBY_MAIN_POWER_ON;
#endif
    }

    else
        return FAPI_ERR_FEATURE_NOT_SUPPORTED;

    return FAPI_OK;
}

/*!
********************************************************************************
** \brief Controls resource-sharing of some GPIOs
**
** this might be a 'particular' control for the Fujitsu dev-kit
** running various 'particular test applications'.
** Usually products do not need to 'share' GPIOs for different functions
** \note  example call
**  (void)FAPI_BOARD_EXT_CONTROL_RESOURCE(0, FAPI_BOARD_PREPARE_TS_OUTP, 0);
** \param index   Optional parameter, not used here
** \param command Enumerate for specific resources which could be shared
** \param optPtr  Optional parameters (not used yet).
**
** \return
** - FAPI_ERR_FEATURE_NOT_SUPPORTED if there is no function registered
** - any other value is the result of the registered function call
********************************************************************************
*/
static int32_t BOARD_FME_H61DK01_ControlGpioResourceSharing(
                    int32_t index, FAPI_BOARD_ResourceCommandT command, void* optPtr )
{
    FAPI_BOARD_CompbinedBitFieldsT  cmb_index;
    uint32_t ts_idx, ccir_idx, ctrl_val;
    int32_t ret_val;

    index  = index;
    optPtr = optPtr;
    ret_val = FAPI_OK;
    
    cmb_index.u32 = (uint32_t) index;   // seperate combined parameters in case of 'CCIR-controls'
    ts_idx   = cmb_index.cbfld.main_index;    // this is TS-PathIndex (TS-A, B, C)
    ccir_idx = cmb_index.cbfld.sub_index;
    ctrl_val = cmb_index.cbfld.main_ctrl;
    // this means 'route TS-Input[x] to 'CCIR-Capture[x]' for 'normal capture-mode'
    if      (command == FAPI_BOARD_PREPARE_CCIR_INP)
    {
        ret_val = BOARD_FME_H61DK01_RSO_Cfg_GpioTsInp_2_CcirInp_NormOp(ts_idx, ccir_idx);
    }
    // this means 'route TS-Input[x] to 'CCIR-Capture[x]' for 'special sync-mode'
    else if (command == FAPI_BOARD_SPECIAL_CCIR_CFG1)
    {
        ret_val = BOARD_FME_H61DK01_RSO_Cfg_GpioTsInp_2_CcirInp_SyncOp(ts_idx, ccir_idx);
    }
    //-- special control of 'clock-signal' enable/disable --
    else if (command == FAPI_BOARD_SPECIAL_CCIR_CFG2)
    {
        BOARD_FME_H61DK01_RSO_Cfg_Gpio_2_CcirInp_ClkCrtrl(ts_idx, ccir_idx, ctrl_val );
    }
    //-- special control of 'serial TS-Output' generation --
    else if (command == FAPI_BOARD_PREPARE_SER_TS_OUTP)
    {
        BOARD_FME_H61DK01_RSO_Cfg_SerTsOutpPins(ts_idx, ccir_idx, ctrl_val);
    }
    else if (command == FAPI_BOARD_PREPARE_TS_OUTP)
    {
        BOARD_FME_H61DK01_RSO_TsOut_ExtADac_TdaIcc(BRDCOM_BC_RegisterGpioMode_PreInit, 0);
    }
    else if (command == FAPI_BOARD_PREPARE_EXT_AUDDAC)
    {
        BOARD_FME_H61DK01_RSO_TsOut_ExtADac_TdaIcc(BRDCOM_BC_RegisterGpioMode_PreInit, 1);
    }
    else if (command == FAPI_BOARD_PREPARE_ICC_TDA)
    {
        BOARD_FME_H61DK01_RSO_TsOut_ExtADac_TdaIcc(BRDCOM_BC_RegisterGpioMode_PreInit, 2);
    }

    else if (command == FAPI_BOARD_PREPARE_P_FLASH)
    {
        BOARD_FME_H61DK01_RSO_PFlash_or_SdIo(BRDCOM_BC_RegisterGpioMode_PreInit, 0);
    }
    else if (command == FAPI_BOARD_PREPARE_SD_CARD)
    {
        BOARD_FME_H61DK01_RSO_PFlash_or_SdIo(BRDCOM_BC_RegisterGpioMode_PreInit, 1);
    }
    else if (command == FAPI_BOARD_PREPARE_D_RGB_OUTP)
    {
        BOARD_FME_H61DK01_RSO_DigitalRGB_or_RestoreDefaults(BRDCOM_BC_RegisterGpioMode_PreInit, 0);
    }
    return ret_val;
}


/*!
********************************************************************************
** \brief register the 'MB86H61Dk01' HW-platform board-information
**
**  Registers the 'MB86H61Dk01' HW-platform board-information structure
**  (BOARDS_HwPlatformInfosT)
**  the 'init/detect function' hook (FAPI_BOARD_InitDetectHookT)
**  and other 'control function' hooks into the
**  generic 'board interface control' (see fapi/board/board_ifc.c)
**
** \note This function can be called even before FAPI-drivers are initialized.
**  Actually some init-parameters for the initialization of FAPI-drivers later-on,
**  might be obtained be retreiving the registered information/configuration-data
**
** \param registerFlags          set to '0' (for future use)
** \param optRegisterDataPtr      pointer to optional data (future use)
**
** \return  usually FAPI_OK
**
********************************************************************************
*/
int32_t BOARD_FME_H61DK01_BoardIfc_Register(uint32_t registerFlags, void *optRegisterDataPtr)
{
    registerFlags         = registerFlags;          // not used yet
    optRegisterDataPtr    = optRegisterDataPtr;     // not used yet

#ifdef BOARD_FME_H61DK01_WAKEUP_HARDWARE_WORKAROUND
    /* Retrieve wakeup workaround pin status. */
    GpioStartupValue63_32 = FREG_GPIO_GetInput_63_32();     // actually save 'P55_WAKUP_TSO7' pin-status
#endif

//    FME_H61TB01_CbsfPtr = BRDCOM_GetIoExtFncsPtr_FapiUpiAta(); // would be like 'old H60'
                                                            // -> should not work, but also not crash !!!
    FME_H61TB01_CbsfPtr = BRDCOM_GetIoExtFncsPtr_ExtSerShiftRegs();    // we use these functions H61 Testbaord
    FAPI_BOARD_SetBoardInfo( (void*) &FmeMb86H61Dk01_HwInfo );

    FAPI_BOARD_SetBoardInitDetectHook  ( BOARD_FME_H61DK01_BoardIfc_InitDetect );
    FAPI_BOARD_SetControlTestLedKeyHook( BOARD_FME_H61DK01_ControlAlifeLed     );
    FAPI_BOARD_SetControlFeResetHook   ( BOARD_FME_H61DK01_ControlFeResetPower );
    FAPI_BOARD_SetControlHdResetHook   ( BOARD_FME_H61DK01_ControlHdResetPower );
    FAPI_BOARD_SetControlScartHook     ( BOARD_FME_H61DK01_ControlScartPin8A16 );
    FAPI_BOARD_SetControlPowerMode     ( BOARD_FME_H61DK01_ControlSystemPowerMode );
    FAPI_BOARD_SetControlUsbPhy        ( BOARD_FME_H61DK01_CtrlUsbPhy );
    FAPI_BOARD_SetResourceShare        ( BOARD_FME_H61DK01_ControlGpioResourceSharing );
//  FAPI_BOARD_SetControlTsPathHook    (  );    // NOTE: see 'mb86h61_dk01_ci.c'

    BOARD_FME_H61DK01_FrontPanel_Register();

#ifdef JUST_REGISTER_PINS   // this will just-preconfigure GPIOs before 'FAPI_GPIO_Init()' is called
    BOARD_FME_H61DK01_RegisterGpioPins(FAPI_GPIO_SetPinFunction);
#else                       // this will call 'FAPI_GPIO_SetPinFunction' but also 'directly pre-configure' the pins
    BOARD_FME_H61DK01_RegisterGpioPins(BRDCOM_BC_RegisterGpioMode_PreInit);
#endif

#ifdef NEW_TS_PATH_MODEL_4CI_PLUS       // if for 'CiPlus' application, we assume 'P-Flash-application'
    BOARD_FME_H61DK01_RSO_TsOut_ExtADac_TdaIcc(BRDCOM_BC_RegisterGpioMode_PreInit, 0);  // we need TS-Outp.
    BOARD_FME_H61DK01_RSO_PFlash_or_SdIo(BRDCOM_BC_RegisterGpioMode_PreInit, 1);        // and want SDIO
#endif

#ifdef INSTALL_HW_TEST_ICC_DRIVER
    BOARD_H61_DK01_InitInstall_HwTest_IccDriver(0);
#endif

    BOARD_FME_H61DK01_RegisterSpecialIrDataInfo();

    return FAPI_OK;
}


/*!
********************************************************************************
** \brief initialization of 'MB86H61Dk01' HW-platform specific control functions
**
**  this function will perform eventually necessary initializations
**  before board-control functions can be used
** \note usually this function should be called after FAPI-drivers were initialized
**  so FAPI-drivers (e.g. like I2C-driver) could be used here.
**  Ideally, this function should be able to verify or detect,
**  that the 'present HW' really matches to the board-specific driver.
**  However, for this 'Fujitsu Dev.Kit', this feature is not implemented yet
**  and the return-code always signals that the detected HW is fine.
**  (even the SW is currently accidently running on a different HW-platform)
**
** \param initDetectFlags     enum for special install conditions, not considered here
** \param optInitDataPtr      pointer to optional data, not used here
**
** \return
** - BOARDS_INSTALL_ASSUME_OK, always
**
********************************************************************************
*/
static int32_t BOARD_FME_H61DK01_BoardIfc_InitDetect
                     (uint32_t initDetectFlags, void *optInitDataPtr)
{
    initDetectFlags = initDetectFlags;
    optInitDataPtr  = optInitDataPtr;

    FME_H61TB01_CbsfPtr->InitAccessExtLatch((void*) &H61DkSerIoExtInfos);
    FME_H61TB01_CbsfPtr->WriteExtLatch(LATCH_INIT_MASK, LATCH_INIT_BITS);

#ifndef TEST_PATA_INTERFACE_VIA_J900_ADAPTER
    (void) BOARD_FME_H61DK01_SetupCiLibHwInterfaceFunctions();
#endif

    (void) BOARDS_CONTROL_FP(0, BOARDS_FP_SET_PWR_MODE_OPERATION, (void*) 0); // activate FrontPanel

#ifdef TEST_PATA_INTERFACE_VIA_J900_ADAPTER
    // NOTE: this will re-configure different to 'BOARD_FME_H61DK01_SetupCiLibHwInterfaceFunctions'
    BRDCOM_BC_DirectPreInitUpiCs2ViaAtaControler();
    BOARD_FME_H61DK01_RSO_ConfigGpiosPAtaAdapter(PIN_REGISTER_PREINIT_FUNC);
#endif

    return FAPI_OK;
}

