/*!
********************************************************************************
**
** \file        ./boards/fme/mb86h61_dk01/mb86h61_dk01.h
**
** \version     $Id: mb86h61_dk01.h 5986 2011-03-31 18:35:29Z jrende $
**
** \brief       header-file for 'MB86H61 Dev.Kit' install function
**
** This file contains the definitions for the 'install function'
** which will then register all board-contro functions into the BIC-driver
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/
#ifndef FME_MB86H61_DK01
#define FME_MB86H61_DK01

#ifdef __cplusplus
extern "C" {
#endif

/*
   NOTE1: The 'application/init' should include the relevant 'board-header-file'
   preferabley using 'compiler command-line-define ' like
   -D'BOARD_INCLUDE_FILE="boards/mb86h61_dk01.h"'

   This define is actually setup via the 'MAKE ENVIRONMENT' taking the information from the
   definition of 'BOARDNAME' in the 'config.mak.xxx'-file

   So, then the 'application/init' could contain the following lines:
   #include "boards/boards_common.h"   // general / common defines
   #include BOARD_INCLUDE_FILE         // should be defined via 'Config.mak.user' and 'the MAKE-system'

   and call the 'install-function' using the macro 'BOARDS_INSTALL_PLATFORM_SUPPORT', e.g.
   BOARDS_INSTALL_PLATFORM_SUPPORT(BOARDS_INIT_DEFAULT, 0);

   then changing the 'HW-platform/board' requires only to change the
   header-file specification in 'config.mak.xxx' and to re-compile the 'application/init'

   NOTE: even the 'boards-drivers' are stored in 'dedicaded directories'
         like '..\boards\fme_devkit'  or '..\boards\CustomerSpecific'
         when doing 'MAKE INSTALL', the board-header files like '..\boards\fme_devkit\mb86h61_dk01.h'
         are copied into '..\install\MB86H61\include\boards'
         so the 'dedicaded directory-name' like 'fme_devkit' is not relevant/used !
*/

#define USE_N_SHIFT_CLOCKS 32   // NOTE: we assmue 'ext.32 shift register' (on-board is's just 8-bit)

//-- defines for 'shift-register output-controls' --
#define GPO_SFTRG_CI_IN_ENA_BIT     BIT0
#define GPO_SFTRG_CI_IN_MUX_BIT     BIT1
#define GPO_SFTRG_CI_RESET_BIT      BIT2
#define GPO_SFTRG_SCART_PIN8_12V    BIT3
#define GPO_SFTRG_SCART_PIN8_5V     BIT4
#define GPO_SFTRG_SCART_PIN16       BIT5
#define GPO_SFTRG_USB_CBU0          BIT6
#define GPO_SFTRG_USB_CBU1          BIT7
//-- defines for 'GPIO-pins' --
#define GPIO_OP_FE_RES_A 12
#define GPIO_OP_FE_RES_B 26
#define GPIO_INP_CI_INS0 59
#define GPIN72_CI_IRQ 72
#define GPIO_UPI_IOWR       60
#define GPIO_UPI_IORD       61
#define GPIO_UPI_BUF_DIR    62
#define GPIO_UPI_CI_WAIT    63
#define GPIO_UPI_CLE        71
#define GPIO_INP_CI_IRQ     72


#define P55_WAKUP_TSO7  55
#define P56_TIC0RS_TSOY  56      // Dev.Kit J575-pin7 (FAPI_GPIO_OUT_ICC0_RST)
#define P57_TIC0VC_TSEN  57      // Dev.Kit J575-pin6 (FAPI_GPIO_OUT_ICC0_VCC)
#define P58_TIC0XX_TSCK  58      // 'old' data-in, not used anymore due to 'bi-directional'

//-- pin-functions in case 'old TDA8004 SmartCard-Adapter' connected to J575
#define OP4_TDA_ICC0_CLK    4   // Dev.Kit J575-pin10 (FAPI_GPIO_OUT_ICC0_CLK)
#define OP5_TDA_ICC0_VPEN   5   // Dev.Kit J575-pin9  (FAPI_GPIO_OUT_ICC0_VPEN)

//-- GPIO-Pins-usage on Dev.Kit for 'TDA-ICC' and 'dummy-TS-Output via Serializer' 
#define IP6_TIC0IN_SERIN  6      // Dev.Kit J575-pin5 (FAPI_GPIO_IN_ICC0_IN)
#define OP7_TIC0YY_SEROT  7      // Dev.Kit J575-pin2 ()
#define IO8_TIC0DA_SERCK  8      // Dev.Kit J575-pin8 (FAPI_GPIO_BIDI_ICC0_DATA)

//-- if TS-A-GPIO-Input taken directly from Front-End --
#define IP16FE_TSA_D7  16
#define IP17FE_TSA_D6  17
#define IP18FE_TSA_D5  18
#define IP19FE_TSA_D4  19
#define IP20FE_TSA_D3  20
#define IP21FE_TSA_D2  21
#define IP22FE_TSA_D1  22
#define IP23FE_TSA_D0  23
#define IP24FE_TSA_VAL 24
#define IP25FE_TSA_SYN 25
#define IP15FE_TSA_CLK 15
//-- if TS-A-GPIO-Input taken from CI-Module-Output (only used for 'old' non-CI+ concept) --
#define IP88CI_TSA_D7  88
#define IP89CI_TSA_D6  89
#define IP90CI_TSA_D5  90
#define IP91CI_TSA_D4  91
#define IP92CI_TSA_D3  92
#define IP93CI_TSA_D2  93
#define IP94CI_TSA_D1  94
#define IP95CI_TSA_D0  95
#define IP85CI_TSA_VAL 85
#define IP86CI_TSA_SYN 86
#define IP87CI_TSA_CLK 87
//-- pins for TS-B Input --
#define IP30FE_TSB_D7  30
#define IP31FE_TSB_D6  31
#define IP32FE_TSB_D5  32
#define IP33FE_TSB_D4  33
#define IP35FE_TSB_D3  35
#define IP34FE_TSB_D2  34
#define IP36FE_TSB_D1  36
#define IP37FE_TSB_D0  37
#define IP29FE_TSB_VAL 29
#define IP28FE_TSB_SYN 28
#define IP27FE_TSB_CLK 27
//-- pins for TS-C Input --
#define IP88FE_TSC_D7  88
#define IP89FE_TSC_D6  89
#define IP90FE_TSC_D5  90
#define IP91FE_TSC_D4  91
#define IP92FE_TSC_D3  92
#define IP93FE_TSC_D2  93
#define IP94FE_TSC_D1  94
#define IP95FE_TSC_D0  95
#define IP85FE_TSC_VAL 85
#define IP86FE_TSC_SYN 86
#define IP87FE_TSC_CLK 87

int32_t BOARD_FME_H61DK01_BoardIfc_Register(uint32_t registerFlags, void *optRegisterDataPtr);
int32_t BOARD_FME_H61DK01_BoardRegister_Mini(uint32_t registerFlags, void *optRegisterDataPtr);
void    BOARD_FME_H61DK01_BoardIfc_Register_CiSlave(void);
void    BOARD_H61_DK01_InitInstall_HwTest_IccDriver(uint32_t flagNativeOrTerridian);
//int32_t BOARD_FME_H61DK01_BoardIfc_Regi_DRT(uint32_t registerFlags, void *optRegisterDataPtr);

void    BOARD_FME_H61DK01_BoardIfc_SetAddInitHook(BOARDS_AddInitHookT addInitCbf);
void    BOARD_FME_H61DK01_BoardIfc_InstallSpecialHwTestFuncs(uint32_t flags);
int32_t BOARD_FME_H61DK01_RegisterFrontends(uint32_t registerFlags, void *optInstallDataPtr);
void    BOARD_FME_H61DK01_FrontPanel_Register(void);

#define BOARDS_MAGIC_GPIO_CTRL_DATA 0x12344321
typedef void (*BOARDS_GpioEventCallbackT)(uint32_t value);
typedef struct {
    uint32_t             MagicBoardsGpioCtrlData;
    BOARDS_GpioEventCallbackT  GpioEventCbf;
} BOARDS_GPIO_ControlDataT;

int32_t BOARD_FME_H61DK01_CntrlUserGpios
                 ( int32_t index, uint32_t CtrlCode, void* optPtr );

extern BOARDS_CommonSupportFunctionsT  *FME_H61TB01_CbsfPtr;   //
extern const char_t BOARD_FME_H61DK01_ModelStr_Initial[];
extern const char_t BOARD_FME_H61DK01_ModelStr4CiPlus[];

/*!
** \brief in case a 'further customer specific or extended install-function' exists
** \sa #BOARDS_INSTALL_PLATFORM_SUPPORT_EXT
*/
int32_t BOARD_FME_H61DK01_RegisterBoardIfc_CstExt(uint32_t registerFlags, void *optInstallDataPtr);

void     BOARD_FME_H61DK01_GetTestGpioInput(void *optPtr);
void     BOARD_FME_H61DK01_RegisterGpioPins(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction);
void     BOARD_FME_H61DK01_RSO_TsAInpSelect_Fe_or_TSG(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t inpSelFlag);
void     BOARD_FME_H61DK01_RSO_TsOut_ExtADac_TdaIcc (BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t modeSelFlag);
void     BOARD_FME_H61DK01_RSO_PFlash_or_SdIo        (BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t modeSel);
void     BOARD_FME_H61DK01_RSO_InitGpio_TdaSmartCard (BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction);
void     BOARD_FME_H61DK01_RSO_ConfigGpiosPAtaAdapter(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction);
int32_t  BOARD_FME_H61DK01_RSO_Cfg_GpioTsInp_2_CcirInp_NormOp(uint32_t TsIdx, uint32_t CcirIdx);
int32_t  BOARD_FME_H61DK01_RSO_Cfg_GpioTsInp_2_CcirInp_SyncOp(uint32_t TsIdx, uint32_t CcirIdx);
void     BOARD_FME_H61DK01_RSO_Cfg_Gpio_2_CcirInp_ClkCrtrl(uint32_t TsIdx, uint32_t ccir_idx, uint32_t disable_enable);
void     BOARD_FME_H61DK01_RSO_Cfg_SerTsOutpPins(uint32_t inp2ts, uint32_t serOutIdx, uint32_t ctrlCode );
void     BOARD_FME_H61DK01_RSO_DigitalRGB_or_RestoreDefaults(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t modeSelFlag);
int32_t  BOARD_FME_H61DK01_SetupCiLibHwInterfaceFunctions(void);
int32_t  BOARD_FME_H61DK01_GetTsPinsInputLevel(BOARDS_FMEDEF_RD_GPIO_INPREG_CBF  RdGpioInpFnc,
                                              int32_t tsPathIndex,
                                              uint16_t *dstTsInpLevelsPtr);
void BOARD_FME_H61DK01_RegisterSpecialIrDataInfo(void);

#define BOARDS_INSTALL_MIN_PLATFORM_SUPPORT  BOARD_FME_H61DK01_BoardRegister_Mini           // special purpose
#define BOARDS_INSTALL_PLATFORM_SUPPORT      BOARD_FME_H61DK01_BoardIfc_Register            // normal case
#define BOARDS_INSTALL_PLATFORM_SUPPORT_CISL BOARD_FME_H61DK01_BoardIfc_Register_CiSlave    // for special test-program
#define BOARDS_INSTALL_PLATFORM_SUPPORT_EXT  BOARD_FME_H61DK01_BoardIfc_Register_CstExt     // special customer-purpose

#define BOARDS_INSTALL_FRONTEND_SUPPORT     BOARD_FME_H61DK01_RegisterFrontends        //!< register supported frontends

#ifdef __cplusplus
}
#endif

/*
********************************************************************************
** End of file
********************************************************************************
*/
#endif
