/*!
********************************************************************************
**
** \file        ./boards/fme/mb86h61_dk01/mb86h61_dk01_ccir.c
**
** \version     $Id: mb86h61_dk01_ccir.c 3333 2010-09-21 08:20:33Z ebende $
**
** \brief       functions to configure CCIR-GPIOs for 'PVR work-around'
**
** \note        there exist two methods
**              a) via ..\fapi\sys\src\sys_init_mb86h61_devkit.h
**              b) using this 'baords' driver
**              Actually method b) should be preferred. If the file which includes
**              '<fapi/sys_init.h>' has the define
**              '#define FAPI_SYS_INIT_USE_BOARDS 1', then the GPIO-registration
**              form 'sys_init_mb86h61_devkit.h' will be disabled
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/

#include <fapi/reg_gpio.h>
#include <fapi/drv_gpio.h>

#include <boards/boards_common.h>       // some defines needed for 'mb86h61_dk01.h'
#include <boards/boards_com_dirx.h>     // functions for 'fast direct register access'
#include <boards/mb86h61_dk01.h>

#define PIN_CFG_INP_FUNC  BRDCOM_DX_ConfigureInputGpios   // direct (no-FREG-Lib)
//#define PIN_CFG_INP_FUNC    BRDCOM_BC_ConfigureInputGpios   // will need FREG-Lib

/*lint -save -e923 */

//================================================
//==    'straight' connections                  ==
//================================================
// inpPinNumP2,     inpCfg
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsA2Ccir0_normal[] = {
 { 2+IP24FE_TSA_VAL,BDEF_GPIO_IN_CCIR0_EXT_EN  }
,{ 2+IP15FE_TSA_CLK,BDEF_GPIO_IN_CCIR0_CLK     }
,{ 2+IP16FE_TSA_D7, BDEF_GPIO_IN_CCIR0_DATA_7  }
,{ 2+IP17FE_TSA_D6, BDEF_GPIO_IN_CCIR0_DATA_6  }
,{ 2+IP18FE_TSA_D5, BDEF_GPIO_IN_CCIR0_DATA_5  }
,{ 2+IP19FE_TSA_D4, BDEF_GPIO_IN_CCIR0_DATA_4  }
,{ 2+IP20FE_TSA_D3, BDEF_GPIO_IN_CCIR0_DATA_3  }
,{ 2+IP21FE_TSA_D2, BDEF_GPIO_IN_CCIR0_DATA_2  }
,{ 2+IP22FE_TSA_D1, BDEF_GPIO_IN_CCIR0_DATA_1  }
,{ 2+IP23FE_TSA_D0, BDEF_GPIO_IN_CCIR0_DATA_0  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsA2Ccir0_syncmode[] = {
 { 2+IP24FE_TSA_VAL,BDEF_GPIO_IN_CCIR0_EXT_EN  }
,{ 2+IP15FE_TSA_CLK,BDEF_GPIO_IN_CCIR0_CLK     }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_7  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_6  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_5  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_4  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_3  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_2  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_1  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR0_DATA_0  }
};
#define N_PINS_CAPTURE (sizeof(InpPins_TsA2Ccir0_normal) / sizeof(BRDCOM_DX_GpioInputConfigDataT))

// inpPinNumP2,     inpCfg
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsB2Ccir1_normal[] = {
 { 2+IP29FE_TSB_VAL,BDEF_GPIO_IN_CCIR1_EXT_EN  }
,{ 2+IP27FE_TSB_CLK,BDEF_GPIO_IN_CCIR1_CLK     }
,{ 2+IP30FE_TSB_D7, BDEF_GPIO_IN_CCIR1_DATA_7  }
,{ 2+IP31FE_TSB_D6, BDEF_GPIO_IN_CCIR1_DATA_6  }
,{ 2+IP32FE_TSB_D5, BDEF_GPIO_IN_CCIR1_DATA_5  }
,{ 2+IP33FE_TSB_D4, BDEF_GPIO_IN_CCIR1_DATA_4  }
,{ 2+IP35FE_TSB_D3, BDEF_GPIO_IN_CCIR1_DATA_3  }
,{ 2+IP34FE_TSB_D2, BDEF_GPIO_IN_CCIR1_DATA_2  }
,{ 2+IP36FE_TSB_D1, BDEF_GPIO_IN_CCIR1_DATA_1  }
,{ 2+IP37FE_TSB_D0, BDEF_GPIO_IN_CCIR1_DATA_0  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsB2Ccir1_syncmode[] = {
 { 2+IP29FE_TSB_VAL,BDEF_GPIO_IN_CCIR1_EXT_EN  }
,{ 2+IP27FE_TSB_CLK,BDEF_GPIO_IN_CCIR1_CLK     }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_7  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_6  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_5  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_4  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_3  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_2  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_1  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR1_DATA_0  }
};

// inpPinNumP2,     inpCfg
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsC2Ccir1_normal[] = {
 { 2+IP85FE_TSC_VAL,BDEF_GPIO_IN_CCIR1_EXT_EN  }
,{ 2+IP87FE_TSC_CLK,BDEF_GPIO_IN_CCIR1_CLK     }
,{ 2+IP88FE_TSC_D7, BDEF_GPIO_IN_CCIR1_DATA_7  }
,{ 2+IP89FE_TSC_D6, BDEF_GPIO_IN_CCIR1_DATA_6  }
,{ 2+IP90FE_TSC_D5, BDEF_GPIO_IN_CCIR1_DATA_5  }
,{ 2+IP91FE_TSC_D4, BDEF_GPIO_IN_CCIR1_DATA_4  }
,{ 2+IP92FE_TSC_D3, BDEF_GPIO_IN_CCIR1_DATA_3  }
,{ 2+IP93FE_TSC_D2, BDEF_GPIO_IN_CCIR1_DATA_2  }
,{ 2+IP94FE_TSC_D1, BDEF_GPIO_IN_CCIR1_DATA_1  }
,{ 2+IP95FE_TSC_D0, BDEF_GPIO_IN_CCIR1_DATA_0  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsC2Ccir1_syncmode[] = {
 { 2+IP85FE_TSC_VAL,BDEF_GPIO_IN_CCIR1_EXT_EN  }
,{ 2+IP87FE_TSC_CLK,BDEF_GPIO_IN_CCIR1_CLK     }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_7  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_6  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_5  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_4  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_3  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_2  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_1  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR1_DATA_0  }
};

//================================================
//==    'cross' connections                     ==
//================================================
// inpPinNumP2,     inpCfg
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsA2Ccir1_normal[] = {
 { 2+IP24FE_TSA_VAL,BDEF_GPIO_IN_CCIR1_EXT_EN  }
,{ 2+IP15FE_TSA_CLK,BDEF_GPIO_IN_CCIR1_CLK     }
,{ 2+IP16FE_TSA_D7, BDEF_GPIO_IN_CCIR1_DATA_7  }
,{ 2+IP17FE_TSA_D6, BDEF_GPIO_IN_CCIR1_DATA_6  }
,{ 2+IP18FE_TSA_D5, BDEF_GPIO_IN_CCIR1_DATA_5  }
,{ 2+IP19FE_TSA_D4, BDEF_GPIO_IN_CCIR1_DATA_4  }
,{ 2+IP20FE_TSA_D3, BDEF_GPIO_IN_CCIR1_DATA_3  }
,{ 2+IP21FE_TSA_D2, BDEF_GPIO_IN_CCIR1_DATA_2  }
,{ 2+IP22FE_TSA_D1, BDEF_GPIO_IN_CCIR1_DATA_1  }
,{ 2+IP23FE_TSA_D0, BDEF_GPIO_IN_CCIR1_DATA_0  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsA2Ccir1_syncmode[] = {
 { 2+IP24FE_TSA_VAL,BDEF_GPIO_IN_CCIR1_EXT_EN  }
,{ 2+IP15FE_TSA_CLK,BDEF_GPIO_IN_CCIR1_CLK     }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_7  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_6  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_5  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_4  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_3  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_2  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_1  }
,{ 2+IP25FE_TSA_SYN,BDEF_GPIO_IN_CCIR1_DATA_0  }
};

// inpPinNumP2,     inpCfg
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsB2Ccir0_normal[] = {
 { 2+IP29FE_TSB_VAL,BDEF_GPIO_IN_CCIR0_EXT_EN  }
,{ 2+IP27FE_TSB_CLK,BDEF_GPIO_IN_CCIR0_CLK     }
,{ 2+IP30FE_TSB_D7, BDEF_GPIO_IN_CCIR0_DATA_7  }
,{ 2+IP31FE_TSB_D6, BDEF_GPIO_IN_CCIR0_DATA_6  }
,{ 2+IP32FE_TSB_D5, BDEF_GPIO_IN_CCIR0_DATA_5  }
,{ 2+IP33FE_TSB_D4, BDEF_GPIO_IN_CCIR0_DATA_4  }
,{ 2+IP35FE_TSB_D3, BDEF_GPIO_IN_CCIR0_DATA_3  }
,{ 2+IP34FE_TSB_D2, BDEF_GPIO_IN_CCIR0_DATA_2  }
,{ 2+IP36FE_TSB_D1, BDEF_GPIO_IN_CCIR0_DATA_1  }
,{ 2+IP37FE_TSB_D0, BDEF_GPIO_IN_CCIR0_DATA_0  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsB2Ccir0_syncmode[] = {
 { 2+IP29FE_TSB_VAL,BDEF_GPIO_IN_CCIR0_EXT_EN  }
,{ 2+IP27FE_TSB_CLK,BDEF_GPIO_IN_CCIR0_CLK     }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_7  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_6  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_5  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_4  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_3  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_2  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_1  }
,{ 2+IP28FE_TSB_SYN,BDEF_GPIO_IN_CCIR0_DATA_0  }
};

// inpPinNumP2,     inpCfg
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsC2Ccir0_normal[] = {
 { 2+IP85FE_TSC_VAL,BDEF_GPIO_IN_CCIR0_EXT_EN  }
,{ 2+IP87FE_TSC_CLK,BDEF_GPIO_IN_CCIR0_CLK     }
,{ 2+IP88FE_TSC_D7, BDEF_GPIO_IN_CCIR0_DATA_7  }
,{ 2+IP89FE_TSC_D6, BDEF_GPIO_IN_CCIR0_DATA_6  }
,{ 2+IP90FE_TSC_D5, BDEF_GPIO_IN_CCIR0_DATA_5  }
,{ 2+IP91FE_TSC_D4, BDEF_GPIO_IN_CCIR0_DATA_4  }
,{ 2+IP92FE_TSC_D3, BDEF_GPIO_IN_CCIR0_DATA_3  }
,{ 2+IP93FE_TSC_D2, BDEF_GPIO_IN_CCIR0_DATA_2  }
,{ 2+IP94FE_TSC_D1, BDEF_GPIO_IN_CCIR0_DATA_1  }
,{ 2+IP95FE_TSC_D0, BDEF_GPIO_IN_CCIR0_DATA_0  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_TsC2Ccir0_syncmode[] = {
 { 2+IP85FE_TSC_VAL,BDEF_GPIO_IN_CCIR0_EXT_EN  }
,{ 2+IP87FE_TSC_CLK,BDEF_GPIO_IN_CCIR0_CLK     }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_7  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_6  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_5  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_4  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_3  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_2  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_1  }
,{ 2+IP86FE_TSC_SYN,BDEF_GPIO_IN_CCIR0_DATA_0  }
};
int32_t BOARD_FME_H61DK01_RSO_Cfg_GpioTsInp_2_CcirInp_NormOp(uint32_t TsIdx, uint32_t CcirIdx)
{
    int32_t ret_val = 0;
    if      (TsIdx==0) {                            // related to TS-A
        if  (CcirIdx==0) {                          // -> CCIR0
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsA2Ccir0_normal); }
        else {
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsA2Ccir1_normal); }
        ret_val = IP24FE_TSA_VAL;
    }
    else if (TsIdx==1) {                            // related to TS-B
        if  (CcirIdx==0) {                          // -> CCIR0
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsB2Ccir0_normal); }
        else {                                      // -> CCIR1
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsB2Ccir1_normal); }
        ret_val = IP29FE_TSB_VAL;
    }
    else if (TsIdx==2) {                            // related to TS-C
        if  (CcirIdx==0) {                          // -> CCIR0
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsC2Ccir0_normal); }
        else {                                      // -> CCIR1
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsC2Ccir1_normal); }
        ret_val = IP85FE_TSC_VAL;
    }
    return ret_val;
}
int32_t BOARD_FME_H61DK01_RSO_Cfg_GpioTsInp_2_CcirInp_SyncOp(uint32_t TsIdx, uint32_t CcirIdx)
{
    int32_t ret_val = 0;
    if      (TsIdx==0) {                            // related to TS-A
        if  (CcirIdx==0) {                          // -> CCIR0
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsA2Ccir0_syncmode); }
        else {                                      // -> CCIR1
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsA2Ccir1_syncmode); }
        ret_val = IP25FE_TSA_SYN;
    }
    else if (TsIdx==1) {                            // related to TS-B
        if  (CcirIdx==0) {                          // -> CCIR0
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsB2Ccir0_syncmode); }
        else {
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsB2Ccir1_syncmode); }
        ret_val = IP28FE_TSB_SYN;
    }
    else if (TsIdx==2) {                            // related to TS-C
        if  (CcirIdx==0) {                          // -> CCIR0
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsC2Ccir0_syncmode); }
        else {
            PIN_CFG_INP_FUNC(N_PINS_CAPTURE, InpPins_TsC2Ccir1_syncmode); }
        ret_val = IP86FE_TSC_SYN;
    }
    return ret_val;
}

void BOARD_FME_H61DK01_RSO_Cfg_Gpio_2_CcirInp_ClkCrtrl(uint32_t TsIdx, uint32_t ccir_idx, uint32_t flg_enable)
{
    volatile uint32_t * inpCfgReg;
    uint32_t regBase;
    uint32_t from_inp_pin;
    uint32_t func;

    func = 0;                           // default: no action
    from_inp_pin = 0;                   // defautl: from 'virtual always-0 input-pin' -> disable
    if      (TsIdx==0) {
        if  (ccir_idx==0) {
            func = BDEF_GPIO_IN_CCIR0_CLK;
            if (flg_enable) {               // if 'enable'
                from_inp_pin  = 2+IP15FE_TSA_CLK; }
        }
        else {
            func = BDEF_GPIO_IN_CCIR1_CLK;
            if (flg_enable) {               // if 'enable'
                from_inp_pin  = 2+IP15FE_TSA_CLK; }
        }
    }
    else if (TsIdx==1) {
        if  (ccir_idx==0) {
            func = BDEF_GPIO_IN_CCIR0_CLK;
            if (flg_enable) {               // if 'enable'
                from_inp_pin  = 2+IP27FE_TSB_CLK; }
        }
        else {
            func = BDEF_GPIO_IN_CCIR1_CLK;
            if (flg_enable) {               // if 'enable'
                from_inp_pin  = 2+IP27FE_TSB_CLK; }
        }
    }
    else if (TsIdx==2) {
        if  (ccir_idx==0) {
            func = BDEF_GPIO_IN_CCIR0_CLK;
            if (flg_enable) {               // if 'enable'
                from_inp_pin  = 2+IP87FE_TSC_CLK; }
        }
        else {
            func = BDEF_GPIO_IN_CCIR1_CLK;
            if (flg_enable) {               // if 'enable'
                from_inp_pin  = 2+IP87FE_TSC_CLK; }
        }
    }

    if (func) {
        regBase = FREG_GPIO_DATA_IN + (func*4);
        inpCfgReg = (volatile uint32_t *) regBase;
        *inpCfgReg = from_inp_pin;
    }
}

