/*!
********************************************************************************
**
** \file        ./boards/fme/mb86h61_dk01/mb86h61_dk01_gpio.c
**
** \version     $Id: mb86h61_dk01_gpio.c 5986 2011-03-31 18:35:29Z jrende $
**
** \brief       function to initialize the GPIO-configuration registers
**
** \note        there exist two methods
**              a) via ..\fapi\sys\src\sys_init_mb86h61_devkit.h
**              b) using this 'baords' driver
**              Actually method b) should be preferred. If the file which includes
**              '<fapi/sys_init.h>' has the define
**              '#define FAPI_SYS_INIT_USE_BOARDS 1', then the GPIO-registration
**              form 'sys_init_mb86h61_devkit.h' will be disabled
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/

#define SUPPORT_DIGIT_RGB_OUTPUT

#include <fapi/reg_gpio.h>
#include <fapi/drv_gpio.h>
//#include <fapi/drv_ir.h>
//#include <fapi/board_ifc.h>

#include <boards/boards_common.h>          // some defines needed for 'mb86h61_dk01.h'
#include <boards/boards_com_dirx.h>
#include <boards/mb86h61_dk01.h>

#define PIN_CFG_INP_FUNC  BRDCOM_DX_ConfigureInputGpios   // direct (no-FREG-Lib)
//#define PIN_CFG_INP_FUNC    BRDCOM_BC_ConfigureInputGpios   // will need FREG-Lib

#define PIN_CFG_OUT_FUNC  BRDCOM_DX_ConfigureOutputGpios   // direct (no-FREG-Lib)
//#define PIN_CFG_OUT_FUNC    BRDCOM_BC_ConfigureOutputGpios   // will need FREG-Lib

#define NO_DIG_RGB_OR_CI_SLAVE_TEST     // comment this out when using 'digital RBG-signals'
                                        // or UPI-Slave,CAM-Emulation Test

//#define USE_P_ATA_ADAPTER             // active when using P-Ata-Adapter

//============================================================================
//== some GPIO-usage defintions for 'dynamic / resource shared GPIO pins    ==
//============================================================================
typedef struct {
  uint32_t pinNum;
  uint32_t config;
} GPIO_CONFIG_INFO_T;

static  GPIO_CONFIG_INFO_T  GpioConfig_TsAInput_FromFe[] = {
 { IP16FE_TSA_D7,  FAPI_GPIO_IN_TS_A_DATA_7  }
,{ IP17FE_TSA_D6,  FAPI_GPIO_IN_TS_A_DATA_6  }
,{ IP18FE_TSA_D5,  FAPI_GPIO_IN_TS_A_DATA_5  }
,{ IP19FE_TSA_D4,  FAPI_GPIO_IN_TS_A_DATA_4  }
,{ IP20FE_TSA_D3,  FAPI_GPIO_IN_TS_A_DATA_3  }
,{ IP21FE_TSA_D2,  FAPI_GPIO_IN_TS_A_DATA_2  }
,{ IP22FE_TSA_D1,  FAPI_GPIO_IN_TS_A_DATA_1  }
,{ IP23FE_TSA_D0,  FAPI_GPIO_IN_TS_A_DATA_0  }
,{ IP24FE_TSA_VAL, FAPI_GPIO_IN_TS_A_EN      }
,{ IP25FE_TSA_SYN, FAPI_GPIO_IN_TS_A_PSTART  }
,{ IP15FE_TSA_CLK, FAPI_GPIO_IN_TS_A_CLK     }
};

static  GPIO_CONFIG_INFO_T  GpioConfig_TsAInput_FromCi[] = {    // only used for 'old concept' (not CI+)
 { IP88CI_TSA_D7,  FAPI_GPIO_IN_TS_A_DATA_7  }
,{ IP89CI_TSA_D6,  FAPI_GPIO_IN_TS_A_DATA_6  }
,{ IP90CI_TSA_D5,  FAPI_GPIO_IN_TS_A_DATA_5  }
,{ IP91CI_TSA_D4,  FAPI_GPIO_IN_TS_A_DATA_4  }
,{ IP92CI_TSA_D3,  FAPI_GPIO_IN_TS_A_DATA_3  }
,{ IP93CI_TSA_D2,  FAPI_GPIO_IN_TS_A_DATA_2  }
,{ IP94CI_TSA_D1,  FAPI_GPIO_IN_TS_A_DATA_1  }
,{ IP95CI_TSA_D0,  FAPI_GPIO_IN_TS_A_DATA_0  }
,{ IP85CI_TSA_VAL, FAPI_GPIO_IN_TS_A_EN      }
,{ IP86CI_TSA_SYN, FAPI_GPIO_IN_TS_A_PSTART  }
,{ IP87CI_TSA_CLK, FAPI_GPIO_IN_TS_A_CLK     }
};

static  GPIO_CONFIG_INFO_T  GpioConfig_TdaSmartCard[] = {
// NOTE: 1st-line is 'just dummy' to remove 'output from 'FAPI_GPIO_OUT_TS_CLK'
 { P58_TIC0XX_TSCK,   FAPI_GPIO_BIDI_ICC0_DATA }    // 1st-line
,{ P57_TIC0VC_TSEN,   FAPI_GPIO_OUT_ICC0_VCC   }
,{ P56_TIC0RS_TSOY,   FAPI_GPIO_OUT_ICC0_RST   }
,{ OP4_TDA_ICC0_CLK,  FAPI_GPIO_OUT_ICC0_CLK   }
,{ OP5_TDA_ICC0_VPEN, FAPI_GPIO_OUT_ICC0_VPEN  }
,{ IP6_TIC0IN_SERIN,  FAPI_GPIO_IN_ICC0_IN     }
,{ IO8_TIC0DA_SERCK,  FAPI_GPIO_BIDI_ICC0_DATA }
};

static  GPIO_CONFIG_INFO_T  GpioConfig_4TsOutput[] = {
 { 48, FAPI_GPIO_OUT_TS_DATA_0   }
,{ 49, FAPI_GPIO_OUT_TS_DATA_1   }
,{ 50, FAPI_GPIO_OUT_TS_DATA_2   }
,{ 51, FAPI_GPIO_OUT_TS_DATA_3   }
,{ 52, FAPI_GPIO_OUT_TS_DATA_4   }
,{ 53, FAPI_GPIO_OUT_TS_DATA_5   }
,{ 54, FAPI_GPIO_OUT_TS_DATA_6   }
,{ P55_WAKUP_TSO7,  FAPI_GPIO_OUT_TS_DATA_7   }
,{ P56_TIC0RS_TSOY, FAPI_GPIO_OUT_TS_PSTART   }
,{ P57_TIC0VC_TSEN, FAPI_GPIO_OUT_TS_EN       }
,{ P58_TIC0XX_TSCK, FAPI_GPIO_OUT_TS_CLK      }
};
static  GPIO_CONFIG_INFO_T  GpioConfig_4ExtAudioDac[] = {
 { 48, FAPI_GPIO_OUT_0             }
,{ 49, FAPI_GPIO_OUT_AD3_DATA      }
,{ 50, FAPI_GPIO_OUT_AD2_DATA      }
,{ 51, FAPI_GPIO_OUT_AD1_DATA      }
,{ 52, FAPI_GPIO_OUT_AD0_DATA      }
,{ 53, FAPI_GPIO_OUT_AD0_LRCLK     }
,{ 54, FAPI_GPIO_OUT_AD0_BCLK      }
,{ P55_WAKUP_TSO7, FAPI_GPIO_OUT_AD0_CLK       }    // NOTE: also used for 'wakeup-reason' P55_WUP_TSO
};

static  GPIO_CONFIG_INFO_T  GpioConfig_4PFlash[] = {
 { 65, FAPI_GPIO_OUT_UPI_ADDRX0  }
,{ 66, FAPI_GPIO_OUT_UPI_ADDRX1  }
,{ 67, FAPI_GPIO_OUT_UPI_ADDRX2  }
,{ 68, FAPI_GPIO_OUT_UPI_ADDRX3  }
};
static  GPIO_CONFIG_INFO_T  GpioConfig_4SdIo[] = {
 { 65, FAPI_GPIO_BIDI_SDIO_DATA_3  }
,{ 66, FAPI_GPIO_BIDI_SDIO_DATA_2  }
,{ 67, FAPI_GPIO_BIDI_SDIO_DATA_1  }
,{ 68, FAPI_GPIO_IN_SDIO_CD        }
};

//------------------------------------------------------------------------------------
//--  pin-functions in case 'dummy-TS-Output via Serializer' GPIO-Pins on Dev.Kit   --
//------------------------------------------------------------------------------------
static  BRDCOM_DX_GpioOutputConfigDataT  OutPins_SerTsLoop[] = {
// pinNum,           upperBits,         progValue
 { IO8_TIC0DA_SERCK, 0,                 BDEF_GPIO_OUT_LCD_CLK   } // -> Dev.Kit J575-pin8
,{ OP7_TIC0YY_SEROT, 0,                 BDEF_GPIO_OUT_HD_DE } // -> Dev.Kit J575-pin2
};
#define N_SER_OUT_PINS (sizeof(OutPins_SerTsLoop) / sizeof(BRDCOM_DX_GpioOutputConfigDataT))

static  BRDCOM_DX_GpioInputConfigDataT  InpPins_SerTsLoop4TsA[] = { // NOTE: Input not really used
// inpPinNumP2      , inpCfg
 { 2+IP6_TIC0IN_SERIN,BDEF_GPIO_IN_IR_DATA   } // -> Dev.Kit J575-pin5
,{ 2+IO8_TIC0DA_SERCK,BDEF_GPIO_IN_TS_A_CLK     } //
,{ 1                , BDEF_GPIO_IN_TS_A_EN      } // (fixed '1' level)
,{ 1                , BDEF_GPIO_IN_TS_A_PSTART  } // (fixed '1' level)
,{ 2+OP7_TIC0YY_SEROT,BDEF_GPIO_IN_TS_A_DATA_0  } //
};

static  BRDCOM_DX_GpioInputConfigDataT  InpPins_SerTsLoop4TsB[] = { // NOTE: Input not really used
// inpPinNumP2      , inpCfg
 { 2+IP6_TIC0IN_SERIN,BDEF_GPIO_IN_IR_DATA   } // -> Dev.Kit J575-pin5
,{ 2+IO8_TIC0DA_SERCK,BDEF_GPIO_IN_TS_B_CLK     } //
,{ 1                , BDEF_GPIO_IN_TS_B_EN      } // (fixed '1' level)
,{ 1                , BDEF_GPIO_IN_TS_B_PSTART  } // (fixed '1' level)
,{ 2+OP7_TIC0YY_SEROT,BDEF_GPIO_IN_TS_B_DATA_0  } //
};

static  BRDCOM_DX_GpioInputConfigDataT  InpPins_SerTsLoop4TsC[] = { // NOTE: Input not really used
// inpPinNumP2      ,   inpCfg
 { 2+IP6_TIC0IN_SERIN,BDEF_GPIO_IN_IR_DATA   } // -> Dev.Kit J575-pin5
,{ 2+IO8_TIC0DA_SERCK,BDEF_GPIO_IN_TS_C_CLK     } //
,{ 1                , BDEF_GPIO_IN_TS_C_EN      } // (fixed '1' level)
,{ 1                , BDEF_GPIO_IN_TS_C_PSTART  } // (fixed '1' level)
,{ 2+OP7_TIC0YY_SEROT,BDEF_GPIO_IN_TS_C_DATA_0  } //
};

#define N_SR_INP_PINS (sizeof(InpPins_SerTsLoop4TsA) / sizeof(BRDCOM_DX_GpioInputConfigDataT))

static  BRDCOM_DX_GpioInputConfigDataT  InpPins_ParTsLoop4TsA[] = {
// inpPinNumP2     ,   inpCfg
   { 2+IP15FE_TSA_CLK,   BDEF_GPIO_IN_TS_A_CLK     }
  ,{ 2+IP24FE_TSA_VAL,   BDEF_GPIO_IN_TS_A_EN      }
  ,{ 2+IP25FE_TSA_SYN,   BDEF_GPIO_IN_TS_A_PSTART  }
  ,{ 2+IP23FE_TSA_D0 ,   BDEF_GPIO_IN_TS_A_DATA_0  }
//,{ 2+IP22FE_TSA_D1 ,   BDEF_GPIO_IN_TS_A_DATA_1  }
//,{ 2+IP21FE_TSA_D2 ,   BDEF_GPIO_IN_TS_A_DATA_2  }
//,{ 2+IP20FE_TSA_D3 ,   BDEF_GPIO_IN_TS_A_DATA_3  }
//,{ 2+IP19FE_TSA_D4 ,   BDEF_GPIO_IN_TS_A_DATA_4  }
//,{ 2+IP18FE_TSA_D5 ,   BDEF_GPIO_IN_TS_A_DATA_5  }
//,{ 2+IP17FE_TSA_D6 ,   BDEF_GPIO_IN_TS_A_DATA_6  }
//,{ 2+IP16FE_TSA_D7 ,   BDEF_GPIO_IN_TS_A_DATA_7  }
};

static  BRDCOM_DX_GpioInputConfigDataT  InpPins_ParTsLoop4TsB[] = {
// inpPinNumP2     ,   inpCfg
   { 2+IP27FE_TSB_CLK,   BDEF_GPIO_IN_TS_B_CLK     }
  ,{ 2+IP29FE_TSB_VAL,   BDEF_GPIO_IN_TS_B_EN      }
  ,{ 2+IP28FE_TSB_SYN,   BDEF_GPIO_IN_TS_B_PSTART  }
  ,{ 2+IP37FE_TSB_D0 ,   BDEF_GPIO_IN_TS_B_DATA_0  }
//,{ 2+IP36FE_TSB_D1 ,   BDEF_GPIO_IN_TS_B_DATA_1  }
//,{ 2+IP34FE_TSB_D2 ,   BDEF_GPIO_IN_TS_B_DATA_2  }
//,{ 2+IP35FE_TSB_D3 ,   BDEF_GPIO_IN_TS_B_DATA_3  }
//,{ 2+IP33FE_TSB_D4 ,   BDEF_GPIO_IN_TS_B_DATA_4  }
//,{ 2+IP32FE_TSB_D5 ,   BDEF_GPIO_IN_TS_B_DATA_5  }
//,{ 2+IP31FE_TSB_D6 ,   BDEF_GPIO_IN_TS_B_DATA_6  }
//,{ 2+IP30FE_TSB_D7 ,   BDEF_GPIO_IN_TS_B_DATA_7  }
};

static  BRDCOM_DX_GpioInputConfigDataT  InpPins_ParTsLoop4TsC[] = {
// inpPinNumP2     ,   inpCfg
   { 2+IP87FE_TSC_CLK,   BDEF_GPIO_IN_TS_C_CLK     }
  ,{ 2+IP85FE_TSC_VAL,   BDEF_GPIO_IN_TS_C_EN      }
  ,{ 2+IP86FE_TSC_SYN,   BDEF_GPIO_IN_TS_C_PSTART  }
  ,{ 2+IP95FE_TSC_D0 ,   BDEF_GPIO_IN_TS_C_DATA_0  }
//,{ 2+IP94FE_TSC_D1 ,   BDEF_GPIO_IN_TS_C_DATA_1  }
//,{ 2+IP93FE_TSC_D2 ,   BDEF_GPIO_IN_TS_C_DATA_2  }
//,{ 2+IP92FE_TSC_D3 ,   BDEF_GPIO_IN_TS_C_DATA_3  }
//,{ 2+IP91FE_TSC_D4 ,   BDEF_GPIO_IN_TS_C_DATA_4  }
//,{ 2+IP90FE_TSC_D5 ,   BDEF_GPIO_IN_TS_C_DATA_5  }
//,{ 2+IP89FE_TSC_D6 ,   BDEF_GPIO_IN_TS_C_DATA_6  }
//,{ 2+IP88FE_TSC_D7 ,   BDEF_GPIO_IN_TS_C_DATA_7  }
};

#define N_PR_INP_PINS (sizeof(InpPins_ParTsLoop4TsA) / sizeof(BRDCOM_DX_GpioInputConfigDataT))

//-- config-data to disable TS-Inputs (for test purpose) --
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_DisableParTsTsA[] = {
// inpPinNumP2,inpCfg
 { 0,          BDEF_GPIO_IN_TS_A_CLK     }
,{ 0,          BDEF_GPIO_IN_TS_A_EN      }
,{ 0,          BDEF_GPIO_IN_TS_A_PSTART  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_0  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_1  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_2  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_3  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_4  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_5  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_6  }
,{ 0,          BDEF_GPIO_IN_TS_A_DATA_7  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_DisableParTsTsB[] = {
// inpPinNumP2,inpCfg
 { 0,          BDEF_GPIO_IN_TS_B_CLK     }
,{ 0,          BDEF_GPIO_IN_TS_B_EN      }
,{ 0,          BDEF_GPIO_IN_TS_B_PSTART  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_0  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_1  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_2  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_3  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_4  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_5  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_6  }
,{ 0,          BDEF_GPIO_IN_TS_B_DATA_7  }
};
static  BRDCOM_DX_GpioInputConfigDataT  InpPins_DisableParTsTsC[] = {
// inpPinNumP2,inpCfg
 { 0,          BDEF_GPIO_IN_TS_C_CLK     }
,{ 0,          BDEF_GPIO_IN_TS_C_EN      }
,{ 0,          BDEF_GPIO_IN_TS_C_PSTART  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_0  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_1  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_2  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_3  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_4  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_5  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_6  }
,{ 0,          BDEF_GPIO_IN_TS_C_DATA_7  }
};
#define N_PAR_DIS_INP_PINS (sizeof(InpPins_DisableParTsTsA) / sizeof(BRDCOM_DX_GpioInputConfigDataT))

//------------------------------------------------------------------------------------
// NOTE: the P-ATA-Adapter needs to re-configure some GPIO-pins otherwise used for CI-Interface
//------------------------------------------------------------------------------------
#ifdef USE_P_ATA_ADAPTER
  #define PIN_PATA_IRQ    (GPIO_UPI_CLE)
  #define PIN_UPI_WAIT    (GPIO_UPI_CI_WAIT)
  #define PIN_UPI_DMARQ   (GPIO_UPI_IORD)         //
  #define PIN_UPI_DMAAC   (GPIO_UPI_IOWR)         //
  #define PIN_UPI_CS2     (GPIO_UPI_BUF_DIR)      // this will be used as HD-ChipSelect

  static  GPIO_CONFIG_INFO_T  GpioConfig_Pins4PAtaTest[] = {
  /* { PIN_PATA_IRQ,  FAPI_GPIO_IN_IRQ_IN0   }
  ,*/{ PIN_UPI_WAIT,  FAPI_GPIO_IN_UPI_WAIT    }
  ,{ PIN_UPI_DMARQ, FAPI_GPIO_IN_UPI_DMARQ   }
  ,{ PIN_UPI_DMAAC, FAPI_GPIO_OUT_UPI_CLE    }  // note: actuall 'DMACK'
  ,{ PIN_UPI_CS2,   FAPI_GPIO_OUT_UPI_CS     }
  };
#endif


//============================================================================
//== default GPIO-usage defintions (before resource share functions used)   ==
//============================================================================
static  GPIO_CONFIG_INFO_T  BOARD_FME_H61DK01_GpioConfig[] = {
 { IP16FE_TSA_D7, FAPI_GPIO_IN_TS_A_DATA_7    }
,{ IP17FE_TSA_D6, FAPI_GPIO_IN_TS_A_DATA_6    }
,{ IP18FE_TSA_D5, FAPI_GPIO_IN_TS_A_DATA_5    }
,{ IP19FE_TSA_D4, FAPI_GPIO_IN_TS_A_DATA_4    }
,{ IP20FE_TSA_D3, FAPI_GPIO_IN_TS_A_DATA_3    }
,{ IP21FE_TSA_D2, FAPI_GPIO_IN_TS_A_DATA_2    }
,{ IP22FE_TSA_D1, FAPI_GPIO_IN_TS_A_DATA_1    }
,{ IP23FE_TSA_D0, FAPI_GPIO_IN_TS_A_DATA_0    }
,{ IP24FE_TSA_VAL,FAPI_GPIO_IN_TS_A_EN        }
,{ IP25FE_TSA_SYN,FAPI_GPIO_IN_TS_A_PSTART    }
,{ IP15FE_TSA_CLK,FAPI_GPIO_IN_TS_A_CLK       }

,{ IP30FE_TSB_D7, FAPI_GPIO_IN_TS_B_DATA_7 }
,{ IP31FE_TSB_D6, FAPI_GPIO_IN_TS_B_DATA_6 }
,{ IP32FE_TSB_D5, FAPI_GPIO_IN_TS_B_DATA_5 }
,{ IP33FE_TSB_D4, FAPI_GPIO_IN_TS_B_DATA_4 }
,{ IP35FE_TSB_D3, FAPI_GPIO_IN_TS_B_DATA_3 }
,{ IP34FE_TSB_D2, FAPI_GPIO_IN_TS_B_DATA_2 }
,{ IP36FE_TSB_D1, FAPI_GPIO_IN_TS_B_DATA_1 }
,{ IP37FE_TSB_D0, FAPI_GPIO_IN_TS_B_DATA_0 }
,{ IP29FE_TSB_VAL,FAPI_GPIO_IN_TS_B_EN     }
,{ IP28FE_TSB_SYN,FAPI_GPIO_IN_TS_B_PSTART }
,{ IP27FE_TSB_CLK,FAPI_GPIO_IN_TS_B_CLK    }

,{ IP88FE_TSC_D7, FAPI_GPIO_IN_TS_C_DATA_7 }
,{ IP89FE_TSC_D6, FAPI_GPIO_IN_TS_C_DATA_6 }
,{ IP90FE_TSC_D5, FAPI_GPIO_IN_TS_C_DATA_5 }
,{ IP91FE_TSC_D4, FAPI_GPIO_IN_TS_C_DATA_4 }
,{ IP92FE_TSC_D3, FAPI_GPIO_IN_TS_C_DATA_3 }
,{ IP93FE_TSC_D2, FAPI_GPIO_IN_TS_C_DATA_2 }
,{ IP94FE_TSC_D1, FAPI_GPIO_IN_TS_C_DATA_1 }
,{ IP95FE_TSC_D0, FAPI_GPIO_IN_TS_C_DATA_0 }
,{ IP85FE_TSC_VAL,FAPI_GPIO_IN_TS_C_EN     }
,{ IP86FE_TSC_SYN,FAPI_GPIO_IN_TS_C_PSTART }
,{ IP87FE_TSC_CLK,FAPI_GPIO_IN_TS_C_CLK    }

,{ 13, FAPI_GPIO_BIDI_I2C0_DATA    }
,{ 14, FAPI_GPIO_BIDI_I2C0_CLK     }

,{  2, FAPI_GPIO_IN_UART0_DATA     }
,{  3, FAPI_GPIO_OUT_UART0_DATA    }
,{  9, FAPI_GPIO_OUT_UART1_DATA    }
,{ 10, FAPI_GPIO_IN_UART1_DATA     }

,{ 39, FAPI_GPIO_RESERVED_39       }
,{ 41, FAPI_GPIO_RESERVED_41       }

,{ 42, FAPI_GPIO_OUT_SPDIF_OUT     }

,{ 43, FAPI_GPIO_IN_SFLASH_DATA    }
,{ 44, FAPI_GPIO_OUT_SFLASH_HOLD   }
,{ 45, FAPI_GPIO_OUT_SFLASH_CS     }
,{ 46, FAPI_GPIO_OUT_SFLASH_CLK    }
,{ 47, FAPI_GPIO_OUT_SFLASH_DATA   }

,{ 65, FAPI_GPIO_OUT_SDIO_DATA_3   }    // NOTE: these appear again as 'BIDI'
,{ 66, FAPI_GPIO_OUT_SDIO_DATA_2   }    // so this is probably scrap
,{ 67, FAPI_GPIO_OUT_SDIO_DATA_1   }
,{ 74, FAPI_GPIO_OUT_SDIO_DATA_0   }
,{ 75, FAPI_GPIO_OUT_SDIO_CMD      }

,{ 65, FAPI_GPIO_BIDI_SDIO_DATA_3  }
,{ 66, FAPI_GPIO_BIDI_SDIO_DATA_2  }
,{ 67, FAPI_GPIO_BIDI_SDIO_DATA_1  }
,{ 68, FAPI_GPIO_IN_SDIO_CD        }

#ifdef NO_DIG_RGB_OR_CI_SLAVE_TEST
,{ 74, FAPI_GPIO_BIDI_SDIO_DATA_0  }
,{ 75, FAPI_GPIO_BIDI_SDIO_CMD     }
,{ 76, FAPI_GPIO_OUT_SDIO_CLK      }
,{ 77, FAPI_GPIO_IN_FPC_KEY_1      }
,{ 78, FAPI_GPIO_IN_FPC_KEY_0      }
,{ 79, FAPI_GPIO_OUT_FPC_SEG_3     }
,{ 80, FAPI_GPIO_OUT_FPC_SEG_2     }
,{ 81, FAPI_GPIO_OUT_FPC_SEG_1     }
,{ 82, FAPI_GPIO_OUT_FPC_SEG_0     }
,{ 83, FAPI_GPIO_OUT_FPC_DATA      }
,{ 84, FAPI_GPIO_OUT_FPC_CLK       }
#endif

,{ 85, FAPI_GPIO_IN_TS_C_EN      }
,{ 86, FAPI_GPIO_IN_TS_C_PSTART  }
,{ 87, FAPI_GPIO_IN_TS_C_CLK     }
,{ 88, FAPI_GPIO_IN_TS_C_DATA_7  }
,{ 89, FAPI_GPIO_IN_TS_C_DATA_6  }
,{ 90, FAPI_GPIO_IN_TS_C_DATA_5  }
,{ 91, FAPI_GPIO_IN_TS_C_DATA_4  }
,{ 92, FAPI_GPIO_IN_TS_C_DATA_3  }
,{ 93, FAPI_GPIO_IN_TS_C_DATA_2  }
,{ 94, FAPI_GPIO_IN_TS_C_DATA_1  }
,{ 95, FAPI_GPIO_IN_TS_C_DATA_0  }

};


/*!
********************************************************************************
** \brief just a 'security function' should generate a linker-warning (but does not work)
********************************************************************************
#ifdef NO_DIG_RGB_OR_CI_SLAVE_TEST

uint32_t BOARD_FME_H61DK01_GenLinkError_ViolatesWithNormalGpioInit;

void BOARD_FME_H61DK01_GenerateLinkError_ViolatesWithNormalGpioInit(void)
{

}
#endif
*/

#ifdef SUPPORT_TDA_SMART_CARD
/*!
********************************************************************************
** \brief initialize all platform/board specific GPIO pins
**
********************************************************************************
*/
void BOARD_FME_H61DK01_RSO_InitGpio_TdaSmartCard(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction)
{
    GPIO_CONFIG_INFO_T *gpc_ptr = GpioConfig_TdaSmartCard;
    uint32_t ix, num;
    uint32_t pin, func;

    if (useThisRegistFunction==NULL) return;    // would be severe programming mistake

    num = (sizeof(GpioConfig_TdaSmartCard) / sizeof(GPIO_CONFIG_INFO_T));
    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}
#endif

/*!
********************************************************************************
** \brief initialize all platform/board specific GPIO pins
**
** this is likely different on all different HW-platforms
**
********************************************************************************
*/
void BOARD_FME_H61DK01_RegisterGpioPins(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction)
{
    GPIO_CONFIG_INFO_T *gpc_ptr = BOARD_FME_H61DK01_GpioConfig;
    uint32_t ix, num;
    uint32_t pin, func;

    if (useThisRegistFunction==NULL) return;    // would be severe programming mistake

    num = (sizeof(BOARD_FME_H61DK01_GpioConfig) / sizeof(GPIO_CONFIG_INFO_T));
    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}

/*!
********************************************************************************
** \brief (re-)configure GPIO-Pins providing TS-A Input
**
** taking TS either from FrontEnd or from CI-Slot output
**
********************************************************************************
*/
void BOARD_FME_H61DK01_RSO_TsAInpSelect_Fe_or_TSG(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t inpSelFlag)
{
    GPIO_CONFIG_INFO_T *gpc_ptr;
    uint32_t ix, num;
    uint32_t pin, func;

    if (!inpSelFlag)
        gpc_ptr = GpioConfig_TsAInput_FromFe;
    else
        gpc_ptr = GpioConfig_TsAInput_FromCi;

    num = (sizeof(GpioConfig_TsAInput_FromFe) / sizeof(GPIO_CONFIG_INFO_T));
    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}

/*!
********************************************************************************
** \brief (re-)configure GPIO-Pins for Ts-Ouput or Ext.Audio-DAC support
**
********************************************************************************
*/
void BOARD_FME_H61DK01_RSO_TsOut_ExtADac_TdaIcc(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t modeSelFlag)
{
    GPIO_CONFIG_INFO_T *gpc_ptr;
    uint32_t ix, num;
    uint32_t pin, func;

    if      (modeSelFlag==0) {
        num = (sizeof(GpioConfig_4TsOutput) / sizeof(GPIO_CONFIG_INFO_T));
        gpc_ptr = GpioConfig_4TsOutput; }
    else if (modeSelFlag==1) {
        num = (sizeof(GpioConfig_4ExtAudioDac) / sizeof(GPIO_CONFIG_INFO_T));
        gpc_ptr = GpioConfig_4ExtAudioDac; }
    else {
        num = (sizeof(GpioConfig_TdaSmartCard) / sizeof(GPIO_CONFIG_INFO_T));
        gpc_ptr = GpioConfig_TdaSmartCard; }

    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}
/*!
********************************************************************************
** \brief (re-)configure GPIO-Pins for PFlash or SD-IO usage
** \note  digital RGB-out can not work in parallel to 'GpioConfig_4SdIo' 
********************************************************************************
*/
void BOARD_FME_H61DK01_RSO_PFlash_or_SdIo(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t modeSel)
{
    GPIO_CONFIG_INFO_T *gpc_ptr;
    uint32_t ix, num;
    uint32_t pin, func;

    if (!modeSel) {
        num = (sizeof(GpioConfig_4PFlash) / sizeof(GPIO_CONFIG_INFO_T));
        gpc_ptr = GpioConfig_4PFlash; }
    else {
        num = (sizeof(GpioConfig_4SdIo) / sizeof(GPIO_CONFIG_INFO_T));
        gpc_ptr = GpioConfig_4SdIo; }

    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}

/*!
********************************************************************************
** \brief (re-)configure GPIO-Pins for 'special Serial TS-Output'
**
********************************************************************************
*/
void BOARD_FME_H61DK01_RSO_Cfg_SerTsOutpPins(uint32_t inp2ts, uint32_t serOutIdx, uint32_t ctrlCode )
{
    serOutIdx = serOutIdx;                      // not used
    if (!ctrlCode)
    {
        // Reconfigure for Serializer Mode
        PIN_CFG_OUT_FUNC(N_SER_OUT_PINS, OutPins_SerTsLoop);

        if (inp2ts==0)
        {   // for TS-A
            PIN_CFG_INP_FUNC(N_SR_INP_PINS, InpPins_SerTsLoop4TsA);
        }
        else if(inp2ts==1)
        {   // for TS-B
            PIN_CFG_INP_FUNC(N_SR_INP_PINS, InpPins_SerTsLoop4TsB);
        }
        else if(inp2ts==2)
        {   // for TS-C
            PIN_CFG_INP_FUNC(N_SR_INP_PINS, InpPins_SerTsLoop4TsC);
        }
    }
    else if (ctrlCode==1)
    {
        // Reconfigure to Parallel Mode
        if (inp2ts==0)
        {   // for TS-A
            PIN_CFG_INP_FUNC(N_PR_INP_PINS, InpPins_ParTsLoop4TsA);
        }
        else if(inp2ts==1)
        {   // for TS-B
            PIN_CFG_INP_FUNC(N_PR_INP_PINS, InpPins_ParTsLoop4TsB);
        }
        else if(inp2ts==2)
        {   // for TS-C
            PIN_CFG_INP_FUNC(N_PR_INP_PINS, InpPins_ParTsLoop4TsC);
        }
    }
    else
    {
        // disable TS-Input
        if (inp2ts==0)
        {   // for TS-A
            PIN_CFG_INP_FUNC(N_PAR_DIS_INP_PINS, InpPins_DisableParTsTsA);
        }
        else if(inp2ts==1)
        {   // for TS-B
            PIN_CFG_INP_FUNC(N_PAR_DIS_INP_PINS, InpPins_DisableParTsTsB);
        }
        else if(inp2ts==2)
        {   // for TS-C
            PIN_CFG_INP_FUNC(N_PAR_DIS_INP_PINS, InpPins_DisableParTsTsC);
        }
    }
}

/*!
********************************************************************************
** \brief compose a 'BitPatternValue' corresponding to TS-Input signals
**
** this function can be used for diagnostic purpose to see if 'TS-input' active
**
** \param tsPathIndex        Index of the TS-path
** \param dstTsInpLevelsPtr  pointer to 'U16' where to store value
**
** \return
** - FAPI_OK                if index is OK
** - FAPI_ERR_BAD_PARAMETER if illegal TS-path index
********************************************************************************
*/
int32_t BOARD_FME_H61DK01_GetTsPinsInputLevel(BOARDS_FMEDEF_RD_GPIO_INPREG_CBF  RdGpioInpFnc,
                                              int32_t tsPathIndex,
                                              uint16_t *dstTsInpLevelsPtr)
{
    uint32_t gpio_regdat;
    uint32_t test_data;
    int32_t  ret_stat;

    test_data   = 0;            // just init
    ret_stat    = FAPI_OK;
    tsPathIndex = tsPathIndex;

    if (!RdGpioInpFnc) return FAPI_ERR_BAD_PARAMETER;
//    gpio_regdat = BOARDS_GPIO_GET_INP_31_0();
    gpio_regdat = RdGpioInpFnc();

    if (tsPathIndex==0)
    {
        test_data  = ((gpio_regdat>>5)&0x400);     // GPIO15 -5 -> TS-Data10 (CLK)
        test_data |= ((gpio_regdat>>15)&0x200);    // GPIO24 -15-> TS-Data9  (VAL)
        test_data |= ((gpio_regdat>>17)&0x100);    // GPIO25 -17-> TS-Data8  (SYN)
        test_data |= ((gpio_regdat>>9) & 0x80);    // GPIO16 -9 -> TS-Data7
        test_data |= ((gpio_regdat>>11)& 0x40);    // GPIO17 -11-> TS-Data6
        test_data |= ((gpio_regdat>>13)& 0x20);    // GPIO18 -13-> TS-Data5
        test_data |= ((gpio_regdat>>15)& 0x10);    // GPIO19 -15-> TS-Data4
        test_data |= ((gpio_regdat>>17)& 0x08);    // GPIO20 -17-> TS-Data3
        test_data |= ((gpio_regdat>>19)& 0x04);    // GPIO21 -19-> TS-Data2
        test_data |= ((gpio_regdat>>21)& 0x02);    // GPIO22 -21-> TS-Data1
        test_data |= ((gpio_regdat>>23)& 0x01);    // GPIO23 -23-> TS-Data0
    }
    else if (tsPathIndex==1)
    {
        test_data  = 0;                             // not implemented yet
    }
    else
        ret_stat = FAPI_ERR_BAD_PARAMETER;

    if (ret_stat == FAPI_OK)
        *dstTsInpLevelsPtr = test_data;

    return ret_stat;
}

#ifdef USE_P_ATA_ADAPTER
/*!
********************************************************************************
** \brief re-configure relevant GPIO for 'special J900-PAta-Adapter-Test'
**
** this is likely different on all different HW-platforms
**
********************************************************************************
*/
void BOARD_FME_H61DK01_RSO_ConfigGpiosPAtaAdapter(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction)
{
    GPIO_CONFIG_INFO_T *gpc_ptr = GpioConfig_Pins4PAtaTest;
    uint32_t ix, num;
    uint32_t pin, func;

    if (useThisRegistFunction==NULL) return;    // would be severe programming mistake

//    BOARDS_GPIO_SET_OUTP(GPIO_UPI_BUF_DIR, (FAPI_GPIO_OUT_UPI_CS - FAPI_GPIO_OUT_OFFSET));

    num = (sizeof(GpioConfig_Pins4PAtaTest) / sizeof(GPIO_CONFIG_INFO_T));
    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}
#endif

#ifdef SUPPORT_DIGIT_RGB_OUTPUT
/*!
********************************************************************************
** \brief (re-)configure dedicated GPIO-Pins for Digital-RGB-output
** \note  in this caes SD-Card and CI (TSC-Input) can not be used !!!
********************************************************************************
*/
void BOARD_FME_H61DK01_RSO_DigitalRGB_or_RestoreDefaults(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t modeSelFlag)
{
    uint32_t pin, func;

    useThisRegistFunction = useThisRegistFunction;  // not used (yet)

    if (!modeSelFlag)                       // activation of 'digital RGB-mode'
    {
//        BTLIB_IO_PrStr("\n{RgbConfigCalled}\n");
        func = BDEF_GPIO_OUT_0;             // NOTE: for 'digital RGB-output',
                                            // pins need a 'kind of any/default' output configuration
        for (pin=68; pin<=95; pin++)        // we do this for all RGB-related pins, 'DE-,VS-,HS-,RGB ...
        {
            FREG_GPIO_SetDataOut( pin, func); 
        }
        FREG_GPIO_SetConfig( 0xFFFFFFF0 );  // and this register must configure 'special RGB-mode'
    }                                       // 
}
#endif

