/*!
********************************************************************************
**
** \file        ./boards/fmc/mb86h61_pdkv1/mb86h61_pdkv1_gpio.c
**
** \version     $Id: mb86h61_pdkv1_gpio.c 
**
** \brief       function to initialize the GPIO-configuration registers
**
** \note        there exist two methods
**              a) via ..\fapi\sys\src\sys_init_mb86h61_devkit.h
**              b) using this 'baords' driver
**              Actually method b) should be preferred. If the file which includes
**              '<fapi/sys_init.h>' has the define
**              '#define FAPI_SYS_INIT_USE_BOARDS 1', then the GPIO-registration
**              form 'sys_init_mb86h61_devkit.h' will be disabled
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010-2011 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/

#include <fapi/reg_gpio.h>
#include <fapi/drv_gpio.h>

#include <boards/boards_common.h>          // some defines needed for 'mb86h61_dk01.h'
#include <boards/mb86h61_pdkv1.h>

#define NO_DIG_RGB_OR_CI_SLAVE_TEST     // comment this out when using 'digital RBG-signals'
                                        // or UPI-Slave,CAM-Emulation Test

//#define USE_P_ATA_ADAPTER             // active when using P-Ata-Adapter

typedef struct {
  uint32_t pinNum;
  uint32_t config;
} GPIO_CONFIG_INFO_T;


static  GPIO_CONFIG_INFO_T  GpioConfig_TsAInput_FromFe[] = {
#if defined(PCB_DT270)
 { 15, FAPI_GPIO_IN_TS_A_PSTART  }
,{ 17, FAPI_GPIO_IN_TS_A_DATA_0  }
,{ 22, FAPI_GPIO_IN_TS_A_CLK     }
,{ 25, FAPI_GPIO_IN_TS_A_EN      }
#elif defined(PCB_POE_BA300W_SB1) || defined(PCB_POE_BA300W_SB1_SR1)
 { 89, FAPI_GPIO_IN_TS_A_PSTART  }
,{ 95, FAPI_GPIO_IN_TS_A_DATA_0  }
,{ 94, FAPI_GPIO_IN_TS_A_CLK     }
,{ 90, FAPI_GPIO_IN_TS_A_EN      }
#else
 { 24, FAPI_GPIO_IN_TS_A_DATA_7  }
,{ 21, FAPI_GPIO_IN_TS_A_DATA_6  }
,{ 20, FAPI_GPIO_IN_TS_A_DATA_5  }
,{ 23, FAPI_GPIO_IN_TS_A_DATA_4  }
,{ 19, FAPI_GPIO_IN_TS_A_DATA_3  }
,{ 16, FAPI_GPIO_IN_TS_A_DATA_2  }
,{ 18, FAPI_GPIO_IN_TS_A_DATA_1  }
,{ 17, FAPI_GPIO_IN_TS_A_DATA_0  }
,{ 25, FAPI_GPIO_IN_TS_A_EN      }
,{ 15, FAPI_GPIO_IN_TS_A_PSTART  }
,{ 22, FAPI_GPIO_IN_TS_A_CLK     }
#endif 	// defined(PCB_DT270)
};

static  GPIO_CONFIG_INFO_T  GpioConfig_TsAInput_FromCi[] = {
 { 88, FAPI_GPIO_IN_TS_A_DATA_7  }
,{ 89, FAPI_GPIO_IN_TS_A_DATA_6  }
,{ 90, FAPI_GPIO_IN_TS_A_DATA_5  }
,{ 91, FAPI_GPIO_IN_TS_A_DATA_4  }
,{ 92, FAPI_GPIO_IN_TS_A_DATA_3  }
,{ 93, FAPI_GPIO_IN_TS_A_DATA_2  }
,{ 94, FAPI_GPIO_IN_TS_A_DATA_1  }
,{ 95, FAPI_GPIO_IN_TS_A_DATA_0  }
,{ 85, FAPI_GPIO_IN_TS_A_EN      }
,{ 86, FAPI_GPIO_IN_TS_A_PSTART  }
,{ 87, FAPI_GPIO_IN_TS_A_CLK     }
};


static  GPIO_CONFIG_INFO_T  BOARD_FMC_H61PDKV1_GpioConfig[] = {
#if defined(PCB_DT270)
	 {  2, FAPI_GPIO_IN_UART0_DATA   }
	,{  3, FAPI_GPIO_OUT_UART0_DATA  }
	,{ 47, FAPI_GPIO_OUT_SFLASH_DATA }
	,{ 44, FAPI_GPIO_OUT_SFLASH_HOLD }
	,{ 45, FAPI_GPIO_OUT_SFLASH_CS   }
	,{ 46, FAPI_GPIO_OUT_SFLASH_CLK  }
	,{ 43, FAPI_GPIO_IN_SFLASH_DATA  }
	,{ 17, FAPI_GPIO_IN_TS_A_DATA_0  }
	,{ 25, FAPI_GPIO_IN_TS_A_EN      }
	,{ 15, FAPI_GPIO_IN_TS_A_PSTART  }
	,{ 22, FAPI_GPIO_IN_TS_A_CLK     }
	,{ 36, FAPI_GPIO_IN_TS_B_DATA_0  }
	,{ 27, FAPI_GPIO_IN_TS_B_EN      }
	,{ 29, FAPI_GPIO_IN_TS_B_PSTART  }
	,{ 26, FAPI_GPIO_IN_TS_B_CLK     }
	// GPIO
	 ,{ 10, FAPI_GPIO_OUT_0     }	// [O/L] (gp)USB0d
	 ,{ 11, FAPI_GPIO_OUT_1     }	// [O/H] USB1d : Hi I(musbhCoł̓dIȂAŃI)
	 ,{ 12, FAPI_GPIO_OUT_0     }	// [O/L] Dione HWZbg : Low Zbg
	 ,{ 13, FAPI_GPIO_OUT_1     }	// [O/H] Dione d : Hi It
	 ,{ 14, FAPI_GPIO_OUT_1     }	// [O/H] Dione FW : Hi ݃fBZ[u
#elif defined(PCB_POE_BA300W_SB1)
	// GPIO0`1  @\ݒsv
	 {  2, FAPI_GPIO_IN_UART0_DATA			}	// UART0 RX
	,{  3, FAPI_GPIO_OUT_UART0_DATA			}	// UART0 TX
	,{  4, FAPI_GPIO_BIDI_I2C0_CLK			}	// I2C0 SCL
	,{  5, FAPI_GPIO_OUT_1					}	// [O/L]`[i3.3V ---> [O/H] (I2C1XC5200Ɉ邽)
	,{  6, FAPI_GPIO_BIDI_I2C0_DATA			}	// I2C0 SDA
	,{  7, FAPI_GPIO_OUT_0					}	// [O/L]q`[iZbg
	,{  8, FAPI_GPIO_OUT_0					}	// [O/L]탊Zbg
	,{  9, FAPI_GPIO_BIDI_I2C1_CLK			}	// I2C1 SCL
	,{ 10, FAPI_GPIO_BIDI_I2C1_DATA			}	// I2C1 SDA
	,{ 11, FAPI_GPIO_OUT_0					}	// [O/L]nD`[iZbg
	,{ 12, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 13, FAPI_GPIO_OUT_1					}	// [O/H]LED WLAN 
	,{ 14, FAPI_GPIO_OUT_1					}	// [O/H]LED WLAN 
	,{ 15, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 16, FAPI_GPIO_OUT_1					}	// [O/H]LED d 
	,{ 17, FAPI_GPIO_OUT_0					}	// [O/L]LED d 
	,{ 18, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 19, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 20, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 21, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 22, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 23, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 24, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 25, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 26, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 27, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 28, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 29, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 30, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 31, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 32, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]Product ID
	,{ 33, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]	V
	,{ 34, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]	V
	,{ 35, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]	V
	,{ 36, FAPI_GPIO_OUT_1					}	// [O/H]USBnuZbg
	,{ 37, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]ZbgAbv{^(N.C)
	// GPIO38`42  pullup/downŒ
	,{ 43, FAPI_GPIO_IN_SFLASH_DATA			}	// SFLASH 
	,{ 44, FAPI_GPIO_OUT_SFLASH_HOLD		}	//   V
	,{ 45, FAPI_GPIO_OUT_SFLASH_CS			}	//   V
	,{ 46, FAPI_GPIO_OUT_SFLASH_CLK			}	//   V
	,{ 47, FAPI_GPIO_OUT_SFLASH_DATA		}	//   V
	,{ 48, FAPI_GPIO_OUT_0					}	// [O/L]1.2V
	,{ 49, FAPI_GPIO_OUT_0					}	// [O/L]3.3V
	,{ 50, FAPI_GPIO_OUT_0					}	// [O/L]N.C(NAND)
	,{ 51, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 52, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 53, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 54, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 55, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 56, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 57, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 58, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 59, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 60, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 61, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 62, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 63, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 64, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]LNBV[go
	,{ 65, FAPI_GPIO_OUT_0					}	// [O/L]LNB ON
	,{ 66, FAPI_GPIO_OUT_1					}	// [O/H]C[Tlbg(PHY)Zbg
	,{ 67, FAPI_GPIO_OUT_0					}	// [O/L]N.C(CASV[go)
	,{ 68, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-](lbg[N)Zbg{^  Hi:Zbg, Low:Zbg
	// GPIO69`71  pullup/downŒ
	,{ 72, FAPI_GPIO_OUT_1					}	// [O/H]USBnCTChXCb`Cl[u
	,{ 73, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]USBߓdo
	,{ 74, FAPI_GPIO_OUT_0					}	// [O/L]DIONEZbg
	,{ 75, FAPI_GPIO_OUT_1					}	// [O/H]DIONE FW݃Cl[u
//	,{ 76, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]C[Tlbg肱
	,{ 77, FAPI_GPIO_OUT_0					}	// [O/L]DIONE 1.2V
	,{ 78, FAPI_GPIO_OUT_0					}	// [O/L]DIONE 1.8V
	,{ 79, FAPI_GPIO_IN_TS_B_EN				}	// TSB
	,{ 80, FAPI_GPIO_IN_TS_B_CLK			}	// TSB
	,{ 81, FAPI_GPIO_OUT_0					}	// [O/L]DIONE 3.3V
	,{ 82, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 83, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 84, FAPI_GPIO_IN_TS_B_PSTART			}	// TSB
	,{ 85, FAPI_GPIO_IN_TS_B_DATA_0			}	// TSB
	,{ 86, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 87, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 88, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 89, FAPI_GPIO_IN_TS_A_PSTART			}	// TSA
	,{ 90, FAPI_GPIO_IN_TS_A_EN				}	//    V
	,{ 91, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 92, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 93, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 94, FAPI_GPIO_IN_TS_A_CLK			}	//    V
	,{ 95, FAPI_GPIO_IN_TS_A_DATA_0			}	//    V
#elif defined(PCB_POE_BA300W_SB1_SR1)
	// GPIO0`1  @\ݒsv
	 {  2, FAPI_GPIO_IN_UART0_DATA			}	// UART0 RX
	,{  3, FAPI_GPIO_OUT_UART0_DATA			}	// UART0 TX
	,{  4, FAPI_GPIO_BIDI_I2C0_CLK			}	// I2C0 SCL
	,{  5, FAPI_GPIO_OUT_1					}	// [O/L]`[i3.3V ---> [O/H] (I2C1XC5200Ɉ邽)
	,{  6, FAPI_GPIO_BIDI_I2C0_DATA			}	// I2C0 SDA
	,{  7, FAPI_GPIO_OUT_0					}	// [O/L]q`[iZbg
	,{  8, FAPI_GPIO_OUT_0					}	// [O/L]탊Zbg
	,{  9, FAPI_GPIO_BIDI_I2C1_CLK			}	// I2C1 SCL
	,{ 10, FAPI_GPIO_BIDI_I2C1_DATA			}	// I2C1 SDA
	,{ 11, FAPI_GPIO_OUT_0					}	// [O/L]nD`[iZbg
	,{ 12, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 13, FAPI_GPIO_OUT_1					}	// [O/H]LED WLAN 
	,{ 14, FAPI_GPIO_OUT_1					}	// [O/H]LED WLAN 
	,{ 15, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 16, FAPI_GPIO_OUT_1					}	// [O/H]LED d 
	,{ 17, FAPI_GPIO_OUT_0					}	// [O/L]LED d 
	,{ 18, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 19, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 20, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 21, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 22, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 23, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 24, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 25, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 26, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 27, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 28, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 29, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 30, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 31, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 32, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 33, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 34, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 35, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 36, FAPI_GPIO_OUT_1					}	// [O/H]USBnuZbg
	,{ 37, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]ZbgAbv{^(N.C)
	// GPIO38`42  pullup/downŒ
	,{ 43, FAPI_GPIO_IN_SFLASH_DATA			}	// SFLASH 
	,{ 44, FAPI_GPIO_OUT_SFLASH_HOLD		}	//   V
	,{ 45, FAPI_GPIO_OUT_SFLASH_CS			}	//   V
	,{ 46, FAPI_GPIO_OUT_SFLASH_CLK			}	//   V
	,{ 47, FAPI_GPIO_OUT_SFLASH_DATA		}	//   V
	,{ 48, FAPI_GPIO_OUT_0					}	// [O/L]1.2V
	,{ 49, FAPI_GPIO_OUT_0					}	// [O/L]3.3V
	,{ 50, FAPI_GPIO_OUT_0					}	// [O/L]N.C(NAND)
	,{ 51, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 52, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 53, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 54, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 55, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 56, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 57, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 58, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 59, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 60, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 61, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 62, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 63, FAPI_GPIO_OUT_0					}	// [O/L]    V
	,{ 64, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]LNBV[go
//	,{ 65, FAPI_GPIO_OUT_0					}	// [O/L]LNB ON
	,{ 65, FAPI_GPIO_OUT_1					}	//    -----> [O/H]  LNB ON (SR1p̂ݕ]̂)
	,{ 66, FAPI_GPIO_OUT_1					}	// [O/H]C[Tlbg(PHY)Zbg
	,{ 67, FAPI_GPIO_OUT_0					}	// [O/L]N.C(CASV[go)
	,{ 68, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-](lbg[N)Zbg{^  Hi:Zbg, Low:Zbg
	// GPIO69`71  pullup/downŒ
	,{ 72, FAPI_GPIO_OUT_1					}	// [O/H]USBnCTChXCb`Cl[u
	,{ 73, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 74, FAPI_GPIO_OUT_0					}	// [O/L]DIONEZbg
	,{ 75, FAPI_GPIO_OUT_1					}	// [O/H]DIONE FW݃Cl[u
//	,{ 76, FAPI_GPIO_IN_GENERIC_SW_FUNCTION	}	// [I/-]C[Tlbg肱
	,{ 77, FAPI_GPIO_OUT_0					}	// [O/L]DIONE 1.2V
	,{ 78, FAPI_GPIO_OUT_0					}	// [O/L]DIONE 1.8V
	,{ 79, FAPI_GPIO_IN_TS_B_EN				}	// TSB
	,{ 80, FAPI_GPIO_IN_TS_B_CLK			}	// TSB
	,{ 81, FAPI_GPIO_OUT_0					}	// [O/L]DIONE 3.3V
	,{ 82, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 83, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 84, FAPI_GPIO_IN_TS_B_PSTART			}	// TSB
	,{ 85, FAPI_GPIO_IN_TS_B_DATA_0			}	// TSB
	,{ 86, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 87, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 88, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 89, FAPI_GPIO_IN_TS_A_PSTART			}	// TSA
	,{ 90, FAPI_GPIO_IN_TS_A_EN				}	//    V
	,{ 91, FAPI_GPIO_OUT_0					}	// [O/L]N.C
	,{ 92, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 93, FAPI_GPIO_OUT_0					}	// [O/L]TP
	,{ 94, FAPI_GPIO_IN_TS_A_CLK			}	//    V
	,{ 95, FAPI_GPIO_IN_TS_A_DATA_0			}	//    V
#else	// defined(PCB_DT270)
//TS A
 { 24, FAPI_GPIO_IN_TS_A_DATA_7  }
,{ 21, FAPI_GPIO_IN_TS_A_DATA_6  }
,{ 20, FAPI_GPIO_IN_TS_A_DATA_5  }
,{ 23, FAPI_GPIO_IN_TS_A_DATA_4  }
,{ 19, FAPI_GPIO_IN_TS_A_DATA_3  }
,{ 16, FAPI_GPIO_IN_TS_A_DATA_2  }
,{ 18, FAPI_GPIO_IN_TS_A_DATA_1  }
,{ 17, FAPI_GPIO_IN_TS_A_DATA_0  }
,{ 25, FAPI_GPIO_IN_TS_A_EN      }
,{ 15, FAPI_GPIO_IN_TS_A_PSTART  }
,{ 22, FAPI_GPIO_IN_TS_A_CLK     }

//TS B
,{ 28, FAPI_GPIO_IN_TS_B_DATA_7  }
,{ 30, FAPI_GPIO_IN_TS_B_DATA_6  }
,{ 31, FAPI_GPIO_IN_TS_B_DATA_5  }
,{ 32, FAPI_GPIO_IN_TS_B_DATA_4  }
,{ 33, FAPI_GPIO_IN_TS_B_DATA_3  }
,{ 34, FAPI_GPIO_IN_TS_B_DATA_2  }
,{ 35, FAPI_GPIO_IN_TS_B_DATA_1  }
,{ 36, FAPI_GPIO_IN_TS_B_DATA_0  }
,{ 29, FAPI_GPIO_IN_TS_B_EN      }
,{ 27, FAPI_GPIO_IN_TS_B_PSTART  }
,{ 26, FAPI_GPIO_IN_TS_B_CLK     }


,{ 13, FAPI_GPIO_BIDI_I2C0_DATA  }
,{ 14, FAPI_GPIO_BIDI_I2C0_CLK   }
,{ 54, FAPI_GPIO_OUT_AD0_BCLK    }
,{ 53, FAPI_GPIO_OUT_AD0_LRCLK   }
,{ 55, FAPI_GPIO_OUT_AD0_CLK     }
,{ 52, FAPI_GPIO_OUT_AD0_DATA    }
,{ 51, FAPI_GPIO_OUT_AD1_DATA    }
,{ 50, FAPI_GPIO_OUT_AD2_DATA    }
,{ 49, FAPI_GPIO_OUT_AD3_DATA    }
,{ 48, FAPI_GPIO_OUT_0           }
,{  2, FAPI_GPIO_IN_UART0_DATA   }
,{  3, FAPI_GPIO_OUT_UART0_DATA  }
,{ 65, FAPI_GPIO_OUT_UART1_DATA  }
,{ 64, FAPI_GPIO_IN_UART1_DATA   }
,{ 47, FAPI_GPIO_OUT_SFLASH_DATA }
,{ 44, FAPI_GPIO_OUT_SFLASH_HOLD }
,{ 45, FAPI_GPIO_OUT_SFLASH_CS   }
,{ 46, FAPI_GPIO_OUT_SFLASH_CLK  }
,{ 43, FAPI_GPIO_IN_SFLASH_DATA  }

#if 1
//config smart card GPIO
,{ 59, FAPI_GPIO_OUT_ICC0_RST    }
,{ 60, FAPI_GPIO_IN_ICC0_IN      }
,{ 61, FAPI_GPIO_OUT_ICC0_CLK    }
,{ 62, FAPI_GPIO_BIDI_ICC0_DATA  }
,{ 63, FAPI_GPIO_OUT_ICC0_VCC    }
#endif

//config SDIO
,{ 55, FAPI_GPIO_OUT_SDIO_CMD    }
,{ 55, FAPI_GPIO_IN_SDIO_CMD     }
,{ 56, FAPI_GPIO_IN_SDIO_CD      }
,{ 53, FAPI_GPIO_OUT_SDIO_DATA_0 }
,{ 52, FAPI_GPIO_OUT_SDIO_DATA_1 }
,{ 51, FAPI_GPIO_OUT_SDIO_DATA_2 }
,{ 56, FAPI_GPIO_OUT_SDIO_DATA_3 }
,{ 56, FAPI_GPIO_IN_SDIO_DATA_3  }
,{ 51, FAPI_GPIO_IN_SDIO_DATA_2  }
,{ 54, FAPI_GPIO_OUT_SDIO_CLK    }


#ifdef NO_DIG_RGB_OR_CI_SLAVE_TEST
//,{ 74, FAPI_GPIO_IN_SDIO_DATA_0  }
#endif
//,{ 67, FAPI_GPIO_OUT_SPDIF_OUT  }//,{ 67, FAPI_GPIO_IN_SDIO_DATA_1   }
,{ 42, FAPI_GPIO_IN_SERIALIZER   }  //changed by shenmeng 
#ifdef NO_DIG_RGB_OR_CI_SLAVE_TEST
,{ 70, FAPI_GPIO_OUT_FPC_CLK     }
,{ 77, FAPI_GPIO_IN_FPC_KEY_0    }
,{ 78, FAPI_GPIO_IN_FPC_KEY_1    }
,{ 71, FAPI_GPIO_OUT_FPC_DATA    }
,{ 73, FAPI_GPIO_OUT_FPC_SEG_0   }
,{ 74, FAPI_GPIO_OUT_FPC_SEG_1   }
,{ 75, FAPI_GPIO_OUT_FPC_SEG_2   }
,{ 76, FAPI_GPIO_OUT_FPC_SEG_3   }
#endif
//,{ 65, FAPI_GPIO_OUT_UPI_ADDRX0  }
,{ 66, FAPI_GPIO_OUT_UPI_ADDRX1  }
,{ 67, FAPI_GPIO_OUT_SPDIF_OUT   }//,{ 67, FAPI_GPIO_OUT_UPI_ADDRX2  }
,{ 68, FAPI_GPIO_OUT_UPI_ADDRX3  }
,{ 39, FAPI_GPIO_RESERVED_39     }
,{ 41, FAPI_GPIO_OUT_1           }
,{ 40, FAPI_GPIO_RESERVED_39     }
#endif	// defined(PCB_DT270)
};

/*!
********************************************************************************
** \brief just a 'security function' should generate a linker-warning (but does not work)
********************************************************************************
#ifdef NO_DIG_RGB_OR_CI_SLAVE_TEST

uint32_t BOARD_FMC_H61PDKV1_GenLinkError_ViolatesWithNormalGpioInit;

void BOARD_FMC_H61PDKV1_GenerateLinkError_ViolatesWithNormalGpioInit(void)
{

}
#endif
*/

/*!
********************************************************************************
** \brief initialize all platform/board specific GPIO pins
**
** this is likely different on all different HW-platforms
**
********************************************************************************
*/
void BOARD_FMC_H61PDKV1_RegisterGpioPins(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction)
{
    GPIO_CONFIG_INFO_T *gpc_ptr = BOARD_FMC_H61PDKV1_GpioConfig;
    uint32_t ix, num;
    uint32_t pin, func;

    if (useThisRegistFunction==NULL) return;    // would be severe programming mistake

    num = (sizeof(BOARD_FMC_H61PDKV1_GpioConfig) / sizeof(GPIO_CONFIG_INFO_T));
    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}


/*!
********************************************************************************
** \brief (re-)configure GPIO-Pins providing TS-A Input
**
** taking TS either from FrontEnd or from CI-Slot output
**
********************************************************************************
*/
void BOARD_FMC_H61PDKV1_TsAInpSelect(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction, uint32_t inpSelFlag)
{
    GPIO_CONFIG_INFO_T *gpc_ptr;
    uint32_t ix, num;
    uint32_t pin, func;

    if (!inpSelFlag)
        gpc_ptr = GpioConfig_TsAInput_FromFe;
    else
        gpc_ptr = GpioConfig_TsAInput_FromCi;

    num = (sizeof(GpioConfig_TsAInput_FromFe) / sizeof(GPIO_CONFIG_INFO_T));
    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}

/*!
********************************************************************************
** \brief compose a 'BitPatternValue' corresponding to TS-Input signals
**
** this function can be used for diagnostic purpose to see if 'TS-input' active
**
** \param tsPathIndex        Index of the TS-path
** \param dstTsInpLevelsPtr  pointer to 'U16' where to store value
**
** \return
** - FAPI_OK                if index is OK
** - FAPI_ERR_BAD_PARAMETER if illegal TS-path index
********************************************************************************
*/
int32_t BOARD_FMC_H61PDKV1_GetTsPinsInputLevel(BOARDS_FMEDEF_RD_GPIO_INPREG_CBF  RdGpioInpFnc,
                                              int32_t tsPathIndex,
                                              uint16_t *dstTsInpLevelsPtr)
{
    uint32_t gpio_regdat;
    uint32_t test_data;
    int32_t  ret_stat;

    test_data   = 0;            // just init
    ret_stat    = FAPI_OK;
    tsPathIndex = tsPathIndex;

    if (!RdGpioInpFnc) return FAPI_ERR_BAD_PARAMETER;
//    gpio_regdat = BOARDS_GPIO_GET_INP_31_0();
    gpio_regdat = RdGpioInpFnc();

    if (tsPathIndex==0)
    {
        test_data  = ((gpio_regdat>>5)&0x400);     // GPIO15 -5 -> TS-Data10 (CLK)
        test_data |= ((gpio_regdat>>15)&0x200);    // GPIO24 -15-> TS-Data9  (VAL)
        test_data |= ((gpio_regdat>>17)&0x100);    // GPIO25 -17-> TS-Data8  (SYN)
        test_data |= ((gpio_regdat>>9) & 0x80);    // GPIO16 -9 -> TS-Data7
        test_data |= ((gpio_regdat>>11)& 0x40);    // GPIO17 -11-> TS-Data6
        test_data |= ((gpio_regdat>>13)& 0x20);    // GPIO18 -13-> TS-Data5
        test_data |= ((gpio_regdat>>15)& 0x10);    // GPIO19 -15-> TS-Data4
        test_data |= ((gpio_regdat>>17)& 0x08);    // GPIO20 -17-> TS-Data3
        test_data |= ((gpio_regdat>>19)& 0x04);    // GPIO21 -19-> TS-Data2
        test_data |= ((gpio_regdat>>21)& 0x02);    // GPIO22 -21-> TS-Data1
        test_data |= ((gpio_regdat>>23)& 0x01);    // GPIO23 -23-> TS-Data0
    }
    else if (tsPathIndex==1)
    {
        test_data  = 0;                             // not implemented yet
    }
    else
        ret_stat = FAPI_ERR_BAD_PARAMETER;

    if (ret_stat == FAPI_OK)
        *dstTsInpLevelsPtr = test_data;

    return ret_stat;
}


#ifdef USE_P_ATA_ADAPTER
// NOTE: the P-ATA-Adapter needs to re-configure some GPIO-pins otherwise used for CI-Interface
#define PIN_PATA_IRQ    (GPIO_UPI_CLE)
#define PIN_UPI_WAIT    (GPIO_UPI_CI_WAIT)
#define PIN_UPI_DMARQ   (GPIO_UPI_IORD)         //
#define PIN_UPI_DMAAC   (GPIO_UPI_IOWR)         //
#define PIN_UPI_CS2     (GPIO_UPI_BUF_DIR)      // this will be used as HD-ChipSelect

static  GPIO_CONFIG_INFO_T  GpioConfig_Pins4PAtaTest[] = {
/* { PIN_PATA_IRQ,  FAPI_GPIO_IN_IRQ_IN0   }
,*/{ PIN_UPI_WAIT,  FAPI_GPIO_IN_UPI_WAIT    }
,{ PIN_UPI_DMARQ, FAPI_GPIO_IN_UPI_DMARQ   }
,{ PIN_UPI_DMAAC, FAPI_GPIO_OUT_UPI_CLE    }  // note: actuall 'DMACK'
,{ PIN_UPI_CS2,   FAPI_GPIO_OUT_UPI_CS     }
};

/*!
********************************************************************************
** \brief re-configure relevant GPIO for 'special J900-PAta-Adapter-Test'
**
** this is likely different on all different HW-platforms
**
********************************************************************************
*/
void BOARD_FMC_H61PDKV1_ConfigGpiosPAtaAdapter(BOARDS_FMEDEF_REG_INI_PIN_CBF_T useThisRegistFunction)
{
    GPIO_CONFIG_INFO_T *gpc_ptr = GpioConfig_Pins4PAtaTest;
    uint32_t ix, num;
    uint32_t pin, func;

    if (useThisRegistFunction==NULL) return;    // would be severe programming mistake

//    BOARDS_GPIO_SET_OUTP(GPIO_UPI_BUF_DIR, (FAPI_GPIO_OUT_UPI_CS - FAPI_GPIO_OUT_OFFSET));

    num = (sizeof(GpioConfig_Pins4PAtaTest) / sizeof(GPIO_CONFIG_INFO_T));
    for (ix=0;ix<num;ix++)
    {
        pin  = gpc_ptr->pinNum;
        func = gpc_ptr->config;
        useThisRegistFunction( pin, func );
        gpc_ptr++;
    }
}
#endif
