/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI ADC configuration code.
*/

#include "fapex_stdhdr.h"
#include "fapi/drv_adc.h"
#include "fapi/drv_intr.h"
#include "fapi/reg_adc.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_adc_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_ADC_FuncStrT adc_funcStr;

/*--------------------------------------------------------*/
/* Local Data                                             */
/*--------------------------------------------------------*/
#ifdef __LINUX__
static uint32_t  result;
#endif

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_ADC_Isr(int arg1, void *ptr)
{
#ifndef __LINUX__
    uint32_t result;
#endif
    void *pBsr;

    /* Interrupt service routine for Isr */
    result = adc_funcStr.Isr();

    if(result != 0) {
        /* register bsr */
        pBsr = (void*)FAPEXCFG_INTR_IsBsrFunction(FAPI_INTR_ADC);
        if(pBsr != NULL){
            FAPEXCFG_INTR_RegisterBSR(pBsr);
        }
    }
    FAPICORE_ISRRETVAL(result)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_ADC_Init(void)
{
    int32_t ret = 0;

    adc_funcStr.Init = FAPI_ADC_Init;
    adc_funcStr.Exit = FAPI_ADC_Exit;
    adc_funcStr.Isr = FAPI_ADC_Isr;
    adc_funcStr.Open = FAPI_ADC_Open;
    adc_funcStr.Close = FAPI_ADC_Close;
    adc_funcStr.Read = FAPI_ADC_Read;
    adc_funcStr.PowerOff = FAPI_ADC_PowerOff;
    adc_funcStr.SetClock= FAPI_ADC_SetClock;
   
    /* initialize function of FAPI RAL */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_ADC] == 0){
        FREG_ADC_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_ADC] = 1;
    }

    /* The initialization for ADC pin is set by boards_funcStr.BoardIfc_Register() */

    /* setup interrupt */
    /* adc */
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_ADC,
                               FAPEXCFG_ADC_Isr,
                               NULL,
                               SA_INTERRUPT, (uint8_t*)"ADC",
                               0, 0 );

    /* initialize function of FAPI CDL (adc) */
    ret = adc_funcStr.Init();

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_ADC_Cleanup(void)
{
    /* unregister interrupt */
    FAPEXCFG_INTR_UnregisterISR(FAPI_INTR_ADC);

    adc_funcStr.Exit();
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(adc_funcStr);

EXPORT_SYMBOL(FAPEXCFG_ADC_Init);
EXPORT_SYMBOL(FAPEXCFG_ADC_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
