/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI AUOUT configuration code.
*/

#include "fapex_stdhdr.h"
#include "fapi/drv_auout.h"
#include "fapi/drv_intr.h"
#include "fapi/reg_ao.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_cfdlsaf.h"
#include "fapex_auout_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_AUOUT_FuncStrT auout_funcStr;

/*--------------------------------------------------------*/
/* Local Data                                             */
/*--------------------------------------------------------*/
static FAPEXCFDLSAF_SpnLockStrT  auout_isr_lock;

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_AUOUT_Isr(int arg1, void *ptr)
{
    uint32_t result, flags;
//  void *pBsr;

    FAPEXCFDLSAF_SPIN_LOCK(&auout_isr_lock, &flags);

    /* Interrupt service routine for Isr */
    result = auout_funcStr.Isr();

    /* register bsr */
//  pBsr = (void*)FAPEXCFG_INTR_IsBsrFunction(FAPI_INTR_AUDIO);
//  if(pBsr != NULL){
//      FAPEXCFG_INTR_RegisterBSR(pBsr);
//  }
    FAPEXCFDLSAF_SPIN_UNLOCK(&auout_isr_lock, &flags);

    FAPICORE_ISRRETVAL(result)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/
/* nothing */

/*--------------------------------------------------------*/
/* Set Stream Information.                                */
/*--------------------------------------------------------*/
void FAPEXCFG_AUOUT_SetStreamInfo(FAPI_SYS_HandleT handle,
                                  FAPI_AUOUT_StreamInfoT* infoPtr)
{
    uint32_t  flags;

    FAPEXCFDLSAF_SPIN_LOCK(&auout_isr_lock, &flags);

    auout_funcStr.SetStreamInfo(handle, infoPtr);

    FAPEXCFDLSAF_SPIN_UNLOCK(&auout_isr_lock, &flags);
}

/*--------------------------------------------------------*/
/* Get Stream Information.                                */
/*--------------------------------------------------------*/
FAPI_AUOUT_StreamInfoT FAPEXCFG_AUOUT_GetStreamInfo(FAPI_SYS_HandleT handle)
{
    FAPI_AUOUT_StreamInfoT  stream_info;
    uint32_t  flags;

    FAPEXCFDLSAF_SPIN_LOCK(&auout_isr_lock, &flags);

    stream_info = auout_funcStr.GetStreamInfo(handle);

    FAPEXCFDLSAF_SPIN_UNLOCK(&auout_isr_lock, &flags);

    return stream_info;
}

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_AUOUT_Init(void)
{
    int32_t ret = 0;

    auout_funcStr.Init                   = FAPI_AUOUT_Init;
    auout_funcStr.Exit                   = FAPI_AUOUT_Exit;
    auout_funcStr.Isr                    = FAPI_AUOUT_Isr;
    auout_funcStr.SetInternalDac         = FAPI_AUOUT_SetInternalDac;
    auout_funcStr.SetSpeakers            = FAPI_AUOUT_SetSpeakers;
    auout_funcStr.SetI2sParams           = FAPI_AUOUT_SetI2sParams;
    auout_funcStr.GetI2sParams           = FAPI_AUOUT_GetI2sParams;
    auout_funcStr.SetSpdifParams         = FAPI_AUOUT_SetSpdifParams;
    auout_funcStr.Open                   = FAPI_AUOUT_Open;
    auout_funcStr.Close                  = FAPI_AUOUT_Close;
    auout_funcStr.Start                  = FAPI_AUOUT_Start;
    auout_funcStr.Stop                   = FAPI_AUOUT_Stop;
    auout_funcStr.SetStreamInfo          = FAPI_AUOUT_SetStreamInfo;
    auout_funcStr.GetStreamInfo          = FAPI_AUOUT_GetStreamInfo;
    auout_funcStr.StartHdmi              = FAPI_AUOUT_StartHdmi;
    auout_funcStr.StopHdmi               = FAPI_AUOUT_StopHdmi;
    auout_funcStr.Mute                   = FAPI_AUOUT_Mute;
    auout_funcStr.UnMute                 = FAPI_AUOUT_UnMute;
    auout_funcStr.SetVolume              = FAPI_AUOUT_SetVolume;
    auout_funcStr.GetVolume              = FAPI_AUOUT_GetVolume;
    auout_funcStr.EnableDualMono         = FAPI_AUOUT_EnableDualMono;
    auout_funcStr.DisableDualMono        = FAPI_AUOUT_DisableDualMono;
    auout_funcStr.GetSpeakers            = FAPI_AUOUT_GetSpeakers;
    auout_funcStr.GetI2sBuffers          = FAPI_AUOUT_GetI2sBuffers;
    auout_funcStr.GetSpdifBuffer         = FAPI_AUOUT_GetSpdifBuffer;
    auout_funcStr.GetPcmBuffer           = FAPI_AUOUT_GetPcmBuffer;
    auout_funcStr.RestartIfEmptied       = FAPI_AUOUT_RestartIfEmptied;
    auout_funcStr.SetSpdifCopyProtect    = FAPI_AUOUT_SetSpdifCopyProtect;
    auout_funcStr.GetSpdifCopyProtect    = FAPI_AUOUT_GetSpdifCopyProtect;
    auout_funcStr.SetSpdifLbit           = FAPI_AUOUT_SetSpdifLbit;
    auout_funcStr.GetSpdifLbit           = FAPI_AUOUT_GetSpdifLbit;
    auout_funcStr.SetSpdifStatus         = FAPI_AUOUT_SetSpdifStatus;
    auout_funcStr.SetSpdifPcmLock        = FAPI_AUOUT_SetSpdifPcmLock;
    auout_funcStr.SetSpdifMode           = FAPI_AUOUT_SetSpdifMode;
    auout_funcStr.WritePcm               = FAPI_AUOUT_WritePcm;
    auout_funcStr.SetApt                 = FAPI_AUOUT_SetApt;
    auout_funcStr.AddApt                 = FAPI_AUOUT_AddApt;
    auout_funcStr.RemoveApt              = FAPI_AUOUT_RemoveApt;
    auout_funcStr.SetAptParams           = FAPI_AUOUT_SetAptParams;
    auout_funcStr.GetAptLastError        = FAPI_AUOUT_GetAptLastError;
    auout_funcStr.ResetAptChain          = FAPI_AUOUT_ResetAptChain;
    auout_funcStr.CalculateSkipStallTime = FAPI_AUOUT_CalculateSkipStallTime;
    auout_funcStr.ExecuteSkipStall       = FAPI_AUOUT_ExecuteSkipStall;
   // auout_funcStr.ExecuteSkipStallTime   = FAPI_AUOUT_ExecuteSkipStallTime;
    auout_funcStr.GetSyncState           = FAPI_AUOUT_GetSyncState;
    auout_funcStr.AptVolExecFunc         = FAPI_AUOUT_APT_VOL_ExecFunc;
    auout_funcStr.Apt11DBExecFunc        = FAPI_AUOUT_APT_11DB_ExecFunc;
    auout_funcStr.AptSsrcExecFunc        = FAPI_AUOUT_APT_SSRC_ExecFunc;
    auout_funcStr.AptSsrcAddFunc         = FAPI_AUOUT_APT_SSRC_AddFunc;
    auout_funcStr.AptSsrcRemoveFunc      = FAPI_AUOUT_APT_SSRC_RemoveFunc;
    auout_funcStr.AptSsrcResetFunc       = FAPI_AUOUT_APT_SSRC_ResetFunc;
    /* initialize function of FAPI RAL */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_AO] == 0){
        FREG_AO_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_AO] = 1;
    }

    FAPI_SYS_AUOUT_SET_BOARD_PARAMS

    /* setup interrupt */
    /* auout */
    FAPEXCFDLSAF_SPIN_LOCK_CREATE(&auout_isr_lock);
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_AUDIO, FAPEXCFG_AUOUT_Isr, NULL,
                               SA_INTERRUPT, (uint8_t*)"AUOUT", 0, 0 );

    /* initialize function of FAPI CDL (auout) */
    ret = auout_funcStr.Init();

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_AUOUT_Cleanup(void)
{
    /* unregister interrupt */
    FAPEXCFG_INTR_UnregisterISR(FAPI_INTR_AUDIO);

    FAPEXCFDLSAF_SPIN_LOCK_DELETE(&auout_isr_lock);

    auout_funcStr.Exit();
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(auout_funcStr);

EXPORT_SYMBOL(FAPEXCFG_AUOUT_Init);
EXPORT_SYMBOL(FAPEXCFG_AUOUT_Cleanup);
EXPORT_SYMBOL(FAPEXCFG_AUOUT_GetStreamInfo);
EXPORT_SYMBOL(FAPEXCFG_AUOUT_SetStreamInfo);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
