/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI DMA configuration code.
*/

#include "fapex_stdhdr.h"
#include "fapi/drv_dma.h"
#include "fapi/drv_intr.h"
#include "fapi/reg_dma.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_dma_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_DMA_FuncStrT dma_funcStr;

/*--------------------------------------------------------*/
/* Local Data                                             */
/*--------------------------------------------------------*/
#ifdef __LINUX__
static struct semaphore dma_sem;
static uint32_t         result_dma;
#endif
static int m_isBsrQuit = 0;

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_DMA_Isr(int arg1, void *ptr)
{
#ifndef __LINUX__
    uint32_t result_dma;
    void *pBsr;
#endif

    /* Interrupt service routine for Isr */
    result_dma = dma_funcStr.Isr();

    if(result_dma != 0) {
#ifndef __LINUX__
    /* register bsr */
        pBsr = (void*)FAPEXCFG_INTR_IsBsrFunction(FAPI_INTR_DMA);
        if(pBsr != NULL){
            FAPEXCFG_INTR_RegisterBSR(pBsr);
        }
    }
#else
        /* wake up for FAPEXCFG_DMA_BsrThread() */
        up(&dma_sem);
    }
#endif
    FAPICORE_ISRRETVAL(result_dma)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/
#ifndef __LINUX__
FAPICORE_BSRRET_T FAPEXCFG_DMA_Bsr(unsigned long arg)
{
    dma_funcStr.Bsr(arg);
}

#define DMABSR_FUNCNAME  FAPEXCFG_DMA_Bsr

#else
static int FAPEXCFG_DMA_BsrThread(void *arg)
{
    int32_t  ret;
    struct sched_param  schedparam = { .sched_priority = 90 };
    sched_setscheduler(current, SCHED_FIFO, &schedparam);

    lock_kernel();
    daemonize("fapex_dma_bsr");
    unlock_kernel();
    while(1) {
        /* wait for FAPEXCFG_DMA_Bsr() */
        ret = down_interruptible(&dma_sem);
        if(ret != 0 || m_isBsrQuit) {
            break;
        }

        dma_funcStr.Bsr(result_dma);
    }
    return 0;
}
#endif /* !__LINUX__ */

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_DMA_Init(void)
{
    int32_t ret = 0;
#ifdef __LINUX__
    int32_t dma_pid;
#endif

    dma_funcStr.Init             = FAPI_DMA_Init;
    dma_funcStr.Exit             = FAPI_DMA_Exit;
    dma_funcStr.Isr              = FAPI_DMA_Isr;
    dma_funcStr.Bsr              = FAPI_DMA_Bsr;
    dma_funcStr.AllocateRequest  = FAPI_DMA_AllocateRequest;
    dma_funcStr.ReleaseRequest   = FAPI_DMA_ReleaseRequest;
    dma_funcStr.GetRequestList   = FAPI_DMA_GetRequestList;
    dma_funcStr.CancelRequest    = FAPI_DMA_CancelRequest;
    dma_funcStr.SendRequest      = FAPI_DMA_SendRequest;
    dma_funcStr.SendRequest2     = FAPI_DMA_SendRequest2;
    dma_funcStr.GetRequestStatus = FAPI_DMA_GetRequestStatus;

    dma_funcStr.ATA_ReadData                = FAPI_DMA_ATA_ReadData;
    dma_funcStr.ATA_WriteData               = FAPI_DMA_ATA_WriteData;
    dma_funcStr.StartMemcopy                = FAPI_DMA_StartMemcopy;
    dma_funcStr.StartPFlashcopy             = FAPI_DMA_StartPFlashcopy;
    dma_funcStr.StartMemfill                = FAPI_DMA_StartMemfill;
    dma_funcStr.StartSearchReplace          = FAPI_DMA_StartSearchReplace;
    dma_funcStr.StartCcir656Capture         = FAPI_DMA_StartCcir656Capture;
    dma_funcStr.StartCcir656CaptureHalfMode = FAPI_DMA_StartCcir656CaptureHalfMode;
    dma_funcStr.StopCcir656Capture          = FAPI_DMA_StopCcir656Capture;
    dma_funcStr.StartAudioCapture           = FAPI_DMA_StartAudioCapture;
    dma_funcStr.StopAudioCapture            = FAPI_DMA_StopAudioCapture;
    dma_funcStr.StartCrypto                 = FAPI_DMA_StartCrypto;
    dma_funcStr.StartCsConversion           = FAPI_DMA_StartCsConversion;
    dma_funcStr.StartTsToAPB                = FAPI_DMA_StartTsToAPB;
    dma_funcStr.StopTsToAPB                 = FAPI_DMA_StopTsToAPB;

    /* initialize function of FAPI RAL */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_DMA] == 0){
        FREG_DMA_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_DMA] = 1;
    }

    /* setup interrupt */
    /* dma */
#ifndef __LINUX__
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_DMA, FAPEXCFG_DMA_Isr,
                               (FAPICORE_BSRRET_T (*)(unsigned long))DMABSR_FUNCNAME,
                               SA_INTERRUPT, (uint8_t*)"DMA",
                               4096, 512 );
#else
    sema_init(&dma_sem, 0);
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_DMA,
                               FAPEXCFG_DMA_Isr, NULL,
                               SA_INTERRUPT, (uint8_t*)"DMA",
                               4096, 512 );

    /* create kernel thread because FAPI_DMA_Bsr() use FAPI_SYS_GET_SEMAPHORE(). */
    dma_pid = kernel_thread(FAPEXCFG_DMA_BsrThread, NULL, SIGCHLD | CLONE_KERNEL);
    if(dma_pid < 0) {
        return dma_pid;
    }
#endif

    /* initialize function of FAPI CDL (dma) */
    ret = dma_funcStr.Init();

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_DMA_Cleanup(void)
{
    /* unregister interrupt */
    FAPEXCFG_INTR_UnregisterISR(FAPI_INTR_DMA);
    m_isBsrQuit = 1;
    up(&dma_sem);    // bsr may block, wakeup
    dma_funcStr.Exit();
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(dma_funcStr);

EXPORT_SYMBOL(FAPEXCFG_DMA_Init);
EXPORT_SYMBOL(FAPEXCFG_DMA_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
