/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI GPIO configuration code.
*/

#include "fapex_stdhdr.h"
#include "fapi/drv_gpio.h"
#include "fapi/drv_intr.h"
#include "fapi/reg_gpio.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_gpio_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_GPIO_FuncStrT gpio_funcStr;

/*--------------------------------------------------------*/
/* Local Data                                             */
/*--------------------------------------------------------*/
#ifdef __LINUX__
static uint32_t  result;
#endif

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_GPIO_Isr(int arg1, void *ptr)
{
#ifndef __LINUX__
    uint32_t result;
#endif
    void *pBsr;

    /* Interrupt service routine for Isr */
    result = gpio_funcStr.Isr();

    if(result != 0) {
        /* register bsr */
        pBsr = (void*)FAPEXCFG_INTR_IsBsrFunction(FAPI_INTR_GPIO);
        if(pBsr != NULL){
            FAPEXCFG_INTR_RegisterBSR(pBsr);
        }
    }
    FAPICORE_ISRRETVAL(result)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/
#ifndef __LINUX__
FAPICORE_BSRRET_T FAPEXCFG_GPIO_Bsr(unsigned long arg)
{
    gpio_funcStr.Bsr(arg);
}

#define GPIOBSR_FUNCNAME  FAPEXCFG_GPIO_Bsr

#else
FAPICORE_BSRRET_T FAPEXCFG_GPIO_Bsr(unsigned long arg)
{
    gpio_funcStr.Bsr(result);
}

#define GPIOBSR_FUNCNAME (&FAPEXCFG_GPIO_Tasklet)

static DECLARE_TASKLET( FAPEXCFG_GPIO_Tasklet, FAPEXCFG_GPIO_Bsr, (unsigned long)&result );
#endif /* !__LINUX__ */

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_GPIO_Init(void)
{
    int32_t ret = 0;

    gpio_funcStr.Init = FAPI_GPIO_Init;
    gpio_funcStr.Exit = FAPI_GPIO_Exit;
    gpio_funcStr.SetPinFunction = FAPI_GPIO_SetPinFunction;
    gpio_funcStr.Isr = FAPI_GPIO_Isr;
    gpio_funcStr.Bsr = FAPI_GPIO_Bsr;
    gpio_funcStr.Open = FAPI_GPIO_Open;
    gpio_funcStr.Close = FAPI_GPIO_Close;
    gpio_funcStr.ReadBit = FAPI_GPIO_ReadBit;
    gpio_funcStr.WriteBit = FAPI_GPIO_WriteBit;
    gpio_funcStr.GetPinByFunction = FAPI_GPIO_GetPinByFunction;
   
    /* initialize function of FAPI RAL */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_GPIO] == 0){
        FREG_GPIO_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_GPIO] = 1;
    }

    /* The initialization for GPIO pin is set by boards_funcStr.BoardIfc_Register() */

    /* setup interrupt */
    /* gpio */
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_GPIO,
                               FAPEXCFG_GPIO_Isr,
                               (FAPICORE_BSRRET_T (*)(unsigned long))GPIOBSR_FUNCNAME,
                               SA_INTERRUPT, (uint8_t*)"GPIO",
                               1024, 4 );

    /* initialize function of FAPI CDL (gpio) */
    ret = gpio_funcStr.Init();

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_GPIO_Cleanup(void)
{
    /* unregister interrupt */
    FAPEXCFG_INTR_UnregisterISR(FAPI_INTR_GPIO);

    gpio_funcStr.Exit();
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(gpio_funcStr);

EXPORT_SYMBOL(FAPEXCFG_GPIO_Init);
EXPORT_SYMBOL(FAPEXCFG_GPIO_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
