/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI INTR configuration code.
*/

#include "fapex_stdhdr.h"
#include "fapi/sys_services.h"
#include "fapi/drv_intr.h"
#include "fapi/reg_irq_ctrl.h"
#include "fapi/reg_vic.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_types.h"
#include "fapex_intr_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_INTR_FuncStrT intr_funcStr;

#ifndef __LINUX__
/* the array of all required drivers */
FAPI_SYS_DriverT   driver_info [FAPI_INTR_COUNT];
FAPI_SYS_DriverT  *driver_array[FAPI_INTR_COUNT+1];
uint32_t           driver_count = 0;

/*--------------------------------------------------------*/
/* set drivers to system(RTOS)                            */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_INTR_InitDrivers(void)
{
    /* set last indicator */
    driver_array[driver_count] = NULL;

    return( FAPI_SYS_INIT_DRIVERS(driver_array) );
}

int32_t dummy_func(void)
{
    return 0;
}
#else
/* bsr functions */
FAPICORE_BSRRET_T *pBsrFunction[FAPI_INTR_COUNT];
#endif /* !__LINUX__ */

/*--------------------------------------------------------*/
/* registration of ISR/BSR function at initialization     */
/*--------------------------------------------------------*/
/*!! flag SA_SHIRQ is not supported. !!*/
int32_t FAPEXCFG_INTR_RegisterISR(int32_t irqNo, FAPICORE_ISRRET_T (*pIsr)(int, void*),
                                  FAPICORE_BSRRET_T (*pBsr)(unsigned long),
                                  uint32_t flags, uint8_t *devname,
                                  uint32_t bsrStackSize, uint32_t bsrFifoDepth)
{
    int32_t ret;

    /* register isr,bsr */
    if( (irqNo >= 0) && (pIsr != NULL) ) {
#ifndef __LINUX__

        FAPI_SYS_DriverT  *driver = &driver_info[driver_count];

        driver->irqNo          = irqNo;
        driver->isrFunc        = (uint32_t (*)(void))pIsr;
        driver->bsrFunc        = (void (*)(uint32_t))pBsr;
        driver->bsrExecution   = (FAPI_SYS_BsrExecutionEnumT)flags;
        driver->driverNamePtr  = (const char*)devname;
        driver->initDriverFunc = dummy_func;
        driver->exitDriverFunc = (void (*)(void))dummy_func;
        driver->bsrStackSize   = bsrStackSize;
        driver->bsrFifoDepth   = bsrFifoDepth;

        ret = intr_funcStr.RegisterIrq(irqNo, (FAPI_INTR_IsrFuncT)pIsr, (FAPI_INTR_BsrFuncT)pBsr);

        if(ret < 0) {
            return ret;
        }
        driver_array[driver_count] = driver;
        driver_count ++;

#else /* !__LINUX__ */

        pBsrFunction[irqNo] = pBsr;
        if(pIsr != NULL){
            ret = request_irq(irqNo, pIsr, flags, devname, NULL);
            if(ret < 0) {
                return ret;
            }
        }
        intr_funcStr.EnableIrqCtrl(irqNo); /* enable INTERRUPT_CNTL */

#endif /* !__LINUX__ */
    }
    else {
        return -1;
    }
    return 0;
}

/*--------------------------------------------------------*/
/* unregistration of ISR/BSR function                       */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_INTR_UnregisterISR(int32_t irqNo)
{
    if(irqNo >= 0) {
        intr_funcStr.DisableIrqCtrl(irqNo); /* disable INTERRUPT_CNTL */
#ifndef __LINUX__
        intr_funcStr.UnregisterIrq(irqNo);
#else /* !__LINUX__ */
        free_irq(irqNo, NULL);
        if(pBsrFunction[irqNo] != NULL) {
            tasklet_kill(pBsrFunction[irqNo]);
            pBsrFunction[irqNo] = NULL;
        }
#endif /* !__LINUX__ */
    }
    return 0;
}

/*--------------------------------------------------------*/
/* get BSR function                                       */
/*--------------------------------------------------------*/
void *FAPEXCFG_INTR_IsBsrFunction(int32_t irqNo)
{
#ifndef __LINUX__
    return FAPI_INTR_BsrFunctionTable[irqNo];
#else
    return pBsrFunction[irqNo];
#endif /* !__LINUX__ */
}

/*--------------------------------------------------------*/
/* registration of BSR function as executable             */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_INTR_RegisterBSR(void *ptr)
{
#ifdef __SOFTARCH_REGISTER_BSR_FUNCTION
#ifdef __LINUX__

    tasklet_schedule((struct tasklet_struct*)ptr);

    return 0;
#endif /* __LINUX__ */
#else /* __SOFTARCH_REGISTER_BSR_FUNCTION */
/* FAMOS */

    return 0;
#endif /*__SOFTARCH_REGISTER_BSR_FUNCTION */
}

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_INTR_Isr31(int arg1, void *ptr)
{
    uint32_t result;
//  void *pBsr;

    /* Interrupt service routine for Isr */
    result = intr_funcStr.Isr31();

//  if(result != 0) {
//      /* register bsr */
//      pBsr = (void*)FAPEXCFG_INTR_IsBsrFunction(FAPI_INTR_POLLING_MODE);
//      if(pBsr != NULL){
//          FAPEXCFG_INTR_RegisterBSR(pBsr);
//      }
//  }
    FAPICORE_ISRRETVAL(result)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/
/* nothing */

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_INTR_Init(void)
{
    int32_t ret = 0;

    intr_funcStr.Init                 = FAPI_INTR_Init;
    intr_funcStr.Exit                 = FAPI_INTR_Exit;
    intr_funcStr.SetReceiver          = FAPI_INTR_SetReceiver;
    intr_funcStr.EnableIrqCtrl        = FAPI_INTR_EnableIrqCtrl;
    intr_funcStr.DisableIrqCtrl       = FAPI_INTR_DisableIrqCtrl;
    intr_funcStr.ThrowSwi             = FAPI_INTR_ThrowSwi;
    intr_funcStr.ThrowPollingModeIrq  = FAPI_INTR_ThrowPollingModeIrq;
    intr_funcStr.RegisterPipeCallback = FAPI_INTR_RegisterPipeCallback;
    intr_funcStr.WritePipe            = FAPI_INTR_WritePipe;
    intr_funcStr.Isr31                = FAPI_INTR_Isr31;
#ifndef __LINUX__
    intr_funcStr.SetPriority          = FAPI_INTR_SetPriority;
    intr_funcStr.RegisterIrq          = FAPI_INTR_RegisterIrq;
    intr_funcStr.UnregisterIrq        = FAPI_INTR_UnregisterIrq;
    intr_funcStr.RegisterSwi          = FAPI_INTR_RegisterSwi;
    intr_funcStr.UnregisterSwi        = FAPI_INTR_UnregisterSwi;
#endif /* !__LINUX__ */

    /* initialize function of FAPI RAL */
    /*
    (intr)
     FREG_IRQ_CTRL <----
     FREG_VIC
    (timer)
     FREG_IRQ_CTRL <----
    */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_IRQ_CTRL] == 0){
        FREG_IRQ_CTRL_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_IRQ_CTRL] = 1;
    }
#ifndef __LINUX__
/* for the 1st step */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VIC] == 0){
        FREG_VIC_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VIC] = 1;
    }
#endif /* !__LINUX__ */

    /* setup interrupt */
    /* intr */
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_POLLING_MODE,
                               FAPEXCFG_INTR_Isr31, NULL,
                               SA_INTERRUPT, (uint8_t*)"INTR",
                               0, 0 );

    /* initialize function of FAPI CDL (intr) */
    FAPI_SYS_INTR_SET_BOARD_PARAMS

    ret = intr_funcStr.Init(); /* already called for OS */

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_INTR_Cleanup(void)
{
    /* unregister interrupt */
    FAPEXCFG_INTR_UnregisterISR(FAPI_INTR_POLLING_MODE);

    intr_funcStr.Exit();
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(intr_funcStr);

EXPORT_SYMBOL(FAPEXCFG_INTR_RegisterISR);
EXPORT_SYMBOL(FAPEXCFG_INTR_UnregisterISR);
EXPORT_SYMBOL(FAPEXCFG_INTR_IsBsrFunction);
EXPORT_SYMBOL(FAPEXCFG_INTR_RegisterBSR);
EXPORT_SYMBOL(FAPEXCFG_INTR_Init);
EXPORT_SYMBOL(FAPEXCFG_INTR_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
