/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI configuration code.
*/
#define FAPICORE_C

#include "fapex.h"
#include "fapex_stdhdr.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "boot/rom_symbols.h"

char FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULENUM];

/*!
*******************************************************************************
**
** \brief  Initialize process of the FAPI core modules.
**
** \param  void (can't set any argument)
**
******************************************************************************/
static int32_t fapicore_CDLInit(void)
{
    int32_t ret = 0;

   /* board setup */

    FAPEXDebugPrint(1, "call FAPEXCFG_BOARDS_Init\n");
    ret = FAPEXCFG_BOARD_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_BOARDS_Init %d\n", ret);
        return ret;
    }
	
    FAPEXDebugPrint(1, "call FAPEXCFG_BOARD_Init\n");
    ret = FAPEXCFG_BOARDS_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_BOARD_Init %d\n", ret);
        return ret;
    }

#if FAPICORE_INIT_MMU == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_MMU_Init\n");

    /* initialize function of FAPI CDL (mmu) */
    ret = FAPEXCFG_MMU_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_MMU_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_GPREG == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_GPREG_Init\n");

    /* initialize function of FAPI CDL (gpreg) */
    ret = FAPEXCFG_GPREG_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_GPREG_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_INTR == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_INTR_Init\n");

    /* initialize function of FAPI CDL (intr) */
    ret = FAPEXCFG_INTR_Init(); /* already called for OS */

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_INTR_Init %d\n", ret);
        return ret;
    }
#endif


#if FAPICORE_INIT_TIMER == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_TIMER_Init\n");

    /* initialize function of FAPI CDL (timer) */
    ret = FAPEXCFG_TIMER_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "call FAPEXCFG_TIMER_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_GPIO == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_GPIO_Init\n");

    /* initialize function of FAPI CDL (gpio) */
    ret = FAPEXCFG_GPIO_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_GPIO_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_UART == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_UART_Init\n");

    /* initialize function of FAPI CDL (uart) */
    ret = FAPEXCFG_UART_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_UART_Init %d\n", ret);
        return ret;
    }
#else
    {
        volatile BOOT_ROM_SymbolT *rom_symbols;
        rom_symbols = (volatile BOOT_ROM_SymbolT*)FIO_ADDRESS(SRAM,0x02000000UL);

        /* UART for ARM enabled */
        rom_symbols->uart_sync_init = 1337;    /* UART_CPU_ENABLED */

        FAPEXDebugPrint(1, "enable the connection of ARM and ARC\n");
    }
#endif

#if FAPICORE_INIT_UPI == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_UPI_Init\n");

    /* initialize function of FAPI CDL (upi) */
    ret = FAPEXCFG_UPI_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_UPI_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_CLKPWR == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_CLKPWR_Init\n");

    /* initialize function of FAPI CDL (clkpwr) */
    ret = FAPEXCFG_CLKPWR_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_CLKPWR_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_DMA == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_DMA_Init\n");

    /* initialize function of FAPI CDL (dma) */
    ret = FAPEXCFG_DMA_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_DMA_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_USB == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_USB_Init\n");

    /* initialize function of FAPI CDL (usb) */
    ret = FAPEXCFG_USB_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_USB_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_I2C == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_I2C_Init\n");

    /* initialize function of FAPI CDL (i2c) */
    ret = FAPEXCFG_I2C_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_I2C_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_ETH == 1
//  FAPEXDebugPrint(1, "call FAPEXCFG_ETH_Init\n");

    /* initialize function of FAPI CDL (eth) */
//  ret = FAPEXCFG_ETH_Init();
//
//  if(ret < 0) {
//      FAPEXDebugPrint(2, "error FAPEXCFG_ETH_Init %d\n", ret);
//      return ret;
//  }
#endif

#if FAPICORE_INIT_FLASH == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_FLASH_Init\n");

    /* initialize function of FAPI CDL (flash) */
    ret = FAPEXCFG_FLASH_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_FLASH_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_BOOT == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_BOOT_Init\n");

    /* initialize function of FAPI CDL (boot) */
    ret = FAPEXCFG_BOOT_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_BOOT_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_HDMI == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_HDMI_Init\n");

    /* initialize function of FAPI CDL (hdmi) */
    ret = FAPEXCFG_HDMI_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_HDMI_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_HDMICEC == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_HDMICEC_Init\n");

    /* initialize function of FAPI CDL (hdmicec) */
    ret = FAPEXCFG_HDMICEC_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_HDMICEC_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_BLTE == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_BLTE_Init\n");

    /* initialize function of FAPI CDL (blte) */
    ret = FAPEXCFG_BLTE_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_BLTE_Init %d\n", ret);
        return ret;
    }
#endif
#if FAPICORE_INIT_SSP == 1
 FAPEXDebugPrint(1, "call FAPEXCFG_SSP_Init\n");

    /* initialize function of FAPI CDL (SSP) */
    ret = FAPEXCFG_SSP_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_SSP_Init %d\n", ret);
        return ret;
    }
#endif
#if FAPICORE_INIT_ADC == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_ADC_Init\n");

    /* initialize function of FAPI CDL (adc) */
    ret = FAPEXCFG_ADC_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_ADC_Init %d\n", ret);
        return ret;
    }
#endif

#if FAPICORE_INIT_PV == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_PV_Init\n");

    /* initialize function of FAPI CDL (pv) */
    ret = FAPEXCFG_PV_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_PV_Init %d\n", ret);
        return ret;
    }
#endif

#if  FAPICORE_INIT_CEC == 1
	FAPEXDebugPrint(1, "call FAPEXCFG_CEC_Init\n");

    /* initialize function of FAPI CDL (CEC) */
    ret = FAPEXCFG_CEC_Init();

    if(ret < 0) {
        FAPEXDebugPrint(2, "error FAPEXCFG_CEC_Init %d\n", ret);
        return ret;
    }
#endif
    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FAPI core modules.
**
** \param  void (can't set any argument)
**
******************************************************************************/
static void fapicore_CDLCleanup(void)
{
	/* board clean up */
	 FAPEXDebugPrint(1, "call FAPEXCFG_BOARD_cleanup\n");
	 FAPEXCFG_BOARD_cleanup();
	
	 FAPEXDebugPrint(1, "call FAPEXCFG_BOARDS_Cleanup\n");
	 FAPEXCFG_BOARDS_Cleanup();

#if FAPICORE_INIT_BLTE == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_BLTE_Cleanup\n");

    FAPEXCFG_BLTE_Cleanup();
#endif

#if FAPICORE_INIT_HDMI == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_HDMI_Cleanup\n");

    FAPEXCFG_HDMI_Cleanup();
#endif

#if FAPICORE_INIT_BOOT == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_BOOT_Cleanup\n");

    FAPEXCFG_BOOT_Cleanup();
#endif

#if FAPICORE_INIT_FLASH == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_FLASH_Cleanup\n");

    FAPEXCFG_FLASH_Cleanup();
#endif

#if FAPICORE_INIT_ETH == 1
//  FAPEXDebugPrint(1, "call FAPEXCFG_ETH_Cleanup\n");
//
//  FAPEXCFG_ETH_Cleanup();
#endif

#if FAPICORE_INIT_I2C == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_I2C_Cleanup\n");

    FAPEXCFG_I2C_Cleanup();
#endif

#if FAPICORE_INIT_USB == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_USB_Init\n");

    FAPEXCFG_USB_Init();
#endif

#if FAPICORE_INIT_DMA == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_DMA_Cleanup\n");

    FAPEXCFG_DMA_Cleanup();
#endif

#if FAPICORE_INIT_CLKPWR == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_CLKPWR_Cleanup\n");

    FAPEXCFG_CLKPWR_Cleanup();
#endif

#if FAPICORE_INIT_UPI == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_UPI_Cleanup\n");

    FAPEXCFG_UPI_Cleanup();
#endif

#if FAPICORE_INIT_TIMER == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_TIMER_Cleanup\n");

    FAPEXCFG_TIMER_Cleanup();
#endif

#if FAPICORE_INIT_UART == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_UART_Cleanup\n");

    FAPEXCFG_UART_Cleanup();
#endif

#if FAPICORE_INIT_SSP == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_SSP_Cleanup\n");

    FAPEXCFG_SSP_Cleanup();
#endif

#if FAPICORE_INIT_GPIO == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_GPIO_Cleanup\n");

    FAPEXCFG_GPIO_Cleanup();
#endif

#if FAPICORE_INIT_INTR == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_INTR_Cleanup\n");

    FAPEXCFG_INTR_Cleanup();
#endif

#if FAPICORE_INIT_GPREG == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_GPREG_Cleanup\n");

    FAPEXCFG_GPREG_Cleanup();
#endif

#if FAPICORE_INIT_MMU == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_MMU_Cleanup\n");

    FAPEXCFG_MMU_Cleanup();
#endif

#if FAPICORE_INIT_ADC == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_ADC_Cleanup\n");

    FAPEXCFG_ADC_Cleanup();
#endif

#if FAPICORE_INIT_PV == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_PV_Cleanup\n");

    FAPEXCFG_PV_Cleanup();
#endif

#if FAPICORE_INIT_CEC == 1
    FAPEXDebugPrint(1, "call FAPEXCFG_HDMICEC_Cleanup\n");

    FAPEXCFG_HDMICEC_Cleanup();
#endif
    return;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FAPI register access layer.
**
** \param  void (can't set any argument)
**
******************************************************************************/
static void fapicore_RALCleanup(void)
{
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_AO]       = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_BM]       = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_CLKPWR]   = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_DDRRAMIF] = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_DMA]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_ETH]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_FPC]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_GPIO]     = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_I2C]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_ICC]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_IR]       = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_IRQ_CTRL] = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_SFLASH]   = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_SSP]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_TSD]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_UART]     = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_UPI]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VD]       = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VE]       = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VE_HD]    = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VIC]      = 0;
    FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VO]       = 0;

    return;
}

/*!
*******************************************************************************
**
** \brief  Initialize process of the FAPI core modules.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXCFG_FAPI_Init(void)
{
    int32_t ret, modcnt;

    /* RAL module init indicate flag is initialized */
    for(modcnt=0; modcnt<FAPEX_RAL_MODULENUM; modcnt++){
        FAPEXCFG_RAL_initialized_flag[modcnt] = 0;
    }

    /* initialize Lowlevel CDL (and Lowlevel RAL) */
    ret = fapicore_CDLInit();

    FAPEXDebugPrint(1, "FAPICORE is initialized %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FAPI core modules.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXCFG_FAPI_Cleanup(void)
{
    /* cleanup CDL */
    fapicore_CDLCleanup();

    /* cleanup RAL (clear flag) */
    fapicore_RALCleanup();

    FAPEXDebugPrint(1, "FAPICORE is cleaned up\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(FAPEXCFG_FAPI_Init);
EXPORT_SYMBOL(FAPEXCFG_FAPI_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */

