/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI INTR configuration code.
*/

#include "fapex_stdhdr.h"
#include "fapi/drv_tsd.h"
#include "fapi/drv_intr.h"
#include "fapi/reg_tsd.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_tsd_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_TSD_FuncStrT tsd_funcStr;

/*--------------------------------------------------------*/
/* Local Data                                             */
/*--------------------------------------------------------*/
#ifdef __LINUX__
static uint32_t  result_isr;
#endif

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_TSD_Isr(int arg1, void *ptr)
{
#ifndef __LINUX__
    uint32_t result_isr;
#endif
    void *pBsr;

    /* Interrupt service routine for Isr */
    result_isr = tsd_funcStr.Isr();

    if(result_isr != 0) {
        /* register bsr */
        pBsr = (void*)FAPEXCFG_INTR_IsBsrFunction(FAPI_INTR_TSD);
        if(pBsr != NULL) {
            FAPEXCFG_INTR_RegisterBSR(pBsr);
        }
    }
    FAPICORE_ISRRETVAL(result_isr)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/
#ifndef __LINUX__
FAPICORE_BSRRET_T FAPEXCFG_TSD_Bsr(unsigned long arg)
{
    tsd_funcStr.Bsr(arg);
}

#define TSDBSR_FUNCNAME  FAPEXCFG_TSD_Bsr

#else
FAPICORE_BSRRET_T FAPEXCFG_TSD_Bsr(unsigned long arg)
{
    uint32_t  result;

    result = *(uint32_t *)arg;
    tsd_funcStr.Bsr(result);
}

#define TSDBSR_FUNCNAME (&FAPEXCFG_TSD_Tasklet)

static DECLARE_TASKLET( FAPEXCFG_TSD_Tasklet, FAPEXCFG_TSD_Bsr, (unsigned long)&result_isr );

#endif /* !__LINUX__ */

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_TSD_Init(void)
{
    int32_t ret = 0;

    tsd_funcStr.Init                               = FAPI_TSD_Init;
    tsd_funcStr.Exit                               = FAPI_TSD_Exit;
    tsd_funcStr.Isr                                = FAPI_TSD_Isr;
    tsd_funcStr.Bsr                                = FAPI_TSD_Bsr;
    tsd_funcStr.Open                               = FAPI_TSD_Open;
    tsd_funcStr.Close                              = FAPI_TSD_Close;
    tsd_funcStr.SetChannelPid                      = FAPI_TSD_SetChannelPid;
    tsd_funcStr.GetChannelPid                      = FAPI_TSD_GetChannelPid;
    tsd_funcStr.StartChannel                       = FAPI_TSD_StartChannel;
    tsd_funcStr.StopChannel                        = FAPI_TSD_StopChannel;
    tsd_funcStr.SetPesParser                       = FAPI_TSD_SetPesParser;
    tsd_funcStr.Configure                          = FAPI_TSD_Configure;
    tsd_funcStr.SetSyncChannel                     = FAPI_TSD_SetSyncChannel;
    tsd_funcStr.SetBuffer                          = FAPI_TSD_SetBuffer;
    tsd_funcStr.GetBuffer                          = FAPI_TSD_GetBuffer;
    tsd_funcStr.SetSectionFilter                   = FAPI_TSD_SetSectionFilter;
    tsd_funcStr.GetSectionFilter                   = FAPI_TSD_GetSectionFilter;
    tsd_funcStr.ClearSectionFilter                 = FAPI_TSD_ClearSectionFilter;
    tsd_funcStr.SetSectionFilterMode               = FAPI_TSD_SetSectionFilterMode;
    tsd_funcStr.GetSectionFilterMode               = FAPI_TSD_GetSectionFilterMode;
    tsd_funcStr.SetPacketEndNotifier               = FAPI_TSD_SetPacketEndNotifier;
    tsd_funcStr.SetScrambledPacketNotifier         = FAPI_TSD_SetScrambledPacketNotifier;
    tsd_funcStr.SetPacketTimeoutNotifier           = FAPI_TSD_SetPacketTimeoutNotifier;
    tsd_funcStr.SetValidPacketTimeoutNotifier      = FAPI_TSD_SetValidPacketTimeoutNotifier;
    tsd_funcStr.SetPtsNotifier                     = FAPI_TSD_SetPtsNotifier;
    tsd_funcStr.SetPtsEvaluator                    = FAPI_TSD_SetPtsEvaluator;
    tsd_funcStr.GetPacketControlAddress            = FAPI_TSD_GetPacketControlAddress;
    tsd_funcStr.GetPacketRegisterAddress           = FAPI_TSD_GetPacketRegisterAddress;
    tsd_funcStr.GetLinearPacketRegisterAddress     = FAPI_TSD_GetLinearPacketRegisterAddress;
    tsd_funcStr.GetLinearPacketAutoRegisterAddress = FAPI_TSD_GetLinearPacketAutoRegisterAddress;
    tsd_funcStr.SetFreeChannelPid                  = FAPI_TSD_SetFreeChannelPid;
    tsd_funcStr.ClearChannelPid                    = FAPI_TSD_ClearChannelPid;
    tsd_funcStr.GetChannelId                       = FAPI_TSD_GetChannelId;
    tsd_funcStr.GetBlockId                         = FAPI_TSD_GetBlockId;
    tsd_funcStr.GetPesParserId                     = FAPI_TSD_GetPesParserId;
    tsd_funcStr.GetHandleByPid                     = FAPI_TSD_GetHandleByPid;
    
    /* initialize function of FAPI RAL */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_TSD] == 0){
        FREG_TSD_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_TSD] = 1;
    }

    /* setup interrupt */
    /* tsd */
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_TSD,
                               FAPEXCFG_TSD_Isr,
                               (FAPICORE_BSRRET_T (*)(unsigned long))TSDBSR_FUNCNAME,
                               SA_INTERRUPT, (uint8_t*)"TSD",
                               2048, 256 );

    /* initialize function of FAPI CDL (tsd) */
    ret = tsd_funcStr.Init();

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_TSD_Cleanup(void)
{
    /* unregister interrupt */
    FAPEXCFG_INTR_UnregisterISR(FAPI_INTR_TSD);

    tsd_funcStr.Exit();
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(tsd_funcStr);

EXPORT_SYMBOL(FAPEXCFG_TSD_Init);
EXPORT_SYMBOL(FAPEXCFG_TSD_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
