/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI UART configuration code.
*/

#include "fapex_stdhdr.h"
#include "fapi/drv_uart.h"
#include "fapi/reg_uart.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_uart_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_UART_FuncStrT uart_funcStr;

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_UART_Isr(int arg1, void *ptr)
{
    uint32_t result;
//  void *pBsr;

    /* Interrupt service routine for Isr */
    result = uart_funcStr.Isr();

    FAPICORE_ISRRETVAL(result)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/
/* nothing */

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_UART_Init(void)
{
    int32_t ret = 0;

    uart_funcStr.Init               = FAPI_UART_Init;
    uart_funcStr.Exit               = FAPI_UART_Exit;
    uart_funcStr.Isr                = FAPI_UART_Isr;
    uart_funcStr.SetStdioBlockIndex = FAPI_UART_SetStdioBlockIndex;
    uart_funcStr.SetClockFrequency  = FAPI_UART_SetClockFrequency;
    uart_funcStr.SetInterruptMode   = FAPI_UART_SetInterruptMode;
    uart_funcStr.SetBaudrate        = FAPI_UART_SetBaudrate;
    uart_funcStr.SetDataBits        = FAPI_UART_SetDataBits;
    uart_funcStr.SetStopBits        = FAPI_UART_SetStopBits;
    uart_funcStr.SetParity          = FAPI_UART_SetParity;
    uart_funcStr.SetFlowControl     = FAPI_UART_SetFlowControl;
    uart_funcStr.Open               = FAPI_UART_Open;
    uart_funcStr.Close              = FAPI_UART_Close;
    uart_funcStr.WriteByte          = FAPI_UART_WriteByte;
    uart_funcStr.ReadByte           = FAPI_UART_ReadByte;
    uart_funcStr.CheckTxFifo        = FAPI_UART_CheckTxFifo;
    uart_funcStr.CheckRxFifo        = FAPI_UART_CheckRxFifo;
    uart_funcStr.LockMutex          = FAPI_UART_LockMutex;
    uart_funcStr.UnlockMutex        = FAPI_UART_UnlockMutex;

    /* initialize function of FAPI RAL */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_UART] == 0){
        FREG_UART_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_UART] = 1;
    }

    FAPI_SYS_UART_SET_BOARD_PARAMS
    
    /* setup interrupt */
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_UART, FAPEXCFG_UART_Isr, NULL,
                               SA_INTERRUPT, (uint8_t*)"UART", 0, 0 );

    /* initialize function of FAPI CDL (uart) */
    ret = uart_funcStr.Init();

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_UART_Cleanup(void)
{
    uart_funcStr.Exit();
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(uart_funcStr);

EXPORT_SYMBOL(FAPEXCFG_UART_Init);
EXPORT_SYMBOL(FAPEXCFG_UART_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
