/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FAPI extension code.
*/

#include "fapex_stdhdr.h"
#include "fapi/drv_viscale.h"
#include "fapi/drv_intr.h"
#include "fapi/reg_vo.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_viscale_func.h"

/*--------------------------------------------------------*/
/* Global Data                                            */
/*--------------------------------------------------------*/
FAPEX_VISCALE_FuncStrT viscale_funcStr;

/*--------------------------------------------------------*/
/* ISR.                                                   */
/*--------------------------------------------------------*/
FAPICORE_ISRRET_T FAPEXCFG_VISCALE_Isr(int arg1, void *ptr)
{
    uint32_t result;
//  void *pBsr;

    /* Interrupt service routine for Isr */
    result = viscale_funcStr.Isr();

    /* register bsr */
//  pBsr = (void*)FAPEXCFG_INTR_IsBsrFunction(FAPI_INTR_VO1);
//  if(pBsr != NULL){
//      FAPEXCFG_INTR_RegisterBSR(pBsr);
//  }

    FAPICORE_ISRRETVAL(result)
}

/*--------------------------------------------------------*/
/* BSR.                                                   */
/*--------------------------------------------------------*/
/* nothing */

/*--------------------------------------------------------*/
/* INIT.                                                  */
/*--------------------------------------------------------*/
int32_t FAPEXCFG_VISCALE_Init(void)
{
    int32_t ret = 0;

    viscale_funcStr.SetIpcMemory             = FAPI_VISCALE_SetIpcMemory;
    viscale_funcStr.Init                     = FAPI_VISCALE_Init;
    viscale_funcStr.Exit                     = FAPI_VISCALE_Exit;
    viscale_funcStr.Isr                      = FAPI_VISCALE_Isr;
    viscale_funcStr.Bsr                      = FAPI_VISCALE_Bsr;
    viscale_funcStr.Open                     = FAPI_VISCALE_Open;
    viscale_funcStr.Close                    = FAPI_VISCALE_Close;
    viscale_funcStr.BackgroundColour         = FAPI_VISCALE_BackgroundColour;
    viscale_funcStr.PlaneOrder               = FAPI_VISCALE_PlaneOrder;
    viscale_funcStr.KillOutputs              = FAPI_VISCALE_KillOutputs;
    viscale_funcStr.SetBrightness            = FAPI_VISCALE_SetBrightness;
    viscale_funcStr.SetContrast              = FAPI_VISCALE_SetContrast;
    viscale_funcStr.SetSaturation            = FAPI_VISCALE_SetSaturation;
    viscale_funcStr.Start                    = FAPI_VISCALE_Start;
    viscale_funcStr.Stop                     = FAPI_VISCALE_Stop;
    viscale_funcStr.EnableDisable            = FAPI_VISCALE_EnableDisable;
    viscale_funcStr.Alpha                    = FAPI_VISCALE_Alpha;
    viscale_funcStr.Resize                   = FAPI_VISCALE_Resize;
    viscale_funcStr.Hilight                  = FAPI_VISCALE_Hilight;
    viscale_funcStr.AutoPlaneFading          = FAPI_VISCALE_AutoPlaneFading;
    viscale_funcStr.AutoPlaneFadingStepSize  = FAPI_VISCALE_AutoPlaneFadingStepSize;
    viscale_funcStr.UpdateClut               = FAPI_VISCALE_UpdateClut;
    viscale_funcStr.UpdateSingleClut         = FAPI_VISCALE_UpdateSingleClut;
    viscale_funcStr.AutoScaler               = FAPI_VISCALE_AutoScaler;
    viscale_funcStr.SetScalerToHdEncoder     = FAPI_VISCALE_SetScalerToHdEncoder;
    viscale_funcStr.SetScalingMethod         = FAPI_VISCALE_SetScalingMethod;
    viscale_funcStr.SetDisplayAspectRatio    = FAPI_VISCALE_SetDisplayAspectRatio;
    viscale_funcStr.SetWssState              = FAPI_VISCALE_SetWssState;
    viscale_funcStr.SetParamChangeCallback   = FAPI_VISCALE_SetParamChangeCallback;
    viscale_funcStr.SetWbuCaptureMode        = FAPI_VISCALE_SetWbuCaptureMode;
    viscale_funcStr.ClearScalerToHdEncoder   = FAPI_VISCALE_ClearScalerToHdEncoder;
    viscale_funcStr.ClearScalingMethod       = FAPI_VISCALE_ClearScalingMethod;
    viscale_funcStr.ClearDisplayAspectRatio  = FAPI_VISCALE_ClearDisplayAspectRatio;
    viscale_funcStr.ClearParamChangeCallback = FAPI_VISCALE_ClearParamChangeCallback;
    viscale_funcStr.ClearWbuCaptureMode      = FAPI_VISCALE_ClearWbuCaptureMode;
    viscale_funcStr.GetSystemType            = FAPI_VISCALE_GetSystemType;
    viscale_funcStr.GetWssSignal             = FAPI_VISCALE_GetWssSignal;
    viscale_funcStr.GetScalerParams          = FAPI_VISCALE_GetScalerParams;

    /* initialize function of FAPI RAL */
    if(FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VO] == 0){
        FREG_VO_Init();
        FAPEXCFG_RAL_initialized_flag[FAPEX_RAL_MODULEID_VO] = 1;
    }

    FAPI_SYS_VISCALE_SET_BOARD_PARAMS

    /* setup interrupt */
    /* viscale */
    FAPEXCFG_INTR_RegisterISR( FAPI_INTR_VO1, FAPEXCFG_VISCALE_Isr, NULL,
                               SA_INTERRUPT, (uint8_t*)"VISCALE", 0, 0 );

    /* initialize function of FAPI CDL (viscale) */
    ret = viscale_funcStr.Init();

    return ret;
}

/*--------------------------------------------------------*/
/* CLEANUP.                                               */
/*--------------------------------------------------------*/
void FAPEXCFG_VISCALE_Cleanup(void)
{
    /* unregister interrupt */
    FAPEXCFG_INTR_UnregisterISR(FAPI_INTR_VO1);

    viscale_funcStr.Exit();
}


#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
EXPORT_SYMBOL(viscale_funcStr);

EXPORT_SYMBOL(FAPEXCFG_VISCALE_Init);
EXPORT_SYMBOL(FAPEXCFG_VISCALE_Cleanup);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
