/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */

#ifdef CONFIG_FUJITSU_FAPEX_STATIC_INIT

#include "fapex.h"
#include "fapex_stdhdr.h"
#include "fapex/sys_config.h"
#include "fapex_config_cmn.h"
#include "fapex_common_priv.h"

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FAPex module.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEX_common_exit(void)
{
#ifdef FAPEXFDL_ICC_ENABLE
    FAPEXFDLICC_cleanup();
#endif

#ifdef FAPEXFDL_CLKPWR_ENABLE
    FAPEXFDLCLKPWR_cleanup();
#endif

#ifdef FAPEXFDL_GPIO_ENABLE
    FAPEXFDLGPIO_cleanup();
#endif

#ifdef FAPEXFDL_I2C_ENABLE
    FAPEXFDLI2C_cleanup();
#endif

#ifdef FAPEXFDL_IR_ENABLE
    FAPEXFDLIR_cleanup();
#endif

#ifdef FAPEXFDL_FPC_ENABLE
    FAPEXFDLFPC_cleanup();
#endif

#ifdef FAPEXFDL_HDMICEC_ENABLE
    FAPEXFDLHDMICEC_cleanup();
#endif

#ifdef FAPEXFDL_HDMI_ENABLE
    FAPEXFDLHDMI_cleanup();
#endif

#ifdef FAPEXFDL_TTX_ENABLE
    FAPEXFDLTTX_cleanup();
#endif

#ifdef FAPEXFDL_AUDEC_ENABLE
    FAPEXFDLAUDEC_cleanup();
#endif

#ifdef FAPEXFDL_AUOUT_ENABLE
    FAPEXFDLAUOUT_cleanup();
#endif

#ifdef FAPEXFDL_VIENC_ENABLE
    FAPEXFDLVIENC_cleanup();
#endif

#ifdef FAPEXFDL_VIOUT_ENABLE
    FAPEXFDLVIOUT_cleanup();
#endif

#ifdef FAPEXFDL_VIDEC_ENABLE
    FAPEXFDLVIDEC_cleanup();
#endif

#ifdef FAPEXFDL_TSD_ENABLE
    FAPEXFDLTSD_cleanup();
#endif

#ifdef FAPEXFDL_SSP_ENABLE
    FAPEXFDLSSP_cleanup();
#endif

#ifdef FAPEXFDL_FLASH_ENABLE
    FAPEXFDLFLASH_cleanup();
#endif

#ifdef FAPEXFDL_BLTE_ENABLE
    FAPEXFDLBLTE_cleanup();
#endif

#ifdef FAPEXFDL_ADC_ENABLE
    FAPEXFDLADC_cleanup();
#endif

#ifdef FAPEXFDL_PV_ENABLE
    FAPEXFDLPV_cleanup();
#endif

    FAPEXCFDL_cleanup();
    FAPISAF_CleanupSystemFunctions();
}

/*!
*******************************************************************************
**
** \brief  Initialize process of the FAPex module.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEX_common_init(void)
{
    int32_t ret;

    /* initialize of system functions */
    ret = FAPISAF_InitSystemFunctions();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPISAF_InitSystemFunctions:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }

    /* initialize of CFDL module */
    ret = FAPEXCFDL_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXCFDL_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }

    ret = FAPEXFDLBOARDIFC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXCFDL_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#ifdef FAPEXFDL_TSD_ENABLE
    /* initialize of TS-Demux FDL module */
    ret = FAPEXFDLTSD_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLTSD_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_VIDEC_ENABLE
    /* initialize of Videc FDL module */
    ret = FAPEXFDLVIDEC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLVIDEC_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_VIOUT_ENABLE
    /* initialize of Viout FDL module */
    ret = FAPEXFDLVIOUT_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLVIOUT_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_VIENC_ENABLE
    /* initialize of Vienc FDL module */
    ret = FAPEXFDLVIENC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLVIENC_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_AUOUT_ENABLE
    /* initialize of Auout FDL module */
    ret = FAPEXFDLAUOUT_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLAUOUT_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_AUDEC_ENABLE
    /* initialize of Audec FDL module */
    ret = FAPEXFDLAUDEC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLAUDEC_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_TTX_ENABLE
    /* initialize of TTX FDL module */
    ret = FAPEXFDLTTX_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLTTX_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_HDMI_ENABLE
    /* initialize of HDMI FDL module */
    ret = FAPEXFDLHDMI_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLHDMI_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_HDMICEC_ENABLE
    /* initialize of HDMI-CEC FDL module */
    ret = FAPEXFDLHDMICEC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLHDMICEC_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_FPC_ENABLE
    /* initialize of FPC FDL module */
    ret = FAPEXFDLFPC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLFPC_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_IR_ENABLE
    /* initialize of IR FDL module */
    ret = FAPEXFDLIR_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLIR_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_I2C_ENABLE
    /* initialize of I2C FDL module */
    ret = FAPEXFDLI2C_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLI2C_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_GPIO_ENABLE
    /* initialize of GPIO FDL module */
    ret = FAPEXFDLGPIO_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLGPIO_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_CLKPWR_ENABLE
    /* initialize of CLKPWR FDL module */
    ret = FAPEXFDLCLKPWR_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLCLKPWR_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_ICC_ENABLE
    /* initialize of ICC FDL module */
    ret = FAPEXFDLICC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLICC_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_SSP_ENABLE
    /* initialize of ICC FDL module */
    ret = FAPEXFDLSSP_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLSSP_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_FLASH_ENABLE
    /* initialize of ICC FDL module */
    ret = FAPEXFDLFLASH_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLFLASH_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_BLTE_ENABLE
    /* initialize of ICC FDL module */
    ret = FAPEXFDLBLTE_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLBLTE_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_ADC_ENABLE
    /* initialize of ICC FDL module */
    ret = FAPEXFDLADC_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLADC_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifdef FAPEXFDL_PV_ENABLE
    /* initialize of ICC FDL module */
    ret = FAPEXFDLPV_init();
    if(ret != 0) {
        FAPEXDebugPrint(2, "FAPEXFDLPV_init:error %d\n", ret);
        FAPEX_common_exit();
        return -1;
    }
#endif

#ifndef __LINUX__
    /* set drivers to system(RTOS) */
    ret = FAPEXCFG_INTR_InitDrivers();

    RTOS_InitKernel();
#endif /* !__LINUX__ */

    return ret;
}

#ifdef __LINUX__
module_init(FAPEX_common_init)
module_exit(FAPEX_common_exit)

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_common object)");
#endif /* __LINUX__ */
#endif /* CONFIG_FUJITSU_FAPEX_STATIC_INIT */
