/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (adc)
*/

#include "fapex.h"
#include "fapex/abst_libadc.h"
#include "fapex/sys_config.h"
#include "fapex_fdladc_priv.h"
#include "fapex_config_cmn.h"

/* for adc_funcStr */
#include "fapex_func_ext.h"

#define FAPEX_ADC_ERR_FROM_FAPI(err) ((err) - FAPI_ADC_MODULE_BASE + FAPEX_ADC_ERR_BASE)

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/

#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_ADC_StrT  FAPIGLD_FDLADCCtx[FAPEXDEV_ADC_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user 
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLADC_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                        FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_ADC_StrT *pfunc;
    FAPI_ADC_OpenParamsT openParams;
    FAPI_ADC_DividerCountEnumT divider;
    uint32_t result;
    int32_t  ret = FAPEX_ADC_ERR_BAD_PARAMETER;

    FAPEXDebugPrint(3, "[FAPEXFDLADC_ioctl] cmd:%08x\n",cmd);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_ADC_StrT*)(file->private_data);

    switch(cmd)
	{
        case FAPEX_IOCCOM_ADC_OPEN:
            if(pfunc->pFAPICtxHdl == NULL)
            {
                if(arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&(openParams.channel), (FAPEX_ADC_OpenParamEnumT *)arg, sizeof(FAPEX_ADC_OpenParamEnumT));

                    /* setup open parameter */
                    openParams.version    = FAPI_ADC_VERSION;

                    /* call ADC open() */
                    pfunc->pFAPICtxHdl = adc_funcStr.Open(&openParams, &ret);
                    FAPEXDebugPrint(3, "[FAPEXFDLADC_ioctl] FAPEX_IOCCOM_ADC_OPEN FAPI context:0x%08x\n",
                                       (uint32_t)pfunc->pFAPICtxHdl);
                    if(pfunc->pFAPICtxHdl != NULL)
			{
			    ret = FAPEX_OK;	// when open ok, adc_funcStr.Open() will not assign &ret
			}
			else
			{
                        ret = FAPEX_ADC_ERR_FROM_FAPI(ret);
			}
                }
            }
            else
            {	// have been opened
                ret = FAPEX_ADC_ERR_ALREADY_OPEN;
            }
            break;
            
        case FAPEX_IOCCOM_ADC_READ:
            {
                if (pfunc->pFAPICtxHdl != NULL)
                {
                    if (arg != 0)
                    {
                        result = adc_funcStr.Read( pfunc->pFAPICtxHdl, &ret );
                        if (ret == FAPI_OK)
                	    {
                            /* copy parameters from the kernel space to an user space */
                            FAPEXCFDLSAF_COPY_TO_USR( (uint32_t *) arg, &result, sizeof( uint32_t ) );
                            ret = FAPEX_OK;
                        }
        		    else
        		    {
                            ret = FAPEX_ADC_ERR_FROM_FAPI(ret);
        		    }
                    }
                }
                else
                {
                    ret = FAPEX_ADC_ERR_NOT_OPEN;
                    FAPEXDebugPrint(2, "[FAPEXFDLADC_ioctl] "
                                           "FAPEX_IOCCOM_ADC_READ error: FAPEX_IOCCOM_ADC_OPEN have not done\n");
                }
            }
            break;
            
        case FAPEX_IOCCOM_ADC_POWEROFF:
            {
                if (( ret = adc_funcStr.PowerOff()) == FAPI_OK)
                {
                    ret = FAPEX_OK;
                }
            }
            break;

        case FAPEX_IOCCOM_ADC_SETCLOCK:
            {
                if (pfunc->pFAPICtxHdl != NULL)
                {
                    if (arg != 0)
                    {
                        /* copy parameters from an user space to the kernel space */
                        FAPEXCFDLSAF_COPY_TO_DRV(&divider, (FAPEX_ADC_DividerCountEnumT *)arg, sizeof(FAPEX_ADC_DividerCountEnumT));
                        if ((ret = adc_funcStr.SetClock( divider)) == FAPI_OK)
                        {
                            ret = FAPEX_OK;
                        }
    			    else
			    {
                            ret = FAPEX_ADC_ERR_FROM_FAPI(ret);
			    }
                    }
                }
                else
                {
                    ret = FAPEX_ADC_ERR_NOT_OPEN;
                    FAPEXDebugPrint(2, "[FAPEXFDLADC_ioctl] "
                                           "FAPEX_IOCCOM_ADC_SETCLOCK error: FAPEX_IOCCOM_ADC_OPEN have not done\n");
                }
            }
            break;

        /* actual code which is depend on function is implimented here. */
        /* case ...: */
        default:
            ret = FAPEX_ADC_ERR_FEATURE_NOT_SUPPORTED;
            break;
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL 
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLADC_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_ADC_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLADC_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLADC_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc = (FAPEXFDL_ADC_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc;
        FAPEXDebugPrint(3, "[FAPEXFDLADC_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));

        /* set parameter */
        pfunc->pCfdlHdl = pCfdlHdl;
        pfunc->id       = id;
    }
    else {
        pfunc = (FAPEXFDL_ADC_StrT *)file->private_data;
    }

    /* increment our usage count for the device */
    atomic_inc(&pfunc->used_count);

    FAPEXDebugPrint(3, "[FAPEXFDLADC_open] fapex_fdladc was opened : used_counter %d.\n",
                       atomic_read(&pfunc->used_count));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  pfunc    The pointer to adc
**
******************************************************************************/
static int32_t fdladc_delete(FAPEXFDL_ADC_StrT *pfunc)
{
    int32_t  ret = FAPEX_OK;
    if(pfunc->pFAPICtxHdl != NULL) {
        /* close sequence */
        ret = adc_funcStr.Close(pfunc->pFAPICtxHdl);
        if(ret == FAPI_OK) {
            pfunc->pFAPICtxHdl = NULL;
        }
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLADC_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_ADC_StrT  *pfunc;
    FAPEXFDLRELRET_T    ret = FAPEX_OK;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_ADC_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        if (atomic_dec_and_test(&pfunc->used_count)) {   // close device handle
            ret = fdladc_delete(pfunc);
        }

        FAPEXDebugPrint(3, "[FAPEXFDLADC_release] : used_count %d\n",
                           atomic_read(&pfunc->used_count));
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdladc_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLADC_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_ADC_StrT  *pfunc;
    FAPEXCFDL_UtilStrT  utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_ADC_StrT  *pfunc0;
#endif

    /* set struct file_operations */
    fapex_fdladc_fops.owner   = THIS_MODULE;
    fapex_fdladc_fops.open    = FAPEXFDLADC_open;
    fapex_fdladc_fops.ioctl   = FAPEXFDLADC_ioctl;
    fapex_fdladc_fops.release = FAPEXFDLADC_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_ADC_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_ADC_StrT)*(FAPEXDEV_ADC_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return -1;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLADCCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_ADC_StrT)*(FAPEXDEV_ADC_DESCRIPTOR_NUM));

    for(count=FAPEXDEV_ADC_MINOR_FIRST; count<=FAPEXDEV_ADC_MINOR_LAST; count++){
        atomic_set(&pfunc->used_count, 0);
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdladc_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLADC_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
        pfunc ++;
    }

    ret = FAPEX_OK;

    FAPEXDebugPrint(1, "[FAPEXFDLADC_init] fapex_fdladc was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command, 
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLADC_cleanup(void)
{
    int32_t count;
    void                   *pCfdlHdl;
    FAPEXFDL_ADC_StrT *pfunc;

    FAPEXDebugPrint(1, "[FAPEXFDLADC_cleanup]\n");

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY

    pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
    pfunc    = (FAPEXFDL_ADC_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_ADC_MINOR_FIRST);

    /* relase the FDL context */
    if(pfunc != NULL) {
        FAPI_SYS_FREE(pfunc);
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_ADC_MINOR_FIRST; count<=FAPEXDEV_ADC_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLADC_cleanup] fapex_fdladc was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLADC_init);
module_exit(FAPEXFDLADC_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdladc object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
