/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL Audio Decoder header file (private)
*/

#ifndef _FAPEX_FDLAUDEC_PRIV_H
#define _FAPEX_FDLAUDEC_PRIV_H

/*!
*************************************************************************
** \brief FDL Audec context structure
*************************************************************************/
typedef struct {
    uint32_t    id;              //!< Device identifier
    uint32_t    used_count;      //!< used count
#ifdef __LINUX__
    struct kref kref;            //!< kref struct
    int32_t     close_result;    //!< return code at closing
#endif

    void        *pCfdlHdl;       //!< The pointer to the CFDL context
    int32_t     status;          //!< AUDEC status

    uint32_t    open_flag;       //!< enable flag for calling FAPI_AUDEC_Open()

    void        *pFAPICtxHdl;    //!< FAPI driver's context
    void        *pInTSDCtxHdl;   //!< input FAPI TSD driver's context
    void        *pInBMCtxHdl;    //!< input FAPI BM driver's context

    int32_t     codec_type;      //!< codec type (see #FAPI_AUDEC_DecoderEnumT)
    int32_t     input_id;        //!< the identifier for input device
    int32_t     output_id;       //!< the identifier for output devide
    uint32_t    sample_bits;     //!< number of sample bits

    void        *pAuoutHdl;      //!< FDL AUOUT handler
    /*!--------------
        mixer decoder
        input_id is mixer main input device id(a pcm buffer AUOUT device)
        output_id is mixer output device id(a AUOUT device)
        other above fileds is ignored
       ---------------*/
   int32_t subInputDevId;  //!<mixer sub input device id(a pcm buffer AUOUT device)
} FAPEXFDL_AUDEC_StrT;

/*! \brief AUDEC enable flag */
#define FAPEXFDL_AUDEC_SET_NONE         (0x0000)
#define FAPEXFDL_AUDEC_SET_INPUT        (0x0001)
#define FAPEXFDL_AUDEC_SET_OUTPUT       (0x0002)
#define FAPEXFDL_AUDEC_SET_PTS_NOTIFIER (0x0004)
#define FAPEXFDL_AUDEC_SET_IODONE       (FAPEXFDL_AUDEC_SET_INPUT | FAPEXFDL_AUDEC_SET_OUTPUT)

/*! \brief AUDEC pre-buffered stream size */
#define FAPEXFDL_AUDEC_PREBUFFERED_SIZE (3*1024)  //!< necessary stream size before calling FAPI_AUDEC_Start()

/*! \brief waiting max time for pre-buffering [msec] */
#define FAPEXFDL_AUDEC_PREBUFFERING_WAITING_MAXTIME 1000

/*! \brief waiting max time for getting the stream frequency [msec] */
#define FAPEXFDL_AUDEC_FREQ_INFO_WAITING_MAXTIME    1500  //500

#endif /* !_FAPEX_FDLAUDEC_PRIV_H */
