/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Microelectronics Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Microelectronics Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (boardifc)
*/

#include "fapex.h"
#include "fapex/abst_libblte.h"
#include "fapex/sys_config.h"
#include "fapex_fdlblte_priv.h"
#include "fapex_config_cmn.h"
#include "fapex_func_ext.h"

#include "fapex_func_ext.h"
/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifndef ALLOCATE_DYNAMIC_MEMORY
    static FAPEXFDL_BLTE_StrT  FAPIGLD_FDLBLTECtx[FAPEXDEV_BLTE_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

/*!
*******************************************************************************
**
** \brief  BLTE Open
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the open parameter
**
******************************************************************************/
static int32_t fdlblte_Open(FAPEXFDL_BLTE_StrT *pfunc, FAPEX_BLTE_OpenParamsT* pArg)
{
    int32_t                 ret = 0;
    int32_t                 errCode = 0;
    FAPI_BLTE_OpenParamsT  params;

    FAPEXDebugPrint(3, "fdlblte_Open\n");

    memset(&params, 0, sizeof(FAPI_BLTE_OpenParamsT));
    params.version = FAPI_BLTE_VERSION;
    params.processingMode = FAPI_BLTE_CMD_MODE;
    //params.baseAddr = (uint32_t*)FAPEX_SYS_MMAP_OSD1_PLANE_START;
    params.baseAddr = (uint32_t*)pArg->baseAddr;

    FAPEXCFDLSAF_COPY_TO_DRV(&(params.colourMode),
                             &(pArg->colormode),
                             sizeof(FAPEX_BLTE_ColorFormatEnumT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params.colourOrder),
                             &(pArg->colororder),
                             sizeof(FAPI_BLTE_ColourOrderEnumT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params.xSize),
                             &(pArg->xsize),
                             sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params.ySize),
                             &(pArg->ysize),
                             sizeof(uint32_t));


    if(NULL != pfunc->pFAPICtxHdl)
    {
        FAPEXDebugPrint(2,"[FAPEXFDLBLTE_ioctl] ERROR:fdlblte_Open handle(0x%x) already exist\n",(unsigned int )pfunc->pFAPICtxHdl);
        return -1;
    }

    pfunc->pFAPICtxHdl = blte_funcStr.Open(&params, &errCode);
    if((NULL == pfunc->pFAPICtxHdl) || (0 != errCode))
    {
        ret = -1;
        FAPEXDebugPrint(2,"[FAPEXFDLBLTE_ioctl] ERROR:fdlblte_Open error ret:(%d), handle:(0x%x)\n",errCode,(unsigned int )pfunc->pFAPICtxHdl);
    }
    else
    {
        FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] fdlblte_Open success ret:(%d), handle:(0x%x)\n",errCode,(unsigned int )pfunc->pFAPICtxHdl);
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE ChangeFrameBase and other base parameter
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the change parameter
**
******************************************************************************/
static int32_t fdlblte_SetFrameBase(FAPEXFDL_BLTE_StrT *pfunc, FAPEX_BLTE_SetFrameBaseParamsT* pArg)
{
    FAPI_BLTE_SetFrameBaseParamsT  params;
    int32_t                 ret = 0;

   if(pArg == NULL)
       return -1;

     FAPEXCFDLSAF_COPY_TO_DRV(&params,
                             pArg,
                             sizeof(FAPI_BLTE_SetFrameBaseParamsT));

    ret = blte_funcStr.SetFrameBase(pfunc->pFAPICtxHdl,&params);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE Set Color key feature
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the color keying parameter
**
******************************************************************************/
static int32_t fdlblte_SetColorKey(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_ColorKeyParamT* pArg)
{
    int32_t                 ret = 0;
    uint8_t                 enable;
    uint32_t                keycolor;

    FAPEXCFDLSAF_COPY_TO_DRV(&enable,
                             &(pArg->enable),
                             sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&keycolor,
                             &(pArg->keycolor),
                             sizeof(uint32_t));

    ret = blte_funcStr.SetColourKeying(pfunc->pFAPICtxHdl,enable,keycolor);
    return ret;
}
/*!
*******************************************************************************
**
** \brief  BLTE set Shading parameter
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the shading parameter
**
******************************************************************************/
static int32_t  fdlblte_SetShadingParam(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_ShadingParamsT* pArg)
{
     int32_t                 ret = 0;
     FAPI_BLTE_ShadingParamsT  param;



    FAPEXCFDLSAF_COPY_TO_DRV(&(param.mode),
                             &(pArg->mode),
                             sizeof(FAPEX_BLTE_ShadingModeEnumT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(param.gouraudParams),
                             &(pArg->gouraudParams),
                             sizeof(FAPEX_BLTE_GouraudShadingParamsT));

    ret = blte_funcStr.SetShadingParams(pfunc->pFAPICtxHdl,&param);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE set blending feature
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the blending parameter
**
******************************************************************************/
static int32_t  fdlblte_SetBlendingParam(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_BlendingParamsT* pArg)
{
     int32_t                 ret = 0;
     FAPI_BLTE_BlendingParamsT  param;
    FAPEXCFDLSAF_COPY_TO_DRV(&param,pArg,sizeof(FAPEX_BLTE_BlendingParamsT));

    ret = blte_funcStr.SetBlendingParams(pfunc->pFAPICtxHdl,&param);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE set MTXI or MTXO parameter
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the MTX parameter
**
******************************************************************************/
static int32_t fdlblte_SetMtxParam(FAPEXFDL_BLTE_StrT*  pfunc,  FAPEX_BLTE_MtxParamT* pArg)
{
    int32_t                 ret = 0;
    uint8_t index;
    int32_t value[12];

    FAPEXCFDLSAF_COPY_TO_DRV(&index,&(pArg->index),sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&value[0],&(pArg->value[0]),sizeof(int32_t)*12);

    if(index == 0)
        ret = blte_funcStr.SetInputColorMatrix(pfunc->pFAPICtxHdl,(const FAPI_BLTE_MTXIParamsT*)value);
    else
        ret = blte_funcStr.SetOutputColourMatrix(pfunc->pFAPICtxHdl,(const FAPI_BLTE_MTXOParamsT*)value);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE set CLUT parameter for 8bit mode(only useful for source data)
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the clut parameter
**
******************************************************************************/
static int32_t fdlblte_SetClutParam(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_ClutParamT* pArg)
{
    int32_t                 ret = 0;
    uint32_t index;
    int32_t value[256];

    FAPEXCFDLSAF_COPY_TO_DRV(&index,&(pArg->index),sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&value[0],&(pArg->value[0]),sizeof(int32_t)*256);

    ret = blte_funcStr.SetClut(pfunc->pFAPICtxHdl,index,(void*)value);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE set Clip parameter
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the clip parameter
**
******************************************************************************/
static int32_t fdlblte_SetClipParam(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_ClipParamT* pArg)
{
    int32_t                 ret = 0;
    uint8_t enable;
    FAPI_BLTE_RectT  rectParam;

    FAPEXCFDLSAF_COPY_TO_DRV(&enable,&(pArg->enable),sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&rectParam,&(pArg->clipRect),sizeof(FAPI_BLTE_RectT));

    ret = blte_funcStr.SetClip(pfunc->pFAPICtxHdl,enable,&rectParam);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE set source format
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to the format
**
******************************************************************************/
static int32_t fdlblte_SetSourceFormat(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_FormatParamT* pArg)
{
     int32_t                 ret = 0;
    uint8_t index;
    FAPI_BLTE_LayoutT layParam;

    FAPEXCFDLSAF_COPY_TO_DRV(&index,&(pArg->index),sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&layParam,&(pArg->format),sizeof(FAPI_BLTE_LayoutT));

    ret = blte_funcStr.SetSourceLayout(pfunc->pFAPICtxHdl,index,layParam);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE Draw rect
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to rect
**
******************************************************************************/
static int32_t fdlblte_DrawRect(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_DrawRectParamT* pArg)
{
    int32_t                 ret = 0;
    FAPI_BLTE_RectParamsT rectParam;

    FAPEXCFDLSAF_COPY_TO_DRV(&rectParam,pArg,sizeof(FAPI_BLTE_RectParamsT));

    ret = blte_funcStr.DrawRect(pfunc->pFAPICtxHdl,&rectParam);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE Draw font
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to font
**
******************************************************************************/
static int32_t fdlblte_DrawFont(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_DrawFontParamT* pArg)
{
    uint32_t wordsPerLine,datalen;
    uint32_t*  fontData;
    FAPI_BLTE_FontParamsT fontParam;
    int32_t ret = 0;

    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.xx),&(pArg->x),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.yy),&(pArg->y),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.width),&(pArg->format.width),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.height),&(pArg->format.height),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.fontColour),&(pArg->format.fontColor),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.bgTransparent),&(pArg->format.BgTrans),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.bgColour),&(pArg->format.BgColor),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.xScale),&(pArg->format.xScale),sizeof(FAPEX_BLTE_BitBltScaleEnumT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.yScale),&(pArg->format.yScale),sizeof(FAPEX_BLTE_BitBltScaleEnumT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(fontParam.scaleMode),&(pArg->format.scaleMode),sizeof(FAPEX_BLTE_BitBltScaleModeEnumT));

    wordsPerLine = (fontParam.width+ 31)/32;
    datalen = wordsPerLine * fontParam.height;

    fontData = FAPI_SYS_MALLOC_UNCACHED(datalen<<2);

    if(fontData == NULL)
    {
        ret = FAPEX_BLTE_ERR_OUT_OF_MEMORY;
        FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_DrawFont ret(%d)\n",ret);
        return ret;
    }

    FAPEXCFDLSAF_COPY_TO_DRV((void*)fontData,(void*)pArg->fontData,datalen<<2);

    fontParam.glyphBufPtr = fontData;

    ret = blte_funcStr.DrawFont(pfunc->pFAPICtxHdl,&fontParam);

     FAPI_SYS_FREE(fontData);

    return ret;
}


/*!
*******************************************************************************
**
** \brief  BLTE Draw uline
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to uline
**
******************************************************************************/
static int32_t fdlblte_DrawUline(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_DrawUlineParamT* pArg)
{
    int32_t                 ret = 0;

    FAPI_BLTE_ULineParamsT ulineParam;


    FAPEXCFDLSAF_COPY_TO_DRV(&ulineParam, pArg, sizeof(FAPI_BLTE_ULineParamsT));

    ret = blte_funcStr.DrawULine(pfunc->pFAPICtxHdl,&ulineParam);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE Draw copy
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to copy parameter
**
******************************************************************************/
static int32_t fdlblte_DrawCopy(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_CopyParamT* pArg)
{
    int32_t                 ret = 0;
    FAPI_BLTE_CopyParamsT copyParam;

    FAPEXCFDLSAF_COPY_TO_DRV(&(copyParam.xx),&(pArg->x),sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(copyParam.yy),&(pArg->y),sizeof(uint32_t));

    FAPEXCFDLSAF_COPY_TO_DRV(&(copyParam.srcAddr),&(pArg->format),sizeof(FAPEX_BLTE_CopyParamsT));

    if(copyParam.srcAddr != NULL)
        copyParam.srcAddr =copyParam.srcAddr; //BLTE USE phsical address

    ret = blte_funcStr.Copy(pfunc->pFAPICtxHdl,&copyParam);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  BLTE Draw bitmap
**
** \param  pfunc    pointer to context
** \param  pArg     pointer to bitmap parameter
**
******************************************************************************/
static int32_t fdlblte_DrawBitmap(FAPEXFDL_BLTE_StrT* pfunc, FAPEX_BLTE_DrawBitmapParamT* pArg)
{
    int32_t                 ret = 0;
    FAPI_BLTE_BitmapParamsT bitmapParam;

    FAPEXCFDLSAF_COPY_TO_DRV(&(bitmapParam.dstRect),&(pArg->pos),sizeof(FAPEX_BLTE_RectT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(bitmapParam.srcAddr),&(pArg->format.srcAddr), sizeof(uint32_t*));
    FAPEXCFDLSAF_COPY_TO_DRV(&(bitmapParam.srcStride),&(pArg->format.srcStride), sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(bitmapParam.srcRect),&(pArg->format.srcRect), sizeof(FAPEX_BLTE_RectT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(bitmapParam.mode),&(pArg->format.mode), sizeof(FAPEX_BLTE_ThumbnailModeEnumT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(bitmapParam.postarParams),&(pArg->format.postarParam), sizeof(FAPEX_BLTE_PostParamsT));
    FAPEXCFDLSAF_COPY_TO_DRV(&(bitmapParam.binaryParams),&(pArg->format.binaryParam), sizeof(FAPEX_BLTE_BinParamsT));

    if(bitmapParam.srcAddr != NULL)
        bitmapParam.srcAddr = bitmapParam.srcAddr; //BLTE USE phsical address

    ret = blte_funcStr.DrawBitmap(pfunc->pFAPICtxHdl,&bitmapParam);
    return ret;
}


/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct fill.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLBLTE_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                        FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_BLTE_StrT     *pfunc = NULL;
    int32_t                 ret = 0;

    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_ioctl] cmd:%d\n",cmd);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_BLTE_StrT*)(file->private_data);

    switch(cmd)
    {
        case FAPEX_IOCCOM_BLTE_OPEN:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl == NULL)
#ifdef __LINUX__
                && (atomic_read(&(pfunc->kref.refcount)) == 1)
#endif
                ) {
                ret = fdlblte_Open(pfunc, (FAPEX_BLTE_OpenParamsT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_Open ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_COLOR_KEY:
            if( (pfunc != NULL)  && (pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_SetColorKey(pfunc,(FAPEX_BLTE_ColorKeyParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetColorKey ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_SHADING_PARAM:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL)) {
                    ret = fdlblte_SetShadingParam(pfunc,(FAPEX_BLTE_ShadingParamsT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetShadingParam(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_BLENDING_PARAM:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL) ) {
                ret = fdlblte_SetBlendingParam(pfunc, (FAPEX_BLTE_BlendingParamsT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetBlendingParam ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_MTX_PARAM:
            if( (pfunc != NULL) &&(pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_SetMtxParam(pfunc, (FAPEX_BLTE_MtxParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetMtxParam ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_CLUT_PARAM:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL) ) {
                ret = fdlblte_SetClutParam(pfunc, (FAPEX_BLTE_ClutParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetClutParam ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_CLIP_PARAM:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL) ) {
                ret = fdlblte_SetClipParam(pfunc, (FAPEX_BLTE_ClipParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetClipParam ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_SOURCE_FORMAT:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL) ) {
                ret = fdlblte_SetSourceFormat(pfunc, (FAPEX_BLTE_FormatParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetSourceFormat ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_DRAW_RECT:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_DrawRect(pfunc, (FAPEX_BLTE_DrawRectParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_DrawRect ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_DRAW_FONT:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_DrawFont(pfunc, (FAPEX_BLTE_DrawFontParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_DrawFont ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_DRAW_ULINE:
            if( (pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_DrawUline(pfunc, (FAPEX_BLTE_DrawUlineParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_DrawUline ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_DRAW_COPY:
            if((pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_DrawCopy(pfunc, (FAPEX_BLTE_CopyParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_Copy ret(%d)\n",ret);
            break;

        case FAPEX_IOCCOM_BLTE_DRAW_BITMAP:
            if((pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_DrawBitmap(pfunc, (FAPEX_BLTE_DrawBitmapParamT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_DrawBitmap ret(%d)\n",ret);
            break;
        case FAPEX_IOCCOM_BLTE_SET_FRAME:
            if((pfunc != NULL) && (pfunc->pFAPICtxHdl != NULL)) {
                ret = fdlblte_SetFrameBase(pfunc, (FAPEX_BLTE_SetFrameBaseParamsT*)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLBLTE_ioctl] FAPI_BLTE_SetFrameBase ret(%d)\n",ret);
                break;

        default:
            break;
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLBLTE_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_BLTE_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);
    if(file->private_data != NULL){
        return -1;
    }

    id = FAPEXCFDLSAF_GETID(inode);
    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_open] minor_no:%d\n", id);
    /* Get the context base of the CFDL. */
    pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));
    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_BLTE_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
    file->private_data = pfunc;
    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));
    /* open sequence */
    /* ... */

    /* set parameter */
    pfunc->pCfdlHdl = pCfdlHdl;
    pfunc->id       = id;
#ifdef __LINUX__
    kref_get(&pfunc->kref);
#endif

    /* increment our usage count for the device */
    pfunc->used_count++;

    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_open] fapex_fdlblte was opened : used_counter %d.\n",
                       pfunc->used_count);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
#ifdef __LINUX__
static void fdlblte_delete(struct kref *kref)
{
    FAPEXFDL_BLTE_StrT *pfunc = NULL;
    int32_t             ret = 0;

    pfunc = container_of(kref, FAPEXFDL_BLTE_StrT, kref);

#else
static int32_t fdlblte_delete(FAPEXFDL_BLTE_StrT *pfunc)
{
    int32_t ret = 0;
#endif
    if(pfunc->pFAPICtxHdl != NULL)
    {
#ifdef FAPI_BLTE_CONNECT
        /* close sequence */
        ret = blte_funcStr.Close(pfunc->pFAPICtxHdl);
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdlblte_delete] "
                               "FAPI_BLTE_Close:error %d\n", ret);
        }
        else
        {
            pfunc->pFAPICtxHdl = NULL;
            FAPEXDebugPrint(1, "[fdlblte_delete] "
                   "FAPI_BLTE_Close:success\n");
        }
#endif /* FAPI_BLTE_CONNECT */
    }
    return;
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLBLTE_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_BLTE_StrT *pfunc = NULL;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_BLTE_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        pfunc->used_count--;
#ifdef __LINUX__
        kref_put(&pfunc->kref, fdlblte_delete);
#else
        fdlblte_delete(pfunc);
#endif

        FAPEXDebugPrint(1, "[FAPEXFDLBLTE_release] : used_count %d\n",
                           pfunc->used_count);
    }

    return 0;
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlblte_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLBLTE_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_BLTE_StrT  *pfunc;
    FAPEXCFDL_UtilStrT   utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_BLTE_StrT  *pfunc0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_init]\n");
    /* set struct file_operations */
    fapex_fdlblte_fops.owner   = THIS_MODULE;
    fapex_fdlblte_fops.open    = FAPEXFDLBLTE_open;
    fapex_fdlblte_fops.ioctl   = FAPEXFDLBLTE_ioctl;
    fapex_fdlblte_fops.release = FAPEXFDLBLTE_release;


#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_BLTE_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_BLTE_StrT)*(FAPEXDEV_BLTE_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return FAPEX_BLTE_ERR_OUT_OF_MEMORY;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLBLTECtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_BLTE_StrT)*(FAPEXDEV_BLTE_DESCRIPTOR_NUM));


    for(count=FAPEXDEV_BLTE_MINOR_FIRST; count<=FAPEXDEV_BLTE_MINOR_LAST; count++){
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlblte_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLBLTE_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
#ifdef __LINUX__
        kref_init(&pfunc->kref);
        kref_set(&pfunc->kref, 0);
#endif
        /* init sequence (common this FDL) */
        /* ... */

        pfunc ++;
    }

#ifdef FAPI_BLTE_CONNECT
    FAPEXDebugPrint(1, "call FAPEXCFG_BLTE_Init\n");
    /* call FAPI (initialize) */
    //ret = FAPEXCFG_BLTE_Init();
#else
    ret = 0;
#endif /* FAPI_BOARDIFC_CONNECT */

    FAPEXDebugPrint(1, "[FAPEXCFG_BLTE_Init] fapex_fdlblte was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLBLTE_cleanup(void)
{
    int32_t count;

    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_cleanup]\n");

#ifdef FAPI_BLTE_CONNECT
    /* call FAPI API (cleanup) */
    //FAPEXCFG_BLTE_Cleanup();

    //FAPEXDebugPrint(1, "end FAPEXFDLBLTE_cleanup\n");
#endif /* FAPI_BLTE_CONNECT */

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY
    {
        void                *pCfdlHdl;
        FAPEXFDL_BLTE_StrT *pfunc;

        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        pfunc    = (FAPEXFDL_BLTE_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_BLTE_MINOR_FIRST);

        /* relase the FDL context */
        if(pfunc != NULL) {
            FAPI_SYS_FREE(pfunc);
        }
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_BLTE_MINOR_FIRST; count<=FAPEXDEV_BLTE_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLBLTE_cleanup] fapex_fdlblteifc was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLBLTE_init);
module_exit(FAPEXFDLBLTE_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlblteifc object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */

