/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (boardifc)
*/

#include "fapex.h"
#include "fapex/abst_libflash.h"
#include "fapex/sys_config.h"
#include "fapex_fdlflash_priv.h"
#include "fapex_config_cmn.h"
#include "fapex_func_ext.h"

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifndef ALLOCATE_DYNAMIC_MEMORY
    static FAPEXFDL_FLASH_StrT  FAPIGLD_FDLFLASHCtx[FAPEXDEV_FLASH_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

static int32_t fdlflash_Open(FAPEXFDL_FLASH_StrT *pfunc, uint32_t arg)
{
    int32_t                 ret = 0;
    int32_t                 errCode = 0;
    FAPI_FLASH_OpenParamsT  params;

    memset(&params, 0, sizeof(FAPI_FLASH_OpenParamsT));
    params.firstSector = 0;
    params.forceFlush = 0;
    params.fsysFlag = 0;
    params.sectorCount = 0;
    params.version = FAPI_FLASH_VERSION;
    params.deviceType = arg;

    if(NULL != pfunc->pFAPICtxHdl)
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_Open handle(0x%x) already exist\n",(unsigned int )pfunc->pFAPICtxHdl);
        return -1;
    }
   
    pfunc->pFAPICtxHdl = flash_funcStr.Open(&params, &errCode);
    if((NULL == pfunc->pFAPICtxHdl) || (0 != errCode))
    {
        ret = -1;
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_Open error ret:(%d), handle:(0x%x)\n",errCode,(unsigned int )pfunc->pFAPICtxHdl);
    }
    else
    {
        FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_Open success ret:(%d), handle:(0x%x)\n",errCode,(unsigned int )pfunc->pFAPICtxHdl);
    }

    return ret;
}

static int32_t fdlflash_WriteBufferBytes(FAPEXFDL_FLASH_StrT *pfunc, FAPEX_FLASH_RWFuncStrT *pArg)
{
    FAPEX_FLASH_RWFuncStrT  params;
    uint8_t                 *buffer = NULL;
    int32_t                 ret = 0;

    memset(&params, 0, sizeof(FAPEX_FLASH_RWFuncStrT));

    FAPEXCFDLSAF_COPY_TO_DRV(&params, pArg, sizeof(FAPEX_FLASH_RWFuncStrT));
    
    buffer = FAPI_SYS_MALLOC(sizeof(uint8_t)*(params.byteCount));
    if(NULL == buffer)
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_WriteBufferBytes no memory\n");
        return -1;
    }
    memset(buffer, 0, sizeof(uint8_t)*(params.byteCount));

    FAPEXCFDLSAF_COPY_TO_DRV(buffer, (uint8_t *)pArg->bufferPtr, sizeof(uint8_t)*(params.byteCount));

#if 0
    {
        int i = 0;
    
        printk("[FAPEXFDLFLASH_ioctl] fdlflash_WriteBufferBytes start addr:[0x%08x], cnt:[%d]\n",
            params.address,
            params.byteCount);
        for(i=0; i<16; i++)
           {
               printk("[0x%02x] ", buffer[i]);
           }
           printk("\n");
    }
#endif    

    ret = flash_funcStr.WriteBytes(pfunc->pFAPICtxHdl, 
        params.address, 
        params.byteCount,
        (const void*)buffer);
    if(ret <0)
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_WriteBufferBytes error(%d)\n",ret);
    }
#if 0
    printk("[FAPEXFDLFLASH_ioctl] fdlflash_ReadBufferBytes start addr:[0x%08x], cnt:[%d]\n",
        params.address,
        params.byteCount);
    for(i=0; i<16; i++)
       {
           printk("[0x%02x] ", buffer[i]);
       }
       printk("\n");
    printk("[FAPEXFDLFLASH_ioctl] fdlflash_ReadBufferBytes end\n");
#endif
    params.bufferPtr = (void *)buffer;

    if(NULL != buffer)
    {
        FAPI_SYS_FREE(buffer);
    }

    return ret;
}

static int32_t fdlflash_ReadBufferBytes(FAPEXFDL_FLASH_StrT *pfunc, FAPEX_FLASH_RWFuncStrT *pArg)
{
    FAPEX_FLASH_RWFuncStrT  params;
    uint8_t                 *buffer = NULL;
    int32_t                 ret = 0;

    memset(&params, 0, sizeof(FAPEX_FLASH_RWFuncStrT));

    FAPEXCFDLSAF_COPY_TO_DRV(&params, pArg, sizeof(FAPEX_FLASH_RWFuncStrT));

    buffer = FAPI_SYS_MALLOC(sizeof(uint8_t)*(params.byteCount));
    if(NULL == buffer)
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_ReadBufferBytes no memory\n");
        return -1;
    }
    memset(buffer, 0, sizeof(uint8_t)*(params.byteCount));

    ret = flash_funcStr.ReadBytes(pfunc->pFAPICtxHdl, 
        params.address, 
        params.byteCount,
        (void *)buffer);
    if(ret <0)
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_ReadBufferBytes error(%d)\n",ret);

        if(NULL != buffer)
        {
            FAPI_SYS_FREE(buffer);
        }
        return -1;
    }
#if 0
    {
        int i = 0;
    
        printk("[FAPEXFDLFLASH_ioctl] fdlflash_ReadBufferBytes start addr:[0x%08x], cnt:[%d]\n",
            params.address,
            params.byteCount);
        for(i=0; i<16; i++)
           {
               printk("[0x%02x] ", buffer[i]);
           }
           printk("\n");
    }
#endif
    params.bufferPtr = (void *)buffer;

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg->bufferPtr, params.bufferPtr, sizeof(uint8_t)*(params.byteCount));

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, &params, sizeof(FAPEX_FLASH_RWFuncStrT));

    if(NULL != buffer)
    {
        FAPI_SYS_FREE(buffer);
    }

    return ret;
}

static int32_t fdlflash_EraseChip(FAPEXFDL_FLASH_StrT *pfunc)
{
    int32_t ret = 0;
    
    if(NULL != pfunc->pFAPICtxHdl)
    {
        ret = flash_funcStr.EraseChip(pfunc->pFAPICtxHdl);
        if(ret <0 )
        {
            FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_EraseChip error flashSize:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
        else
        {
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_EraseChip success flashSize:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
    }
    else
    {
        ret = -1;
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:flash handle null\n");
    }
    
    return ret;
}

static int32_t fdlflash_EraseSector(FAPEXFDL_FLASH_StrT *pfunc, uint32_t arg)
{
    int32_t ret = 0;
    
    if(NULL != pfunc->pFAPICtxHdl)
    {
        ret = flash_funcStr.EraseSector(pfunc->pFAPICtxHdl,arg);
        if(ret <0 )
        {
            FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_EraseSector error flashSize:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
        else
        {
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_EraseSector success flashSize:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
    }
    else
    {
        ret = -1;
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:flash handle null\n");
    }
    
    return ret;
}
static int32_t fdlflash_Flush(FAPEXFDL_FLASH_StrT *pfunc, uint32_t arg)
{
    int32_t ret = 0;

    if(NULL != pfunc->pFAPICtxHdl)
    {
        ret = flash_funcStr.Flush(pfunc->pFAPICtxHdl);
        if(ret <0)
        {
            FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_Flush error flashSize:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
        else
        {
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_Flush success flashSize:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
    }
    else
    {
        ret = -1;
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:flash handle null\n");
    }
    return ret;

}
static int32_t fdlflash_GetDeviceSize(FAPEXFDL_FLASH_StrT *pfunc, uint32_t *arg)
{
    int32_t ret = 0;
    
    if(NULL != pfunc->pFAPICtxHdl)
    {
        int32_t flashSize = -1;
        flashSize = flash_funcStr.GetDeviceSize(pfunc->pFAPICtxHdl);
        if(flashSize <0 )
        {
            FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_GetDeviceSize error ret:(%d), handle:(0x%x)\n",flashSize,(unsigned int )pfunc->pFAPICtxHdl);
            ret = -1;
        }
        else
        {
            FAPEXCFDLSAF_COPY_TO_USR(arg, &flashSize, sizeof(int32_t));
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_GetDeviceSize success ret:(0x%x), handle:(0x%x)\n",flashSize,(unsigned int )pfunc->pFAPICtxHdl);
        }
    }
    else
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:flash handle null\n");
    }

    return ret;
}

static int32_t fdlflash_GetSectorCount(FAPEXFDL_FLASH_StrT *pfunc)
{
    int32_t ret = 0;
    
    if(NULL != pfunc->pFAPICtxHdl)
    {
        ret = flash_funcStr.GetSectorCount(pfunc->pFAPICtxHdl);
        if(ret <0 )
        {
            FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_GetSectorCount error ret:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
        else
        {
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_GetSectorCount success ret:(0x%x), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
    }
    else
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:flash handle null\n");
    }

    return ret;
}

static int32_t fdlflash_GetSectorBytes(FAPEXFDL_FLASH_StrT *pfunc, uint32_t arg)
{
    int32_t ret = 0;
    
    if(NULL != pfunc->pFAPICtxHdl)
    {
        ret = flash_funcStr.GetSectorBytes(pfunc->pFAPICtxHdl,arg);
        if(ret <0 )
        {
            FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_GetSectorBytes error ret:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
        else
        {
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_GetSectorBytes success ret:(0x%x), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
    }
    else
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:flash handle null\n");
    }

    return ret;
}
static int32_t fdlflash_GetSectorStart(FAPEXFDL_FLASH_StrT *pfunc, uint32_t arg)
{
    int32_t ret = 0;
    
    if(NULL != pfunc->pFAPICtxHdl)
    {
        ret = flash_funcStr.GetSectorStart(pfunc->pFAPICtxHdl,arg);
        if(ret <0 )
        {
            FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:fdlflash_GetSectorStart error ret:(%d), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
        else
        {
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] fdlflash_GetSectorStart success ret:(0x%x), handle:(0x%x)\n",ret,(unsigned int )pfunc->pFAPICtxHdl);
        }
    }
    else
    {
        FAPEXDebugPrint(2,"[FAPEXFDLFLASH_ioctl] ERROR:flash handle null\n");
    }

    return ret;
}


/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLFLASH_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,

                                        FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_FLASH_StrT     *pfunc = NULL;
    int32_t                 ret = 0;

    FAPEXDebugPrint(1, "[FAPEXFDLFLASH_ioctl] cmd:%d\n",cmd);
    
    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_FLASH_StrT*)(file->private_data);

    switch(cmd)
    {
        case FAPEX_IOCCOM_FLASH_OPEN:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_Open\n");
            if( pfunc != NULL) {
                ret = fdlflash_Open(pfunc, (uint32_t)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_Open ret(%d)\n",ret);
            break;
        case FAPEX_IOCCOM_FLASH_ERASE_CHIP:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_EraseChip\n");
            if( pfunc != NULL) {
                ret = fdlflash_EraseChip(pfunc);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_EraseChip ret(%d)\n",ret);
            break;
        case FAPEX_IOCCOM_FLASH_ERASE_SECTOR:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_EraseSector\n");
            if( pfunc != NULL) {
                ret = fdlflash_EraseSector(pfunc,arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_EraseSector ret(%d)\n",ret);
            break;
        case FAPEX_IOCCOM_FLASH_WRITE_BYTES:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_WriteBytes\n");
            if( (pfunc != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlflash_WriteBufferBytes(pfunc, (FAPEX_FLASH_RWFuncStrT *)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_WriteBytes ret(%d)\n",ret);
            break;
        case FAPEX_IOCCOM_FLASH_READ_BYTES:  
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_ReadBytes\n");
            if( (pfunc != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlflash_ReadBufferBytes(pfunc, (FAPEX_FLASH_RWFuncStrT *)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_ReadBytes ret(%d)\n",ret);
            break;            
        case FAPEX_IOCCOM_FLASH_GET_DEVICE_SIZE:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetDeviceSize\n");
            if( (pfunc != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlflash_GetDeviceSize(pfunc, (uint32_t *)arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetDeviceSize ret(%d)\n",ret);
            break;
        case FAPEX_IOCCOM_FLASH_GET_DEVICE_ID:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetDeviceId\n");
            break;
        case FAPEX_IOCCOM_FLASH_GET_SECTOR_COUNT:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetSectorCount\n");
            if( pfunc != NULL) {
                ret = fdlflash_GetSectorCount(pfunc);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetSectorCount ret(%d)\n",ret);
            break;
       case FAPEX_IOCCOM_FLASH_GET_SECTOR_BYTES:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetSectorBytes\n");
            if( pfunc != NULL) {
                ret = fdlflash_GetSectorBytes(pfunc,arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetSectorBytes ret(%d)\n",ret);
            break;
       case FAPEX_IOCCOM_FLASH_GET_SECTOR_START:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetSectorStart\n");
            if( pfunc != NULL) {
                ret = fdlflash_GetSectorStart(pfunc,arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetSectorStart ret(%d)\n",ret);
            break;
        case FAPEX_IOCCOM_FLASH_FLUSH:
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_FLUSH\n");
            if( pfunc != NULL) {
                ret = fdlflash_Flush(pfunc,arg);
            }
            FAPEXDebugPrint(1,"[FAPEXFDLFLASH_ioctl] FAPI_FLASH_GetSectorStart ret(%d)\n",ret);
            break;
        default:
            break;
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLFLASH_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_FLASH_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(1, "[FAPEXFDLFLASH_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);
    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(1, "[FAPEXFDLFLASH_open] minor_no:%d\n", id);
        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(1, "[FAPEXFDLFLASH_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));
        /* Get the context base of this id's FDL. */
        pfunc = (FAPEXFDL_FLASH_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc;
        FAPEXDebugPrint(1, "[FAPEXFDLFLASH_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));
        /* open sequence */
        /* ... */

        /* set parameter */
        pfunc->pCfdlHdl = pCfdlHdl;
        pfunc->id       = id;
#ifdef __LINUX__
        kref_init(&pfunc->kref);
#endif
    }
    else {
        pfunc = (FAPEXFDL_FLASH_StrT *)file->private_data;
#ifdef __LINUX__
        kref_get(&pfunc->kref);
#endif
    }

    /* increment our usage count for the device */
    pfunc->used_count++;

    FAPEXDebugPrint(1, "[FAPEXFDLFLASH_open] fapex_fdlflash was opened : used_counter %d.\n",
                       pfunc->used_count);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
#ifdef __LINUX__
static void fdlflash_delete(struct kref *kref)
{
    FAPEXFDL_FLASH_StrT *pfunc = NULL;
    int32_t             ret = 0;

    pfunc = container_of(kref, FAPEXFDL_FLASH_StrT, kref);

#else
static int32_t fdlflash_delete(FAPEXFDL_FLASH_StrT *pfunc)
{
    int32_t ret = 0;
#endif
    if(pfunc->pFAPICtxHdl != NULL) 
    {
#ifdef FAPI_FLASH_CONNECT
        /* close sequence */
        ret = flash_funcStr.Close(pfunc->pFAPICtxHdl);
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdflash_delete] "
                               "FAPI_FLASH_Close:error %d\n", ret);
        }
        else
        {
            pfunc->pFAPICtxHdl = NULL;
            FAPEXDebugPrint(1, "[fdflash_delete] "
                   "FAPI_FLASH_Close:success\n");
        }
#endif /* FAPI_FLASH_CONNECT */
    }  
    return;
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLFLASH_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_FLASH_StrT *pfunc = NULL;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_FLASH_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        pfunc->used_count--;
#ifdef __LINUX__
        kref_put(&pfunc->kref, fdlflash_delete);
#else
        fdlflash_delete(pfunc);
#endif

        FAPEXDebugPrint(1, "[FAPEXFDLFLASH_release] : used_count %d\n",
                           pfunc->used_count);
    }

    return 0;
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlflash_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLFLASH_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_FLASH_StrT  *pfunc;
    FAPEXCFDL_UtilStrT   utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_FLASH_StrT  *pfunc0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLFLASH_init]\n");
    /* set struct file_operations */
    fapex_fdlflash_fops.owner   = THIS_MODULE;
    fapex_fdlflash_fops.open    = FAPEXFDLFLASH_open;
    //fapex_fdlboardifc_fops.read    = FAPEXFDLBOARDIFC_read;
    //fapex_fdlboardifc_fops.write   = FAPEXFDLBOARDIFC_write;
    fapex_fdlflash_fops.ioctl   = FAPEXFDLFLASH_ioctl;
    //fapex_fdlboardifc_fops.poll    = FAPEXFDLBOARDIFC_poll;
    //fapex_fdlboardifc_fops.mmap    = FAPEXFDLBOARDIFC_mmap;
    fapex_fdlflash_fops.release = FAPEXFDLFLASH_release;


#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_FLASH_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_FLASH_StrT)*(FAPEXDEV_FLASH_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return FAPEX_FLASH_ERR_OUT_OF_MEMORY;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLFLASHCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_FLASH_StrT)*(FAPEXDEV_FLASH_DESCRIPTOR_NUM));


    for(count=FAPEXDEV_FLASH_MINOR_FIRST; count<=FAPEXDEV_FLASH_MINOR_LAST; count++){
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlflash_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLFLASH_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
        /* init sequence (common this FDL) */
        /* ... */

        pfunc ++;
    }

#ifdef FAPI_FLASH_CONNECT
    FAPEXDebugPrint(1, "call FAPEXCFG_FLASH_Init\n");
    /* call FAPI (initialize) */
    //ret = FAPEXCFG_FLASH_Init();
#else
    ret = 0;
#endif /* FAPI_BOARDIFC_CONNECT */

    FAPEXDebugPrint(1, "[FAPEXCFG_FLASH_Init] fapex_fdlflash was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLFLASH_cleanup(void)
{
    int32_t count;

    FAPEXDebugPrint(1, "[FAPEXFDLFLASH_cleanup]\n");

#ifdef FAPI_FLASH_CONNECT
    /* call FAPI API (cleanup) */
    //FAPEXCFG_FLASH_Cleanup();
    
    FAPEXDebugPrint(1, "end FAPEXFDLFLASH_cleanup\n");
#endif /* FAPI_BOARDIFC_CONNECT */

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY
    {
        void                *pCfdlHdl;
        FAPEXFDL_FLASH_StrT *pfunc;

        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        pfunc    = (FAPEXFDL_FLASH_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_FLASH_MINOR_FIRST);

        /* relase the FDL context */
        if(pfunc != NULL) {
            FAPI_SYS_FREE(pfunc);
        }
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_FLASH_MINOR_FIRST; count<=FAPEXDEV_FLASH_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLFLASH_cleanup] fapex_fdlflashifc was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLFLASH_init);
module_exit(FAPEXFDLFLASH_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlflashifc object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */

