/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (gpio)
*/

#include "fapex.h"
#include "fapex/abst_libgpio.h"
#include "fapex/sys_config.h"
#include "fapex_fdlgpio_priv.h"
#include "fapex_config_cmn.h"

/* for gpio_funcStr */
#include "fapex_func_ext.h"

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_GPIO_StrT  FAPIGLD_FDLGPIOCtx[FAPEXDEV_GPIO_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Read method of this id's FDL.
**         This function is called with the systemcall "read".
**
** If copy data to the argument's buffer, use FAPI_SYS_copyFromUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLREADRET_T FAPEXFDLGPIO_read(FAPEXFDLREADARG1_T file, FAPEXFDLREADARG2_T buf,
                                        FAPEXFDLREADARG3_T size, FAPEXFDLREADARG4_T offset)
{
    FAPEXFDL_GPIO_StrT *pfunc;
    int32_t ret;

    FAPEXDebugPrint(3, "[FAPEXFDLGPIO_read] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_GPIO_StrT*)(file->private_data);

    /* actual code which is depend on function is implimented here. */

    return size;
}

/*!
*******************************************************************************
**
** \brief  Write method of this id's FDL.
**         This function is called with the systemcall "write".
**
** If copy data from the argument's buffer, use FAPI_SYS_copyToUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLWRTRET_T FAPEXFDLGPIO_write(FAPEXFDLWRTARG1_T file, FAPEXFDLWRTARG2_T buf,
                                        FAPEXFDLWRTARG3_T size, FAPEXFDLWRTARG4_T offset)
{
    FAPEXFDL_GPIO_StrT *pfunc;

    FAPEXDebugPrint(3, "[FAPEXFDLGPIO_write] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_GPIO_StrT*)(file->private_data);

    /* actual code which is depend on function is implimented here. */

    return size;
}
#endif /* not supported */
/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user 
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLGPIO_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                        FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_GPIO_StrT *pfunc;
    int32_t  ret = FAPEX_GPIO_ERR_BAD_PARAMETER;

    FAPEXDebugPrint(3, "[FAPEXFDLGPIO_ioctl] cmd:%08x\n",cmd);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_GPIO_StrT*)(file->private_data);

    switch(cmd) {
        case FAPEX_IOCCOM_GPIO_OPEN:
            if(pfunc->pFAPICtxHdl == NULL) {
                if(arg != 0) {
                    FAPI_GPIO_OpenParamsT     params;
                    FAPEX_GPIO_OpenParamStrT  *pArg = (FAPEX_GPIO_OpenParamStrT *)arg;

                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.pin),      &(pArg->pin),      sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.function), &(pArg->function), sizeof(uint32_t));

                    /* setup open parameter */
                    params.version    = FAPI_GPIO_VERSION;
                    params.callback   = NULL;
                    params.execInIsr  = 0;
                    params.irqTrigger = 0;

                    /* call GPIO open() */
                    pfunc->pFAPICtxHdl = gpio_funcStr.Open(&params, &ret);
                    FAPEXDebugPrint(3, "[fdlgpio_Open] FAPI context:0x%08x\n",
                                       (uint32_t)pfunc->pFAPICtxHdl);
                    if(pfunc->pFAPICtxHdl == NULL) {
                        FAPEXDebugPrint(2, "[FAPEXFDLGPIO_ioctl] "
                                           "FAPI_GPIO_Open error:%d\n", ret);
                    }
                }
            }
            else {
                ret = 0;
            }
            break;
        case FAPEX_IOCCOM_GPIO_READBIT:
            {
                int32_t result;
                int32_t *read_bit = (int32_t *)arg;

                result = gpio_funcStr.ReadBit( pfunc->pFAPICtxHdl );
                if ( (result == 0) || (result == 1) )
                {
                    /* copy parameters from the kernel space to an user space */
                    FAPEXCFDLSAF_COPY_TO_USR( read_bit, &result, sizeof( int32_t ) );
                    ret = 0; /* OK */
                }
                else
                {
                    ret = result;
                }
            }
            break;
        case FAPEX_IOCCOM_GPIO_WRITEBIT:
            {
                int32_t result;
                int32_t *pwrite_bit = (int32_t *)arg;
                int32_t write_bit;

                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV( &(write_bit), pwrite_bit, sizeof(int32_t));

                result = gpio_funcStr.WriteBit( pfunc->pFAPICtxHdl,  write_bit );
                if ( result < 0 )
                {
                    ret = result;
                }
                else
                {
                    ret = 0; /* OK */
                }
            }
            break;
        /* actual code which is depend on function is implimented here. */
        /* case ...: */
        default:
            break;
    }

    return ret;
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Poll method of this id's FDL.
**         This function is called with the systemcall "select".
**
** \param  file    The pointer to the struct file.
** \param  ptbl    An internal data.
**
******************************************************************************/
FAPEXFDLPLRET_T FAPEXFDLGPIO_poll(FAPEXFDLPLARG1_T file, FAPEXFDLPLARG2_T ptbl)
{
    FAPEXFDLPLRET_T mask = 0;
    FAPEXFDL_GPIO_StrT *pfunc;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_GPIO_StrT*)(file->private_data);

    FAPEXDebugPrint(4, "[FAPEXFDLGPIO_poll]\n");

    /* actual code which is depend on function is implimented here. */

    return mask;
}

/*!
*******************************************************************************
**
** \brief  Mmap method of this id's FDL.
**         This function is called with the systemcall "mmap".
**
** \param  file    The pointer to the struct file.
** \param  vma     system depend
**
******************************************************************************/
FAPEXFDLMMPRET_T FAPEXFDLGPIO_mmap(FAPEXFDLMMPARG1_T file, FAPEXFDLMMPARG2_T vma)
{
#ifdef __LINUX__
    int32_t ret;

    if (vma->vm_pgoff != 0){
        return -EINVAL;
    }

    /* This area doen't swapout */
    vma->vm_flags |= VM_IO | VM_RESERVED;

    /* This area is IO area. Set noncache, and it doesn't dump to a core
       file of a process */
    if (file->f_flags & O_SYNC){
        vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
        vma->vm_flags |= VM_IO;
    }

    ret = remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff,
                  vma->vm_end - vma->vm_start, vma->vm_page_prot);

    if(ret != 0){
        return -EAGAIN;
    }
    return 0;
#else
    return -1;
#endif /* __LINUX__ */
}
#endif /* not supported */
/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL 
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLGPIO_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_GPIO_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(3, "[FAPEXFDLGPIO_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLGPIO_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLGPIO_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc = (FAPEXFDL_GPIO_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc;
        FAPEXDebugPrint(3, "[FAPEXFDLGPIO_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));

        /* set parameter */
        pfunc->pCfdlHdl = pCfdlHdl;
        pfunc->id       = id;
#ifdef __LINUX__
        kref_init(&pfunc->kref);
#endif
    }
    else {
        pfunc = (FAPEXFDL_GPIO_StrT *)file->private_data;
#ifdef __LINUX__
        kref_get(&pfunc->kref);
#endif
    }

    /* increment our usage count for the device */
    pfunc->used_count++;

    FAPEXDebugPrint(3, "[FAPEXFDLGPIO_open] fapex_fdlgpio was opened : used_counter %d.\n",
                       pfunc->used_count);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
#ifdef __LINUX__
static void fdlgpio_delete(struct kref *kref)
{
    FAPEXFDL_GPIO_StrT  *pfunc;
    int32_t  ret = 0;

    pfunc = container_of(kref, FAPEXFDL_GPIO_StrT, kref);
#else
static int32_t fdlgpio_delete(FAPEXFDL_GPIO_StrT *pfunc)
{
    int32_t  ret = 0;
#endif

    if(pfunc->pFAPICtxHdl != NULL) {
#ifdef FAPI_GPIO_CONNECT

        /* close sequence */
        ret = gpio_funcStr.Close(pfunc->pFAPICtxHdl);
        if(ret != FAPI_OK) {
            goto fdlgpio_delete_end;
        }
#endif /* FAPI_GPIO_CONNECT */
        pfunc->pFAPICtxHdl = NULL;
    }

fdlgpio_delete_end:
#ifdef __LINUX__
    pfunc->close_result = ret;
#else
    return ret;
#endif
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLGPIO_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_GPIO_StrT  *pfunc;
    FAPEXFDLRELRET_T    ret = 0;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_GPIO_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        pfunc->used_count--;
#ifdef __LINUX__
        ret = kref_put(&pfunc->kref, fdlgpio_delete);
        if(ret != 0) {
            ret = pfunc->close_result;
        }
#else
        ret = fdlgpio_delete(pfunc);
#endif

        FAPEXDebugPrint(3, "[FAPEXFDLGPIO_release] : used_count %d\n",
                           pfunc->used_count);
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlgpio_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLGPIO_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_GPIO_StrT  *pfunc;
    FAPEXCFDL_UtilStrT  utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_GPIO_StrT  *pfunc0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLGPIO_init]\n");

    /* set struct file_operations */
    fapex_fdlgpio_fops.owner   = THIS_MODULE;
    fapex_fdlgpio_fops.open    = FAPEXFDLGPIO_open;
//  fapex_fdlgpio_fops.read    = FAPEXFDLGPIO_read;
//  fapex_fdlgpio_fops.write   = FAPEXFDLGPIO_write;
    fapex_fdlgpio_fops.ioctl   = FAPEXFDLGPIO_ioctl;
//  fapex_fdlgpio_fops.poll    = FAPEXFDLGPIO_poll;
//  fapex_fdlgpio_fops.mmap    = FAPEXFDLGPIO_mmap;
    fapex_fdlgpio_fops.release = FAPEXFDLGPIO_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_GPIO_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_GPIO_StrT)*(FAPEXDEV_GPIO_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return -1;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLGPIOCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_GPIO_StrT)*(FAPEXDEV_GPIO_DESCRIPTOR_NUM));

    for(count=FAPEXDEV_GPIO_MINOR_FIRST; count<=FAPEXDEV_GPIO_MINOR_LAST; count++){
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlgpio_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLGPIO_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
        /* init sequence (common this FDL) */
        /* ... */

        pfunc ++;
    }

#ifdef FAPI_GPIO_CONNECT
    // FAPEXDebugPrint(1, "call FAPEXCFG_GPIO_Init\n");

    /* call FAPI (initialize) */
    // ret = FAPEXCFG_GPIO_Init(); /* called from CFDL */
    ret = 0;
#else
    ret = 0;
#endif /* FAPI_GPIO_CONNECT */

    FAPEXDebugPrint(1, "[FAPEXFDLGPIO_init] fapex_fdlgpio was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command, 
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLGPIO_cleanup(void)
{
    int32_t count;

    FAPEXDebugPrint(1, "[FAPEXFDLGPIO_cleanup]\n");

#ifdef FAPI_GPIO_CONNECT
    /* call FAPI API (cleanup) */
    // FAPEXCFG_GPIO_Cleanup(); /* called from CFDL */

    // FAPEXDebugPrint(1, "end FAPEXCFG_GPIO_Cleanup\n");
#endif /* FAPI_GPIO_CONNECT */

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY
    {
        void                   *pCfdlHdl;
        FAPEXFDL_GPIO_StrT *pfunc;

        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        pfunc    = (FAPEXFDL_GPIO_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_GPIO_MINOR_FIRST);

        /* relase the FDL context */
        if(pfunc != NULL) {
            FAPI_SYS_FREE(pfunc);
        }
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_GPIO_MINOR_FIRST; count<=FAPEXDEV_GPIO_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLGPIO_cleanup] fapex_fdlgpio was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLGPIO_init);
module_exit(FAPEXFDLGPIO_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlgpio object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
