/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (i2c)
*/

#include "fapex.h"
#include "fapex/abst_libi2c.h"
#include "fapex/sys_config.h"
#include "fapex_fdli2c_priv.h"
#include "fapex_config_cmn.h"

/* for i2c_funcStr */
#include "fapex_func_ext.h"

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_I2C_StrT  FAPIGLD_FDLI2CCtx[FAPEXDEV_I2C_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Read method of this id's FDL.
**         This function is called with the systemcall "read".
**
** If copy data to the argument's buffer, use FAPI_SYS_copyFromUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLREADRET_T FAPEXFDLI2C_read(FAPEXFDLREADARG1_T file, FAPEXFDLREADARG2_T buf,
                                        FAPEXFDLREADARG3_T size, FAPEXFDLREADARG4_T offset)
{
    FAPEXFDL_I2C_StrT *pfunc;
    int32_t ret;

    FAPEXDebugPrint(3, "[FAPEXFDLI2C_read] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_I2C_StrT*)(file->private_data);

    /* actual code which is depend on function is implimented here. */

    return size;
}

/*!
*******************************************************************************
**
** \brief  Write method of this id's FDL.
**         This function is called with the systemcall "write".
**
** If copy data from the argument's buffer, use FAPI_SYS_copyToUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLWRTRET_T FAPEXFDLI2C_write(FAPEXFDLWRTARG1_T file, FAPEXFDLWRTARG2_T buf,
                                        FAPEXFDLWRTARG3_T size, FAPEXFDLWRTARG4_T offset)
{
    FAPEXFDL_I2C_StrT *pfunc;

    FAPEXDebugPrint(3, "[FAPEXFDLI2C_write] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_I2C_StrT*)(file->private_data);

    /* actual code which is depend on function is implimented here. */

    return size;
}
#endif /* not supported */

/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user 
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLI2C_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                        FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_I2C_StrT *pfunc;
    int32_t  ret = FAPEX_I2C_ERR_BAD_PARAMETER;

    FAPEXDebugPrint(3, "[FAPEXFDLI2C_ioctl] cmd:%08x\n",cmd);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_I2C_StrT*)(file->private_data);

    switch(cmd) {
        case FAPEX_IOCCOM_I2C_OPEN:
            if(pfunc->pFAPICtxHdl == NULL) {
                if(arg != 0) {
                    FAPI_I2C_OpenParamsT     params;
                    FAPEX_I2C_OpenParamStrT  *pArg = (FAPEX_I2C_OpenParamStrT *)arg;

                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.blockIndex),   &(pArg->blockIndex),   sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.speed),        &(pArg->speed),        sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.protocol),     &(pArg->protocol),     sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.slaveAddress), &(pArg->slaveAddress), sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.type),         &(pArg->type),         sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.timeOut),      &(pArg->timeOut),      sizeof(uint32_t));

                    /* setup open parameter */
                    params.version = FAPI_I2C_VERSION;

                    /* call I2C open() */
                    pfunc->pFAPICtxHdl = i2c_funcStr.Open(&params, &ret);
                    FAPEXDebugPrint(3, "[fdli2c_Open] FAPI context:0x%08x\n",
                                       (uint32_t)pfunc->pFAPICtxHdl);
                    if(pfunc->pFAPICtxHdl == NULL) {
                        FAPEXDebugPrint(2, "[FAPEXFDLI2C_ioctl] "
                                           "FAPI_I2C_Open error:%d\n", ret);
                    }
                }
            }
            else {
                ret = 0;
            }
            break;
        case FAPEX_IOCCOM_RESET:
            {
                i2c_funcStr.Reset( pfunc->pFAPICtxHdl );
                ret = 0;
            }
            break;
#if 0 /* not supported */
        case FAPEX_IOCCOM_I2C_CANCELPROCESS:
            {
                i2c_funcStr.CancelProcess( pfunc->pFAPICtxHdl );
                ret = 0;
            }
            break;
        case FAPEX_IOCCOM_I2C_PROCESS:
            {
                int32_t result;
                uint8_t *dataPtr;
                uint32_t dataLength;
                FAPEX_I2C_ProcessParamStrT  *pArg = (FAPEX_I2C_ProcessParamStrT *)arg;

                FAPEXCFDLSAF_COPY_TO_DRV( &(dataLength), &(pArg->dataLength), sizeof(uint32_t));
                dataPtr = FAPI_SYS_MALLOC( dataLength );
                if ( dataPtr == NULL )
                {
                    return -1; /* error allocate Need ADD CODE MACRO */
                }
                FAPEXCFDLSAF_COPY_TO_DRV( dataPtr, (pArg->dataPtr), dataLength );
                result = i2c_funcStr.Process( pfunc->pFAPICtxHdl, dataPtr, dataLength);
                if ( result == 0 )
                {
                    /* copy parameters from the kernel space to an user space */
                    FAPEXCFDLSAF_COPY_TO_USR( (pArg->dataPtr), dataPtr, dataLength );
                    ret = result;
                }
                else
                {
                    ret = 0;
                }
            }
            break;
#endif /* not supported */
        case FAPEX_IOCCOM_I2C_SET_PROTOCOL:
            {
                int32_t result;
                FAPI_I2C_ProtocolT protocol;
                FAPI_I2C_ProtocolT  *pArg = (FAPI_I2C_ProtocolT *)arg;
                FAPEXCFDLSAF_COPY_TO_DRV( &(protocol), pArg, sizeof(FAPI_I2C_ProtocolT));
                result = i2c_funcStr.SetProtocol( pfunc->pFAPICtxHdl, protocol);
                if ( result == 0 )
                {
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
            }
            break;
        case FAPEX_IOCCOM_I2C_READ:
            {
                int32_t result;
                FAPEX_I2C_RWParamStrT  params;
                FAPEX_I2C_RWParamStrT  *pArg = (FAPEX_I2C_RWParamStrT *)arg;

                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.msSlaveAddress), &(pArg->msSlaveAddress), sizeof(uint8_t));
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.txDataLength),   &(pArg->txDataLength),   sizeof(uint32_t));
                params.txDataPtr = (uint8_t *)FAPI_SYS_MALLOC( params.txDataLength );
                if ( params.txDataPtr == NULL )
                {
                    ret = FAPEX_I2C_ERR_OUT_OF_MEMORY;
                    break;
                }
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV( (params.txDataPtr),     (pArg->txDataPtr),    params.txDataLength);
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.rxDataLength), &(pArg->rxDataLength), sizeof(uint32_t));
                params.rxDataPtr = (uint8_t *)FAPI_SYS_MALLOC( params.rxDataLength );
                if ( params.rxDataPtr == NULL )
                {
                    ret = FAPEX_I2C_ERR_OUT_OF_MEMORY;
                    FAPI_SYS_FREE(params.txDataPtr);
                    break;
                }
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV( (params.rxDataPtr), (pArg->rxDataPtr), params.rxDataLength);
                result = i2c_funcStr.Read( pfunc->pFAPICtxHdl, params.msSlaveAddress,
                                           params.txDataPtr, params.txDataLength,
                                           params.rxDataPtr, params.rxDataLength );
                if ( result == 0 )
                {
                    /* copy parameters from the kernel space to an user space */
                    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->rxDataLength), &(params.rxDataLength), sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_USR( (pArg->rxDataPtr),     (params.rxDataPtr),    params.rxDataLength);
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
                FAPI_SYS_FREE(params.rxDataPtr);
                FAPI_SYS_FREE(params.txDataPtr);
            }
            break;
        case FAPEX_IOCCOM_I2C_WRITE:
            {
                int32_t result;
                FAPEX_I2C_RWParamStrT  params;
                FAPEX_I2C_RWParamStrT  *pArg = (FAPEX_I2C_RWParamStrT *)arg;

                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.msSlaveAddress), &(pArg->msSlaveAddress), sizeof(uint8_t));
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.txDataLength),   &(pArg->txDataLength),   sizeof(uint32_t));
                params.txDataPtr = (uint8_t *)FAPI_SYS_MALLOC( params.txDataLength );
                if ( params.txDataPtr == NULL )
                {
                    ret = FAPEX_I2C_ERR_OUT_OF_MEMORY;
                    break;
                }
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV( (params.txDataPtr), (pArg->txDataPtr), params.txDataLength);
                result = i2c_funcStr.Write( pfunc->pFAPICtxHdl, params.msSlaveAddress,
                                            params.txDataPtr, params.txDataLength );
                if ( result == 0 )
                {
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
                FAPI_SYS_FREE(params.txDataPtr);
            }
            break;
        
        /* actual code which is depend on function is implimented here. */
        /* case ...: */
        default:
        break;
    }

    return ret;
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Poll method of this id's FDL.
**         This function is called with the systemcall "select".
**
** \param  file    The pointer to the struct file.
** \param  ptbl    An internal data.
**
******************************************************************************/
FAPEXFDLPLRET_T FAPEXFDLI2C_poll(FAPEXFDLPLARG1_T file, FAPEXFDLPLARG2_T ptbl)
{
    FAPEXFDLPLRET_T mask = 0;
    FAPEXFDL_I2C_StrT *pfunc;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_I2C_StrT*)(file->private_data);

    FAPEXDebugPrint(4, "[FAPEXFDLI2C_poll]\n");

    /* actual code which is depend on function is implimented here. */

    return mask;
}

/*!
*******************************************************************************
**
** \brief  Mmap method of this id's FDL.
**         This function is called with the systemcall "mmap".
**
** \param  file    The pointer to the struct file.
** \param  vma     system depend
**
******************************************************************************/
FAPEXFDLMMPRET_T FAPEXFDLI2C_mmap(FAPEXFDLMMPARG1_T file, FAPEXFDLMMPARG2_T vma)
{
#ifdef __LINUX__
    int32_t ret;

    if (vma->vm_pgoff != 0){
        return -EINVAL;
    }

    /* This area doen't swapout */
    vma->vm_flags |= VM_IO | VM_RESERVED;

    /* This area is IO area. Set noncache, and it doesn't dump to a core
       file of a process */
    if (file->f_flags & O_SYNC){
        vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
        vma->vm_flags |= VM_IO;
    }

    ret = remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff,
                  vma->vm_end - vma->vm_start, vma->vm_page_prot);

    if(ret != 0){
        return -EAGAIN;
    }
    return 0;
#else
    return -1;
#endif /* __LINUX__ */
}
#endif /* not supported */

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL 
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLI2C_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_I2C_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(3, "[FAPEXFDLI2C_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLI2C_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLI2C_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc = (FAPEXFDL_I2C_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc;
        FAPEXDebugPrint(3, "[FAPEXFDLI2C_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));

        /* open sequence */
        /* ... */

        /* set parameter */
        pfunc->pCfdlHdl = pCfdlHdl;
        pfunc->id       = id;
#ifdef __LINUX__
        kref_init(&pfunc->kref);
#endif
    }
    else {
        pfunc = (FAPEXFDL_I2C_StrT *)file->private_data;
#ifdef __LINUX__
        kref_get(&pfunc->kref);
#endif
    }

    /* increment our usage count for the device */
    pfunc->used_count++;

    FAPEXDebugPrint(3, "[FAPEXFDLI2C_open] fapex_fdli2c was opened : used_counter %d.\n",
                       pfunc->used_count);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
#ifdef __LINUX__
static void fdli2c_delete(struct kref *kref)
{
    FAPEXFDL_I2C_StrT  *pfunc;
    int32_t  ret = 0;

    pfunc = container_of(kref, FAPEXFDL_I2C_StrT, kref);
#else
static int32_t fdli2c_delete(FAPEXFDL_I2C_StrT *pfunc)
{
    int32_t  ret = 0;
#endif

    if(pfunc->pFAPICtxHdl != NULL) {
        if(!pfunc->used_count)
        {
#ifdef FAPI_I2C_CONNECT
            /* close sequence */
            ret = i2c_funcStr.Close(pfunc->pFAPICtxHdl);
            if(ret != FAPI_OK) {
                goto fdli2c_delete_end;
            }
#endif /* FAPI_I2C_CONNECT */
            pfunc->pFAPICtxHdl = NULL;
        }
    }

fdli2c_delete_end:
#ifdef __LINUX__
    pfunc->close_result = ret;
#else
    return ret;
#endif
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLI2C_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_I2C_StrT  *pfunc;
    FAPEXFDLRELRET_T   ret = 0;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_I2C_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        pfunc->used_count--;
#ifdef __LINUX__
        ret = kref_put(&pfunc->kref, fdli2c_delete);
        if(ret != 0) {
            ret = pfunc->close_result;
        }
#else
        ret = fdli2c_delete(pfunc);
#endif

        FAPEXDebugPrint(3, "[FAPEXFDLI2C_release] : used_count %d\n",
                           pfunc->used_count);
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdli2c_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLI2C_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_I2C_StrT   *pfunc;
    FAPEXCFDL_UtilStrT  utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_I2C_StrT   *pfunc0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLI2C_init]\n");

    /* set struct file_operations */
    fapex_fdli2c_fops.owner   = THIS_MODULE;
    fapex_fdli2c_fops.open    = FAPEXFDLI2C_open;
//  fapex_fdli2c_fops.read    = FAPEXFDLI2C_read;
//  fapex_fdli2c_fops.write   = FAPEXFDLI2C_write;
    fapex_fdli2c_fops.ioctl   = FAPEXFDLI2C_ioctl;
//  fapex_fdli2c_fops.poll    = FAPEXFDLI2C_poll;
//  fapex_fdli2c_fops.mmap    = FAPEXFDLI2C_mmap;
    fapex_fdli2c_fops.release = FAPEXFDLI2C_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_I2C_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_I2C_StrT)*(FAPEXDEV_I2C_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return -1;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLI2CCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_I2C_StrT)*(FAPEXDEV_I2C_DESCRIPTOR_NUM));

    for(count=FAPEXDEV_I2C_MINOR_FIRST; count<=FAPEXDEV_I2C_MINOR_LAST; count++){
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdli2c_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLI2C_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
        /* init sequence (common this FDL) */
        /* ... */

        pfunc ++;
    }

#ifdef FAPI_I2C_CONNECT
    // FAPEXDebugPrint(1, "call FAPEXCFG_I2C_Init\n");

    /* call FAPI (initialize) */
    // ret = FAPEXCFG_I2C_Init();  /* called from CFDL */
    ret = 0;
#else
    ret = 0;
#endif /* FAPI_I2C_CONNECT */

    FAPEXDebugPrint(1, "[FAPEXFDLI2C_init] fapex_fdli2c was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command, 
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLI2C_cleanup(void)
{
    int32_t count;

    FAPEXDebugPrint(1, "[FAPEXFDLI2C_cleanup]\n");

#ifdef FAPI_I2C_CONNECT
    /* call FAPI API (cleanup) */
    // FAPEXCFG_I2C_Cleanup(); /* called from CFDL */

    // FAPEXDebugPrint(1, "end FAPEXCFG_I2C_Cleanup\n");
#endif /* FAPI_I2C_CONNECT */

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY
    {
        void                   *pCfdlHdl;
        FAPEXFDL_I2C_StrT *pfunc;

        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        pfunc    = (FAPEXFDL_I2C_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_I2C_MINOR_FIRST);

        /* relase the FDL context */
        if(pfunc != NULL) {
            FAPI_SYS_FREE(pfunc);
        }
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_I2C_MINOR_FIRST; count<=FAPEXDEV_I2C_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLI2C_cleanup] fapex_fdli2c was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLI2C_init);
module_exit(FAPEXFDLI2C_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdli2c object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
