/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Microelectronics Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Microelectronics Limited, All Rights Reserved.
 */
/*
 FDL icc Decoder header file (private)
*/

#ifndef _FAPEX_FDLICC_PRIV_H
#define _FAPEX_FDLICC_PRIV_H

#include "fapex/abst_libicc.h"
#include "fapi/drv_icc.h"
#include "fapi/drv_icc_t0.h"
#include "fapi/drv_icc_t1.h"


typedef struct
{
    uint32_t                id;
    uint32_t               inUse;
    uint32_t               cardInitialized;
    uint32_t                blockIndex;
    FAPI_SYS_HandleT       iccHandle;
    FAPI_ICC_T0_OpenParamsT openParams;
    FAPI_ICC_PpsParamsT     ppsParams;
}FAPEXFDL_IccT0_HandleT;


/*!
*************************************************************************
** \brief FDL ICC common context structure
*************************************************************************/
typedef struct {
    int32_t id;           //!< id
    int32_t used_count;   //!< used count
#ifdef __LINUX__
    struct kref kref;     //!< kref struct
#endif

    void    *pCfdlHdl;    //!< The pointer to the CFDL context

    volatile uint32_t   enable_channel;     //!< enable ICC channel flag

    uint32_t  protocolMode;   //!< The number of the protocol which is inserted T=0:0, T=1:1

    FAPI_SYS_SemaphoreT cmn_sem;            //!< Semaphore for common parameter control

} FAPEXFDL_ICC_CmnStrT;

/*!
*************************************************************************
** \brief FDL Viout sub context structure
*************************************************************************/
typedef struct {
    int32_t     id;              //!< id
    int32_t     used_count;      //!< used count
#ifdef __LINUX__
    struct kref kref;            //!< kref struct
#endif

    void        *pCfdlHdl;       //!< The pointer to the CFDL context
    int32_t     status;          //!< ICC channel status

    void        *pCmnHdl;        //!< the pointer to the ICC common context
    void        *pFAPICtxHdl;    //!< FAPI driver's context

    uint8_t     atrBuffer [FAPEX_ICC_ATR_MAX_SIZE];         //!< ATR data buffer
    uint8_t     histBuffer[FAPEX_ICC_ATR_MAX_HIST_SIZE];    //!< Historical data buffer
    uint8_t     *dataBuffer;                                //!< data buffer sent and received
    uint32_t    dataBufferSize;                             //!< data buffer max size

    FAPEX_ICC_IOportParamStrT   ioport;      //!< I/O port parameter
    FAPI_SYS_HandleT*           IccT0Handle; //!<ICC T0 handle

    FAPEXCFDLSAF_EventStrT  dt_event_card;   //!< data event control for card I/O
    FAPEXCFDLSAF_EventStrT  dt_event_reset;  //!< data event control for reset

    FAPI_SYS_SemaphoreT     buff_sem;      //!< Semaphore for buffer allocation

} FAPEXFDL_ICC_SubStrT;

/*! \brief max timeout */
#define FAPEXFDL_ICC_MAX_TIMEOUT     (60000)

#endif /* !_FAPEX_FDLICC_PRIV_H */
