/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (pv)
*/

#include "fapex.h"
#include "fapex/abst_libpv.h"
#include "fapex/sys_config.h"
#include "fapex_fdlpv_priv.h"
#include "fapex_config_cmn.h"

/* for pv_funcStr */
#include "fapex_func_ext.h"

#define FAPEX_PV_ERR_FROM_FAPI(err) ((err) - FAPI_PV_MODULE_BASE + FAPEX_PV_ERR_BASE)

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/

#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_PV_StrT  FAPIGLD_FDLPVCtx[FAPEXDEV_PV_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user 
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLPV_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                        FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_PV_StrT *pfunc;
    FAPI_PV_OpenParamsT openParam;
    FAPEX_PV_RWParamStrT rwParam;
    FAPEX_PV_RWBootParamStrT rwBootParam;
    FAPEX_PV_RWBitFieldParamStrT rwBitFieldParam;
    FAPEX_PV_BitFieldStrT *pBitField;
    FAPEX_PV_BitFieldXrefStrT bitFieldXref;
    FAPEX_PV_BurnCountStrT burnCount;

    int32_t  ret = FAPEX_PV_ERR_BAD_PARAMETER;

    FAPEXDebugPrint(3, "[FAPEXFDLPV_ioctl] cmd:%08x\n",cmd);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_PV_StrT*)(file->private_data);

    switch(cmd)
    {
        case FAPEX_IOCCOM_PV_OPEN:
            if(pfunc->pFAPICtxHdl == NULL)
            {
                /* setup open parameter */
                openParam.version    = FAPI_PV_VERSION;

                /* call PV Open() */
                pfunc->pFAPICtxHdl = pv_funcStr.Open(&openParam, &ret);
                FAPEXDebugPrint(3, "[FAPEXFDLPV_ioctl] FAPEX_IOCCOM_PV_OPEN FAPI context:0x%08x\n",
                                   (uint32_t)pfunc->pFAPICtxHdl);
                if(pfunc->pFAPICtxHdl != NULL)
                {
                    ret = FAPEX_OK;    // when open ok, pv_funcStr.Open() will not assign &ret
                }
                else
                {
                    ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_ALREADY_OPEN;
            }
            break;
            
        case FAPEX_IOCCOM_PV_RESETBURNCOUNTS:
            /* call PV ResetBurnCounts() */
            pv_funcStr.ResetBurnCounts();
            ret = FAPEX_OK;
            
            break;

        case FAPEX_IOCCOM_PV_READDATA:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwParam, (FAPEX_PV_RWParamStrT *) arg, sizeof(FAPEX_PV_RWParamStrT));

                    /* call PV ReadData() */
                    if ((ret = pv_funcStr.ReadData(pfunc->pFAPICtxHdl, rwParam.wordIndex, rwParam.bitMask, &rwParam.data)) == FAPI_OK)
                    {
                        /* copy parameters from the kernel space to an user space */
                        FAPEXCFDLSAF_COPY_TO_USR((FAPEX_PV_RWParamStrT*) arg, &rwParam, sizeof(FAPEX_PV_RWParamStrT));
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_READDATA error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

        case FAPEX_IOCCOM_PV_READSIPROMDATA:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwParam, (FAPEX_PV_RWParamStrT *) arg, sizeof(FAPEX_PV_RWParamStrT));

                    /* call PV ReadSipromData() */
                    if ((ret = pv_funcStr.ReadSipromData(pfunc->pFAPICtxHdl, rwParam.wordIndex, rwParam.bitMask, &rwParam.data)) == FAPI_OK)
                    {
                        /* copy parameters from the kernel space to an user space */
                        FAPEXCFDLSAF_COPY_TO_USR((FAPEX_PV_RWParamStrT *) arg, &rwParam, sizeof(FAPEX_PV_RWParamStrT));
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_READSIPROMDATA error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

        case FAPEX_IOCCOM_PV_READSIPROMBOOT:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwBootParam, (FAPEX_PV_RWBootParamStrT *) arg, sizeof(FAPEX_PV_RWBootParamStrT));

                    /* call PV ReadSipromBoot() */
                    if ((ret = pv_funcStr.ReadSipromBoot(pfunc->pFAPICtxHdl, rwBootParam.sectorIndex, rwBootParam.baseParam.wordIndex,
                            rwBootParam.baseParam.bitMask, &rwBootParam.baseParam.data)) == FAPI_OK)
                    {
                        /* copy parameters from the kernel space to an user space */
                        FAPEXCFDLSAF_COPY_TO_USR((FAPEX_PV_RWBootParamStrT *) arg, &rwBootParam, sizeof(FAPEX_PV_RWBootParamStrT));
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_READSIPROMBOOT error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

        case FAPEX_IOCCOM_PV_READBITFIELD:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwBitFieldParam, (FAPEX_PV_RWBitFieldParamStrT *) arg, sizeof(FAPEX_PV_RWBitFieldParamStrT));

                    /* call PV ReadBitfield() */
                    if ((ret = pv_funcStr.ReadBitfield(pfunc->pFAPICtxHdl, (FAPI_PV_BitfieldT *) &rwBitFieldParam.bitField, &rwBitFieldParam.data)) == FAPI_OK)
                    {
                        /* copy parameters from the kernel space to an user space */
                        FAPEXCFDLSAF_COPY_TO_USR((FAPEX_PV_RWBitFieldParamStrT *) arg, &rwBitFieldParam, sizeof(FAPEX_PV_RWBitFieldParamStrT));
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_READBITFIELD error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

         case FAPEX_IOCCOM_PV_GETBITFIELDBYNAME:
            if (arg != 0)
            {
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&bitFieldXref, (FAPEX_PV_BitFieldXrefStrT *) arg, sizeof(FAPEX_PV_BitFieldXrefStrT));

                /* call PV GetBitfieldByName() */
                if ((pBitField = (FAPEX_PV_BitFieldStrT *) pv_funcStr.GetBitfieldByName(bitFieldXref.aBitfieldName)) != NULL)
                {
                    /* copy parameters from the kernel space to an user space */
                    bitFieldXref.bitField = *pBitField;
                    FAPEXCFDLSAF_COPY_TO_USR((FAPEX_PV_BitFieldXrefStrT *) arg, &bitFieldXref, sizeof(FAPEX_PV_BitFieldXrefStrT));
                    ret = FAPEX_OK;
                }
                else
                {
                    ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                }
            }
            break;
            
         case FAPEX_IOCCOM_PV_GETBURNCOUNT:
            if (arg != 0)
            {
                /* call PV GetBurnCounts() */
                if ((ret = pv_funcStr.GetBurnCounts(&burnCount.burnCount, burnCount.burnRetryCountArray)) == FAPI_OK)
                {
                    /* copy parameters from the kernel space to an user space */
                    FAPEXCFDLSAF_COPY_TO_USR((FAPEX_PV_BurnCountStrT *) arg, &burnCount, sizeof(FAPEX_PV_BurnCountStrT));
                    ret = FAPEX_OK;
                }
            }
            break;

        case FAPEX_IOCCOM_PV_WRITEDATA:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwParam, (FAPEX_PV_RWParamStrT *) arg, sizeof(FAPEX_PV_RWParamStrT));
                    
                    /* call PV WriteData() */
                    if ((ret = pv_funcStr.WriteData(pfunc->pFAPICtxHdl, rwParam.wordIndex, rwParam.bitMask)) == FAPI_OK)
                    {
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_WRITEDATA error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

        case FAPEX_IOCCOM_PV_WRITESIPROMDATA:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwParam, (FAPEX_PV_RWParamStrT *) arg, sizeof(FAPEX_PV_RWParamStrT));

                    /* call PV WriteSipromData() */
                    if ((ret = pv_funcStr.WriteSipromData(pfunc->pFAPICtxHdl, rwParam.wordIndex, rwParam.bitMask)) == FAPI_OK)
                    {
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_WRITESIPROMDATA error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

        case FAPEX_IOCCOM_PV_WRITESIPROMBOOT:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwBootParam, (FAPEX_PV_RWBootParamStrT *) arg, sizeof(FAPEX_PV_RWBootParamStrT));

                    /* call PV WriteSipromBoot() */
                    if ((ret = pv_funcStr.WriteSipromBoot(pfunc->pFAPICtxHdl, rwBootParam.sectorIndex,
                            rwBootParam.baseParam.wordIndex, rwBootParam.baseParam.bitMask)) == FAPI_OK)
                    {
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_WRITESIPROMBOOT error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

        case FAPEX_IOCCOM_PV_WRITEBITFIELD:
            if (pfunc->pFAPICtxHdl != NULL)
            {
                if (arg != 0)
                {
                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&rwBitFieldParam, (FAPEX_PV_RWBitFieldParamStrT *) arg, sizeof(FAPEX_PV_RWBitFieldParamStrT));

                    /* call PV WriteBitfield() */
                    if ((ret = pv_funcStr.WriteBitfield(pfunc->pFAPICtxHdl, (FAPI_PV_BitfieldT *) &rwBitFieldParam.bitField, rwBitFieldParam.data)) == FAPI_OK)
                    {
                        ret = FAPEX_OK;
                    }
                    else
                    {
                        ret = FAPEX_PV_ERR_FROM_FAPI(ret);
                    }
                }
            }
            else
            {
                ret = FAPEX_PV_ERR_NOT_OPEN;
                FAPEXDebugPrint(2, "[FAPEXFDLPV_ioctl] "
                           "FAPEX_IOCCOM_PV_WRITEBITFIELD error: FAPEX_IOCCOM_PV_OPEN have not done\n");
            }
            break;

       /* actual code which is depend on function is implimented here. */
        /* case ...: */
        default:
            ret = FAPEX_PV_ERR_FEATURE_NOT_SUPPORTED;
            break;
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL 
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLPV_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_PV_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLPV_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLPV_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc = (FAPEXFDL_PV_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc;
        FAPEXDebugPrint(3, "[FAPEXFDLPV_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));

        /* set parameter */
        pfunc->pCfdlHdl = pCfdlHdl;
        pfunc->id       = id;
    }
    else {
        pfunc = (FAPEXFDL_PV_StrT *)file->private_data;
    }

    /* increment our usage count for the device */
    atomic_inc(&pfunc->used_count);

    FAPEXDebugPrint(3, "[FAPEXFDLPV_open] fapex_fdlpv was opened : used_counter %d.\n",
                       atomic_read(&pfunc->used_count));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  pfunc    The pointer to pv.
**
******************************************************************************/
static int32_t fdlpv_delete(FAPEXFDL_PV_StrT *pfunc)
{
    int32_t  ret = FAPEX_OK;
    if(pfunc->pFAPICtxHdl != NULL) {
        /* close sequence */
        ret = pv_funcStr.Close(pfunc->pFAPICtxHdl);
        if(ret == FAPI_OK) {
            pfunc->pFAPICtxHdl = NULL;
        }
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLPV_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_PV_StrT  *pfunc;
    FAPEXFDLRELRET_T    ret = FAPEX_OK;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_PV_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        if (atomic_dec_and_test(&pfunc->used_count)) {   // close device handle
            ret = fdlpv_delete(pfunc);
        }

        FAPEXDebugPrint(3, "[FAPEXFDLPV_release] : used_count %d\n",
                           atomic_read(&pfunc->used_count));
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlpv_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLPV_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_PV_StrT  *pfunc;
    FAPEXCFDL_UtilStrT  utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_PV_StrT  *pfunc0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLPV_init]\n");

    /* set struct file_operations */
    fapex_fdlpv_fops.owner   = THIS_MODULE;
    fapex_fdlpv_fops.open    = FAPEXFDLPV_open;
    fapex_fdlpv_fops.ioctl   = FAPEXFDLPV_ioctl;
    fapex_fdlpv_fops.release = FAPEXFDLPV_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_PV_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_PV_StrT)*(FAPEXDEV_PV_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return -1;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLPVCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_PV_StrT)*(FAPEXDEV_PV_DESCRIPTOR_NUM));

    for(count=FAPEXDEV_PV_MINOR_FIRST; count<=FAPEXDEV_PV_MINOR_LAST; count++){
        atomic_set(&pfunc->used_count, 0);
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlpv_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLPV_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
        pfunc ++;
    }

    ret = FAPEX_OK;

    FAPEXDebugPrint(1, "[FAPEXFDLPV_init] fapex_fdlpv was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command, 
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLPV_cleanup(void)
{
    int32_t count;

    FAPEXDebugPrint(1, "[FAPEXFDLPV_cleanup]\n");

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY
   {
       void                   *pCfdlHdl;
        FAPEXFDL_PV_StrT *pfunc;

        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        pfunc    = (FAPEXFDL_PV_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_PV_MINOR_FIRST);

        /* relase the FDL context */
        if(pfunc != NULL) {
            FAPI_SYS_FREE(pfunc);
        }
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_PV_MINOR_FIRST; count<=FAPEXDEV_PV_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLPV_cleanup] fapex_fdlpv was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLPV_init);
module_exit(FAPEXFDLPV_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlpv object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */

