/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (ssp)
*/

#include "fapex.h"
#include "fapex/abst_libssp.h"
#include "fapex/sys_config.h"
#include "fapex_fdlssp_priv.h"
#include "fapex_config_cmn.h"

/* for ssp_funcStr */
#include "fapex_func_ext.h"

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_SSP_StrT  FAPIGLD_FDLSSPCtx[FAPEXDEV_SSP_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */
#if 0 /* not supported */

/*!
*******************************************************************************
**
** \brief  Read method of this id's FDL.
**         This function is called with the systemcall "read".
**
** If copy data to the argument's buffer, use FAPI_SYS_copyFromUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLREADRET_T FAPEXFDLSSP_read(FAPEXFDLREADARG1_T file, FAPEXFDLREADARG2_T buf,
                                        FAPEXFDLREADARG3_T size, FAPEXFDLREADARG4_T offset)
{
    FAPEXFDL_SSP_StrT *pfunc;
    int32_t ret;

    FAPEXDebugPrint(3, "[FAPEXFDLSSP_read] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_SSP_StrT*)(file->private_data);

    /* actual code which is depend on function is implimented here. */

    return size;
}

/*!
*******************************************************************************
**
** \brief  Write method of this id's FDL.
**         This function is called with the systemcall "write".
**
** If copy data from the argument's buffer, use FAPI_SYS_copyToUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLWRTRET_T FAPEXFDLSSP_write(FAPEXFDLWRTARG1_T file, FAPEXFDLWRTARG2_T buf,
                                        FAPEXFDLWRTARG3_T size, FAPEXFDLWRTARG4_T offset)
{
    FAPEXFDL_SSP_StrT *pfunc;

    FAPEXDebugPrint(3, "[FAPEXFDLSSP_write] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_SSP_StrT*)(file->private_data);

    /* actual code which is depend on function is implimented here. */

    return size;
}
#endif /* not supported */

/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLSSP_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                        FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_SSP_StrT *pfunc;
    int32_t  ret = FAPEX_SSP_ERR_BAD_PARAMETER;

    FAPEXDebugPrint(3, "[FAPEXFDLSSP_ioctl] cmd:%d\n",cmd);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_SSP_StrT*)(file->private_data);

    switch(cmd) {
        case FAPEX_IOCCOM_SSP_OPEN:
            if(pfunc->pFAPICtxHdl == NULL) {
                if(arg != 0) {
                    FAPI_SSP_OpenParamsT params;
                    FAPEX_SSP_OpenParamStrT  *pArg = (FAPEX_SSP_OpenParamStrT *)arg;

                    /* copy parameters from an user space to the kernel space */
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.blockIndex),   &(pArg->blockIndex),   sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_DRV(&(params.transferMode),   &(pArg->transferMode),   sizeof(uint32_t));

                    params.finishCallback = NULL;
                    params.optArgPtr = NULL;
                    /* setup open parameter */
                    params.version = FAPI_SSP_VERSION;
                    /* call SSP open() */
                    pfunc->pFAPICtxHdl = ssp_funcStr.Open(&params, &ret);
                    FAPEXDebugPrint(3, "[fdlssp_Open] FAPI context:0x%08x\n",
                                       (uint32_t)pfunc->pFAPICtxHdl);
                    if(pfunc->pFAPICtxHdl == NULL) {
                        FAPEXDebugPrint(2, "[FAPEXFDLSSP_ioctl] "
                                           "FAPI_SSP_Open error:%d\n", ret);
                    }
                }
            }
            break;
        case FAPEX_IOCCOM_SSP_READ:
            {
                int32_t result;
                FAPEX_SSP_RParamStrT  params;
                FAPEX_SSP_RParamStrT  *pArg = (FAPEX_SSP_RParamStrT *)arg;

                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.rxDataLength),   &(pArg->rxDataLength),   sizeof(uint32_t));
                params.rxDataPtr = (void *)FAPI_SYS_MALLOC( params.rxDataLength );
                if ( params.rxDataPtr == NULL )
                {
                    ret = FAPEX_SSP_ERR_OUT_OF_MEMORY;
                    break;
                }
                FAPEXCFDLSAF_COPY_TO_DRV( (params.rxDataPtr), (pArg->rxDataPtr), params.rxDataLength);

                result = ssp_funcStr.Read( pfunc->pFAPICtxHdl,
                                           params.rxDataPtr,
                                           params.rxDataLength );
                if ( result >= 0 )
                {
                    /* copy parameters from the kernel space to an user space */
                    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->rxDataLength), &(params.rxDataLength), sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_USR( (pArg->rxDataPtr),     (params.rxDataPtr),    params.rxDataLength);
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
                FAPI_SYS_FREE(params.rxDataPtr);
            }
            break;
        case FAPEX_IOCCOM_SSP_WRITE:
            {
                int32_t result;
                FAPEX_SSP_WParamStrT  params;
                FAPEX_SSP_WParamStrT  *pArg = (FAPEX_SSP_WParamStrT *)arg;

                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.txDataLength),   &(pArg->txDataLength),   sizeof(uint32_t));
                params.txDataPtr = (void *)FAPI_SYS_MALLOC( params.txDataLength );
                if ( params.txDataPtr == NULL )
                {
                    ret = FAPEX_SSP_ERR_OUT_OF_MEMORY;
                    break;
                }
                FAPEXCFDLSAF_COPY_TO_DRV( (params.txDataPtr), (pArg->txDataPtr), params.txDataLength);

                result = ssp_funcStr.Write( pfunc->pFAPICtxHdl,
                                           params.txDataPtr,
                                           params.txDataLength );
                if ( result >= 0 )
                {
                    /* copy parameters from the kernel space to an user space */
                    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->txDataLength), &(params.txDataLength), sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_USR( (pArg->txDataPtr),     (params.txDataPtr),    params.txDataLength);
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
                FAPI_SYS_FREE(params.txDataPtr);
            }
            break;
        case FAPEX_IOCCOM_SSP_GETDATA:
            {
                int32_t result;
                FAPEX_SSP_GetDataParamStrT params;
                FAPEX_SSP_GetDataParamStrT *pArg = (FAPEX_SSP_GetDataParamStrT *)arg;
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.txDataLength),   &(pArg->txDataLength),   sizeof(uint32_t));
                params.txDataPtr =(void *)FAPI_SYS_MALLOC(params.txDataLength);

                if ( params.txDataPtr == NULL )
                {
                    ret = FAPEX_SSP_ERR_OUT_OF_MEMORY;
                    break;
                }
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV( (params.txDataPtr), (pArg->txDataPtr), params.txDataLength);
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.rxDataLength),   &(pArg->rxDataLength),   sizeof(uint32_t));
                params.rxDataPtr = (void *)FAPI_SYS_MALLOC( params.rxDataLength );
                if ( params.rxDataPtr == NULL )
                {
                    ret = FAPEX_SSP_ERR_OUT_OF_MEMORY;
                    FAPI_SYS_FREE(params.txDataPtr);
                    break;
                }
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV( (params.rxDataPtr), (pArg->rxDataPtr), params.rxDataLength);
                result = ssp_funcStr.GetData(pfunc->pFAPICtxHdl,
                                             params.txDataPtr,
                                             params.txDataLength,
                                             params.rxDataPtr,
                                             params.rxDataLength );
                FAPEXDebugPrint(3, "rx=0x%x\n",params.rxDataPtr);
                if ( result >= 0 )
                {
                    /* copy parameters from the kernel space to an user space */
                    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->rxDataLength), &(params.rxDataLength), sizeof(uint32_t));
                    FAPEXCFDLSAF_COPY_TO_USR( (pArg->rxDataPtr),     (params.rxDataPtr),    params.rxDataLength);
                    FAPEXDebugPrint(3, "rx1=0x%x\n",pArg->rxDataPtr);
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
                FAPI_SYS_FREE(params.rxDataPtr);
                FAPI_SYS_FREE(params.txDataPtr);
            }
            break;
        case FAPEX_IOCCOM_SSP_SET_FRAMEFORMAT:
            {
                int32_t result;
                FAPI_SSP_FrameFormatT frameformat;
                FAPI_SSP_FrameFormatT *pArg = (FAPI_SSP_FrameFormatT *)arg;
                FAPEXCFDLSAF_COPY_TO_DRV( &(frameformat), pArg, sizeof(FAPI_SSP_FrameFormatT));
                result = ssp_funcStr.SetFrameFormat( pfunc->pFAPICtxHdl, &frameformat);
                if ( result == 0 )
                {
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
            }
        break;
        case FAPEX_IOCCOM_SSP_SET_SPEED:
            {
                int32_t result;
                FAPEX_SSP_SpeedParamStrT speed;
                FAPEX_SSP_SpeedParamStrT *pArg =(FAPEX_SSP_SpeedParamStrT *)arg;
                FAPEXCFDLSAF_COPY_TO_DRV(&(speed.serialClock),&(pArg->serialClock),sizeof(uint32_t));
                FAPEXCFDLSAF_COPY_TO_DRV(&(speed.prescale),&(pArg->prescale),sizeof(uint32_t));
                result = ssp_funcStr.SetSpeed(pfunc->pFAPICtxHdl,
                                              speed.serialClock,
                                              speed.prescale);
                if ( result == 0 )
                {
                    ret = 0;
                }
                else
                {
                    ret = result;
                }
            }
        break;

        /* actual code which is depend on function is implimented here. */
        /* case ...: */
        default:
        break;
    }

    return ret;
}
#if 0 /* not supported */

/*!
*******************************************************************************
**
** \brief  Poll method of this id's FDL.
**         This function is called with the systemcall "select".
**
** \param  file    The pointer to the struct file.
** \param  ptbl    An internal data.
**
******************************************************************************/
FAPEXFDLPLRET_T FAPEXFDLSSP_poll(FAPEXFDLPLARG1_T file, FAPEXFDLPLARG2_T ptbl)
{
    FAPEXFDLPLRET_T mask = 0;
    FAPEXFDL_SSP_StrT *pfunc;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_SSP_StrT*)(file->private_data);

    FAPEXDebugPrint(4, "[FAPEXFDLSSP_poll]\n");

    /* actual code which is depend on function is implimented here. */

    return mask;
}

/*!
*******************************************************************************
**
** \brief  Mmap method of this id's FDL.
**         This function is called with the systemcall "mmap".
**
** \param  file    The pointer to the struct file.
** \param  vma     system depend
**
******************************************************************************/
FAPEXFDLMMPRET_T FAPEXFDLSSP_mmap(FAPEXFDLMMPARG1_T file, FAPEXFDLMMPARG2_T vma)
{
#ifdef __LINUX__
    int32_t ret;

    if (vma->vm_pgoff != 0){
        return -EINVAL;
    }

    /* This area doen't swapout */
    vma->vm_flags |= VM_IO | VM_RESERVED;

    /* This area is IO area. Set noncache, and it doesn't dump to a core
       file of a process */
    if (file->f_flags & O_SYNC){
        vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
        vma->vm_flags |= VM_IO;
    }

    ret = remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff,
                  vma->vm_end - vma->vm_start, vma->vm_page_prot);

    if(ret != 0){
        return -EAGAIN;
    }
    return 0;
#else
    return -1;
#endif /* __LINUX__ */
}
#endif /* not supported */

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLSSP_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_SSP_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(3, "[FAPEXFDLSSP_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLSSP_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLSSP_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc = (FAPEXFDL_SSP_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc;
        FAPEXDebugPrint(3, "[FAPEXFDLSSP_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));

        /* open sequence */
        /* ... */

        /* set parameter */
        pfunc->pCfdlHdl = pCfdlHdl;
        pfunc->id       = id;
#ifdef __LINUX__
        kref_init(&pfunc->kref);
#endif
    }
    else {
        pfunc = (FAPEXFDL_SSP_StrT *)file->private_data;
#ifdef __LINUX__
        kref_get(&pfunc->kref);
#endif
    }

    /* increment our usage count for the device */
    pfunc->used_count++;

    FAPEXDebugPrint(3, "[FAPEXFDLSSP_open] fapex_fdlssp was opened : used_counter %d.\n",
                       pfunc->used_count);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
#ifdef __LINUX__
static void fdlssp_delete(struct kref *kref)
{
    FAPEXFDL_SSP_StrT  *pfunc;
    int32_t  ret = 0;

    pfunc = container_of(kref, FAPEXFDL_SSP_StrT, kref);
#else
static int32_t fdlssp_delete(FAPEXFDL_SSP_StrT *pfunc)
{
    int32_t  ret = 0;
#endif

    if(pfunc->pFAPICtxHdl != NULL) {
        if(!pfunc->used_count)
        {
#ifdef FAPI_SSP_CONNECT
            /* close sequence */
            ret = ssp_funcStr.Close(pfunc->pFAPICtxHdl);
            if(ret != FAPI_OK) {
                goto fdlssp_delete_end;
            }
#endif /* FAPI_SSP_CONNECT */
            pfunc->pFAPICtxHdl = NULL;
        }
    }

fdlssp_delete_end:
#ifdef __LINUX__
    pfunc->close_result = ret;
#else
    return ret;
#endif
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLSSP_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_SSP_StrT  *pfunc;
    FAPEXFDLRELRET_T   ret = 0;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_SSP_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        pfunc->used_count--;
#ifdef __LINUX__
        ret = kref_put(&pfunc->kref, fdlssp_delete);
        if(ret != 0) {
            ret = pfunc->close_result;
        }
#else
        ret = fdlssp_delete(pfunc);
#endif

        FAPEXDebugPrint(3, "[FAPEXFDLSSP_release] : used_count %d\n",
                           pfunc->used_count);
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlssp_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLSSP_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_SSP_StrT  *pfunc;
    FAPEXCFDL_UtilStrT      utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_SSP_StrT  *pfunc0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLSSP_init]\n");

    /* set struct file_operations */
    fapex_fdlssp_fops.owner   = THIS_MODULE;
    fapex_fdlssp_fops.open    = FAPEXFDLSSP_open;
    //fapex_fdlssp_fops.read    = FAPEXFDLSSP_read;
    //fapex_fdlssp_fops.write   = FAPEXFDLSSP_write;
    fapex_fdlssp_fops.ioctl   = FAPEXFDLSSP_ioctl;
    //fapex_fdlssp_fops.poll    = FAPEXFDLSSP_poll;
    //fapex_fdlssp_fops.mmap    = FAPEXFDLSSP_mmap;
    fapex_fdlssp_fops.release = FAPEXFDLSSP_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_SSP_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_SSP_StrT)*(FAPEXDEV_SSP_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return FAPEX_SSP_ERR_OUT_OF_MEMORY;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLSSPCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_SSP_StrT)*(FAPEXDEV_SSP_DESCRIPTOR_NUM));

    for(count=FAPEXDEV_SSP_MINOR_FIRST; count<=FAPEXDEV_SSP_MINOR_LAST; count++){
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlssp_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLSSP_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
        /* init sequence (common this FDL) */
        /* ... */

        pfunc ++;
    }

#ifdef FAPI_SSP_CONNECT
    FAPEXDebugPrint(1, "call FAPEX_SSP_Init\n");

    /* call FAPI (initialize) */
    //ret = FAPEXCFG_SSP_Init();
#else
    ret = 0;
#endif /* FAPI_SSP_CONNECT */

    FAPEXDebugPrint(1, "[FAPEXFDLSSP_init] fapex_fdlssp was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLSSP_cleanup(void)
{
    int32_t count;

    FAPEXDebugPrint(1, "[FAPEXFDLSSP_cleanup]\n");

#ifdef FAPI_SSP_CONNECT
    /* call FAPI API (cleanup) */
    //FAPEXCFG_SSP_Cleanup();

    //FAPEXDebugPrint(1, "end FAPEX_SSP_Cleanup\n");
#endif /* FAPI_SSP_CONNECT */

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY
    {
        void                   *pCfdlHdl;
        FAPEXFDL_SSP_StrT *pfunc;

        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        pfunc    = (FAPEXFDL_SSP_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_SSP_MINOR_FIRST);

        /* relase the FDL context */
        if(pfunc != NULL) {
            FAPI_SYS_FREE(pfunc);
        }
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_SSP_MINOR_FIRST; count<=FAPEXDEV_SSP_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLSSP_cleanup] fapex_fdlssp was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLSSP_init);
module_exit(FAPEXFDLSSP_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlssp object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */

