/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 * 
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL (video decoder) header file (private)
*/

#ifndef _FAPEX_FDLVIDEC_PRIV_H
#define _FAPEX_FDLVIDEC_PRIV_H

#include "fapi/drv_videc.h"
#include "fapex/abst_libvidec.h"

/*!
*************************************************************************
** \brief FDL Videc context structure
*************************************************************************/
typedef struct {
    int32_t     id;             //!< Device identifier
    uint32_t    used_count;     //!< used count
#ifdef __LINUX__
    struct kref kref;           //!< kref struct
    int32_t     close_result;   //!< return code at closing
#endif

    void        *pCfdlHdl;      //!< The pointer to the CFDL context
    int32_t     status;         //!< VIDEC status

    void        *pFAPICtxHdl;   //!< FAPI driver's context

    uint32_t    codec_type;     //!< codec type
    int32_t     input_id;       //!< the identifier for input device
    int32_t     pes_parser_no;  //!< PES Parser Number (see #FAPI_VIDEC_PesParserEnumT)

    int32_t     playback_env;   //!< playback environment (see #FAPI_VIDEC_PlaybackEnvironmentEnumT)
    int32_t     playback_mode;  //!< playback mode (see #FAPI_VIDEC_PlaybackModeEnumT)

    FAPEXCFDLSAF_EventStrT  dt_event;   //!< data event control
    uint32_t    event_flags;            //!< event flags

    FAPEXCFDLSAF_SpnLockStrT        lock;       //!< lock for update of frameInfo
    FAPEX_VIDEC_DecFrameInfoStrT    frameInfo;  //!< decoded video frame information
    uint32_t    *pOffsetLut;                    //!< frame buffer Y offset LUT
    uint32_t    c_offset;                       //!< C offset value from Y offset
    uint32_t    bank_size;                      //!< frame buffer bank max size

} FAPEXFDL_VIDEC_StrT;

/*! \brief TSD event type */
#define FAPEXFDL_VIDEC_EVENT_NONE               (0x0000)    //!< none event
#define FAPEXFDL_VIDEC_EVENT_DEC_AVAILABLE      (0x0001)    //!< decoding frame avilable
#define FAPEXFDL_VIDEC_EVENT_DISP_AVAILABLE     (0x0002)    //!< display frame avilable
#define FAPEXFDL_VIDEC_EVENT_DISP_INFO          (0x0004)    //!< frame info enable
#define FAPEXFDL_VIDEC_EVENT_DISP_CHANGE        (0x0010)    //!< stream parameter change
#define FAPEXFDL_VIDEC_EVENT_END                (0x8000)    //!< end event
#define FAPEXFDL_VIDEC_EVENT_FRAME_INFO         (FAPEXFDL_VIDEC_EVENT_DEC_AVAILABLE | FAPEXFDL_VIDEC_EVENT_DISP_AVAILABLE | \
                                                 FAPEXFDL_VIDEC_EVENT_DISP_INFO | FAPEXFDL_VIDEC_EVENT_DISP_CHANGE)
#define FAPEXFDL_VIDEC_EVENT_READ               (FAPEXFDL_VIDEC_EVENT_DISP_INFO)
#define FAPEXFDL_VIDEC_EVENT_WRITE              (FAPEXFDL_VIDEC_EVENT_NONE)
#define FAPEXFDL_VIDEC_EVENT_MESSAGE            (FAPEXFDL_VIDEC_EVENT_DEC_AVAILABLE | FAPEXFDL_VIDEC_EVENT_DISP_AVAILABLE | \
                                                 FAPEXFDL_VIDEC_EVENT_END)

/*! \breaf ramif_v base address */
#define FAPEXFDL_VIDEC_RAMIFV_BASE_ADDR         FAPEX_SYS_MMAP_VIDEO_FRAME_START

#endif /* !_FAPEX_FDLVIDEC_PRIV_H */
