/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 * 
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv
*/

#include "fapex.h"
#include "fapex/abst_libviout.h"
#include "fapex/abst_libvienc.h"
#include "fapex/sys_config.h"
#include "fapex_fdlviout_priv.h"
#include "fapex_fdlvienc_priv.h"
#include "fapex_config_cmn.h"

/* for viscale_funcStr, hdmi_funcStr, dma_funcStr */
#include "fapex_func_ext.h"
#include "linux/dma-mapping.h"
/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifdef VIOUT_USE_DMA
static FAPI_SYS_SemaphoreT  fdlviout_DmaSemaphore = 0;
#endif

#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_VIOUT_CmnStrT  FAPIGLD_FDLVIOUTCmnCtx[FAPEXDEV_VIOUT_CMN_DESCRIPTOR_NUM];
static FAPEXFDL_VIOUT_SubStrT  FAPIGLD_FDLVIOUTSubCtx[FAPEXDEV_VIOUT_SUB_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Read method of this id's FDL.
**         This function is called with the systemcall "read".
**
** If copy data to the argument's buffer, use FAPEXCFDLSAF_COPY_TO_USR().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLREADRET_T FAPEXFDLVIOUT_read(FAPEXFDLREADARG1_T file, FAPEXFDLREADARG2_T buf, FAPEXFDLREADARG3_T size,
                                     FAPEXFDLREADARG4_T offset)
{
    FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_read] size:%d\n",size);

    /* actual code which is depend on function is implimented here. */

    return size;
}
#endif

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Write method of this id's FDL.
**         This function is called with the systemcall "write".
**
** If copy data from the argument's buffer, use FAPEXCFDLSAF_COPY_TO_DRV().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLWRTRET_T FAPEXFDLVIOUT_write(FAPEXFDLWRTARG1_T file, FAPEXFDLWRTARG2_T buf, FAPEXFDLWRTARG3_T size,
                                     FAPEXFDLWRTARG4_T offset)
{
    FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_write] size:%d\n",size);

    /* actual code which is depend on function is implimented here. */

    return size;
}
#endif

/*!
*******************************************************************************
**
** \brief  set VIOUT common running status
**
** \param  pfunc_cmn    pointer to common context
** \param  running_ch   running channel bit flag
**
******************************************************************************/
static void fdlviout_SetCmnRunningStatus(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn, uint32_t running_ch)
{
    /*---------- LOCK ----------*/
    FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

    pfunc_cmn->running_channel |= running_ch;
    pfunc_cmn->status = FAPEX_STATUS_RUNNING;

    /*--------- UNLOCK ---------*/
    FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
}

/*!
*******************************************************************************
**
** \brief  VIOUT start
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
**
******************************************************************************/
static int32_t fdlviout_Start(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn, FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                              uint32_t id)
{
    FAPI_VISCALE_ScalerParamsT  *params;
    uint32_t  enable_cmn, enable_ch, running_ch;
    uint32_t  one_ch_flag;
    int32_t   ret = 0;

    enable_cmn = pfunc_cmn->enable_channel;
    enable_ch  = 0x00000001;
    running_ch = 0;

    if(pfunc_sub != NULL) {
        /* sub descriptor */
        enable_ch <<= id - FAPEXDEV_VIOUT_SUB_MINOR_FIRST;
        enable_cmn &= enable_ch;
        one_ch_flag = 1;
    }
    else {
        /* common descriptor */
        pfunc_sub   = (FAPEXFDL_VIOUT_SubStrT *)FAPEXCFDL_GetFdlHdl(pfunc_cmn->pCfdlHdl, id+1);
        one_ch_flag = 0;
    }

    /* start channel */
    while(enable_cmn) {
        if(pfunc_sub->status == FAPEX_STATUS_READY) {

            params = &pfunc_sub->scaler_params;

#ifdef FAPI_VIOUT_CONNECT

            if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_VIDEO) {

                if(pfunc_sub->scaler_type == FAPEX_VIOUT_SCALER_VIDEO) {
                    ret = viscale_funcStr.SetWssState(pfunc_sub->pFAPICtxHdl,
                                                      FAPI_VISCALE_WSS_ENABLE_16_9_ANAMORPHIC);
                    if(ret != FAPI_OK) {
                        FAPEXDebugPrint(2, "[fdlviout_Start] FAPI_VISCALE_SetWssState error:%d\n",
                                           ret);
                        return ret;
                    }
                }

                /* start VISCALE layer */
                ret = viscale_funcStr.Start(pfunc_sub->pFAPICtxHdl, params);
                if(ret != FAPI_OK) {
                    if(ret != FAPI_VISCALE_ERR_INVALID_INITIALIZATION_PARAMS) {
                        FAPEXDebugPrint(2, "[fdlviout_Start] FAPI_VISCALE_Start error:%d(%d)\n",
                                           ret, pfunc_sub->scaler_type);
                        return ret;
                    }
                }

                /* resize VISCALE layer */
                ret = viscale_funcStr.Resize(pfunc_sub->pFAPICtxHdl, params);
                if(ret != FAPI_OK) {
                    if(ret != FAPI_VISCALE_ERR_INVALID_INITIALIZATION_PARAMS) {
                        FAPEXDebugPrint(2, "[fdlviout_Start] FAPI_VISCALE_Resize error:%d\n", ret);
                        return ret;
                    }
                    ret = FAPI_OK;
                }
            }
            else if(pfunc_sub->scaler_type != FAPI_VISCALE_SCALER_CURSOR) {
                uint32_t picBase;
                /* start VISCALE layer */
                if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3 || pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD4)
                {
                   picBase = params->picBase;
                    params->picBase = (uint32_t)phys_to_virt(pfunc_sub->dma_addr);
                }
                ret = viscale_funcStr.Start(pfunc_sub->pFAPICtxHdl, params);
                if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3 || pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD4)
                {
                    params->picBase = picBase;
                }

                if(ret != FAPI_OK) {
                    FAPEXDebugPrint(2, "[fdlviout_Start] FAPI_VISCALE_Start error:%d(%d)\n",
                                       ret, pfunc_sub->scaler_type);
                    return ret;
                }

               
                /* resize VISCALE layer */
                #if 1
                ret = viscale_funcStr.Resize(pfunc_sub->pFAPICtxHdl, params);
                if(ret != FAPI_OK) {
                    FAPEXDebugPrint(2, "[fdlviout_Start] FAPI_VISCALE_Resize error:%d\n", ret);
                    return ret;
                }
                #endif

                /* enable VISCALE layer output */
                ret = viscale_funcStr.EnableDisable(pfunc_sub->pFAPICtxHdl, FAPI_VISCALE_PARAMETER_ON);
                if(ret != FAPI_OK) {
                    FAPEXDebugPrint(2, "[fdlviout_Start] FAPI_VISCALE_EnableDisable error:%d\n", ret);
                    return ret;
                }
            }
#endif /* FAPI_VIOUT_CONNECT */

            /* update channel status */
            if(pfunc_sub->scaler_type != FAPI_VISCALE_SCALER_CURSOR) {
                pfunc_sub->status  = FAPEX_STATUS_RUNNING;
                running_ch        |= enable_ch;
                FAPEXDebugPrint(3, "[fdlviout_Start] enable_ch= 0x%x\n", enable_ch);
            }
        }
        else if(one_ch_flag != 0) {
            if(pfunc_sub->status != FAPEX_STATUS_RUNNING) {
                FAPEXDebugPrint(2, "[fdlviout_Start] status error:%d\n", pfunc_sub->status);
                return FAPEX_VIOUT_ERR_INVALID_STATUS;
            }
        }
        enable_cmn &= ~enable_ch;

        /* setup for next channel */
        pfunc_sub ++;
        enable_ch <<= 1;
    }

    /* update block status */
    if(running_ch != 0) {
        fdlviout_SetCmnRunningStatus(pfunc_cmn, running_ch);
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  VIOUT stop
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
**
******************************************************************************/
static int32_t fdlviout_Stop(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn, FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                             uint32_t id)
{
    uint32_t  enable_cmn, disable_ch, stop_channel;
    int32_t   ret = 0;

    enable_cmn   = pfunc_cmn->enable_channel;
    disable_ch   = 0x00000001;
    stop_channel = 0;

    if(pfunc_sub != NULL) {
        /* sub descriptor */
        disable_ch <<= id - FAPEXDEV_VIOUT_SUB_MINOR_FIRST;
        enable_cmn  &= disable_ch;
    }
    else {
        /* common descriptor */
        pfunc_sub = (FAPEXFDL_VIOUT_SubStrT *)FAPEXCFDL_GetFdlHdl(pfunc_cmn->pCfdlHdl, id+1);
    }

    /* stop channel */
    while(enable_cmn) {
        if( (enable_cmn & disable_ch)
         && (pfunc_sub->status == FAPEX_STATUS_RUNNING) ) {

#ifdef FAPI_VIOUT_CONNECT
            if(pfunc_sub->scaler_type != FAPI_VISCALE_SCALER_VIDEO) {
                /* call FAPI (stop) */
                ret = viscale_funcStr.Stop(pfunc_sub->pFAPICtxHdl);
                if(ret < 0) {
                    FAPEXDebugPrint(2, "[fdlviout_Stop] "
                                       "FAPI_VISCALE_Stop error:%d\n", ret);
                }
            }
#endif /* FAPI_VIOUT_CONNECT */

            /* update channel status */
            pfunc_sub->status = FAPEX_STATUS_READY;
            stop_channel     |= disable_ch;
        }
        enable_cmn &= ~disable_ch;

        /* setup for next channel */
        pfunc_sub ++;
        disable_ch <<= 1;
    }

    /* update block status */
    if(stop_channel != 0) {
        /*---------- LOCK ----------*/
        FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

        pfunc_cmn->running_channel &= ~stop_channel;
        if(pfunc_cmn->running_channel == 0) {
            pfunc_cmn->status = FAPEX_STATUS_READY;
        }

        /*--------- UNLOCK ---------*/
        FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Get HDMI video code.
**
** \param  pfunc_cmn      pointer to common context
** \param  pictureAr      Intended picture aspect ratio.
**
******************************************************************************/
static uint32_t fdlviout_GetHdmiVideoCode(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                          uint32_t pictureAr)
{
    FAPEXFDL_VIENC_CmnStrT  *pfunc_vienc_cmn;
    FAPI_HDMI_EdidInfoT     edidInfo;
    int32_t  system_mode;
    int32_t  hdmi_video_code;
    int32_t  ret;
#ifdef MB86H60
    int32_t  freq_mode;
#endif

    pfunc_vienc_cmn = (FAPEXFDL_VIENC_CmnStrT*)FAPEXCFDL_GetFdlHdl(pfunc_cmn->pCfdlHdl,
                                                                   FAPEXDEV_VIENC_CMN_MINOR);
    if(pfunc_vienc_cmn == NULL) {
        FAPEXDebugPrint(2, "[fdlviout_GetHdmiVideoCode] vienc link error\n");
        return 0;
    }
    system_mode = pfunc_vienc_cmn->system_mode;
#ifdef MB86H60
    freq_mode   = system_mode & FAPEX_VIENC_SYSTEM_MODE_FREQ_MSK;
#endif

    /* check EDID information */
    ret = hdmi_funcStr.ReadEDID(pfunc_cmn->pHdmiCtxtHdl);
    if(ret == FAPI_OK) {
        ret = hdmi_funcStr.GetEDID_Info(pfunc_cmn->pHdmiCtxtHdl, &edidInfo);
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdlviout_GetHdmiVideoCode] "
                               "FAPI_HDMI_GetEDID_Info error:%d\n", ret);
            edidInfo.videoCode1 = FAPEXFDL_VIOUT_HDMI_SVD_NONE;
        }
    }
    else {
        FAPEXDebugPrint(2, "[fdlviout_GetHdmiVideoCode] "
                           "FAPI_HDMI_ReadEDID error:%d\n", ret);
        edidInfo.videoCode1 = FAPEXFDL_VIOUT_HDMI_SVD_NONE;
    }

    switch((system_mode & FAPEX_VIENC_SYSTEM_MODE_HDMI_MSK)) {
        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x24P:
            hdmi_video_code = 32;
#ifdef MB86H60
            if(freq_mode == FAPEX_VIENC_SYSTEM_MODE_FREQ2397) {
                hdmi_video_code |= 0x80000000;
            }
#endif
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x25P:
            hdmi_video_code = 33;
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x30P:
            hdmi_video_code = 34;
#ifdef MB86H60
            if(freq_mode == FAPEX_VIENC_SYSTEM_MODE_FREQ2997) {
                hdmi_video_code |= 0x80000000;
            }
#endif
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x50P:
            hdmi_video_code = 31;
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x50I:
            hdmi_video_code = 20;
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60P:
            hdmi_video_code = 16;
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60I:
            hdmi_video_code = 5;
#ifdef MB86H60
            if(freq_mode == FAPEX_VIENC_SYSTEM_MODE_FREQ5994) {
                hdmi_video_code |= 0x80000000;
            }
#endif
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x24P:
            hdmi_video_code = 60;
#ifdef MB86H60
            if(freq_mode == FAPEX_VIENC_SYSTEM_MODE_FREQ2397) {
                hdmi_video_code |= 0x80000000;
            }
#endif
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x25P:
            hdmi_video_code = 61;
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x30P:
            hdmi_video_code = 62;
#ifdef MB86H60
            if(freq_mode == FAPEX_VIENC_SYSTEM_MODE_FREQ2997) {
                hdmi_video_code |= 0x80000000;
            }
#endif
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x50P:
            hdmi_video_code = 19;
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1360x768x60P:
        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x60P:
        case FAPEX_VIENC_SYSTEM_MODE_HDMI_1024x768x60P:
            hdmi_video_code = 4;
#ifdef MB86H60
            if(freq_mode == FAPEX_VIENC_SYSTEM_MODE_FREQ5994) {
                hdmi_video_code |= 0x80000000;
            }
#endif
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_720x576x50I:
            if( (pictureAr == FAPEX_VIOUT_DISPLAY_AR_NONE)
             || (pictureAr == FAPEX_VIOUT_DISPLAY_AR_4_3) ) {
                hdmi_video_code = 21;
            }
            else {
                if(edidInfo.videoCode1 & FAPEXFDL_VIOUT_HDMI_SVD_720x576x50I_16_9) {
                    hdmi_video_code = 22;
                }
                else {
                    hdmi_video_code = 21;
                }
            }
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_720x576x50P:
            if( (pictureAr == FAPEX_VIOUT_DISPLAY_AR_NONE)
             || (pictureAr == FAPEX_VIOUT_DISPLAY_AR_4_3) ) {
                hdmi_video_code = 17;
            }
            else {
                if(edidInfo.videoCode1 & FAPEXFDL_VIOUT_HDMI_SVD_720x576x50P_16_9) {
                    hdmi_video_code = 18;
                }
                else {
                    hdmi_video_code = 17;
                }
            }
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_720x480x60I:
            if( (pictureAr == FAPEX_VIOUT_DISPLAY_AR_NONE)
             || (pictureAr == FAPEX_VIOUT_DISPLAY_AR_4_3) ) {
                hdmi_video_code = 6;
            }
            else {
                if(edidInfo.videoCode1 & FAPEXFDL_VIOUT_HDMI_SVD_720x480x5994I_16_9) {
                    hdmi_video_code = 7;
                }
                else {
                    hdmi_video_code = 6;
                }
            }
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_800x600x60P:
        case FAPEX_VIENC_SYSTEM_MODE_HDMI_720x480x60P:
            if( (pictureAr == FAPEX_VIOUT_DISPLAY_AR_NONE)
             || (pictureAr == FAPEX_VIOUT_DISPLAY_AR_4_3) ) {
                hdmi_video_code = 2;
            }
            else {
                if(edidInfo.videoCode1 & FAPEXFDL_VIOUT_HDMI_SVD_720x480x5994P_16_9) {
                    hdmi_video_code = 3;
                }
                else {
                    hdmi_video_code = 2;
                }
            }
            break;

        case FAPEX_VIENC_SYSTEM_MODE_HDMI_640x480x60P:
            hdmi_video_code = 1;
            break;

        default:
            hdmi_video_code = 20;
    }
    return hdmi_video_code;
}

/*!
*******************************************************************************
**
** \brief  HDMI Video Start
**
** \param  pfunc_cmn    pointer to common context
** \param  pHdmiParams  pointer to FAPEX_VIOUT_HdmiParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_HdmiStart(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                  FAPEX_VIOUT_HdmiParamStrT *pHdmiParams,
                                  int32_t save_flag)
{
    FAPI_HDMI_VideoParamsT  videoParams;
    int32_t  ret;

    /* set HDMI Video Parameter */
    videoParams.inputType    = pHdmiParams->inputType;
    videoParams.outputType   = pHdmiParams->outputType;
    videoParams.scanInfo     = pHdmiParams->scanInfo;
    videoParams.activeFormat = pHdmiParams->activeFormat;
    videoParams.videoCode    = fdlviout_GetHdmiVideoCode(pfunc_cmn,
                                                         pHdmiParams->pictureAr);
    ret = hdmi_funcStr.SetVideoParameters(pfunc_cmn->pHdmiCtxtHdl, &videoParams);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlviout_HdmiStart] "
                           "FAPI_HDMI_SetVideoParameters error:%d\n", ret);
        return ret;
    }

    ret = hdmi_funcStr.VIDEO_Start(pfunc_cmn->pHdmiCtxtHdl);
    if(ret == FAPI_OK) {
        if(save_flag != 0) {
            /* backup HDMI video parameter */
            memcpy(&pfunc_cmn->hdmi_param, &videoParams, sizeof(FAPI_HDMI_VideoParamsT));
        }
    }
    else {
        FAPEXDebugPrint(2, "[fdlviout_HdmiStart] "
                           "FAPI_HDMI_VIDEO_Start error:%d\n", ret);
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  VIOUT common descriptor open
**
** \param  pfunc_cmn    pointer to common context
** \param  id           Device identifier
** \param  pArg         pointer to FAPEX_VIOUT_OpenCmnParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_OpenCmn(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn, uint32_t id,
                                FAPEX_VIOUT_OpenCmnParamStrT *pArg)
{
    FAPEX_VIOUT_OpenCmnParamStrT params;
    FAPEXFDL_VIENC_CmnStrT       *pfunc_vienc_cmn;
    FAPEXFDL_VIENC_SubStrT       *pfunc_vienc;
    int32_t  ret;

    if(pfunc_cmn->output_id > 0) {
        return 0;
    }
    if(pArg == NULL) {
        return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }
    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&params, pArg, sizeof(FAPEX_VIOUT_OpenCmnParamStrT));

    /* check device id */
    if( (params.outputId < FAPEXDEV_VIENC_SUB_MINOR_FIRST)
     || (params.outputId > FAPEXDEV_VIENC_SUB_MINOR_LAST) ) {
        return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }
    pfunc_cmn->output_id = params.outputId;

    pfunc_vienc = (FAPEXFDL_VIENC_SubStrT*)FAPEXCFDL_GetFdlHdl(pfunc_cmn->pCfdlHdl,
                                                               params.outputId);
    if(pfunc_vienc == NULL) {
        return FAPEX_VIOUT_ERR_INVALID_STATUS;
    }
    pfunc_vienc_cmn = (FAPEXFDL_VIENC_CmnStrT*)pfunc_vienc->pCmnHdl;

    switch((pfunc_vienc_cmn->system_mode & FAPEX_VIENC_SYSTEM_MODE_FREQ_MSK)) {
        case FAPEX_VIENC_SYSTEM_MODE_FREQ2500:
        case FAPEX_VIENC_SYSTEM_MODE_FREQ5000:
            pfunc_cmn->encoder_type = FAPI_VISCALE_ENCODER_PAL;
            break;
        default:
            pfunc_cmn->encoder_type = FAPI_VISCALE_ENCODER_NTSC;
    }

    if(params.enableFlag & FAPEX_VIOUT_ENABLE_HDMI) {
#ifdef FAPI_VIOUT_CONNECT
        FAPI_HDMI_OpenParamsT  hdmiOpenParams;

        /* open HDMI video driver */
        hdmiOpenParams.version = FAPI_HDMI_VERSION;
        hdmiOpenParams.block   = FAPI_HDMI_VIDEO;

        pfunc_cmn->pHdmiCtxtHdl = hdmi_funcStr.Open(&hdmiOpenParams, &ret);
        if(pfunc_cmn->pHdmiCtxtHdl == NULL) {
            FAPEXDebugPrint(2, "[fdlviout_OpenCmn] FAPI_HDMI_Open error:%d\n", ret);
        }
        else {
            fdlviout_HdmiStart(pfunc_cmn, &params.hdmiParams, 1);
        }
#endif /* FAPI_VIOUT_CONNECT */
    }
    else {
        pfunc_cmn->pHdmiCtxtHdl = NULL;
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  setup VIOUT output
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
**
** \sa FAPI_VISCALE_SetScalerToHdEncoder
******************************************************************************/
static int32_t fdlviout_SetOutput(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                  FAPEXFDL_VIOUT_SubStrT *pfunc_sub)
{
    FAPEXFDL_VIENC_SubStrT  *pfunc_vienc;
    int32_t  ret = 0;

    pfunc_vienc = (FAPEXFDL_VIENC_SubStrT*)FAPEXCFDL_GetFdlHdl(pfunc_sub->pCfdlHdl,
                                                               pfunc_cmn->output_id);
    if(pfunc_vienc == NULL) {
        FAPEXDebugPrint(2, "[fdlviout_SetOutput] vienc handler error %d(%d)\n",
                           pfunc_cmn->output_id, pfunc_sub->scaler_type);
        return FAPEX_VIOUT_ERR_INVALID_STATUS;
    }
#ifdef FAPI_VIOUT_CONNECT
    {
 #ifdef COMBINATION_WITH_FB
        if(pfunc_sub->scaler_type == FAPEX_VIOUT_SCALER_OSD1) {
            /*---------- LOCK ----------*/
            FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

            if(pfunc_cmn->pOsd1CtxHdl == NULL) {
                /* call FAPI (set encoder) */
                ret = viscale_funcStr.SetScalerToHdEncoder(pfunc_sub->pFAPICtxHdl,
                                                           pfunc_vienc->pFAPICtxHdl);
            }

            /*--------- UNLOCK ---------*/
            FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
        }
        else
 #endif
        {
            /* call FAPI (set encoder) */
            ret = viscale_funcStr.SetScalerToHdEncoder(pfunc_sub->pFAPICtxHdl,
                                                       pfunc_vienc->pFAPICtxHdl);
        }
        if(ret == FAPI_OK) {
            pfunc_sub->pViencHdl = pfunc_vienc;

            pfunc_cmn->system_mode = viscale_funcStr.GetSystemType(pfunc_sub->pFAPICtxHdl);

            switch((FAPI_VISCALE_SystemModeEnumT)pfunc_cmn->system_mode) {
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x2397P:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x24P:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x25P:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x2997P:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x30P:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x50P:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x5994P:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x60P:
                    pfunc_sub->max_hres = 1920;
                    pfunc_sub->max_vres = 1080;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x50I:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x5994I:
                case FAPI_VISCALE_SYSTEM_MODE_1920x1080x60I:
                    pfunc_sub->max_hres = 1920;
                    pfunc_sub->max_vres =  540;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_VGA_1360x768x5994P:
                    pfunc_sub->max_hres = 1360;
                    pfunc_sub->max_vres =  768;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_1280x720x2397P:
                case FAPI_VISCALE_SYSTEM_MODE_1280x720x24P:
                case FAPI_VISCALE_SYSTEM_MODE_1280x720x25P:
                case FAPI_VISCALE_SYSTEM_MODE_1280x720x2997P:
                case FAPI_VISCALE_SYSTEM_MODE_1280x720x30P:
                case FAPI_VISCALE_SYSTEM_MODE_1280x720x50P:
                case FAPI_VISCALE_SYSTEM_MODE_1280x720x5994P:
                case FAPI_VISCALE_SYSTEM_MODE_1280x720x60P:
                    pfunc_sub->max_hres = 1280;
                    pfunc_sub->max_vres =  720;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_VGA_1024x768x5994P:
                    pfunc_sub->max_hres = 1024;
                    pfunc_sub->max_vres =  768;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_VGA_800x600x5994P:
                    pfunc_sub->max_hres = 800;
                    pfunc_sub->max_vres = 600;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_720x576x50I:
                    pfunc_sub->max_hres = 720;	// MODIFY by tiger from 1440
                    pfunc_sub->max_vres =  288;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_720x576x50P:
                    pfunc_sub->max_hres =  720;
                    pfunc_sub->max_vres =  576;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_720x480x5994I:
                    pfunc_sub->max_hres = 720;	// MODIFY by tiger from 1440
                    pfunc_sub->max_vres =  240;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_720x480x5994P:
                    pfunc_sub->max_hres =  720;
                    pfunc_sub->max_vres =  480;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_VGA_640x480x5994P:
                case FAPI_VISCALE_SYSTEM_MODE_LCD_640x480x50P_LQ080V3DG01:
                    pfunc_sub->max_hres =  640;
                    pfunc_sub->max_vres =  480;
                    break;

                case FAPI_VISCALE_SYSTEM_MODE_LCD_1024x768x50P_SVA150XG10TB:
                    pfunc_sub->max_hres = 1024;
                    pfunc_sub->max_vres =  768;
                    break;

                default:
                    FAPEXDebugPrint(2, "[fdlviout_SetOutput] FAPI_VISCALE_GetSystemType error:%d\n",
                                       pfunc_cmn->system_mode);
            }
        }
        else {
            FAPEXDebugPrint(2, "[fdlviout_SetOutput] "
                               "FAPI_VISCALE_SetScalerToHdEncoder error:%d\n", ret);
        }
    }
#endif /* FAPI_VIOUT_CONNECT */

    return ret;
}

/*!
*******************************************************************************
**
** \brief  VIOUT sub descriptor open
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
** \param  pArg         pointer to FAPEX_VIOUT_OpenSubParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_OpenSub(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn, FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                uint32_t id, FAPEX_VIOUT_OpenSubParamStrT *pArg)
{
    FAPI_VISCALE_OpenParamsT  params;
    int32_t  ret = 0;

    if(pfunc_sub->pFAPICtxHdl != NULL) {
        return 0;
    }
    if(pArg == NULL) {
        return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }
    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&(params.scalerType),   &(pArg->scalerType),   sizeof(int32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params.topologyType), &(pArg->topologyType), sizeof(int32_t));
    
    params.version     = FAPI_VISCALE_VERSION;
    params.encoderType = (FAPI_VISCALE_EncoderEnumT)pfunc_cmn->encoder_type;

    pfunc_sub->scaler_type = params.scalerType;
    pfunc_sub->scaler_params.layerEnable    = 0; /* disable */
    pfunc_sub->scaler_params.hilightOnOff   = 0;
    pfunc_sub->scaler_params.useGlobalAlpha = 1;
    pfunc_sub->scaler_params.globalAlpha    = 255;
    pfunc_sub->scaler_params.endianness = FAPI_VISCALE_ENDIAN_DEFAULT;
    pfunc_sub->scaler_params.pixelOrder = FAPI_VISCALE_PIXEL_ORDER_DEFAULT;
#ifdef FAPI_VIOUT_CONNECT

 #ifdef COMBINATION_WITH_FB
    if(pfunc_sub->scaler_type == FAPEX_VIOUT_SCALER_OSD1) {
        /*---------- LOCK ----------*/
        FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

        if(pfunc_cmn->pOsd1CtxHdl != NULL) {
            pfunc_sub->pFAPICtxHdl = pfunc_cmn->pOsd1CtxHdl;
        }
        else {
            /* call FAPI (open) */
            pfunc_sub->pFAPICtxHdl = viscale_funcStr.Open(&params, &ret);
        }

        /*--------- UNLOCK ---------*/
        FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
    }
    else
 #endif
    {
        /* call FAPI (open) */
        pfunc_sub->pFAPICtxHdl = viscale_funcStr.Open(&params, &ret); 
    }
    FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_ioctl] Viout context:0x%08x,ret=%d\n",
                       (uint32_t)pfunc_sub->pFAPICtxHdl,ret);

    if(pfunc_sub->pFAPICtxHdl != NULL) {
        /* set link to output device */
        ret = fdlviout_SetOutput(pfunc_cmn, pfunc_sub);
        if(ret < 0) {
            return ret;
        }
#else
    {
#endif /* FAPI_VIOUT_CONNECT */
        /*---------- LOCK ----------*/
        FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

        pfunc_cmn->enable_channel |= 1 << (id - FAPEXDEV_VIOUT_SUB_MINOR_FIRST);
        if(pfunc_cmn->status == FAPEX_STATUS_OPENED) {
            pfunc_cmn->status = FAPEX_STATUS_READY;
        }

        /*--------- UNLOCK ---------*/
        FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);

        pfunc_sub->max_hres = FAPEX_VIOUT_MAX_HRES;
        pfunc_sub->max_vres = FAPEX_VIOUT_MAX_VRES;
        pfunc_sub->status   = FAPEX_STATUS_READY;
    }
    else {
        FAPEXDebugPrint(2, "[fdlviout_OpenSub] FAPI_VISCALE_Open error:%d\n", ret);
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  VIOUT sub descriptor close
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
**
******************************************************************************/
static int32_t fdlviout_CloseSub(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                 FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                 uint32_t id)
{
    uint32_t ch_mask;
    int32_t  ret;

    if(pfunc_sub->pFAPICtxHdl == NULL) {
        return 0;
    }

#ifdef FAPI_VIOUT_CONNECT
 #ifdef COMBINATION_WITH_FB
    if(pfunc_sub->scaler_type == FAPEX_VIOUT_SCALER_OSD1) {
        /*---------- LOCK ----------*/
        FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

        if(pfunc_cmn->pOsd1CtxHdl == NULL) {
            ret = viscale_funcStr.Close(pfunc_sub->pFAPICtxHdl);
        }
        else {
            ret = 0;
        }

        /*--------- UNLOCK ---------*/
        FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
    }
    else
 #endif
    {
        /* call FAPI (close) */
        ret = viscale_funcStr.Close(pfunc_sub->pFAPICtxHdl);
    }
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlviout_CloseSub] "
                           "FAPI_VISCALE_Close error:%d [%d]\n", ret, id);
        return ret;
    }
#endif /* FAPI_VIOUT_CONNECT */
    pfunc_sub->pFAPICtxHdl = NULL;
    pfunc_sub->pViencHdl   = NULL;
  //  pfunc_sub->scaler_type = FAPI_VISCALE_UNKNOWN_SCALER; //move to the later

    if(pfunc_sub->plane_ptr != NULL) {
#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
  #ifndef __LINUX__
        FAPI_SYS_FREE(pfunc_sub->plane_ptr);
  #endif
#else
        /*---------- LOCK ----------*/
        FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

        if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD1) {
            pfunc_cmn->used_bytes_osd1 = 0;
        }
        else if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD2) {
            pfunc_cmn->used_bytes_osd2 = 0;
        }
        /*---------- UNLOCK ----------*/
        FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
#endif /* VIOUT_ALLOCATE_OSD_CURSOR_PLANE */

        pfunc_sub->plane_ptr = NULL;
    }

#ifdef __LINUX__
  #ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    if(pfunc_sub->plane_size_order != 0) {
        __free_pages(pfunc_sub->plane_pg, (pfunc_sub->plane_size_order - PAGE_SHIFT));
      
        pfunc_sub->plane_pg         = NULL;
        pfunc_sub->plane_size_order = 0;
    }
  #endif
  #ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    if( (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3) ||
        (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD4)){
       
         if(pfunc_sub->virt_addr != 0){
       //      __free_pages(pfunc_sub->plane_pg, (pfunc_sub->plane_size_order - PAGE_SHIFT));
            dma_free_coherent(NULL,pfunc_sub->plane_max_size,(void*)pfunc_sub->virt_addr,pfunc_sub->dma_addr);
            pfunc_sub->virt_addr = 0;
            pfunc_sub->dma_addr = 0;
            pfunc_sub->plane_pg         = NULL;
            pfunc_sub->plane_size_order = 0;
        }
    }  

  #endif


#endif /* __LINUX__ */


    pfunc_sub->scaler_type = FAPI_VISCALE_UNKNOWN_SCALER;

    pfunc_sub->plane_size     = 0;
    pfunc_sub->plane_max_size = 0;

    ch_mask = ~(1 << (id - FAPEXDEV_VIOUT_SUB_MINOR_FIRST));

    /*---------- LOCK ----------*/
    FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

    /* update status */
    pfunc_cmn->enable_channel  &= ch_mask;
    pfunc_cmn->running_channel &= ch_mask;

    pfunc_sub->status = FAPEX_STATUS_LOADED;
    if(pfunc_cmn->enable_channel == 0) {
        pfunc_cmn->status = FAPEX_STATUS_OPENED;
    }
    else if(pfunc_cmn->running_channel == 0) {
        pfunc_cmn->status = FAPEX_STATUS_READY;
    }

    /*--------- UNLOCK ---------*/
    FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  start Corsor Plane
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
**
******************************************************************************/
static int32_t fdlviout_StartCP(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                uint32_t id)
{
    int32_t  ret;

#ifdef FAPI_VIOUT_CONNECT
    /* start Cursor Plane */
    ret = viscale_funcStr.Start(pfunc_sub->pFAPICtxHdl, &pfunc_sub->scaler_params);
#else
    ret = FAPI_OK;
#endif
    if(ret == FAPI_OK) {
        if(pfunc_sub->status != FAPEX_STATUS_RUNNING) {
            pfunc_sub->status = FAPEX_STATUS_RUNNING;

            fdlviout_SetCmnRunningStatus(pfunc_cmn, (1 << (id - FAPEXDEV_VIOUT_SUB_MINOR_FIRST)));
        }
    }
    else {
        FAPEXDebugPrint(2, "[fdlviout_StartCP] FAPI_VISCALE_Start error:%d\n", ret);
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  enable VIOUT plane
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
**
******************************************************************************/
static int32_t fdlviout_Enable(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                               FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                               uint32_t id)
{
    int32_t  ret = FAPI_OK;

    if(pfunc_sub->scaler_type != FAPI_VISCALE_SCALER_CURSOR) {
#ifdef FAPI_VIOUT_CONNECT
        /* enable layer */
        ret = viscale_funcStr.EnableDisable(pfunc_sub->pFAPICtxHdl,
                                            FAPI_VISCALE_PARAMETER_ON);
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdlviout_Enable] "
                               "FAPI_VISCALE_EnableDisable error:%d\n", ret);
            return ret;
        }
#endif
        pfunc_sub->scaler_params.layerEnable = 1; /* enable */
    }
    else {
        pfunc_sub->scaler_params.layerEnable = 1; /* enable */

        if(pfunc_sub->scaler_params.picBase != 0) {
            FAPI_SYS_FlushDataCache(pfunc_sub->scaler_params.picBase,
                                    pfunc_sub->plane_size);
            ret = fdlviout_StartCP(pfunc_cmn, pfunc_sub, id);
        }
    }
    return ret;
}

#ifdef VIOUT_USE_DMA
/*!
*****************************************************************************
** \brief   Sets the fdlviout_DmaSemaphore. Procedure is intented to be used
**          as callback when working with DMA.
**
*****************************************************************************
*/
static void fdlviout_DmaFinish(FAPI_DMA_RequestT* damRequestPtr, void* optArgPtr)
{
    FAPI_SYS_SET_SEMAPHORE(fdlviout_DmaSemaphore, FAPI_SYS_NO_SUSPEND);
}
#endif /* VIOUT_USE_DMA */

/*!
*******************************************************************************
**
** \brief fill OSD/Cursor Plane
**
** \param  pfunc_sub    pointer to sub context
**
******************************************************************************/
static void fdlviout_FillPlane(FAPEXFDL_VIOUT_SubStrT *pfunc_sub)
{
#ifdef VIOUT_USE_DMA
    FAPI_DMA_MemoryFillParamsT  *dma_ptr;
    FAPI_DMA_RequestT           *dma_request = NULL;
    uint32_t  dma_ok = 1;
#endif
    FAPI_VISCALE_ScalerParamsT  *scaler_ptr;
    uint32_t  bpp = pfunc_sub->bpp;
    uint32_t  colour32, colour;
    uint32_t  ii;
    uint32_t  count;
    uint32_t  *buf_ptr;

    colour32 = pfunc_sub->fill_colour;

    if(bpp != 32) {
        colour32 &= ((1 << bpp) - 1);

        colour = colour32;
        for(ii = ((32 / bpp) - 1); ii > 0; ii--) {
            colour32 = ((colour32 << bpp) | colour);
        }
    }
    count      = pfunc_sub->plane_size >> 2;
    scaler_ptr = &pfunc_sub->scaler_params;

#ifdef VIOUT_USE_DMA
    dma_ptr = pfunc_sub->dma_params;
    dma_ptr->fillValue = colour32;

    ii = scaler_ptr->inputPicHeight; // ii serves as a line counter here

    buf_ptr = (uint32_t*)scaler_ptr->picBase;

    if(((scaler_ptr->inputPicWidth * pfunc_sub->bpp) >> 5) > 992)
    {
        // Split the DMA fill into a left and a right OSD side because
        // the lineLength of the DMA controller is limited to
        // 0x00000FFF = 4095 bytes.
        // Since each line is a multiple of 128 bytes, just divide
        // the line length by two in order to get balanced partitions.
        // Therefore it will be possible to write as many lines as
        // possible in a single burst.

        dma_ptr->lineWords = (scaler_ptr->inputPicWidth * pfunc_sub->bpp) >> 6;
        dma_ptr->skipWords = dma_ptr->lineWords;
        while(ii) {
            // try to write as many lines as possible:
            dma_ptr->lineCount = 0x7FFFF / dma_ptr->lineWords;

            // check if the end has been reached:
            if(dma_ptr->lineCount > ii) {
                dma_ptr->lineCount = ii;
            }

            // check for DMA maximum line number constraints:
            if(dma_ptr->lineCount > 0x000003FFUL) {
                dma_ptr->lineCount = 0x000003FFUL;
            }

            // perform the burst for the left side:
            dma_request = dma_funcStr.StartMemfill(buf_ptr,
                                                   dma_ptr,
                                                   fdlviout_DmaFinish);
            if(dma_request) {
                // wait until the request has finished:
                FAPI_SYS_GET_SEMAPHORE(fdlviout_DmaSemaphore, FAPI_SYS_SUSPEND);
            }
            else {
                dma_ok = 0; // DMA processing failed
                break;
            }

            // perform the burst for the right side:
            dma_request = dma_funcStr.StartMemfill(buf_ptr + dma_ptr->lineWords,
                                                   dma_ptr,
                                                   fdlviout_DmaFinish);
            if(dma_request) {
                // wait until the request has finished:
                FAPI_SYS_GET_SEMAPHORE(fdlviout_DmaSemaphore, FAPI_SYS_SUSPEND);
            }
            else {
                dma_ok = 0; // DMA processing failed
                break;
            }
            buf_ptr += dma_ptr->lineCount * dma_ptr->lineWords * 2;
            ii -= dma_ptr->lineCount;
        }
    }
    else {
        // serve a whole line in each burst
        dma_ptr->skipWords = 0;
        dma_ptr->lineWords = (scaler_ptr->inputPicWidth * pfunc_sub->bpp) >> 5;
        while(ii) {
            // try to write as many lines as possible:
            dma_ptr->lineCount = 0x7FFFF / dma_ptr->lineWords;

            // check if the end has been reached:
            if(dma_ptr->lineCount > ii) {
                dma_ptr->lineCount = ii;
            }

            // check for DMA maximum line number constraints:
            if(dma_ptr->lineCount > 0x000003FFUL) {
                dma_ptr->lineCount = 0x000003FFUL;
            }

            dma_request = dma_funcStr.StartMemfill(buf_ptr,
                                                   dma_ptr,
                                                   fdlviout_DmaFinish);
            if(dma_request) {
                // wait until the request has finished:
                FAPI_SYS_GET_SEMAPHORE(fdlviout_DmaSemaphore, FAPI_SYS_SUSPEND);
                buf_ptr += dma_ptr->lineCount * dma_ptr->lineWords;
                ii -= dma_ptr->lineCount;
            }
            else {
                dma_ok = 0; // DMA processing failed
                break;
            }
        }
    }
    dma_funcStr.ReleaseRequest(dma_request);

    if(!dma_ok) {
        FAPEXDebugPrint(2, "[fdlviout_FillPlane] DMA failed\n");

        //-------- fallback: CPU based procedure:
        buf_ptr = (uint32_t*)scaler_ptr->picBase;
        ii = count; // ii serves as a long word counter here
        while(ii--) {
            *buf_ptr++ = colour32;
        }
        FAPI_SYS_FlushDataCache(scaler_ptr->picBase, pfunc_sub->plane_size);
        //-------- end of fallback procedure
    }
#else
    buf_ptr = (uint32_t*)scaler_ptr->picBase;

    while(count--) {
        *buf_ptr++ = colour32;
    }
    FAPI_SYS_FlushDataCache(scaler_ptr->picBase, pfunc_sub->plane_size);

#endif /* VIOUT_USE_DMA */
}

/*!
*******************************************************************************
**
** \brief  setup VIOUT OSD Plane
**
** \param  pfunc_sub    pointer to sub context
** \param  params       pointer to FAPI_VISCALE_ScalerParamsT parameter
** \param  pBpp         pointer to bpp value (bit per pixel)
** \param  pPlaneSize   pointer to plane size value
**
******************************************************************************/
static int32_t fdlviout_SetupOsdPlane(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                      FAPI_VISCALE_ScalerParamsT *params,
                                      uint32_t *pBpp, uint32_t *pPlaneSize)
{
    uint32_t  pic_h_res, pic_v_res;
    uint32_t  mask, size, h_size;
    int32_t   ret = 0;
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    FAPEXFDL_VIOUT_CmnStrT  *pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT *)pfunc_sub->pCmnHdl;
    uint32_t                aligned_size, osd_free_bytes;
#endif

    /* set Bits Per Pixel */
    switch(params->osdMode) {
        case FAPI_VISCALE_OSD_MODE_32_24BIT:
            *pBpp = 32;
            break;
        case FAPI_VISCALE_OSD_MODE_5551:
        case FAPI_VISCALE_OSD_MODE_4444:
        case FAPI_VISCALE_OSD_MODE_16BIT:
            *pBpp = 16;
            break;
        case FAPI_VISCALE_OSD_MODE_CLUT_8BIT:
            *pBpp = 8;
            break;
        case FAPI_VISCALE_OSD_MODE_CLUT_4BIT:
            *pBpp = 4;
            break;
        default:
            FAPEXDebugPrint(2, "[fdlviout_SetupOsdPlane] error mode\n");
            return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }
    pic_h_res = params->inputWindowWidth;
    pic_v_res = params->inputWindowHeight;

    FAPEXDebugPrint(3, "[fdlviout_SetupOsdPlane] %d x %d, bpp %d\n",
                       pic_h_res, pic_v_res, *pBpp);

    /* one line must allocate a multiple of 64 bytes */
    // go above:
    mask = (64 / *pBpp) - 1; //changed for h61 for 8 bytes aligned
    if((pic_h_res & mask) != 0) {
        pic_h_res = (pic_h_res + mask) & ~mask;
    }
    // v_res must be a multiple of 2
    if(pic_v_res & 0x00000001) {
        pic_v_res ++;
    }

#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    /*---------- LOCK ----------*/
    FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);
#endif

    /* OSD plane size check */
    h_size = (pic_h_res * *pBpp) >> 3;
    size   = h_size * pic_v_res;
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    aligned_size = (size + (FAPEX_SYS_MMAP_OSD_BORDER_SIZE - 1)) & FAPEX_SYS_MMAP_OSD_BORDER_MASK;
#endif

    if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD1) {
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE

        osd_free_bytes = FAPEX_SYS_MMAP_OSD1_PLANE_BYTES;

#endif
        if( (h_size > FAPEX_VIOUT_OSD1_MAX_LINE_BYTE)
         || (size   > FAPEX_VIOUT_OSD1_MAX_SIZE)
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
         || (aligned_size > osd_free_bytes)
#endif
        ) {
            FAPEXDebugPrint(2, "[fdlviout_SetupOsdPlane] out of memory (OSD1) (%d x %d) %08x\n",
                               pic_h_res, pic_v_res, size);
            ret = FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
        }
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
        else {

            pfunc_sub->plane_ptr = (uint8_t*)FAPEX_SYS_MMAP_OSD1_PLANE_START;    /* physical address */

            pfunc_sub->plane_max_size = pfunc_cmn->used_bytes_osd1 = aligned_size;
        }
#endif
    }
    else if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD2) {
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE

        osd_free_bytes = FAPEX_SYS_MMAP_OSD2_PLANE_BYTES;

#endif
        if( (h_size > FAPEX_VIOUT_OSD2_MAX_LINE_BYTE)
         || (size   > FAPEX_VIOUT_OSD2_MAX_SIZE)
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
         || (aligned_size > osd_free_bytes)
#endif
        ) {
            FAPEXDebugPrint(2, "[fdlviout_SetupOsdPlane] out of memory (OSD2) (%d x %d) %08x, MAX_LINE_SIZE = %d, MAX_SIZE = %d\n",
                               pic_h_res, pic_v_res, size, FAPEX_VIOUT_OSD2_MAX_LINE_BYTE, FAPEX_VIOUT_OSD2_MAX_SIZE);
            ret = FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
        }
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
        else {

            pfunc_sub->plane_ptr = (uint8_t*)FAPEX_SYS_MMAP_OSD2_PLANE_START;                /* physical address */

            pfunc_sub->plane_max_size = pfunc_cmn->used_bytes_osd2 = aligned_size;
        }
#endif
    }

   else if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3){    /* For OSD3, we always use allocation memory*/
       if( (h_size > FAPEX_VIOUT_OSD3_MAX_LINE_BYTE)
        || (size > FAPEX_VIOUT_OSD3_MAX_SIZE)) {
           FAPEXDebugPrint(2, "[fdlviout_SetupOsdPlane] out of memory (OSD3) (%d x %d) %08x\n",
                               pic_h_res, pic_v_res, size);
           ret = FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
       }
        size = (size +  PAGE_SIZE - 1) & PAGE_MASK;
   }
   else{    /* for OSD4,we always use allocation memory */
       if( (h_size > FAPEX_VIOUT_OSD4_MAX_LINE_BYTE)   
        || (size > FAPEX_VIOUT_OSD4_MAX_SIZE)) {
           FAPEXDebugPrint(2, "[fdlviout_SetupOsdPlane] out of memory (OSD3) (%d x %d) %08x\n",
                               pic_h_res, pic_v_res, size);
           ret = FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
       }  
       size = (size + PAGE_SIZE - 1) & PAGE_MASK;   
    
   }
    if(ret == 0) {
        *pPlaneSize = size;
        params->inputPicWidth  = pic_h_res;
        params->inputPicHeight = pic_v_res;
        FAPEXDebugPrint(3, "[fdlviout_SetupOsdPlane] inputPicWidth %d, inputPicHeight %d\n",
                           pic_h_res, pic_v_res);
       
        #ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
        if( (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD1) ||
            (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD2) ){
            if((uint32_t)pfunc_sub->plane_ptr < FAPEX_SYS_MMAP_ARM_APPLICATION_START) {
                params->picBase = FIO_ADDRESS(BUF, (uint32_t)pfunc_sub->plane_ptr);
            }
            else {
                params->picBase = FIO_ADDRESS(DDR2, (uint32_t)pfunc_sub->plane_ptr);
            }
            FAPEXDebugPrint(3, "[fdlviout_SetupOsdPlane] size : %08x %08x\n",
                               pfunc_cmn->used_bytes_osd1, pfunc_cmn->used_bytes_osd2);
        }
       #endif
    }
    /*---------- UNLOCK ----------*/
    #ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
    #endif

    return ret;
}

/*!
*******************************************************************************
**
** \brief  setup VIOUT Cursor Plane
**
** \param  params       pointer to FAPI_VISCALE_ScalerParamsT parameter
** \param  pBpp         pointer to bpp value (bit per pixel)
** \param  pPlaneSize   pointer to plane size value
**
******************************************************************************/
static int32_t fdlviout_SetupCursorPlane(FAPI_VISCALE_ScalerParamsT *params,
                                         uint32_t *pBpp, uint32_t *pPlaneSize)
{
    uint32_t  pic_x_res, pic_y_res;
    uint32_t  mask, size;

    /* set Bits Per Pixel */
    switch(params->cpMode) {
        case FAPI_VISCALE_CURSOR_MODE_1BIT:
            *pBpp = 1;
            break;
        case FAPI_VISCALE_CURSOR_MODE_2BIT:
            *pBpp = 2;
            break;
        case FAPI_VISCALE_CURSOR_MODE_4BIT:
            *pBpp = 4;
            break;
        case FAPI_VISCALE_CURSOR_MODE_8BIT:
            *pBpp = 8;
            break;
        default:
            FAPEXDebugPrint(2, "[fdlviout_SetupCursorPlane] error mode\n");
            return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }
    pic_x_res = params->inputWindowWidth;
    pic_y_res = params->inputWindowHeight;

    FAPEXDebugPrint(3, "[fdlviout_SetupCursorPlane] %d x %d, bpp %d\n",
                       pic_x_res, pic_y_res, *pBpp);

    // need for 32 bits per line on the CP:
    mask = (32 / *pBpp) - 1;
    if((pic_x_res & mask) != 0) {
        pic_x_res = (pic_x_res + mask) & ~mask;
    }
    if(pic_y_res & 0x00000001) {
        pic_y_res ++;
    }

    /* the Cursor Plane must NOT exceed 4 KB of memory */
    size = (pic_x_res * pic_y_res * *pBpp) >> 3;
    if( (size > FAPEX_VIOUT_CURSOR_RAM_MAX_SIZE)
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
     || (size > FAPEX_SYS_MMAP_CURSOR_PLANE_BYTES)
#endif
    ) {
        FAPEXDebugPrint(2, "[fdlviout_SetupCursorPlane] out of memory (%d x %d)\n",
                           pic_x_res, pic_y_res);
        return FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
    }
    *pPlaneSize = size;
    params->inputPicWidth  = pic_x_res;
    params->inputPicHeight = pic_y_res;
    FAPEXDebugPrint(3, "[fdlviout_SetupCursorPlane] inputPicWidth %d, inputPicHeight %d\n",
                       pic_x_res, pic_y_res);
    return 0;
}

//#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
/*!
*******************************************************************************
**
** \brief  allocate Plane buffer
**
** \param  pfunc_sub    pointer to sub context
**
******************************************************************************/
static int32_t fdlviout_allocatePlane(FAPEXFDL_VIOUT_SubStrT *pfunc_sub)
{
#ifndef __LINUX__
    uint32_t  max_size, aligned_rest;

    if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_CURSOR) {
        aligned_rest = 3;
    }
    else {
        aligned_rest = 1023;
    }

    if((pfunc_sub->plane_size + aligned_rest) > pfunc_sub->plane_max_size) {
        max_size = pfunc_sub->plane_size + aligned_rest;

        if(pfunc_sub->plane_ptr != NULL) {
            FAPI_SYS_FREE(pfunc_sub->plane_ptr);
        }

        /* plane buffer allocation */
        pfunc_sub->plane_ptr = (uint8_t *)FAPI_SYS_MALLOC(max_size);
        if(pfunc_sub->plane_ptr == NULL) {
            pfunc_sub->scaler_params.picBase = 0;
            pfunc_sub->plane_size            = 0;
            pfunc_sub->plane_max_size        = 0;
            return FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
        }
        /* alignment of plane buffer address */
        pfunc_sub->scaler_params.picBase = ((uint32_t)pfunc_sub->plane_ptr + aligned_rest) & ~aligned_rest;
        pfunc_sub->plane_max_size        = max_size;

        FAPEXDebugPrint(2, "[fdlviout_allocatePlane] VIOUT buffer :0x%08x(0x%08x)\n",
                           pfunc_sub->scaler_params.picBase, pfunc_sub->plane_ptr);
#else /* !__LINUX__ */
#if 0
    uint32_t order;
    order = PAGE_SHIFT;
    while( (order < 31) && (pfunc_sub->plane_size > (1 << order)) ) {
        order ++;
    }
#endif

    if(pfunc_sub->plane_size > pfunc_sub->plane_max_size) {
        if(pfunc_sub->plane_max_size != 0) {
           // __free_pages(pfunc_sub->plane_pg, (pfunc_sub->plane_size_order - PAGE_SHIFT));
           dma_free_coherent(NULL,pfunc_sub->plane_max_size,(void*)pfunc_sub->virt_addr,pfunc_sub->dma_addr);
        }

        /* plane buffer allocation */
       // pfunc_sub->plane_pg = alloc_pages((GFP_HIGHUSER | GFP_DMA), (order - PAGE_SHIFT));
        pfunc_sub->virt_addr =(uint32_t)dma_alloc_coherent(NULL,pfunc_sub->plane_size,
                                          &(pfunc_sub->dma_addr),GFP_KERNEL);
        if(pfunc_sub->virt_addr == 0) {
            pfunc_sub->scaler_params.picBase = 0;
            pfunc_sub->plane_size            = 0;
            pfunc_sub->plane_max_size        = 0;
         //   pfunc_sub->plane_size_order      = 0;
            pfunc_sub->dma_addr = 0;
            return FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
        }
        pfunc_sub->scaler_params.picBase = pfunc_sub->virt_addr;
        FAPEXDebugPrint(3, "picBase:0x%x\n",pfunc_sub->scaler_params.picBase);
        FAPEXDebugPrint(3, "dma addr:0x%x\n",pfunc_sub->dma_addr);
        pfunc_sub->plane_max_size        = pfunc_sub->plane_size;
       // pfunc_sub->plane_size_order      = order;
        FAPEXDebugPrint(2, "[fdlviout_allocatePlane] VIOUT buffer :0x%08x(0x%08x) %d/%d\n",
                           pfunc_sub->scaler_params.picBase, (uint32_t)page_to_phys(pfunc_sub->plane_pg),
                           pfunc_sub->plane_size, pfunc_sub->bpp);
#endif /* !__LINUX__ */
    }
    return 0;
}
//#endif /* VIOUT_ALLOCATE_OSD_CURSOR_PLANE */

/*!
*******************************************************************************
**
** \brief  set VIOUT scaler parameter
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_ScalerParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_SetScalerParams(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                        FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                        FAPEX_VIOUT_ScalerParamStrT *pArg)
{
    FAPI_VISCALE_ScalerParamsT  *params;
    int32_t  ret = 0;

    params = &pfunc_sub->scaler_params;

    pfunc_sub->plane_size = 0;
    pfunc_sub->bpp        = 0;

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->frameField),         &(pArg->flagFrameField),     sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->inputWindowX0),      &(pArg->inputWindowX0),      sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->inputWindowY0),      &(pArg->inputWindowY0),      sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->inputWindowWidth),   &(pArg->inputWindowWidth),   sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->inputWindowHeight),  &(pArg->inputWindowHeight),  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->outputWindowX0),     &(pArg->outputWindowX0),     sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->outputWindowY0),     &(pArg->outputWindowY0),     sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->outputWindowWidth),  &(pArg->outputWindowWidth),  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->outputWindowHeight), &(pArg->outputWindowHeight), sizeof(uint32_t));

    // Set up the output resolution if it is set to 0:
    if(params->outputWindowWidth == 0) {
        params->outputWindowWidth = params->inputWindowWidth;
    }
    if(params->outputWindowHeight == 0) {
        params->outputWindowHeight = (params->frameField != 0) ?
                                      params->inputWindowHeight : (params->inputWindowHeight / 2);
    }
    /* For 720x...i modes the horizontal width is multiplied with 2
    ** for hardware limitation reasons (constant hsync frequency).
    if( (pfunc_cmn->system_mode == FAPI_VISCALE_SYSTEM_MODE_720x576x50I)
     || (pfunc_cmn->system_mode == FAPI_VISCALE_SYSTEM_MODE_720x480x5994I) ) {
        params->inputWindowX0     <<= 1;
        params->outputWindowWidth <<= 1;
    }
    */	// COMMENT by tiger

    switch(pfunc_sub->scaler_type) {
        case FAPI_VISCALE_SCALER_OSD1:
        case FAPI_VISCALE_SCALER_OSD2:
        case FAPI_VISCALE_SCALER_OSD3:
        case FAPI_VISCALE_SCALER_OSD4:
            FAPEXCFDLSAF_COPY_TO_DRV(&params->osdMode,   &(pArg->mode),         sizeof(uint32_t));
            FAPEXCFDLSAF_COPY_TO_DRV(&params->osdLayout, &(pArg->osdByteOrder), sizeof(uint32_t));

            ret = fdlviout_SetupOsdPlane(pfunc_sub, params, &pfunc_sub->bpp, &pfunc_sub->plane_size);
            if(ret == 0) {
#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
                /* allocate plane image buffer */
                ret = fdlviout_allocatePlane(pfunc_sub);
#endif

#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
                if( (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3) ||
                    (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD4) )
                    ret = fdlviout_allocatePlane(pfunc_sub);
            }
#endif
            break;

        case FAPI_VISCALE_SCALER_CURSOR:
            FAPEXCFDLSAF_COPY_TO_DRV(&params->cpMode,     &(pArg->mode),       sizeof(uint32_t));
            FAPEXCFDLSAF_COPY_TO_DRV(&params->cpTileNumX, &(pArg->cpTileNumX), sizeof(uint32_t));
            FAPEXCFDLSAF_COPY_TO_DRV(&params->cpTileNumY, &(pArg->cpTileNumY), sizeof(uint32_t));
            params->osdLayout = FAPI_VISCALE_OSD_LAYOUT_CbYCrA;

            ret = fdlviout_SetupCursorPlane(params, &pfunc_sub->bpp, &pfunc_sub->plane_size);
            if(ret == 0) {
#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
                /* allocate plane image buffer */
                ret = fdlviout_allocatePlane(pfunc_sub);
#else
                pfunc_sub->plane_ptr = (uint8_t*)FAPEX_SYS_MMAP_CURSOR_PLANE_START;  /* physical address */
                pfunc_sub->plane_max_size = FAPEX_SYS_MMAP_CURSOR_PLANE_BYTES;
                pfunc_sub->scaler_params.picBase = FIO_ADDRESS(BUF, FAPEX_SYS_MMAP_CURSOR_PLANE_START);
#endif
            }
            break;

        default:
#ifdef FAPI_VIOUT_CONNECT
            if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_VIDEO) {
                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&params->osdLayout, &(pArg->mode), sizeof(int32_t));

                ret = viscale_funcStr.SetScalingMethod(
                            pfunc_sub->pFAPICtxHdl,
                            (FAPI_VISCALE_ScalingMethodEnumT)params->osdLayout);
                if(ret < 0) {
                    FAPEXDebugPrint(2, "[fdlviout_SetScalerParams] "
                                       "FAPI_VISCALE_SetScalingMethod error:%d\n", ret);
                    return ret;
                }
            }
            else
#endif /* FAPI_VIOUT_CONNECT */
            {
                params->osdLayout = FAPI_VISCALE_UNKNOWN_OSD_LAYOUT;
            }
            params->osdMode = FAPI_VISCALE_UNKNOWN_OSD_MODE;
            params->cpMode  = FAPI_VISCALE_UNKNOWN_CURSOR_MODE;
            params->inputPicWidth  = params->inputWindowWidth;
            params->inputPicHeight = params->inputWindowWidth;
    }

    if((ret == 0) && (pfunc_sub->plane_size > 0)) {
        /* fill the OSD/Cursor Plane with fill_colour */
        fdlviout_FillPlane(pfunc_sub);
    }

    if(ret == 0) {
        if (pfunc_sub->status != FAPEX_STATUS_READY && \
            pfunc_sub->status != FAPEX_STATUS_RUNNING) {
            pfunc_sub->status = FAPEX_STATUS_READY;
        }
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  get VIOUT scaler parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_ScalerParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_GetScalerParams(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                        FAPEX_VIOUT_ScalerParamStrT *pArg)
{
    FAPI_VISCALE_ScalerParamsT  *params;
    FAPI_VISCALE_GetScalerParamsT getParams;
    int32_t  temp;
    int32_t  ret;

    params = &pfunc_sub->scaler_params;

    if (pfunc_sub->status == FAPEX_STATUS_RUNNING && pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_VIDEO){
        
        ret = viscale_funcStr.GetScalerParams(pfunc_sub->pFAPICtxHdl, &getParams);

        if(ret != FAPI_OK){
            FAPEXDebugPrint(2, "[fdlviout_GetScalerParams] "
                                "FAPI_VISCALE_GetScalerParams error: %d\n", ret);
            return ret;
        }

        /*Scaling Method kept in the params->osdLayout*/

        params->inputWindowX0      = getParams.inputWindowX0;
        params->inputWindowY0      = getParams.inputWindowY0;
        params->inputWindowWidth   = getParams.inputWindowWidth;
        params->inputWindowHeight  = getParams.inputWindowHeight;
        params->outputWindowX0     = getParams.outputWindowX0;
        params->outputWindowY0     = getParams.outputWindowY0;
        params->outputWindowWidth  = getParams.outputWindowWidth;
        params->outputWindowHeight = getParams.outputWindowHeight;
        params->osdMode            = getParams.osdMode;
        params->cpMode             = getParams.cpMode;
        params->cpTileNumX         = getParams.cpTileNumX;
        params->cpTileNumY         = getParams.cpTileNumY;
        params->inputPicWidth      = getParams.inputWindowWidth;
        params->inputPicHeight     = getParams.inputWindowHeight;
        params->inputSarWidth      = getParams.inputSarWidth;
        params->inputSarHeight     = getParams.inputSarHeight;
    }

    /* copy parameters from the kernel space to an user space */
    switch(pfunc_sub->scaler_type) {
        case FAPI_VISCALE_SCALER_OSD1:
        case FAPI_VISCALE_SCALER_OSD2:
        case FAPI_VISCALE_SCALER_OSD3:
        case FAPI_VISCALE_SCALER_OSD4:
            FAPEXCFDLSAF_COPY_TO_USR(&(pArg->mode), &params->osdMode, sizeof(int32_t));
            temp = params->osdLayout;
            break;

        case FAPI_VISCALE_SCALER_CURSOR:
            FAPEXCFDLSAF_COPY_TO_USR(&(pArg->mode), &params->cpMode, sizeof(int32_t));
            temp = params->osdLayout;
            break;

        case FAPI_VISCALE_SCALER_VIDEO:
            FAPEXCFDLSAF_COPY_TO_USR(&(pArg->mode), &params->osdLayout, sizeof(int32_t));
            temp = FAPEX_VIOUT_OSD_LAYOUT_UNKNOWN;
            break;

        default:
            temp = -1;
            FAPEXCFDLSAF_COPY_TO_USR(&(pArg->mode), &temp, sizeof(int32_t));
            temp = FAPEX_VIOUT_OSD_LAYOUT_UNKNOWN;
    }
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->osdByteOrder),       &temp,                         sizeof(int32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->cpTileNumX),         &params->cpTileNumX,           sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->cpTileNumY),         &params->cpTileNumY,           sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->flagFrameField),     &(params->frameField),         sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->bpp),                &(pfunc_sub->bpp),             sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->inputWindowX0),      &(params->inputWindowX0),      sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->inputWindowY0),      &(params->inputWindowY0),      sizeof(uint32_t));
    if (pfunc_sub->scaler_type != FAPI_VISCALE_SCALER_VIDEO) {
        FAPEXCFDLSAF_COPY_TO_USR(&(pArg->inputWindowWidth),   &(params->inputPicWidth),      sizeof(uint32_t));
        FAPEXCFDLSAF_COPY_TO_USR(&(pArg->inputWindowHeight),  &(params->inputPicHeight),     sizeof(uint32_t));
    } else {
        FAPEXCFDLSAF_COPY_TO_USR(&(pArg->inputWindowWidth),   &(params->inputWindowWidth),   sizeof(uint32_t));
        FAPEXCFDLSAF_COPY_TO_USR(&(pArg->inputWindowHeight),  &(params->inputWindowHeight),  sizeof(uint32_t));
    }
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->outputWindowX0),     &(params->outputWindowX0),     sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->outputWindowY0),     &(params->outputWindowY0),     sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->outputWindowWidth),  &(params->outputWindowWidth),  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->outputWindowHeight), &(params->outputWindowHeight), sizeof(uint32_t));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT display aspect ratio parameter
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
** \param  aspectFlag   display aspect ratio flag
**
******************************************************************************/
static int32_t fdlviout_SetAspectRatio(
                    FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn, FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                    uint32_t id, FAPI_VISCALE_DisplayAspectRatioEnumT aspectFlag)
{
    uint32_t  enable_cmn, enable_ch;
    uint32_t  one_ch_flag;
    int32_t   ret = 0;

    enable_ch  = 0x00000001;
    enable_cmn = pfunc_cmn->enable_channel;

    if(pfunc_sub != NULL) {
        /* sub descriptor */
        enable_ch <<= id - FAPEXDEV_VIOUT_SUB_MINOR_FIRST;
        enable_cmn &= enable_ch;
        one_ch_flag = 1;
    }
    else {
        /* common descriptor */
        pfunc_sub   = (FAPEXFDL_VIOUT_SubStrT *)FAPEXCFDL_GetFdlHdl(pfunc_cmn->pCfdlHdl, id+1);
        one_ch_flag = 0;
    }

    /* setup channel */
    while(enable_cmn) {
        if(enable_cmn & enable_ch) {

            if( (one_ch_flag)
             || (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_VIDEO)
             || (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_SD) ) {
#ifdef FAPI_VIOUT_CONNECT
                /* set display aspect ratio */
                ret = viscale_funcStr.SetDisplayAspectRatio(pfunc_sub->pFAPICtxHdl, aspectFlag);
                if(ret != FAPI_OK) {
                    FAPEXDebugPrint(2, "[fdlviout_SetAspectRatio] "
                                       "FAPI_VISCALE_SetDisplayAspectRatio(%d) error:%d\n",
                                       aspectFlag, ret);
                    return ret;
                }
                if(pfunc_sub->status == FAPEX_STATUS_RUNNING) {
                    /* set resize */
                    ret = viscale_funcStr.Resize(pfunc_sub->pFAPICtxHdl, &pfunc_sub->scaler_params);
                    if(ret != FAPI_OK) {
                        if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_SD) {
                            FAPEXDebugPrint(2, "[fdlviout_SetAspectRatio] "
                                               "FAPI_VISCALE_Resize error:%d\n", ret);
                            return ret;
                        }
                        ret = FAPI_OK;
                    }
                }
#endif
            }
            enable_cmn &= ~enable_ch;
        }

        /* setup for next channel */
        pfunc_sub ++;
        enable_ch <<= 1;
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT alpha parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_AlphaParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_SetAlphaParams(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                       FAPEX_VIOUT_AlphaParamStrT *pArg)
{
    FAPEX_VIOUT_AlphaParamStrT  params;
    int32_t  ret = 0;

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&params, pArg, sizeof(FAPEX_VIOUT_AlphaParamStrT));

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (set alpha) */
    ret = viscale_funcStr.Alpha(pfunc_sub->pFAPICtxHdl,
                                (FAPI_VISCALE_ParameterModeEnumT)params.mode,
                                params.alpha);
    if(ret == FAPI_OK) {
        if(params.mode == FAPI_VISCALE_PARAMETER_ON) {
            ret = viscale_funcStr.Alpha(pfunc_sub->pFAPICtxHdl,
                                        FAPI_VISCALE_PARAMETER_SET,
                                        params.alpha);

            pfunc_sub->scaler_params.useGlobalAlpha = 1;
            pfunc_sub->scaler_params.globalAlpha    = ret;
        }
        else if(params.mode == FAPI_VISCALE_PARAMETER_OFF) {
            pfunc_sub->scaler_params.useGlobalAlpha = 0;
        }
    }
#endif /* FAPI_VIOUT_CONNECT */

    if(ret >= 0) {
        return 0;
    }
    else {
        FAPEXDebugPrint(2, "[fdlviout_SetAlphaParams] FAPI_VISCALE_Alpha error:%d\n", ret);
        return ret;
    }
}

/*!
*******************************************************************************
**
** \brief  get VIOUT alpha parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_AlphaParamStrT parameter
**
******************************************************************************/
static void fdlviout_GetAlphaParams(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                    uint32_t *pArg)
{
    uint32_t alpha;

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (get alpha) */
    alpha = viscale_funcStr.Alpha(pfunc_sub->pFAPICtxHdl, FAPI_VISCALE_PARAMETER_GET, 0);
#endif /* FAPI_VIOUT_CONNECT */

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, &alpha, sizeof(uint32_t));
}

/*!
*******************************************************************************
**
** \brief  set VIOUT fading parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_FadingParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_SetFadingParams(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                        FAPEX_VIOUT_FadingParamStrT *pArg)
{
    FAPEX_VIOUT_FadingParamStrT params;
    int32_t ret = 0;

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&params, pArg, sizeof(FAPEX_VIOUT_FadingParamStrT));

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI () */
    ret = viscale_funcStr.AutoPlaneFadingStepSize(pfunc_sub->pFAPICtxHdl,
                                                  FAPI_VISCALE_PARAMETER_SET,
                                                  &params.stepSize);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlviout_SetFadingParams] "
                           "FAPI_VISCALE_AutoPlaneFadingStepSize error:%d\n", ret);
        return ret;
    }

    ret = viscale_funcStr.AutoPlaneFading(pfunc_sub->pFAPICtxHdl,
                                          (FAPI_VISCALE_VideoFadingModeEnumT)params.mode);
    if(ret == FAPI_OK) {
        pfunc_sub->fading_mode = params.mode;
    }
    else {
        FAPEXDebugPrint(2, "[fdlviout_SetFadingParams] "
                           "FAPI_VISCALE_AutoPlaneFading error:%d\n", ret);
    }
#endif /* FAPI_VIOUT_CONNECT */

    return ret;
}

/*!
*******************************************************************************
**
** \brief  get VIOUT fading parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_FadingParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_GetFadingParams(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                        FAPEX_VIOUT_FadingParamStrT *pArg)
{
    FAPEX_VIOUT_FadingParamStrT params;
    int32_t ret = 0;

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI () */
    ret = viscale_funcStr.AutoPlaneFadingStepSize(pfunc_sub->pFAPICtxHdl,
                                                  FAPI_VISCALE_PARAMETER_GET,
                                                  &params.stepSize);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlviout_GetFadingParams] "
                           "FAPI_VISCALE_AutoPlaneFadingStepSize error:%d\n", ret);
        return ret;
    }
#endif /* FAPI_VIOUT_CONNECT */

    params.mode = pfunc_sub->fading_mode;

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, &params, sizeof(FAPEX_VIOUT_FadingParamStrT));

    return ret;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT resize parameter
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_ResizeParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_SetResizeParams(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                        FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                        FAPEX_VIOUT_ResizeParamStrT *pArg)
{
    FAPI_VISCALE_ScalerParamsT   new_params, *org_params;
    FAPEX_VIOUT_ResizeParamStrT  params;
    uint32_t  bpp, new_plane_size;
    int32_t   ret = 0;

//#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    uint32_t  org_plane_size = pfunc_sub->plane_size;
//#endif

    org_params = &pfunc_sub->scaler_params;
    memcpy(&new_params, org_params, sizeof(FAPI_VISCALE_ScalerParamsT));

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&params, pArg, sizeof(FAPEX_VIOUT_ResizeParamStrT));

    new_params.inputWindowX0     = params.inputWindowX0;
    new_params.inputWindowY0     = params.inputWindowY0;
    new_params.inputWindowWidth  = params.inputWindowWidth;
    new_params.inputWindowHeight = params.inputWindowHeight;
    new_params.outputWindowX0    = params.outputWindowX0;
    new_params.outputWindowY0    = params.outputWindowY0;

    if(params.outputWindowWidth > pfunc_sub->max_hres) {
        new_params.outputWindowWidth = pfunc_sub->max_hres;
    }
    else {
        new_params.outputWindowWidth = params.outputWindowWidth;
    }
    if(params.outputWindowHeight > pfunc_sub->max_vres) {
        new_params.outputWindowHeight = pfunc_sub->max_vres;
    }
    else {
        new_params.outputWindowHeight = params.outputWindowHeight;
    }

    /* For 720x...i modes the horizontal width is multiplied with 2
    ** for hardware limitation reasons (constant hsync frequency).
    if( (pfunc_cmn->system_mode == FAPI_VISCALE_SYSTEM_MODE_720x576x50I)
     || (pfunc_cmn->system_mode == FAPI_VISCALE_SYSTEM_MODE_720x480x5994I) ) {
        new_params.outputWindowWidth <<= 1;
        if(new_params.outputWindowWidth > pfunc_sub->max_hres) {
            new_params.outputWindowWidth = pfunc_sub->max_hres;
        }
    }
    */	// COMMENT by tiger

    switch(pfunc_sub->scaler_type) {
        case FAPI_VISCALE_SCALER_OSD1:
        case FAPI_VISCALE_SCALER_OSD2:
        case FAPI_VISCALE_SCALER_OSD3:
        case FAPI_VISCALE_SCALER_OSD4:
            ret = fdlviout_SetupOsdPlane(pfunc_sub, &new_params, &bpp, &new_plane_size);
#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
            if((ret == 0) && (org_plane_size < new_plane_size)) {
                ret = fdlviout_allocatePlane(pfunc_sub);
            }
#endif
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE   //OSD3 & OSD4 always use allocation memory                
        if( (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3) || 
            (pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD4) )
        {
            if((ret == 0) && (org_plane_size < new_plane_size)){
                ret = fdlviout_allocatePlane(pfunc_sub);
            }
        } 
#endif
            break;

        case FAPI_VISCALE_SCALER_CURSOR:
            ret = fdlviout_SetupCursorPlane(&new_params, &bpp, &new_plane_size);
#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
            if((ret == 0) && (org_plane_size < new_plane_size)) {
                ret = fdlviout_allocatePlane(pfunc_sub);
            }
#endif
            break;

        default:
            bpp            = pfunc_sub->bpp;
            new_plane_size = pfunc_sub->plane_size;
    }
    if(ret != 0) {
        return ret;
    }

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI () */
    ret = viscale_funcStr.Resize(pfunc_sub->pFAPICtxHdl, &new_params);
    if(ret == FAPI_OK) {
        /* update parameter */
        org_params->inputWindowX0      = new_params.inputWindowX0;
        org_params->inputWindowY0      = new_params.inputWindowY0;
        org_params->inputWindowWidth   = new_params.inputWindowWidth;
        org_params->inputWindowHeight  = new_params.inputWindowHeight;
        org_params->inputPicWidth      = new_params.inputPicWidth;
        org_params->inputPicHeight     = new_params.inputPicHeight;
        org_params->outputWindowX0     = new_params.outputWindowX0;
        org_params->outputWindowY0     = new_params.outputWindowY0;
        org_params->outputWindowWidth  = new_params.outputWindowWidth;
        org_params->outputWindowHeight = new_params.outputWindowHeight;
        org_params->picBase            = new_params.picBase;
        pfunc_sub->bpp        = bpp;
        pfunc_sub->plane_size = new_plane_size;
    }
    else {
        FAPEXDebugPrint(2, "[fdlviout_SetResizeParams] FAPI_VISCALE_Resize error:%d\n", ret);
    }
#endif /* FAPI_VIOUT_CONNECT */

    return ret;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT position parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_PosParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_SetPositionParams(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                          FAPEX_VIOUT_PosParamStrT *pArg)
{
    FAPI_VISCALE_ScalerParamsT  *params;
    int32_t  ret = 0;

    params = &pfunc_sub->scaler_params;

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->inputWindowX0),  &(pArg->inputWindowX0),  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->inputWindowY0),  &(pArg->inputWindowY0),  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->outputWindowX0), &(pArg->outputWindowX0), sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(params->outputWindowY0), &(pArg->outputWindowY0), sizeof(uint32_t));

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI () */
    ret = viscale_funcStr.Resize(pfunc_sub->pFAPICtxHdl, params);

    if(ret < 0) {
        FAPEXDebugPrint(2, "[fdlviout_SetPositionParams] FAPI_VISCALE_Resize error:%d\n", ret);
    }
#endif /* FAPI_VIOUT_CONNECT */

    return ret;
}

/*!
*******************************************************************************
**
** \brief  get VIOUT position parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_PosParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_GetPositionParams(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                          FAPEX_VIOUT_PosParamStrT *pArg)
{
    FAPI_VISCALE_ScalerParamsT  *params;

    params = &pfunc_sub->scaler_params;

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(&pArg->inputWindowX0,  &params->inputWindowX0,  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&pArg->inputWindowY0,  &params->inputWindowY0,  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&pArg->outputWindowX0, &params->outputWindowX0, sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&pArg->outputWindowY0, &params->outputWindowY0, sizeof(uint32_t));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT plane order
**
** \param  pArg         pointer to plane order array parameter
**
******************************************************************************/
static int32_t fdlviout_SetPlaneOrder(int32_t *pArg)
{
    int32_t  layer[6];
    uint32_t select[6], set_flag = 0;
    int32_t  i;

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(layer, pArg, sizeof(int32_t) * 6);

    for(i=5; i>=0; i--) {
        if(layer[i] >= 0) {
            if(layer[i] < 6) {
                select[layer[i]] = i;
                set_flag |= 1 << layer[i];
            }
            else {
                return FAPEX_VIOUT_ERR_BAD_PARAMETER;
            }
        }
    }
    if(set_flag != 0x3F) {
        return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI () */
    viscale_funcStr.PlaneOrder(FAPI_VISCALE_PARAMETER_SET,
                               (FAPI_VISCALE_PlaneDataSelectorEnumT)select[5],
                               (FAPI_VISCALE_PlaneDataSelectorEnumT)select[4],
                               (FAPI_VISCALE_PlaneDataSelectorEnumT)select[3],
                               (FAPI_VISCALE_PlaneDataSelectorEnumT)select[2],
                               (FAPI_VISCALE_PlaneDataSelectorEnumT)select[1],
                               (FAPI_VISCALE_PlaneDataSelectorEnumT)select[0]);
#endif /* FAPI_VIOUT_CONNECT */

    return 0;
}

/*!
*******************************************************************************
**
** \brief  get VIOUT plane order
**
** \param  pArg         pointer to plane order array parameter
**
******************************************************************************/
static int32_t fdlviout_GetPlaneOrder(int32_t *pArg)
{
    uint32_t  i, plane_no, select;
    int32_t   layer[] = {-1, -1, -1, -1, -1, -1};

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI () */
    select = viscale_funcStr.PlaneOrder(FAPI_VISCALE_PARAMETER_GET,
                                        (FAPI_VISCALE_PlaneDataSelectorEnumT)0,
                                        (FAPI_VISCALE_PlaneDataSelectorEnumT)0,
                                        (FAPI_VISCALE_PlaneDataSelectorEnumT)0,
                                        (FAPI_VISCALE_PlaneDataSelectorEnumT)0,
                                        (FAPI_VISCALE_PlaneDataSelectorEnumT)0,
                                        (FAPI_VISCALE_PlaneDataSelectorEnumT)0);
#endif /* FAPI_VIOUT_CONNECT */

    for(i=0; i<6; i++) {
        plane_no = select & 0x07;
        if(layer[plane_no] < 0) {
            layer[plane_no] = i;
        }
        select >>= 4;
    }
    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, layer, (6 * sizeof(int32_t)));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT background colour
**
** \param  bgcolour   background colour value
**
******************************************************************************/
static int32_t fdlviout_SetBackgroundColour(uint32_t bgcolour)
{
#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (set back ground colour) */
    viscale_funcStr.BackgroundColour(FAPI_VISCALE_PARAMETER_SET,
                                     (bgcolour>>16)&0xFF, (bgcolour>>8)&0xFF, bgcolour&0xFF);
#endif /* FAPI_VIOUT_CONNECT */
    return 0;
}

/*!
*******************************************************************************
**
** \brief  get VIOUT background colour
**
** \param  pArg         pointer to background colour value
**
******************************************************************************/
static int32_t fdlviout_GetBackgroundColour(uint32_t *pArg)
{
    uint32_t bgcolour = 0;

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (get back ground colour) */
    bgcolour = viscale_funcStr.BackgroundColour(FAPI_VISCALE_PARAMETER_GET, 0, 0, 0);
#endif /* FAPI_VIOUT_CONNECT */

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, &bgcolour, sizeof(uint32_t));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT fill colour
**
** \param  pfunc_sub    pointer to sub context
** \param  fill_colour  fill colour value
**
******************************************************************************/
static int32_t fdlviout_SetFillColour(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                      uint32_t fill_colour)
{
    pfunc_sub->fill_colour = fill_colour;

    if(pfunc_sub->scaler_params.picBase != 0) {
        /* fill the OSD/Cursor Plane with fill_colour */
        fdlviout_FillPlane(pfunc_sub);
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  get VIOUT fill colour
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to fill colour value
**
******************************************************************************/
static int32_t fdlviout_GetFillColour(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                      uint32_t *pArg)
{
    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, &pfunc_sub->fill_colour, sizeof(uint32_t));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  set VIOUT OSD/CursorPlane layer Colour Look Up Table
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to CLUT array parameter
**
******************************************************************************/
static int32_t fdlviout_SetColourLUT(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                     uint32_t *pArg)
{
    uint32_t *colour_ptr;
    int32_t  ret = 0;

    colour_ptr = pfunc_sub->scaler_params.colourLut;

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(colour_ptr, pArg, (sizeof(uint32_t) * 256));

#ifdef FAPI_VIOUT_CONNECT
    if(pfunc_sub->status == FAPEX_STATUS_RUNNING) {
        /* call FAPI (update CLUT) */
        ret = viscale_funcStr.UpdateClut(pfunc_sub->pFAPICtxHdl, colour_ptr);

        if(ret < 0) {
            FAPEXDebugPrint(2, "[fdlviout_SetColourLUT] "
                               "FAPI_VISCALE_UpdateClut error:%d\n", ret);
        }
    }
#endif /* FAPI_VIOUT_CONNECT */

    return ret;
}

/*!
*******************************************************************************
**
** \brief  fill rectangle area
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_VIOUT_FillRectParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_FillRectangle(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
                                      FAPEX_VIOUT_FillRectParamStrT *pArg)
{
#ifdef VIOUT_USE_DMA
    FAPI_DMA_MemoryFillParamsT  *dma_ptr;
    FAPI_DMA_RequestT           *dma_request = NULL;
    uint32_t  dma_ok = 1;   // indicates whether all executed DMA operations
#endif
    FAPEX_VIOUT_FillRectParamStrT  params;
    FAPI_VISCALE_ScalerParamsT     *scaler_ptr;
    int32_t   ret = 0;
    uint32_t  ii, kk;
    uint32_t  colour;
    uint32_t  colour32;     // 4-byte colour code
    uint32_t  bpp;          // count of bits per pixel
    uint32_t  base_addr;    // 32-bit base address of the top left corner
    uint32_t  addr_offset;  // adress offset from the base address of the top left corner
    uint32_t  line_offset;  // offset in bytes for line jumps
    uint32_t  line_count;   // temporary line counter
    uint32_t  colour_mask;  // mask for writing pixels partially
    uint32_t  *buf_ptr;     // memory-pointer

    scaler_ptr = &pfunc_sub->scaler_params;

    if(scaler_ptr->picBase != 0) {
        /* copy parameters from an user space to the kernel space */
        FAPEXCFDLSAF_COPY_TO_DRV(&params, pArg, sizeof(FAPEX_VIOUT_FillRectParamStrT));

        /* size check */
        if( ((params.x0 + params.width)  > pfunc_sub->scaler_params.inputPicWidth)
         || ((params.y0 + params.height) > pfunc_sub->scaler_params.inputPicHeight) ) {
            return FAPEX_VIOUT_ERR_BAD_PARAMETER;
        }

        bpp      = pfunc_sub->bpp;
        colour32 = params.colour;
        if(bpp != 32) {
            colour32 &= ((1 << bpp) - 1);

            colour = colour32;
            for(ii = ((32 / bpp) - 1); ii > 0; ii--) {
                colour32 = ((colour32 << bpp) | colour);
            }
        }

        // 1. calculate parameter for steps 2 and 3:
        base_addr   = scaler_ptr->picBase
                    + ((params.y0 * scaler_ptr->inputPicWidth * bpp) >> 3)
                    + ((params.x0 * bpp) >> 3);  // /8 ==> byte pointer
        base_addr  &= 0xFFFFFFFC;                //4-byte aligned address
        addr_offset = (params.x0 * bpp) & 0x0000001F;
        line_offset = (scaler_ptr->inputPicWidth * bpp) >> 5;

        // 2. write out the left stripe of the rectangle with cpu
        colour_mask = 0;
        if(addr_offset != 0) {
            colour_mask = (1 << (32 - addr_offset)) - 1;
            if((addr_offset + (params.width * bpp)) < 32) {
                /* mention the case that the the stripe is very 'thin' */
                ii = addr_offset + (params.width * bpp);
                colour_mask &= ((1 << ii) - 1) << (32 - ii);
            }
        }
        /* also consider 'thin' stripes that start at offset 0 */
        else if((params.width * bpp) < 32) {
            ii = params.width * bpp;
            colour_mask = ((1 << ii) - 1) << (32 - ii);
        }

        if(colour_mask) {
            buf_ptr = (uint32_t*)base_addr;
            line_count = params.height;
            while(line_count--)
            {
                *buf_ptr = (*buf_ptr & ~colour_mask) | (colour32 & colour_mask);
                buf_ptr += line_offset;
            }
        }

        // 3. write out the right stripe of the rectangle with cpu
        // calculate the address of the first pixel that can not be written 32-bit wise
        buf_ptr = (uint32_t*)base_addr;
        ii = addr_offset + (params.width * bpp); // ii: count of bits per line to write
        if(ii >> 5) {
            // don't do anything for 'thin' stripes
            buf_ptr += (ii >> 5);
            ii &= 0x0000001F; // ii indicates the count of remaining bits in the last 32-bit word
            if(ii) {
                colour_mask = ((1 << ii) - 1) << (32 - ii);
                line_count = params.height; // ii serves as a line counter here
                while(line_count--)
                {
                    *buf_ptr = (*buf_ptr & ~colour_mask) | (colour32 & colour_mask);
                    buf_ptr += line_offset;
                }
            }
        }

        // 4. do the main part below: DMA or CPU based
        ii = addr_offset + (params.width * bpp); // ii: count of total bits from the first dword
        ii >>= 5;
        if(addr_offset) {
            ii = ii ? (ii - 1) : 0;
            buf_ptr = (uint32_t*)(base_addr + 4);
        }
        else {
            buf_ptr = (uint32_t*)base_addr;
        }
        // ii: count of 32-bit words per line to write
        // buf_ptr: points to the first double word in the rectangle.

        if(ii) { // check whether there are any 32-bit words to write out
            // calculate the line offset considering the written 32-bit words in between
            line_offset -= ii;

#ifdef VIOUT_USE_DMA
            dma_ptr = pfunc_sub->dma_params;
            dma_ptr->fillValue = colour32;

            line_count = params.height;
            if(ii > 0x3FFUL) {   // serve half a line per burst
                // Split the DMA fill into a left and a right OSD side because
                // the lineLength of the DMA controller is limited to
                // 0x000003FF = 1023 4-byte words.
                // Divide the line length by two in order to get balanced partitions.
                // Therefore it will be possible to write as many lines as
                // possible in a single burst.

                kk = ii / 2; // kk: rounded down count of 32-bit words per burst
                while(line_count) {
                    // try to write as many lines as possible:
                    dma_ptr->lineCount = 0x7FFFF / (kk + 1);

                    // check if the end has been reached:
                    if(dma_ptr->lineCount > line_count) {
                        dma_ptr->lineCount = line_count;
                    }

                    // check for DMA maximum line number constraints:
                    if(dma_ptr->lineCount > 0x000003FFUL) {
                        dma_ptr->lineCount = 0x000003FFUL;
                    }

                    // perform the burst for the left side:
                    dma_ptr->lineWords = kk;
                    dma_ptr->skipWords = kk + line_offset;

                    dma_request = dma_funcStr.StartMemfill(buf_ptr,
                                                           dma_ptr,
                                                           fdlviout_DmaFinish);
                    if(dma_request) {
                        // wait until the request has finished:
                        FAPI_SYS_GET_SEMAPHORE(fdlviout_DmaSemaphore, FAPI_SYS_SUSPEND);
                    }
                    else {
                        dma_ok = 0; // DMA processing failed
                        break;
                    }

                    // perform the burst for the right side:
                    dma_ptr->lineWords = (ii % 2) ? (kk + 1) : kk;
                    dma_ptr->skipWords = dma_ptr->lineWords + line_offset;

                    dma_request = dma_funcStr.StartMemfill(buf_ptr + kk,
                                                           dma_ptr,
                                                           fdlviout_DmaFinish);
                    if(dma_request) {
                        // wait until the request has finished:
                        FAPI_SYS_GET_SEMAPHORE(fdlviout_DmaSemaphore, FAPI_SYS_SUSPEND);
                    }
                    else {
                        dma_ok = 0; // DMA processing failed
                        break;
                    }
                    buf_ptr    += dma_ptr->lineCount * (ii + line_offset);
                    line_count -= dma_ptr->lineCount;
                }
            }
            else {
                // serve a whole line per burst
                dma_ptr->lineWords = ii;
                dma_ptr->skipWords = line_offset;
                while(line_count) {
                    // try to write as many lines as possible:
                    dma_ptr->lineCount = 0x7FFFF / dma_ptr->lineWords;

                    // check if the end has been reached:
                    if(dma_ptr->lineCount > line_count) {
                        dma_ptr->lineCount = line_count;
                    }

                    // check for DMA maximum line number constraints:
                    if(dma_ptr->lineCount > 0x000003FFUL) {
                        dma_ptr->lineCount = 0x000003FFUL;
                    }

                    dma_request = dma_funcStr.StartMemfill(buf_ptr,
                                                           dma_ptr,
                                                           fdlviout_DmaFinish);
                    if(dma_request) {
                        // wait until the request has finished:
                        FAPI_SYS_GET_SEMAPHORE(fdlviout_DmaSemaphore, FAPI_SYS_SUSPEND);
                        buf_ptr    += dma_ptr->lineCount * (dma_ptr->lineWords + line_offset);
                        line_count -= dma_ptr->lineCount;
                    }
                    else {
                        dma_ok = 0; // DMA processing failed
                        break;
                    }
                }
            }
            dma_funcStr.ReleaseRequest(dma_request);

            if(!dma_ok) {
                FAPEXDebugPrint(2, "[fdlviout_FillRectangle] DMA failed\n");

                //-------- fallback: CPU based procedure:
                buf_ptr = (uint32_t*) (addr_offset ? (base_addr + 4) : base_addr);
                line_count = params.height;
                while(line_count--) {
                    kk = ii;
                    while(kk--) {
                        *buf_ptr++ = colour32;
                    }
                    buf_ptr += line_offset;
                }
                FAPI_SYS_FlushDataCache(scaler_ptr->picBase,
                                        (scaler_ptr->inputPicHeight *
                                        (scaler_ptr->inputPicWidth * bpp)) >> 3);
                //-------- end of fallback procedure
            }
#else
            line_count = params.height;
            while(line_count--) {
                kk = ii;
                while(kk--) {
                    *buf_ptr++ = colour32;
                }
                buf_ptr += line_offset;
            }

#endif /* VIOUT_USE_DMA */
        }
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  update HDMI AVI InfoFrame for aspect ratio handling.
**
** \param  pfunc_cmn    pointer to common context
** \param  pArg         pointer to FAPEX_VIOUT_HdmiParamStrT parameter
**
******************************************************************************/
static int32_t fdlviout_UpdateHdmiAviInfo(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn,
                                          FAPEX_VIOUT_HdmiParamStrT *pArg)
{
    FAPI_HDMI_VideoParamsT  curr_param, update_param;
    uint32_t  pictureAr;
    int32_t   ret;

    if(pfunc_cmn->pHdmiCtxtHdl == NULL) {
        return FAPEX_VIOUT_ERR_INVALID_STATUS;
    }

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&update_param.inputType,    &pArg->inputType,    sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&update_param.outputType,   &pArg->outputType,   sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&update_param.scanInfo,     &pArg->scanInfo,     sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&update_param.activeFormat, &pArg->activeFormat, sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&pictureAr,                 &pArg->pictureAr,    sizeof(uint32_t));

    update_param.videoCode = fdlviout_GetHdmiVideoCode(pfunc_cmn, pictureAr);

    /* get current Video Parameters */
    ret = hdmi_funcStr.GetVideoParameters(pfunc_cmn->pHdmiCtxtHdl,
                                          &curr_param);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlviout_UpdateHdmiAviInfo] "
                           "FAPI_HDMI_GetVideoParameters error:%d\n", ret);
        return ret;
    }

    if( (update_param.videoCode    != curr_param.videoCode)
     || (update_param.activeFormat != curr_param.activeFormat)
     || (update_param.inputType    != curr_param.inputType)
     || (update_param.outputType   != curr_param.outputType)
     || (update_param.scanInfo     != curr_param.scanInfo) ) {
        /* change HDMI Video Parameters */
        ret = hdmi_funcStr.SetVideoParameters(pfunc_cmn->pHdmiCtxtHdl, &update_param);
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdlviout_UpdateHdmiAviInfo] "
                               "FAPI_HDMI_SetVideoParameters error:%d\n", ret);
            return ret;
        }

        /* update AVI InfoFrame data. */
        ret = hdmi_funcStr.UpdateAviInfoFrame(pfunc_cmn->pHdmiCtxtHdl);
        if(ret == FAPI_OK) {
            /* backup HDMI video parameter */
            FAPEXCFDLSAF_COPY_TO_DRV(&pfunc_cmn->hdmi_param, &update_param,
                                     sizeof(FAPI_HDMI_VideoParamsT));
        }
        else {
            FAPEXDebugPrint(2, "[fdlviout_UpdateHdmiAviInfo] "
                               "FAPI_HDMI_UpdateAviInfoFrame error:%d\n", ret);
        }
        return ret;
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  Restart HDMI Video
**
** \param  pfunc_cmn    pointer to common context
**
******************************************************************************/
static int32_t fdlviout_HdmiRestart(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn)
{
    int32_t  ret;

    hdmi_funcStr.VIDEO_Stop(pfunc_cmn->pHdmiCtxtHdl);

    ret = hdmi_funcStr.SetVideoParameters(pfunc_cmn->pHdmiCtxtHdl,
                                          &pfunc_cmn->hdmi_param);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlviout_HdmiRestart] "
                           "FAPI_HDMI_SetVideoParameters error:%d\n", ret);
        return ret;
    }

    ret = hdmi_funcStr.VIDEO_Start(pfunc_cmn->pHdmiCtxtHdl);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlviout_HdmiRestart] "
                           "FAPI_HDMI_VIDEO_Start error:%d\n", ret);
    }
    return ret;
}

/*!
 * *******************************************************************************
 * **
 * ** \brief  set VIOUT brightness
 * **
 * ** \param  level   brightness value
 * **
 * ******************************************************************************/
static int32_t fdlviout_SetBrightness(int32_t level) 
{
#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (set brightness) */
    viscale_funcStr.SetBrightness(FAPI_VISCALE_PARAMETER_SET,level);
#endif /* FAPI_VIOUT_CONNECT */
    return 0;
}

/*!
 *  ********************************************************************************
 *  **
 *  ** \brief  set VIOUT brightness
 *  **
 *  ** \param  level   contrast value
 *  **
 *  ********************************************************************************/
static int32_t fdlviout_SetContrast(int32_t level)
{
#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (set contrast) */
    viscale_funcStr.SetContrast(FAPI_VISCALE_PARAMETER_SET,level);
#endif /* FAPI_VIOUT_CONNECT */
    return 0;
}

/*!
 *  *********************************************************************************
 *  **
 *  ** \brief  set VIOUT saturation
 *  **
 *  ** \param  level   saturation value
 *  **
 *  ********************************************************************************/
static int32_t fdlviout_SetSaturation(int32_t level)
{
#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (set saturation) */
    viscale_funcStr.SetSaturation(FAPI_VISCALE_PARAMETER_SET,level);
#endif /* FAPI_VIOUT_CONNECT */
    return 0;
}

/*!
*******************************************************************************
**
** \brief  video output scale set Wss state.
**
** \param  pfunc_cmn    pointer to common context
** \param  pArg         pointer to FAPEX_VISCALE_WssStateEnumT parameter
**
******************************************************************************/
static int32_t fdlviout_Viscale_SetWssState(FAPEXFDL_VIOUT_SubStrT *pfunc_sub,
										FAPEX_VISCALE_WssStateEnumT *pArg)
{
	int32_t  ret;
	FAPEX_VISCALE_WssStateEnumT wssState =  *pArg;

	if(pfunc_sub->pFAPICtxHdl == NULL) {
		return FAPEX_VIOUT_ERR_INVALID_STATUS;
	}

	ret = viscale_funcStr.SetWssState(pfunc_sub->pFAPICtxHdl, wssState);
	if(ret != FAPI_OK) {
		FAPEXDebugPrint(2, "[fdlviout_Viscale_SetWssState] FAPI_VISCALE_SetWssState error:%d\n", ret);
		return ret;
	}
	return 0;
}

/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user 
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLVIOUT_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                     FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn;
    FAPEXFDL_VIOUT_SubStrT *pfunc_sub;
    uint32_t id;
    int32_t  ret = FAPEX_VIOUT_ERR_BAD_PARAMETER;

    id = FAPEXCFDLSAF_GETID(inode);

    // FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_ioctl] %3d cmd:%08x\n", id, cmd);

    /* Get the context base of this id's FDL. */
    if(id < FAPEXDEV_VIOUT_SUB_MINOR_FIRST) {
        pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT *)(file->private_data);
        pfunc_sub = NULL;
    }
    else {
        pfunc_sub = (FAPEXFDL_VIOUT_SubStrT *)(file->private_data);
        if(pfunc_sub != NULL) {
            pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT *)pfunc_sub->pCmnHdl;
        }
        else {
            pfunc_cmn = NULL;
        }
    }

    switch(cmd){
        /* actual code which is depend on function is implimented here. */
        /* case ...: */
        case FAPEX_IOCCOM_START:
            ret = fdlviout_Start(pfunc_cmn, pfunc_sub, id);
            break;

        case FAPEX_IOCCOM_STOP:
            ret = fdlviout_Stop(pfunc_cmn, pfunc_sub, id);
            break;

        case FAPEX_IOCCOM_VIOUT_OPEN:
            if(pfunc_sub == NULL) {
                ret = fdlviout_OpenCmn(pfunc_cmn, id, (FAPEX_VIOUT_OpenCmnParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_OPEN_SUBID:
            if(pfunc_sub != NULL) {
                ret = fdlviout_OpenSub(pfunc_cmn, pfunc_sub, id,
                                       (FAPEX_VIOUT_OpenSubParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_CLOSE_SUBID:
            if(pfunc_sub != NULL) {
                ret = fdlviout_CloseSub(pfunc_cmn, pfunc_sub, id);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_ENABLE:
            if(pfunc_sub != NULL) {
                ret = fdlviout_Enable(pfunc_cmn, pfunc_sub, id);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_DISABLE:
            if(pfunc_sub != NULL) {
#ifdef FAPI_VIOUT_CONNECT
                /* call FAPI (disable) */
                ret = viscale_funcStr.EnableDisable(pfunc_sub->pFAPICtxHdl, FAPI_VISCALE_PARAMETER_OFF);

                if(ret < 0) {
                    FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_ioctl] "
                                       "FAPI_VISCALE_EnableDisable error:%d\n", ret);
                }
                else
#endif /* FAPI_VIOUT_CONNECT */
                {
                    pfunc_sub->scaler_params.layerEnable = 0;
                }
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_SCALER_PARAMS:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_SetScalerParams(pfunc_cmn, pfunc_sub,
                                               (FAPEX_VIOUT_ScalerParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_SCALER_PARAMS:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_GetScalerParams(pfunc_sub, (FAPEX_VIOUT_ScalerParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO:
            ret = fdlviout_SetAspectRatio(pfunc_cmn, pfunc_sub, id,
                                          (FAPI_VISCALE_DisplayAspectRatioEnumT)arg);
            break;

        case FAPEX_IOCCOM_VIOUT_SET_ALPHA:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_SetAlphaParams(pfunc_sub, (FAPEX_VIOUT_AlphaParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_ALPHA:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                fdlviout_GetAlphaParams(pfunc_sub, (uint32_t*)arg);
                ret = 0;
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_FADING:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_SetFadingParams(pfunc_sub, (FAPEX_VIOUT_FadingParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_FADING:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_GetFadingParams(pfunc_sub, (FAPEX_VIOUT_FadingParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_RESIZE:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_SetResizeParams(pfunc_cmn, pfunc_sub,
                                               (FAPEX_VIOUT_ResizeParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_POSITION:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_SetPositionParams(pfunc_sub, (FAPEX_VIOUT_PosParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_POSITION:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_GetPositionParams(pfunc_sub, (FAPEX_VIOUT_PosParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_PLANEORDER:
            if((pfunc_sub == NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_SetPlaneOrder((int32_t*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_PLANEORDER:
            if((pfunc_sub == NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_GetPlaneOrder((int32_t*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_BGCOLOUR:
            if(pfunc_sub == NULL) {
                ret = fdlviout_SetBackgroundColour((uint32_t)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_BGCOLOUR:
            if((pfunc_sub == NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_GetBackgroundColour((uint32_t*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_SYSTEM_MODE:
            if((pfunc_cmn != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                int32_t *pArg = (int32_t *)arg;

                /* copy parameters from the kernel space to an user space */
                FAPEXCFDLSAF_COPY_TO_USR(pArg, &pfunc_cmn->system_mode, sizeof(int32_t));
                ret = 0;
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_FILLCOLOUR:
            if(pfunc_sub != NULL) {
                ret = fdlviout_SetFillColour(pfunc_sub, (uint32_t)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_FILLCOLOUR:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_GetFillColour(pfunc_sub, (uint32_t*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_CLUT:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_SetColourLUT(pfunc_sub, (uint32_t*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_GET_CLUT:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                uint32_t  *pArg = (uint32_t*)arg;

                /* copy parameters from the kernel space to an user space */
                FAPEXCFDLSAF_COPY_TO_USR(pArg, pfunc_sub->scaler_params.colourLut,
                                         (sizeof(uint32_t) * 256));
                ret = 0;
            }
            break;

        case FAPEX_IOCCOM_VIOUT_SET_HILIGHT:
            if(pfunc_sub != NULL) {
                FAPI_VISCALE_ParameterModeEnumT  hilight;
                uint32_t  mode = (uint32_t)arg;

                if(mode == 0) {
                    hilight = FAPI_VISCALE_PARAMETER_OFF;
                }
                else {
                    hilight = FAPI_VISCALE_PARAMETER_ON;
                    mode = 1;
                }
#ifdef FAPI_VIOUT_CONNECT
                /* call FAPI () */
                ret = viscale_funcStr.Hilight(pfunc_sub->pFAPICtxHdl, hilight);

                if(ret < 0) {
                    FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_ioctl] "
                                       "FAPI_VISCALE_Hilight error:%d\n", ret);
                }
                else
#endif /* FAPI_VIOUT_CONNECT */
                {
                    pfunc_sub->scaler_params.hilightOnOff = mode;
                }
            }
            break;

        case FAPEX_IOCCOM_VIOUT_FILL_RECTANGLE:
            if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_FillRectangle(pfunc_sub, (FAPEX_VIOUT_FillRectParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_FLUSH_PLANE:
            if(pfunc_sub != NULL) {
                if(pfunc_sub->scaler_params.picBase != 0) {
                    FAPI_SYS_FlushDataCache(pfunc_sub->scaler_params.picBase,
                                            pfunc_sub->plane_size);

                    if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_CURSOR) {
                        /* start Cursor Plane */
                        ret = fdlviout_StartCP(pfunc_cmn, pfunc_sub, id);
                    }
                    else {
                        ret = 0;
                    }
                }
            }
            break;

        case FAPEX_IOCCOM_VIOUT_UPDATE_AVI_INFO:
            if((pfunc_sub == NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
                ret = fdlviout_UpdateHdmiAviInfo(pfunc_cmn, (FAPEX_VIOUT_HdmiParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_VIOUT_HDMI_RESTART:
            if((pfunc_cmn != NULL) && (pfunc_sub == NULL)) {
                ret = fdlviout_HdmiRestart(pfunc_cmn);
            }
			break;

		case FAPEX_IOCCOM_VIOUT_SCALER_SET_WSSSTATE:
			if((pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL)) {
				ret = fdlviout_Viscale_SetWssState(pfunc_sub, (FAPEX_VISCALE_WssStateEnumT*)arg);
			}
			break;

		case FAPEX_IOCCOM_VIOUT_SET_BRIGHTNESS:
			if(pfunc_sub == NULL) {
				ret = fdlviout_SetBrightness((int32_t)arg);
			}
			break;

		case FAPEX_IOCCOM_VIOUT_SET_CONTRAST:
			if(pfunc_sub == NULL){
				ret = fdlviout_SetContrast((int32_t)arg);
			}
			break;

		case FAPEX_IOCCOM_VIOUT_SET_SATURATION:
			if(pfunc_sub == NULL){
				ret = fdlviout_SetSaturation((int32_t)arg);
			}
			break;

		default:
            ;
    }

    return ret;
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Poll method of this id's FDL.
**         This function is called with the systemcall "select".
**
** \param  file    The pointer to the struct file.
** \param  ptbl    An internal data.
**
******************************************************************************/
FAPEXFDLPLRET_T FAPEXFDLVIOUT_poll(FAPEXFDLPLARG1_T file, FAPEXFDLPLARG2_T ptbl)
{
    FAPEXFDLPLRET_T mask = 0;

    FAPEXDebugPrint(4, "[FAPEXFDLVIOUT_poll]\n");

    /* actual code which is depend on function is implimented here. */

    return mask;
}
#endif

/*!
*******************************************************************************
**
** \brief  Mmap method of this id's FDL.
**         This function is called with the systemcall "mmap".
**
** \param  file    The pointer to the struct file.
** \param  vma     system depend
**
******************************************************************************/
FAPEXFDLMMPRET_T FAPEXFDLVIOUT_mmap(FAPEXFDLMMPARG1_T file, FAPEXFDLMMPARG2_T vma)
{
#ifdef __LINUX__
    FAPEXFDL_VIOUT_SubStrT *pfunc_sub;
    int32_t  ret;

    FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_mmap]\n");

    /* Get the context base of this id's FDL. */
    pfunc_sub = (FAPEXFDL_VIOUT_SubStrT *)(file->private_data);

    if(pfunc_sub == NULL) {
        FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_mmap] context error\n");
        return FAPEX_VIOUT_ERR_INVALID_STATUS;
    }
    if(pfunc_sub->id < FAPEXDEV_VIOUT_SUB_MINOR_FIRST) {
        FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_mmap] id error\n");
        return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }

    if( (vma->vm_pgoff != 0) || ((vma->vm_end - vma->vm_start) > pfunc_sub->plane_max_size) ) {
        FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_mmap] size:%d\n", pfunc_sub->plane_max_size);
        return -EINVAL;
    }

    /* This area doen't swapout */
    vma->vm_flags |= VM_RESERVED;

    /* This area is IO area. Set noncache, and it doesn't dump to a core
       file of a process */
    if(file->f_flags & O_SYNC) {
        vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
        vma->vm_flags |= VM_IO;
    }

#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
   if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3 ||
       pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD4 ){
       ret = remap_pfn_range(vma,
                             vma->vm_start,
                             (uint32_t)pfunc_sub->dma_addr >> PAGE_SHIFT,  /* physical page */
                             vma->vm_end - vma->vm_start,
                             vma->vm_page_prot);
    }
    else{
         ret = remap_pfn_range(vma,
                          vma->vm_start,
                          (uint32_t)pfunc_sub->plane_ptr >> PAGE_SHIFT,  /* physical page */
                          vma->vm_end - vma->vm_start,
                          vma->vm_page_prot);
    }
#else
    if(pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD3 ||
       pfunc_sub->scaler_type == FAPI_VISCALE_SCALER_OSD4 ){
        ret = remap_pfn_range(vma,
                              vma->vm_start,
                              (uint32_t)pfunc_sub->dma_addr >> PAGE_SHIFT,  /* physical page */
                              vma->vm_end - vma->vm_start,
                              vma->vm_page_prot);
    }
    else{
        ret = remap_pfn_range(vma,
                              vma->vm_start,
                              page_to_phys(pfunc_sub->plane_pg) >> PAGE_SHIFT,  /* physical page */
                              vma->vm_end - vma->vm_start,
                              vma->vm_page_prot);
    }
#endif
    if(ret != 0) {
        FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_mmap] error:%d\n", ret);
        return -EAGAIN;
    }

#else

    FAPEXFDL_VIOUT_SubStrT *pfunc_sub;

    /* Get the context base of this id's FDL. */
    pfunc_sub = (FAPEXFDL_VIOUT_SubStrT *)(file->private_data);

    if( (vma->vm_length > pfunc_sub->plane_size)
     || (vma->vm_offset != 0) ) {
        return FAPEX_VIOUT_ERR_BAD_PARAMETER;
    }

    vma->vm_start = (void *)pfunc_sub->scaler_params.picBase;

#endif /* __LINUX__ */

    return 0;
}

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL 
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLVIOUT_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_open] pCfdlHdl:0x%08x\n",(uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT *)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc_cmn;
        FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_open] pFdlHdl:0x%08x\n",(uint32_t)(pfunc_cmn));

        /* set parameter */
        pfunc_cmn->pCfdlHdl = pCfdlHdl;
        pfunc_cmn->id       = id;
        pfunc_cmn->status   = FAPEX_STATUS_OPENED;
#ifdef __LINUX__
        kref_init(&pfunc_cmn->kref);
#endif
    }
    else {
        pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT *)file->private_data;
#ifdef __LINUX__
        kref_get(&pfunc_cmn->kref);
#endif
    }

    /* increment our usage count for the device */
    pfunc_cmn->used_count++;

    FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_open] fapex_fdlviout was opened : used_counter %d.\n",
                       pfunc_cmn->used_count);
    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
#ifdef __LINUX__
static void fdlviout_delete(struct kref *kref)
{
    FAPEXFDL_VIOUT_CmnStrT  *pfunc_cmn;
    int32_t  ret = 0;

    pfunc_cmn = container_of(kref, FAPEXFDL_VIOUT_CmnStrT, kref);
#else
static int32_t fdlviout_delete(FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn)
{
    int32_t  ret = 0;
#endif

    if(pfunc_cmn->id == FAPEXDEV_VIOUT_CMN_MINOR) {
        if(pfunc_cmn->pHdmiCtxtHdl != NULL) {
#ifdef FAPI_VIOUT_CONNECT

            hdmi_funcStr.VIDEO_Stop(pfunc_cmn->pHdmiCtxtHdl);

            ret = hdmi_funcStr.Close(pfunc_cmn->pHdmiCtxtHdl);
            if(ret != FAPI_OK) {
                FAPEXDebugPrint(2, "[fdlviout_delete] "
                                   "FAPI_HDMI_Close error:%d\n", ret);
                goto fdlviout_delete_end;
            }
            FAPEXDebugPrint(3, "[fdlviout_delete] FAPI_HDMI_Close\n");
#endif /* FAPI_VIOUT_CONNECT */
            pfunc_cmn->pHdmiCtxtHdl = NULL;
        }
        pfunc_cmn->output_id = -1;
    }

fdlviout_delete_end:
#ifdef __LINUX__
    pfunc_cmn->close_result = ret;
#else
    return ret;
#endif
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLVIOUT_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn;
    int32_t  ret = 0;

    pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT *)(file->private_data);

    if(pfunc_cmn != NULL) {
        if(pfunc_cmn->status > FAPEX_STATUS_OPENED) {
            FAPEXFDL_RETVAL( FAPEX_VIOUT_ERR_INVALID_STATUS );
        }
        pfunc_cmn->status = FAPEX_STATUS_TERMINATED;

        /* decrement the count on our device */
        pfunc_cmn->used_count--;
#ifdef __LINUX__
        ret = kref_put(&pfunc_cmn->kref, fdlviout_delete);
        if(ret != 0) {
            ret = pfunc_cmn->close_result;
        }
#else
        ret = fdlviout_delete(pfunc_cmn);
#endif

        FAPEXDebugPrint(3, "[FAPEXFDLVIOUT_release] id %d, used_count %d\n",
                           pfunc_cmn->id, pfunc_cmn->used_count);
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlviout_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLVIOUT_init(void)
{
    int32_t ret;
    int32_t count;
    void    *pCfdlHdl;
    FAPEXFDL_VIOUT_CmnStrT  *pfunc_cmn;
    FAPEXFDL_VIOUT_SubStrT  *pfunc_sub;
    FAPEXCFDL_UtilStrT      utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_VIOUT_SubStrT  *pfunc_sub0;
#endif
#ifdef VIOUT_USE_DMA
    FAPI_DMA_MemoryFillParamsT  *dma_ptr;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLVIOUT_init]\n");

    /* set struct file_operations */
    fapex_fdlviout_fops.owner   = THIS_MODULE;
    fapex_fdlviout_fops.open    = FAPEXFDLVIOUT_open;
//  fapex_fdlviout_fops.read    = FAPEXFDLVIOUT_read;
//  fapex_fdlviout_fops.write   = FAPEXFDLVIOUT_write;
    fapex_fdlviout_fops.ioctl   = FAPEXFDLVIOUT_ioctl;
//  fapex_fdlviout_fops.poll    = FAPEXFDLVIOUT_poll;
    fapex_fdlviout_fops.mmap    = FAPEXFDLVIOUT_mmap;
    fapex_fdlviout_fops.release = FAPEXFDLVIOUT_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT *)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_VIOUT_CmnStrT));
    if(pfunc_cmn == NULL) {
        return FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
    }
    pfunc_sub = (FAPEXFDL_VIOUT_SubStrT *)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_VIOUT_SubStrT)*FAPEXDEV_VIOUT_SUB_DESCRIPTOR_NUM);
    if(pfunc_sub == NULL) {
        FAPI_SYS_FREE(pfunc_cmn);
        return FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
    }
    pfunc_sub0 = pfunc_sub;
#else
    pfunc_cmn = FAPIGLD_FDLVIOUTCmnCtx;
    pfunc_sub = FAPIGLD_FDLVIOUTSubCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc_cmn, sizeof(FAPEXFDL_VIOUT_CmnStrT));
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc_sub, sizeof(FAPEXFDL_VIOUT_SubStrT)*FAPEXDEV_VIOUT_SUB_DESCRIPTOR_NUM);

    pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();

    pfunc_cmn->pCfdlHdl  = pCfdlHdl;
//  pfunc_cmn->status    = FAPEX_STATUS_LOADED;
    pfunc_cmn->output_id = -1;
    pfunc_cmn->cmn_sem   = FAPI_SYS_CREATE_SEMAPHORE(1);
#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    pfunc_cmn->used_bytes_osd1 = 0;
    pfunc_cmn->used_bytes_osd2 = 0;
#endif
    if(pfunc_cmn->cmn_sem == (FAPI_SYS_SemaphoreT)0) {
        ret = FAPEX_VIOUT_ERR_SEMAPHORE_CREATE;
        goto FAPEXFDLVIOUT_init_error;
    }

    /* Register this id's FDL device to CFDL. */
    ret = FAPEXCFDL_RegisterFdl(&fapex_fdlviout_fops, pfunc_cmn, &utility, FAPEXDEV_VIOUT_CMN_MINOR);
    if(ret) {
        goto FAPEXFDLVIOUT_init_error;
    }

#ifdef VIOUT_USE_DMA
    dma_ptr = (FAPI_DMA_MemoryFillParamsT*)FAPI_SYS_MALLOC(sizeof(FAPI_DMA_MemoryFillParamsT) * FAPEXDEV_VIOUT_SUB_DESCRIPTOR_NUM);
    if(dma_ptr == NULL) {
        ret = FAPEX_VIOUT_ERR_OUT_OF_MEMORY;
        goto FAPEXFDLVIOUT_init_error;
    }
#endif

    for(count=FAPEXDEV_VIOUT_SUB_MINOR_FIRST; count<=FAPEXDEV_VIOUT_SUB_MINOR_LAST; count++) {
        pfunc_sub->pCfdlHdl = pCfdlHdl;
        pfunc_sub->pCmnHdl  = pfunc_cmn;
//      pfunc_sub->status   = FAPEX_STATUS_LOADED;

#ifdef VIOUT_USE_DMA
        pfunc_sub->dma_params = dma_ptr ++;
#endif
        pfunc_sub->scaler_type = FAPI_VISCALE_UNKNOWN_SCALER;

        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlviout_fops, pfunc_sub, &utility, count);
        if(ret) {
            goto FAPEXFDLVIOUT_init_error;
        }
        pfunc_sub ++;
    }

#ifdef FAPI_VIOUT_CONNECT
    FAPEXDebugPrint(1, "call FAPEXCFG_VISCALE_Init\n");

    /* call FAPI (initialize) */
    ret = FAPEXCFG_VISCALE_Init();
#else
    ret = 0;
#endif /* FAPI_VIOUT_CONNECT */

#ifdef VIOUT_USE_DMA
    fdlviout_DmaSemaphore = FAPI_SYS_CREATE_SEMAPHORE(0);
    if(fdlviout_DmaSemaphore == (FAPI_SYS_SemaphoreT)0) {
        ret = FAPEX_VIOUT_ERR_SEMAPHORE_CREATE;
        goto FAPEXFDLVIOUT_init_error;
    }
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLVIOUT_init] fapex_fdlviout was loaded. %d\n", ret);

    return ret;

FAPEXFDLVIOUT_init_error:
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPI_SYS_FREE(pfunc_cmn);
    FAPI_SYS_FREE(pfunc_sub0);
#endif
    FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command, 
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLVIOUT_cleanup(void)
{
    int32_t count;
    void    *pCfdlHdl;
    FAPEXFDL_VIOUT_CmnStrT *pfunc_cmn;
    FAPEXFDL_VIOUT_SubStrT *pfunc_sub;

    FAPEXDebugPrint(1, "[FAPEXFDLVIOUT_cleanup]\n");

#ifdef VIOUT_USE_DMA
    FAPI_SYS_DESTROY_SEMAPHORE(fdlviout_DmaSemaphore);
    fdlviout_DmaSemaphore = (FAPI_SYS_SemaphoreT)0;
#endif

#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI API (cleanup) */
    FAPEXCFG_VISCALE_Cleanup();

    FAPEXDebugPrint(1, "end FAPEXCFG_VISCALE_Cleanup\n");
#endif /* FAPI_VIOUT_CONNECT */

    /* cleanup sequence (common this FDL) */
    pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();

    /* relase the FDL context */
    pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_VIOUT_CMN_MINOR);
    if(pfunc_cmn != NULL) {
        FAPI_SYS_DESTROY_SEMAPHORE(pfunc_cmn->cmn_sem);
        pfunc_cmn->cmn_sem = (FAPI_SYS_SemaphoreT)0;

#ifdef ALLOCATE_DYNAMIC_MEMORY
        FAPI_SYS_FREE(pfunc_cmn);
#endif
    }

    pfunc_sub = (FAPEXFDL_VIOUT_SubStrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_VIOUT_SUB_MINOR_FIRST);
    if(pfunc_sub != NULL) {
#ifdef VIOUT_USE_DMA
        FAPI_SYS_FREE(pfunc_sub->dma_params);
#endif

#ifdef ALLOCATE_DYNAMIC_MEMORY
        FAPI_SYS_FREE(pfunc_sub);
#endif
    }

    /* Unregister this id's FDL device to CFDL. */
    FAPEXCFDL_UnregisterFdl(FAPEXDEV_VIOUT_CMN_MINOR);

    for(count=FAPEXDEV_VIOUT_SUB_MINOR_FIRST; count<=FAPEXDEV_VIOUT_SUB_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLVIOUT_cleanup] fapex_fdlviout was unloaded.\n");
    return;
}

#ifdef COMBINATION_WITH_FB
/*!
*******************************************************************************
**
** \brief  Setup ... of the FDL module.
**         This function is
**
** \param  handle
**
******************************************************************************/
int32_t FAPEXFDLVIOUT_SetLinuxFB(void *handle)
{
    void                    *pCfdlHdl;
    FAPEXFDL_VIOUT_CmnStrT  *pfunc_cmn;
    FAPEXFDL_VIENC_SubStrT  *pfunc_vienc;
    int32_t  ret = 0;

    pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();

    /* get the FDL context */
    pfunc_cmn = (FAPEXFDL_VIOUT_CmnStrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_VIOUT_CMN_MINOR);
    if(pfunc_cmn == NULL) {
        return FAPEX_VIOUT_ERR_INVALID_STATUS;
    }

    if(handle == NULL) {
        /*---------- LOCK ----------*/
        FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

        pfunc_cmn->pOsd1CtxHdl = NULL;

        /*--------- UNLOCK ---------*/
        FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);

        return 0;
    }

    pfunc_vienc = (FAPEXFDL_VIENC_SubStrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl,
                                                               pfunc_cmn->output_id);
    if(pfunc_vienc == NULL) {
        return FAPEX_VIOUT_ERR_INVALID_STATUS;
    }
#ifdef FAPI_VIOUT_CONNECT
    /* call FAPI (set encoder) */
    ret = viscale_funcStr.SetScalerToHdEncoder(handle, pfunc_vienc->pFAPICtxHdl);

    if(ret < 0) {
        FAPEXDebugPrint(2, "[FAPEXFDLVIOUT_SetLinuxFB] "
                           "FAPI_VISCALE_SetScalerToHdEncoder error:%d\n", ret);
    }
    else
#endif
    if(ret == 0) {
        /*---------- LOCK ----------*/
        FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

        pfunc_cmn->pOsd1CtxHdl = handle;

        /*--------- UNLOCK ---------*/
        FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);
    }

    return ret;
}
#endif /* COMBINATION_WITH_FB */

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLVIOUT_init);
module_exit(FAPEXFDLVIOUT_cleanup);

#ifdef COMBINATION_WITH_FB
EXPORT_SYMBOL(FAPEXFDLVIOUT_SetLinuxFB);
#endif

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlviout object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
