/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 * 
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL Video Output header file (private)
*/

#ifndef _FAPEX_FDLVIOUT_PRIV_H
#define _FAPEX_FDLVIOUT_PRIV_H

#include "fapi/drv_viscale.h"
#include "fapi/drv_hdmi.h"

#ifdef VIOUT_USE_DMA
#include "fapi/drv_dma.h"
#endif

/*! \brief HDMI Video Aspect Ratio information flags */
#define FAPEXFDL_VIOUT_HDMI_SVD_NONE               0
#define FAPEXFDL_VIOUT_HDMI_SVD_720x576x50I_16_9   (1 << 22)
#define FAPEXFDL_VIOUT_HDMI_SVD_720x576x50P_16_9   (1 << 18)
#define FAPEXFDL_VIOUT_HDMI_SVD_720x480x5994I_16_9 (1 <<  7)
#define FAPEXFDL_VIOUT_HDMI_SVD_720x480x5994P_16_9 (1 <<  3)

/*!
*************************************************************************
** \brief FDL Viout common context structure
*************************************************************************/
typedef struct {
    int32_t     id;                 //!< id
    int32_t     used_count;         //!< used count
#ifdef __LINUX__
    struct kref kref;               //!< kref struct
    int32_t     close_result;       //!< return code at closing
#endif

    void        *pCfdlHdl;          //!< The pointer to the CFDL context
    int32_t     status;             //!< VIOUT block status

    volatile uint32_t   enable_channel;     //!< enable VIOUT channel flag
    volatile uint32_t   running_channel;    //!< running channel flag

    uint32_t    encoder_type;       //!< encoder type (see #FAPI_VISCALE_EncoderEnumT)

    void        *pHdmiCtxtHdl;          //!< HDMI video out driver's context
    FAPI_HDMI_VideoParamsT  hdmi_param; //!< HDMI video parameter

    int32_t     output_id;          //!< output Device minor number
    int32_t     system_mode;        //!< system mode (see #FAPI_VISCALE_SystemModeEnumT)

    FAPI_SYS_SemaphoreT  cmn_sem;   //!< Semaphore for common parameter control

#ifndef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    uint32_t    used_bytes_osd1;    //!< OSD1 used bytes
    uint32_t    used_bytes_osd2;    //!< OSD2 used bytes
#endif

#ifdef COMBINATION_WITH_FB
    void        *pOsd1CtxHdl;       //!< FAPI OSD1 driver's context
#endif
} FAPEXFDL_VIOUT_CmnStrT;

/*!
*************************************************************************
** \brief FDL Viout sub context structure
*************************************************************************/
typedef struct {
    int32_t     id;              //!< id
    int32_t     used_count;      //!< used count
#ifdef __LINUX__
    struct kref kref;            //!< kref struct
    int32_t     close_result;    //!< return code at closing
#endif

    void        *pCfdlHdl;       //!< The pointer to the CFDL context
    int32_t     status;          //!< VIOUT channel status

    void        *pCmnHdl;        //!< the pointer to the VIOUT common context
    void        *pFAPICtxHdl;    //!< FAPI driver's context
    void        *pViencHdl;      //!< FDL VIENC handler

    int32_t                     scaler_type;     //!< scaler type (see #FAPI_VISCALE_ScalerEnumT)
    FAPI_VISCALE_ScalerParamsT  scaler_params;   //!< Scaling params for this scaler.

    int32_t     fading_mode;        //!< fading mode (see #FAPI_VISCALE_VideoFadingModeEnumT)

    uint32_t    fill_colour;        //!< fill colour value
    uint32_t    bpp;                //!< bits per pixel

    uint32_t    max_hres;           //!< max horizontal resolution
    uint32_t    max_vres;           //!< max vertical resolution
    uint32_t    plane_size;         //!< used size of plane area
    uint32_t    plane_max_size;     //!< max size of plane area
    uint8_t     *plane_ptr;         //!< pointer for plane buffer area
#ifdef __LINUX__
//#ifdef VIOUT_ALLOCATE_OSD_CURSOR_PLANE
    uint32_t    plane_size_order;   //!< order of plane_max_size
    struct page *plane_pg;          //!< plane area page
//#endif
    uint32_t    virt_addr;
    dma_addr_t  dma_addr;
#endif

#ifdef VIOUT_USE_DMA
    FAPI_DMA_MemoryFillParamsT  *dma_params;    //!< pointer for DMA memory fill parameter
#endif

} FAPEXFDL_VIOUT_SubStrT;

#endif /* !_FAPEX_FDLVIOUT_PRIV_H */
