/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 * 
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 fapi common header file
*/
#ifndef _FAPEX_H
#define _FAPEX_H

/**
 header files 
**/
#ifndef __LINUX__

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>

#define FAPEXFDL_RETVAL(val) {return val;}

#else /* !__LINUX__ */

#if !defined(__KERNEL__)
#define __KERNEL__
#endif

#ifdef BUILD_AS_MODULE
#if !defined(MODULE)
#define MODULE
#endif
#endif /* BUILD_AS_MODULE */

#include "fapex_stdhdr.h"

#define FAPEXFDL_RETVAL(val) {return ((val == 0) ? 0 : -ENODEV);}

#endif /* !__LINUX__ */

#include "fapex_types.h"
#include "fapex_cfdl.h"
#include "fapi/sys_services.h"

/**
 definitions 
**/
/* function type */
#define FAPEX_FDLTYPE_1          (0)
#define FAPEX_FDLTYPE_2          (1)

/*** set major_no, minor_no ***/
/* FAPI mejor_no */
#define FAPEX_MAJOR_DEFAULT       (244)

/* minimum minor_no */
#define FAPEX_MINIMUM_MINOR       (0)
/* maximum minor_no */
#define FAPEX_MAXIMUM_MINOR       (255)
#define FAPEX_DESCRIPTOR_NUM      (FAPEX_MAXIMUM_MINOR - FAPEX_MINIMUM_MINOR + 1)

/* CFDL : (minor_no)=255 */
#define FAPEX_CFDL_MINOR          (255)

#if (DEBUG > 0)
 #ifndef __LINUX__
  #include "fapi/drv_timer.h"
  #define FAPEXDebugPrint(level,...)  \
         if( (DEBUG) >= (level) ) {   \
             if( (DEBUG) == 1 ) {     \
                 printf("%lld ", FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC)); \
             }                        \
             printf(__VA_ARGS__);     \
         }
 #else /* !__LINUX__ */
  #define FAPEXDebugPrint(level,fmt,...)          \
         if( (DEBUG) >= (level) ) {               \
             printk(KERN_INFO fmt,##__VA_ARGS__); \
         }
 #endif /* !__LINUX__ */
#else
 #define FAPEXDebugPrint(level,...)
#endif /* DEBUG */

/**
 RTOS specific
**/
#ifndef __LINUX__

/*!
*******************************************************************************
** \brief file structure (for RTOS)<br>
**        On Linux, it is pre-defined by kernel.
******************************************************************************/
struct file {
    void *private_data; /*!< private data (the pointer to this id's FDL context) */
    int32_t id;         /*!< id */
};

#endif /* !__LINUX__ */

/*!
*******************************************************************************
** \brief event structure
******************************************************************************/
typedef struct {
    /*!
        initialized flag
    */
    uint32_t                    initialized;
    /*!
        event data
    */
    FAPEXCFDLSAF_EventT         ivt;
    /*!
        lock for event data
    */
    FAPEXCFDLSAF_SpnLockStrT    lock;
    /*!
        wait bit pattern
    */
    uint32_t                    wai_bitptn;
    /*!
        event bit pattern
    */
    volatile uint32_t           evt_bitptn;
    /*!
        event waiting mode<br>
        - (FAPEXCFDLSAF_WAITEVENT_OR | FAPEXCFDLSAF_WAITEVENT_SYNC)
        - (FAPEXCFDLSAF_WAITEVENT_OR | FAPEXCFDLSAF_WAITEVENT_ASYNC)
        - (FAPEXCFDLSAF_WAITEVENT_AND | FAPEXCFDLSAF_WAITEVENT_SYNC)
        - (FAPEXCFDLSAF_WAITEVENT_AND | FAPEXCFDLSAF_WAITEVENT_ASYNC)
        - (FAPEXCFDLSAF_WAITEVENT_QUEUE | FAPEXCFDLSAF_WAITEVENT_SYNC)
        - (FAPEXCFDLSAF_WAITEVENT_QUEUE | FAPEXCFDLSAF_WAITEVENT_ASYNC)
    */
    uint32_t                    mode;
    /*!
        read index of event queue
    */
    volatile uint32_t           read_idx;
    /*!
        write index of event queue
    */
    volatile uint32_t           write_idx;
    /*!
        timeout [msec] (0 : without timeout)
    */
    uint32_t                    tout;

} FAPEXCFDLSAF_EventStrT;

#define FAPEXCFDLSAF_WAITEVENT_OR           (0x00)  //!< waiting mode : event flag (OR type)
#define FAPEXCFDLSAF_WAITEVENT_AND          (0x01)  //!< waiting mode : event flag (AND type)
#define FAPEXCFDLSAF_WAITEVENT_QUEUE        (0x02)  //!< waiting mode : buffer queue
#define FAPEXCFDLSAF_WAITEVENT_SYNC         (0x00)  //!< synchronous mode
#define FAPEXCFDLSAF_WAITEVENT_ASYNC        (0x10)  //!< asynchronous mode
#define FAPEXCFDLSAF_WAITEVENT_QUEUE_WRITE  (0x02)  //!< increment of the write index for buffer queue
#define FAPEXCFDLSAF_WAITEVENT_QUEUE_READ   (0x22)  //!< increment of the read index for buffer queue

/** io memory **/
#define FAPIDRV_MAX_REGIONNUM (2)

#include "fapex_cfdlsaf.h"
#endif /* !_FAPEX_H */
