/*!
*************************************************************************
**
** \file    fapex/abst_lib.h
**
** \brief   abstraction library header file
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIB_H
#define _FAPEX_ABST_LIB_H

/*
*************************************************************************
** device minor numbers
*************************************************************************/
/* minimum minor_no */
#define FAPEX_MINIMUM_MINOR       (0)
/* maximum minor_no */
#define FAPEX_MAXIMUM_MINOR       (255)

#define FAPEX_DESCRIPTOR_NUM      (FAPEX_MAXIMUM_MINOR - FAPEX_MINIMUM_MINOR + 1)

/* DEV_TSD : (minor_no)=(0..131) */
#define FAPEXDEV_TSD_MINOR_FIRST            (0)
#define FAPEXDEV_TSD_MINOR_LAST             (131)   /* 4 + 4x32 - 1 */
#define FAPEXDEV_TSD_DESCRIPTOR_NUM         (FAPEXDEV_TSD_MINOR_LAST - FAPEXDEV_TSD_MINOR_FIRST + 1)
#define FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM  (32)
#define FAPEXDEV_TSD_CMN_DESCRIPTOR_NUM     (4)
#define FAPEXDEV_TSD_SUB_DESCRIPTOR_NUM     (FAPEXDEV_TSD_CMN_DESCRIPTOR_NUM * FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM)
#define FAPEXDEV_TSD0_CMN_MINOR             (FAPEXDEV_TSD_MINOR_FIRST + 0)
#define FAPEXDEV_TSD1_CMN_MINOR             (FAPEXDEV_TSD_MINOR_FIRST + 1)
#define FAPEXDEV_TSD2_CMN_MINOR             (FAPEXDEV_TSD_MINOR_FIRST + 2)
#define FAPEXDEV_TSD3_CMN_MINOR             (FAPEXDEV_TSD_MINOR_FIRST + 3)
#define FAPEXDEV_TSD_CMN_MINOR_FIRST        (FAPEXDEV_TSD0_CMN_MINOR)
#define FAPEXDEV_TSD_CMN_MINOR_LAST         (FAPEXDEV_TSD3_CMN_MINOR)
#define FAPEXDEV_TSD_SUB_MINOR_FIRST        (FAPEXDEV_TSD_MINOR_FIRST + FAPEXDEV_TSD_CMN_DESCRIPTOR_NUM)
#define FAPEXDEV_TSD_SUB_MINOR_LAST         (FAPEXDEV_TSD_MINOR_LAST)
#define FAPEXDEV_TSD0_SUB_MINOR_FIRST       (FAPEXDEV_TSD_SUB_MINOR_FIRST)
#define FAPEXDEV_TSD1_SUB_MINOR_FIRST       (FAPEXDEV_TSD0_SUB_MINOR_FIRST + FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM)
#define FAPEXDEV_TSD2_SUB_MINOR_FIRST       (FAPEXDEV_TSD1_SUB_MINOR_FIRST + FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM)
#define FAPEXDEV_TSD3_SUB_MINOR_FIRST       (FAPEXDEV_TSD2_SUB_MINOR_FIRST + FAPEXDEV_TSD_CH_SUB_DESCRIPTOR_NUM)

/* DEV_VIDEC : (minor_no)=(132..133) */
#define FAPEXDEV_VIDEC_MINOR_FIRST          (132)
#define FAPEXDEV_VIDEC_MINOR_LAST           (133)
#define FAPEXDEV_VIDEC_DESCRIPTOR_NUM       (FAPEXDEV_VIDEC_MINOR_LAST - FAPEXDEV_VIDEC_MINOR_FIRST + 1)

/* DEV_VIOUT : (minor_no)=(134..141) */
#define FAPEXDEV_VIOUT_MINOR_FIRST          (134)
#define FAPEXDEV_VIOUT_MINOR_LAST           (141)
#define FAPEXDEV_VIOUT_DESCRIPTOR_NUM       (FAPEXDEV_VIOUT_MINOR_LAST - FAPEXDEV_VIOUT_MINOR_FIRST + 1)
#define FAPEXDEV_VIOUT_CMN_DESCRIPTOR_NUM   (1)
#define FAPEXDEV_VIOUT_SUB_DESCRIPTOR_NUM   (7)  /* VIDEO, OSD1, OSD2, OSD3,OSD4,SD, CP */
#define FAPEXDEV_VIOUT_CMN_MINOR            (FAPEXDEV_VIOUT_MINOR_FIRST)
#define FAPEXDEV_VIOUT_SUB_MINOR_FIRST      (FAPEXDEV_VIOUT_MINOR_FIRST + FAPEXDEV_VIOUT_CMN_DESCRIPTOR_NUM)
#define FAPEXDEV_VIOUT_SUB_MINOR_LAST       (FAPEXDEV_VIOUT_MINOR_LAST)

/* DEV_VIENC : (minor_no)=(142..144) */
#define FAPEXDEV_VIENC_MINOR_FIRST          (142)
#define FAPEXDEV_VIENC_MINOR_LAST           (144)
#define FAPEXDEV_VIENC_DESCRIPTOR_NUM       (FAPEXDEV_VIENC_MINOR_LAST - FAPEXDEV_VIENC_MINOR_FIRST + 1)
#define FAPEXDEV_VIENC_CMN_DESCRIPTOR_NUM   (1)
#define FAPEXDEV_VIENC_SUB_DESCRIPTOR_NUM   (2)  /* HD, SD */
#define FAPEXDEV_VIENC_CMN_MINOR            (FAPEXDEV_VIENC_MINOR_FIRST)
#define FAPEXDEV_VIENC_SUB_MINOR_FIRST      (FAPEXDEV_VIENC_MINOR_FIRST + FAPEXDEV_VIENC_CMN_DESCRIPTOR_NUM)
#define FAPEXDEV_VIENC_SUB_MINOR_LAST       (FAPEXDEV_VIENC_MINOR_LAST)

/* DEV_AUDEC : (minor_no)=(145..151) */
#define FAPEXDEV_AUDEC_MINOR_FIRST          (145)
#define FAPEXDEV_AUDEC_MINOR_LAST           (151)
#define FAPEXDEV_AUDEC_DESCRIPTOR_NUM       (FAPEXDEV_AUDEC_MINOR_LAST - FAPEXDEV_AUDEC_MINOR_FIRST + 1)

/* DEV_AUOUT : (minor_no)=(152..163) */
#define FAPEXDEV_AUOUT_MINOR_FIRST          (152)
#define FAPEXDEV_AUOUT_PCM_MINOR_FIRST (156)
#define FAPEXDEV_AUOUT_PCM_MINOR_LAST (163)
#define FAPEXDEV_AUOUT_MINOR_LAST           FAPEXDEV_AUOUT_PCM_MINOR_LAST
#define FAPEXDEV_AUOUT_DESCRIPTOR_NUM       (FAPEXDEV_AUOUT_MINOR_LAST - FAPEXDEV_AUOUT_MINOR_FIRST + 1)

/* DEV_HDMI : (minor_no)=(164) */
#define FAPEXDEV_HDMI_MINOR_FIRST           (164)
#define FAPEXDEV_HDMI_MINOR_LAST            (164)
#define FAPEXDEV_HDMI_DESCRIPTOR_NUM        (FAPEXDEV_HDMI_MINOR_LAST - FAPEXDEV_HDMI_MINOR_FIRST + 1)

/* DEV_HDMICEC : (minor_no)=(165) */
#define FAPEXDEV_HDMICEC_MINOR_FIRST        (165)
#define FAPEXDEV_HDMICEC_MINOR_LAST         (165)
#define FAPEXDEV_HDMICEC_DESCRIPTOR_NUM     (FAPEXDEV_HDMICEC_MINOR_LAST - FAPEXDEV_HDMICEC_MINOR_FIRST + 1)

/* DEV_TTX : (minor_no)=(166..167) */
#define FAPEXDEV_TTX_MINOR_FIRST            (166)
#define FAPEXDEV_TTX_MINOR_LAST             (167)
#define FAPEXDEV_TTX_DESCRIPTOR_NUM         (FAPEXDEV_TTX_MINOR_LAST - FAPEXDEV_TTX_MINOR_FIRST + 1)

/* DEV_IR : (minor_no)=(168..169) */
#define FAPEXDEV_IR_MINOR_FIRST             (168)
#define FAPEXDEV_IR_MINOR_LAST              (169)
#define FAPEXDEV_IR_DESCRIPTOR_NUM          (FAPEXDEV_IR_MINOR_LAST - FAPEXDEV_IR_MINOR_FIRST + 1)

/* DEV_FPC : (minor_no)=(170..171) */
#define FAPEXDEV_FPC_MINOR_FIRST            (170)
#define FAPEXDEV_FPC_MINOR_LAST             (171)
#define FAPEXDEV_FPC_DESCRIPTOR_NUM         (FAPEXDEV_FPC_MINOR_LAST - FAPEXDEV_FPC_MINOR_FIRST + 1)

/* DEV_SSP : (minor_no)=(172..173) */
#define FAPEXDEV_SSP_MINOR_FIRST            (172)
#define FAPEXDEV_SSP_MINOR_LAST             (173)
#define FAPEXDEV_SSP_DESCRIPTOR_NUM         (FAPEXDEV_SSP_MINOR_LAST - FAPEXDEV_SSP_MINOR_FIRST + 1)

/* DEV_I2C : (minor_no)=(174..185) */
#define FAPEXDEV_I2C_MINOR_FIRST            (174)
#define FAPEXDEV_I2C_MINOR_LAST             (185)
#define FAPEXDEV_I2C_DESCRIPTOR_NUM         (FAPEXDEV_I2C_MINOR_LAST - FAPEXDEV_I2C_MINOR_FIRST + 1)

/* DEV_GPIO : (minor_no)=(186..195) */
#define FAPEXDEV_GPIO_MINOR_FIRST           (186)
#define FAPEXDEV_GPIO_MINOR_LAST            (195)
#define FAPEXDEV_GPIO_DESCRIPTOR_NUM        (FAPEXDEV_GPIO_MINOR_LAST - FAPEXDEV_GPIO_MINOR_FIRST + 1)

/* DEV_ICC : (minor_no)=(196..198) */
#define FAPEXDEV_ICC_MINOR_FIRST            (196)
#define FAPEXDEV_ICC_MINOR_LAST             (198)
#define FAPEXDEV_ICC_DESCRIPTOR_NUM         (FAPEXDEV_ICC_MINOR_LAST - FAPEXDEV_ICC_MINOR_FIRST + 1)
#define FAPEXDEV_ICC_CMN_DESCRIPTOR_NUM     (1)
#define FAPEXDEV_ICC_SUB_DESCRIPTOR_NUM     (2)  /* ICC0, ICC1 */
#define FAPEXDEV_ICC_CMN_MINOR              (FAPEXDEV_ICC_MINOR_FIRST)
#define FAPEXDEV_ICC_SUB_MINOR_FIRST        (FAPEXDEV_ICC_MINOR_FIRST + FAPEXDEV_ICC_CMN_DESCRIPTOR_NUM)
#define FAPEXDEV_ICC_SUB_MINOR_LAST         (FAPEXDEV_ICC_MINOR_LAST)

/* DEV_CLKPWR : (minor_no)=(199) */
#define FAPEXDEV_CLKPWR_MINOR_FIRST         (199)
#define FAPEXDEV_CLKPWR_MINOR_LAST          (199)
#define FAPEXDEV_CLKPWR_DESCRIPTOR_NUM      (FAPEXDEV_CLKPWR_MINOR_LAST - FAPEXDEV_CLKPWR_MINOR_FIRST + 1)

/* DEV_BOARDIFC : (minor_no)=(200) */
#define FAPEXDEV_BOARDIFC_MINOR_FIRST         (200)
#define FAPEXDEV_BOARDIFC_MINOR_LAST          (200)
#define FAPEXDEV_BOARDIFC_DESCRIPTOR_NUM      (FAPEXDEV_BOARDIFC_MINOR_LAST - FAPEXDEV_BOARDIFC_MINOR_FIRST + 1)

/* DEV_FLASH : (minor_no)=(201) */
#define FAPEXDEV_FLASH_MINOR_FIRST         (201)
#define FAPEXDEV_FLASH_MINOR_LAST          (201)
#define FAPEXDEV_FLASH_DESCRIPTOR_NUM      (FAPEXDEV_FLASH_MINOR_LAST - FAPEXDEV_FLASH_MINOR_FIRST + 1)

/* DEV_FLASH : (minor_no)=(202) */
#define FAPEXDEV_BLTE_MINOR_FIRST         (202)
#define FAPEXDEV_BLTE_MINOR_LAST          (202)
#define FAPEXDEV_BLTE_DESCRIPTOR_NUM      (FAPEXDEV_BLTE_MINOR_LAST - FAPEXDEV_BLTE_MINOR_FIRST + 1)

/* DEV_ADC : (minor_no)=(203) */
#define FAPEXDEV_ADC_MINOR_FIRST         (203)
#define FAPEXDEV_ADC_MINOR_LAST          (203)
#define FAPEXDEV_ADC_DESCRIPTOR_NUM      (FAPEXDEV_ADC_MINOR_LAST - FAPEXDEV_ADC_MINOR_FIRST + 1)

/* DEV_PV : (minor_no)=(204) */
#define FAPEXDEV_PV_MINOR_FIRST         (204)
#define FAPEXDEV_PV_MINOR_LAST          (204)
#define FAPEXDEV_PV_DESCRIPTOR_NUM      (FAPEXDEV_PV_MINOR_LAST - FAPEXDEV_PV_MINOR_FIRST + 1)

/* DEV_CEC : (minor_no)=(205) */
#define FAPEXDEV_CEC_MINOR_FIRST           (205)
#define FAPEXDEV_CEC_MINOR_LAST            (205)
#define FAPEXDEV_CEC_DESCRIPTOR_NUM        (FAPEXDEV_CEC_MINOR_LAST - FAPEXDEV_CEC_MINOR_FIRST + 1)

/*
*************************************************************************
** select mode
*************************************************************************/
#define FAPEX_SELECT_MODE_READ  (0x1)   //!< read mode
#define FAPEX_SELECT_MODE_WRITE (0x2)   //!< write mode
#define FAPEX_SELECT_MODE_EXPT  (0x4)   //!< exception mode

#ifndef _LINUX_FAPI
/*
*************************************************************************
** return value of select
*************************************************************************/
#define POLLIN      0x0001
#define POLLPRI     0x0002
#define POLLOUT     0x0004
#define POLLERR     0x0008
#define POLLHUP     0x0010
#define POLLNVAL    0x0020
#define POLLRDNORM  0x0040
#define POLLRDBAND  0x0080
#define POLLWRNORM  0x0100
#define POLLWRBAND  0x0200
#define POLLMSG     0x0400
#define POLLREMOVE  0x1000
#define POLLRDHUP   0x2000

#ifndef __LINUX__
/*
*************************************************************************
** open flag
*************************************************************************/
#define O_RDONLY    0x0
#define O_WRONLY    0x1
#define O_RDWR      0x2
#define O_SYNC      0x4

/*
*************************************************************************
** mmap prot flag
*************************************************************************/
#define PROT_READ   0x1
#define PROT_WRITE  0x2

/*
*************************************************************************
** mmap map flag
*************************************************************************/
#define MAP_SHARED  0x1
#define MAP_PRIVATE 0x2

#endif /* !__LINUX__ */
#endif /* !_LINUX_FAPI */

/*
*************************************************************************
** ioctl common command
*************************************************************************/
#define FAPEX_DEV_TYPE                  0xFF
#define FAPEX_DEV1_TYPE                 0xFE /* due to 0xff is occupied */

/* common command value */
#define FAPEX_IOCCOM_OPEN               _IO (FAPEX_DEV_TYPE, 0x00)
#define FAPEX_IOCCOM_START              _IO (FAPEX_DEV_TYPE, 0x01)
#define FAPEX_IOCCOM_STOP               _IO (FAPEX_DEV_TYPE, 0x02)
#define FAPEX_IOCCOM_PAUSE              _IO (FAPEX_DEV_TYPE, 0x03)
#define FAPEX_IOCCOM_RESET              _IOW(FAPEX_DEV_TYPE, 0x04, uint32_t)
#define FAPEX_IOCCOM_FORCE_RESET        _IO (FAPEX_DEV_TYPE, 0x05)
#define FAPEX_IOCCOM_CLOSE_SUBID        _IO (FAPEX_DEV_TYPE, 0x06)

/* command number for each modules */
#define FAPEX_IOCCOM_NB_AUDEC           0x08
#define FAPEX_IOCCOM_NB_AUOUT           0x10
#define FAPEX_IOCCOM_NB_CLKPWR          0x20
#define FAPEX_IOCCOM_NB_FPC             0x30
#define FAPEX_IOCCOM_NB_GPIO            0x40
#define FAPEX_IOCCOM_NB_HDMI            0x48
#define FAPEX_IOCCOM_NB_HDMICEC         0x50
#define FAPEX_IOCCOM_NB_CEC             0x52
#define FAPEX_IOCCOM_NB_I2C             0x60
#define FAPEX_IOCCOM_NB_ICC             0x70
#define FAPEX_IOCCOM_NB_IR              0x80
#define FAPEX_IOCCOM_NB_TSD             0x90
#define FAPEX_IOCCOM_NB_TTX             0xB0
#define FAPEX_IOCCOM_NB_VIDEC           0xB8
#define FAPEX_IOCCOM_NB_VIENC           0xD0
#define FAPEX_IOCCOM_NB_VIOUT           0xD8
#define FAPEX_IOCCOM_NB_BOARDIFC        0xE0
#define FAPEX_IOCCOM_NB_FLASH           0xF0

/* FAPEX_DEV1_TYPE */
#define FAPEX_IOCCOM_NB_BLTE            0x08
#define FAPEX_IOCCOM_NB_SSP             0x18
#define FAPEX_IOCCOM_NB_ADC             0x20
#define FAPEX_IOCCOM_NB_PV              0x28

#define FAPEX_IOCCOM_NB_MAX             0xFF
#define FAPEX_IOCCOM_NB_SUBID_ICC       (FAPEX_IOCCOM_NB_MAX - 3)
#define FAPEX_IOCCOM_NB_SUBID_TSD       (FAPEX_IOCCOM_NB_MAX - 2)
#define FAPEX_IOCCOM_NB_SUBID_VIENC     (FAPEX_IOCCOM_NB_MAX - 1)
#define FAPEX_IOCCOM_NB_SUBID_VIOUT     (FAPEX_IOCCOM_NB_MAX - 0)
#define FAPEX_IOCCOM_NB_SUBID_NUM       4
#define FAPEX_IOCCOM_NB_SUBID           (FAPEX_IOCCOM_NB_MAX - FAPEX_IOCCOM_NB_SUBID_NUM + 1)

/*
*************************************************************************
** driver status
*************************************************************************/
#define FAPEX_STATUS_TERMINATED   (-1)  //!< terminated status
#define FAPEX_STATUS_LOADED       (0)   //!< loaded status
#define FAPEX_STATUS_OPENED       (1)   //!< opened status
#define FAPEX_STATUS_READY        (2)   //!< ready starts
#define FAPEX_STATUS_WAITING      (3)   //!< waiting status
#define FAPEX_STATUS_RUNNING      (4)   //!< running status


#ifndef _LINUX_FAPI
/*
*************************************************************************
** prototype
*************************************************************************/
int32_t FAPEX_common_init(void);
int32_t FAPEX_common_exit(void);
#endif /* !_LINUX_FAPI */

#endif /* !_FAPEX_ABST_LIB_H */

