/*!
*****************************************************************************
**
** \file        fapex/abst_libaudec_mixer.h
** 
** \version     $Id: abst_libaudec_mixer.h 4190 2009-07-02 16:14:11Z spurke $
**
** \brief       mixer codec specific parameters.
**
** This file contains the declaration of specific parameters for the MIXER codec.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** (C) Copyright 2006-2010 Fujitsu Semiconductor Europe GmbH, All Rights Reserved.
** (C) Copyright 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*****************************************************************************
*/

#ifndef _FAPEX_ABST_LIBAUDEC_PCM_H
    #define _FAPEX_ABST_LIBAUDEC_PCM_H

    #include <fapex/abst_lib.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    /*!
    *************************************************************************
    ** \brief Parameters for the PCM to PCM copier.
    **
    ** This data structure covers all parameters which can be set for the PCM
    ** to PCM copier.
    ** Setting these parameters is optional. If not the codec applies the 
    ** mentioned defaults. 
    **
    ** \sa FAPI_AUDEC_SetCodecParams
    *************************************************************************
    */
    typedef struct {

        /*!
           The audio decoder ID.
           \note This field must be set to FAPEX_AUDEC_CODEC_PCMOUTPUT.
        */
        uint32_t decoder;

        /*!
           The stream's sample frequency.
        */
        uint32_t sampleFreq;

        /*!
           The channel sequence starting from zero.
           -1 if channel is not assigned.
        */
        int8_t channelFormatArr[8]; // TODO: define macro in abst_libauout.h, see drv_auout.h

        /*!
           The bits per sample.
           This field provides the sample size in bits. -1 if unknown.
        */
        int32_t sampleBits;
        
        /*!
           The number of channels provided by the stream. #FAPI_AUOUT_CHANNEL_UNDEF if unknown.
           note: maximum two channels are supported at the moment.
        */
        int32_t channels;
        
        /*!
           The flag indicating if wave header is attached in file.
           - 0: wave header not attached
           - 1: wave header attached
           (default: 1).
        */
        int8_t  ifWavFormat;
        
        /*!
           The flag indicating if the pcm copier outputs the certain pcm data in
           the buffer in a loop, the reader buffer pointer resets back to the starting
           position after a certain samples are readed.
           - 0: normal mode
           - 1: loop mode
           (default: 0).
        */
        int8_t loopMode;
        
        /*!
           The number of bytes to output in loop mode.
           This value is only valid when loopMode is one.
           This value has to be smaller than the total buffer size.
        */
        uint32_t loopSizeInByte;
        
    } FAPEX_AUDEC_PcmParamStrT;

#endif
