/*!
*************************************************************************
**
** \file    fapex_audec_func.h
**
** \brief   function pointer definiton header file 
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
** 
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_AUDEC_FUNC_H
#define _FAPEX_AUDEC_FUNC_H

#include "fapi/drv_audec.h"

typedef struct {
    FAPI_AUDEC_CapabilitiesT (*GetCapabilities)( void );
    int32_t                  (*Init)( void );
    void                     (*Exit)( void );
    FAPI_SYS_HandleT         (*Open)( const FAPI_AUDEC_OpenParamsT* paramsPtr, int32_t* errorCodePtr );
    int32_t                  (*Close)( FAPI_SYS_HandleT handle );
    int32_t                  (*Start)( FAPI_SYS_HandleT handle );
    int32_t                  (*Stop)( FAPI_SYS_HandleT handle );
    FAPI_AUDEC_StatusEnumT   (*GetStatus)( FAPI_SYS_HandleT handle );
    int32_t                  (*SetCodecParams)( FAPI_SYS_HandleT handle,
                                                void* codecParamsPtr,
                                                uint32_t length);
    void                     (*PtsNotifier)( uint64_t pts, void* inputStreamHandle );
    int32_t                  (*Reboot)( uint32_t restoreFlag );
    int32_t                  (*Suspend)( void );
    int32_t                  (*Resume)( void );
    FAPI_SYS_HandleT         (*OpenMixer)( FAPI_SYS_HandleT audioMainInpHandle,
                                           FAPI_SYS_HandleT audioSubInpHandle,
                                           FAPI_SYS_HandleT audioOutHandle,
                                           int32_t* errorCodePtr);
    int32_t                  (*InvalidatePts)( FAPI_SYS_HandleT handle );
    int32_t                  (*SetOutputDelay)( FAPI_SYS_HandleT handle, int32_t outputDelay );

    uint32_t                 firmware_type;     //!< 0:external boot, 1:embedded boot
} FAPEX_AUDEC_FuncStrT;

#endif /* _FAPEX_AUDEC_FUNC_H */
