/*!
*************************************************************************
**
** \file    fapex_blte_func.h
**
** \brief   function pointer definiton header file 
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Microelectronics Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Microelectronics Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_BLTE_FUNC_H
#define _FAPEX_BLTE_FUNC_H

#include "fapi/drv_blte.h"
   
typedef struct {
    int32_t    (*Init)(void);
    void       (*Exit)(void);
    uint32_t   (*Isr)(void);
    void       (*Bsr)(uint32_t value);

    FAPI_SYS_HandleT (*Open)(const FAPI_BLTE_OpenParamsT* paramsPtr, int32_t* errorCodePtr );
    int32_t    (*Close)(FAPI_SYS_HandleT handle);
    int32_t    (* SetColourKeying)(FAPI_SYS_HandleT handle,uint8_t enable, uint32_t keyColor);
    int32_t    (*SetShadingParams)(FAPI_SYS_HandleT handle,
                                            const FAPI_BLTE_ShadingParamsT* shadingParams);
    int32_t    (*SetBlendingParams)(FAPI_SYS_HandleT handle,
                                            const FAPI_BLTE_BlendingParamsT* blendingParams);
    int32_t    (*SetInputColorMatrix)(FAPI_SYS_HandleT handle,
                                            const FAPI_BLTE_MTXIParamsT* mtxiPtr);
    int32_t    (*SetOutputColourMatrix)(FAPI_SYS_HandleT handle,
                                            const FAPI_BLTE_MTXOParamsT* mtxoPtr);
    int32_t    (*SetClut)(FAPI_SYS_HandleT handle, uint32_t index, const uint32_t* clutPtr);
    int32_t    (*SetClip)(FAPI_SYS_HandleT handle, uint32_t enable, const FAPI_BLTE_RectT* clipRect);
    int32_t    (*SetSourceLayout)(FAPI_SYS_HandleT handle,uint32_t  index,
                                           FAPI_BLTE_LayoutT layout);
    int32_t    (*DrawRect)(FAPI_SYS_HandleT handle,
                                const FAPI_BLTE_RectParamsT* rectParams);
    int32_t    (*DrawFont)(FAPI_SYS_HandleT handle, 
                                const FAPI_BLTE_FontParamsT* fontParamsPtr);
    int32_t    (*DrawULine)(FAPI_SYS_HandleT handle,
                                const FAPI_BLTE_ULineParamsT* ulineParamsPtr);
    int32_t    (*Copy)(FAPI_SYS_HandleT handle,
                                const FAPI_BLTE_CopyParamsT* copyParamsPtr);
    int32_t    (*DrawBitmap)(FAPI_SYS_HandleT handle, 
                                const FAPI_BLTE_BitmapParamsT* bitmapParamsPtr);
    int32_t    (*SetProcessingMode)( FAPI_SYS_HandleT handle, 
                                FAPI_BLTE_ProcessingModeT processingMode);
    int32_t    (*Begin)( FAPI_SYS_HandleT handle);
    int32_t    (*End)(FAPI_SYS_HandleT handle);  
    int32_t    (*SetFrameBase)(FAPI_SYS_HandleT handle,
                                   const FAPI_BLTE_SetFrameBaseParamsT * paramsPtr);  
} FAPEX_BLTE_FuncStrT;

#endif /* _FAPEX_FLASH_FUNC_H */
