/*!
*************************************************************************
**
** \file    fapex_bm_func.h
**
** \brief   function pointer definiton header file 
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
** 
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_BM_FUNC_H
#define _FAPEX_BM_FUNC_H

#include "fapi/drv_bm.h"

typedef struct {
    int32_t          (*Init)( void );
    void             (*Exit)( void );
    uint32_t         (*Isr)( void );
    void             (*Bsr)( uint32_t privData );
    FAPI_SYS_HandleT (*Open)( const FAPI_BM_OpenParamsT* pParams,
                                         int32_t* errorCodePtr );
    int32_t          (*Close)( FAPI_SYS_HandleT handle );
    int32_t          (*GetBufferFillLevel)( const FAPI_SYS_HandleT handle );
    uint32_t         (*GetBufferReadPointerPosition)( const FAPI_SYS_HandleT handle );
    uint32_t         (*GetBufferWritePointerPosition)( const FAPI_SYS_HandleT handle );
    uint8_t*         (*GetBufferReadPointer)( const FAPI_SYS_HandleT handle );
    uint8_t*         (*GetBufferWritePointer)( const FAPI_SYS_HandleT handle );
    uint32_t         (*GetHardwareControlledBlock)( void );
    int32_t          (*MoveBufferReadPointer)( FAPI_SYS_HandleT handle, const int32_t readPtrDistance );
    int32_t          (*MoveBufferWritePointer)( FAPI_SYS_HandleT handle, const int32_t writePtrDistance );
    int32_t          (*SetBufferReadPointer)( FAPI_SYS_HandleT handle,
                                              const uint32_t readPtrPosition,
                                              const uint32_t enableOutOfBounds );
    int32_t          (*SetBufferWritePointer)( const FAPI_SYS_HandleT handle,
                                               const uint32_t writePtrPosition,
                                               const uint32_t enableOutOfBounds );
    void             (*SetHardwareControlledBlock)( const uint32_t blockIndex );
    int32_t          (*SetReadEnable)( FAPI_SYS_HandleT handle, uint32_t readEnable );
    int32_t          (*Write)( FAPI_SYS_HandleT handle, void* writeBuffer,
                               uint32_t length, uint32_t timeout );
    int32_t          (*WriteMtxCol)( FAPI_SYS_HandleT handle, 
                                     void* mtxLinePtrArr[],
                                     uint16_t mtxColOffsetArr[],
                                     uint16_t lineCount,
                                     uint16_t colCount,
                                     uint16_t wordSize,
                                     uint32_t timeout);
    int32_t          (*Read)( FAPI_SYS_HandleT handle, void* readBuffer,
                              uint32_t length, uint32_t timeout );
    
    int32_t          (*SetAudCallback)(FAPI_SYS_HandleT handle,
                                       FAPI_BM_AudCallbackT callbackFunction,
                                       void* optData,
                                       const uint32_t searchPattern,
                                       const uint32_t searchPatternMask);
   // int32_t          (*GetAudChannelAssignment)( uint32_t blockIndex,
   //                                              const uint32_t audIndex );
    int32_t          (*StartAudFiltering)( const FAPI_SYS_HandleT handle );
    int32_t          (*StopAudFiltering)( const FAPI_SYS_HandleT handle );
    
    int32_t          (*ClearAudFiltering)(const FAPI_SYS_HandleT handle);
    int32_t          (*GetInfo)( FAPI_SYS_HandleT handle, FAPI_BM_InfoT* info );
    int32_t          (*SetBufferPurpose)(const FAPI_BM_PurposeT bufferPurpose,
                                         const uint32_t blockIndex,
                                         const uint32_t address,
                                         const uint32_t size );
    uint32_t         (*GetAddressByBufferPurpose)( const FAPI_BM_PurposeT bufferPurpose );
    int32_t          (*GetBlockIndexByBufferPurpose)( const FAPI_BM_PurposeT bufferPurpose );
    uint32_t         (*GetSizeByBufferPurpose)( const FAPI_BM_PurposeT bufferPurpose );
    int32_t          (*SetHighLevelIrq)( const FAPI_SYS_HandleT handle, FAPI_BM_CallbackT callback, void* optData, uint32_t level, uint32_t enable );
    int32_t          (*GetHighLevelIrq)( const FAPI_SYS_HandleT handle );
    int32_t          (*SetLowLevelIrq)( const FAPI_SYS_HandleT handle, FAPI_BM_CallbackT callback, void* optData, uint32_t level, uint32_t enable );
    int32_t          (*GetLowLevelIrq)( const FAPI_SYS_HandleT handle );
    int32_t          (*SetReadSwap)( const FAPI_SYS_HandleT handle, uint32_t swap,
                                     uint32_t mode );
    int32_t          (*SetWriteSwap)( const FAPI_SYS_HandleT handle, uint32_t swap );
    int32_t          (*SetOverflowIrq)( const FAPI_SYS_HandleT handle, FAPI_BM_CallbackT callback, void* optData, uint32_t enable );
    int32_t          (*Reconfigure)( FAPI_SYS_HandleT handle, const FAPI_BM_ReconfigureT* const configPtr, uint32_t checkBufferOverlap );
} FAPEX_BM_FuncStrT;

#endif /* _FAPEX_BM_FUNC_H */
