/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 fapex SAF header file
*/

#ifndef _FAPEX_CFDLSAF_H_
#define _FAPEX_CFDLSAF_H_

#include "fapex.h"

#define FAPEXCFDLSAF_GETID(id) \
    ((FAPEXCFDLSAF_Services.getid != NULL)  \
      ? FAPEXCFDLSAF_Services.getid(id)     \
      : (-1) )

#define FAPEXCFDLSAF_CLEAR_MEMORY(ptr,size) \
    if (FAPEXCFDLSAF_Services.clear_memory != NULL){   \
        FAPEXCFDLSAF_Services.clear_memory(ptr,size);  \
    }

#define FAPEXCFDLSAF_CREATE_EVENT(pIvt) \
    ((FAPEXCFDLSAF_Services.event_create != NULL)  \
      ? FAPEXCFDLSAF_Services.event_create(pIvt)   \
      : (-1) )

#define FAPEXCFDLSAF_DELETE_EVENT(pIvt) \
    ((FAPEXCFDLSAF_Services.event_delete != NULL)  \
      ? FAPEXCFDLSAF_Services.event_delete(pIvt)   \
      : (-1) )

#define FAPEXCFDLSAF_WAIT_EVENT(pIvt) \
    ((FAPEXCFDLSAF_Services.event_wait != NULL)    \
      ? FAPEXCFDLSAF_Services.event_wait(pIvt)     \
      : (-1) )

#define FAPEXCFDLSAF_SET_EVENT(pIvt,bitptn,mode) \
    ((FAPEXCFDLSAF_Services.event_set != NULL)            \
      ? FAPEXCFDLSAF_Services.event_set(pIvt,bitptn,mode) \
      : (-1) )

#define FAPEXCFDLSAF_POLL_EVENT(pFile,pIvt_read,pIvt_write,ptbl) \
    ((FAPEXCFDLSAF_Services.event_poll != NULL)                           \
      ? FAPEXCFDLSAF_Services.event_poll(pFile,pIvt_read,pIvt_write,ptbl) \
      : (-1) )

#define FAPEXCFDLSAF_COPY_TO_USR(pTo,pFrom,bytes) \
    ((FAPEXCFDLSAF_Services.copy_to_user != NULL)           \
      ? FAPEXCFDLSAF_Services.copy_to_user(pTo,pFrom,bytes) \
      : (-1) )

#define FAPEXCFDLSAF_COPY_TO_DRV(pTo,pFrom,bytes) \
    ((FAPEXCFDLSAF_Services.copy_to_drv != NULL)           \
      ? FAPEXCFDLSAF_Services.copy_to_drv(pTo,pFrom,bytes) \
      : (-1) )

#define FAPEXCFDLSAF_SPIN_LOCK_CREATE(pLock) \
    if (FAPEXCFDLSAF_Services.spinlock_create != NULL){  \
        FAPEXCFDLSAF_Services.spinlock_create(pLock);    \
    }

#define FAPEXCFDLSAF_SPIN_LOCK_DELETE(pLock) \
    if (FAPEXCFDLSAF_Services.spinlock_delete != NULL){ \
        FAPEXCFDLSAF_Services.spinlock_delete(pLock);   \
    }

#define FAPEXCFDLSAF_SPIN_LOCK(pLock,pFlags) \
    if (FAPEXCFDLSAF_Services.spinlock != NULL){      \
        FAPEXCFDLSAF_Services.spinlock(pLock,pFlags); \
    }

#define FAPEXCFDLSAF_SPIN_UNLOCK(pLock,pFlags) \
    if (FAPEXCFDLSAF_Services.spinunlock != NULL){      \
        FAPEXCFDLSAF_Services.spinunlock(pLock,pFlags); \
    }

typedef struct {
    int32_t (*getid)(FAPEXinode_S id);
    void    (*clear_memory)(void *ptr, int32_t size);
    int32_t (*event_create)(FAPEXCFDLSAF_EventStrT *pIvt);
    int32_t (*event_delete)(FAPEXCFDLSAF_EventStrT *pIvt);
    int32_t (*event_wait)(FAPEXCFDLSAF_EventStrT *pIvt);
    int32_t (*event_set)(FAPEXCFDLSAF_EventStrT *pIvt, uint32_t bitptn, uint32_t mode);
    int32_t (*event_poll)(FAPEXfile_S *pFile,
                          FAPEXCFDLSAF_EventStrT *pIvt_read,
                          FAPEXCFDLSAF_EventStrT *pIvt_write,
                          FAPEXFDLPLARG2_T ptbl);
    int32_t (*copy_to_user)(void* pFrom, void* pTo, uint32_t bytes);
    int32_t (*copy_to_drv)(void* pFrom, void* pTo, uint32_t bytes);
    void    (*spinlock_create)(FAPEXCFDLSAF_SpnLockStrT *pLock);
    void    (*spinlock_delete)(FAPEXCFDLSAF_SpnLockStrT *pLock);
    void    (*spinlock)(FAPEXCFDLSAF_SpnLockStrT *pLock, uint32_t *pFlags);
    void    (*spinunlock)(FAPEXCFDLSAF_SpnLockStrT *pLock, uint32_t *pFlags);
} FAPEXCFDLSAF_ServicesT;

#ifndef __FAPEX_CFDL_SAF_C__
/* A global data for the FAPEX System Abstraction Functions (SAF) */
extern FAPEXCFDLSAF_ServicesT FAPEXCFDLSAF_Services;
#endif /* __FAPEX_CFDL_SAF_C__ */

#ifdef __LINUX__
/*!
*************************************************************************
** \brief static buffer management control structure
*************************************************************************/
typedef struct _FAPISAF_BuffCtrlT      FAPISAF_BuffCtrlT;

struct _FAPISAF_BuffCtrlT {
    uint32_t   buff_top;    //!< buffer area physical address
    uint32_t   buff_size;   //!< buffer area size;

    FAPISAF_BuffCtrlT   *prev_info;     //!< previous buffer information
    FAPISAF_BuffCtrlT   *next_info;     //!< next buffer information

};

typedef struct _FAPISAF_BuffManageT {
    struct semaphore    sem;
    uint32_t            align_mask;

    FAPISAF_BuffCtrlT   *base_info;     //!< base management information
    FAPISAF_BuffCtrlT   *alloc_area;    //!< alocated area management information
    FAPISAF_BuffCtrlT   *free_area;     //!< free area management information

} FAPISAF_BuffManageT;

#endif /* __LINUX__ */
/* prototype */
void FAPEXCFDLSAF_SetServices(FAPEXCFDLSAF_ServicesT* pSys);

#ifdef __LINUX__
FAPISAF_BuffManageT *FAPISAF_InitSBuffArea(
                            uint32_t baseAddress, uint32_t totalSize,
                            uint32_t alignSize, uint32_t maxNum);
int32_t  FAPISAF_ExitSBuffArea(FAPISAF_BuffManageT *pManage);
uint32_t FAPISAF_AllocSBuffArea(FAPISAF_BuffManageT *pManage, uint32_t size);
void     FAPISAF_FreeSBuffArea(FAPISAF_BuffManageT *pManage, uint32_t address);
#endif /* _FAPEX_CFDLSAF_H_ */
#endif /* _FAPEX_CFDLSAF_H_ */
