/*!
*************************************************************************
**
** \file    fapex_fpc_func.h
**
** \brief   function pointer definiton header file 
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_FPC_FUNC_H
#define _FAPEX_FPC_FUNC_H

#include "fapi/drv_fpc.h"

typedef struct {
    int32_t          (*Init)(void);
    void             (*Exit)(void);
    void             (*Bsr)(uint32_t pressedButtons);
    FAPI_SYS_HandleT (*Open)(const FAPI_FPC_OpenParamsT* pParams,
                             int32_t* errorCodePtr);
    int32_t          (*Close)(FAPI_SYS_HandleT handle);

    // preinit functions
    int32_t          (*SetMode)(uint32_t newMode);
    int32_t          (*SetSelectBitInversion)(uint32_t inverting);
    int32_t          (*SetDividerClock)(uint32_t divider_clk);
    int32_t          (*SetDividerSeg)(uint32_t divider_seg);
    int32_t          (*SetInitialBrightness)(uint32_t initialBrightness);
    void             (*SetLeadingZeros)(uint32_t leadingZeros);
    // end of preinit functions

    int32_t          (*SetButtonFunction)(const FAPI_SYS_HandleT handle,
                                          const uint32_t buttonIndex,
                                          FAPI_FPC_CallbackT function,
                                          void* optDataPtr);
    int32_t          (*SetBrightness)(const FAPI_SYS_HandleT handle,
                                      const uint32_t value);
    int32_t          (*IncreaseBrightness)(const FAPI_SYS_HandleT handle,
                                           const uint32_t steps);
    int32_t          (*DecreaseBrightness)(const FAPI_SYS_HandleT handle,
                                           const uint32_t steps);
    int32_t          (*SetDisplayWord)(const FAPI_SYS_HandleT handle,
                                       const uint8_t sign3, const uint8_t sign2,
                                       const uint8_t sign1, const uint8_t sign0,
                                       const uint32_t dot3, const uint32_t dot2,
                                       const uint32_t dot1, const uint32_t dot0);
    int32_t          (*SetDigitAt)(const FAPI_SYS_HandleT handle,
                                   const uint8_t sign, const uint32_t dot,
                                   const uint32_t position);
    int32_t          (*IncrementCipherAt)(const FAPI_SYS_HandleT handle,
                                           const uint32_t position);
    int32_t          (*DecrementCipherAt)(const FAPI_SYS_HandleT handle,
                                           const uint32_t position);
    int32_t          (*GetCipherAt)(const FAPI_SYS_HandleT handle,
                                    const uint32_t position);
    int32_t          (*SetCipherAt)(const FAPI_SYS_HandleT handle,
                                    const uint32_t cipher,
                                    const uint32_t position);
    int32_t          (*IsDotActiveAt)(const FAPI_SYS_HandleT handle,
                                      const uint32_t position);
    int32_t          (*ToggleDotAt)(const FAPI_SYS_HandleT handle,
                                    const uint32_t position);
    int32_t          (*GetIntegerValue)(const FAPI_SYS_HandleT handle);
    int32_t          (*SetIntegerValue)(const FAPI_SYS_HandleT handle,
                                        const int32_t value);
    int32_t          (*IncrementIntegerValue)(const FAPI_SYS_HandleT handle);
    int32_t          (*DecrementIntegerValue)(const FAPI_SYS_HandleT handle);
    int32_t          (*SetScrollText)(const FAPI_SYS_HandleT handle,
                                      const uint8_t* message,
                                      const uint32_t delay);
} FAPEX_FPC_FuncStrT;

#endif /* _FAPEX_FPC_FUNC_H */
