/*!
*************************************************************************
**
** \file    fapex_icc_func.h
**
** \brief   function pointer definiton header file
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_ICC_FUNC_H
#define _FAPEX_ICC_FUNC_H

#include "fapi/drv_icc.h"
#include "fapi/drv_icc_t0.h"
#include "fapi/drv_icc_t1.h"

typedef struct {
    int32_t          (*Init)(void);
    void             (*Exit)(void);
    uint32_t         (*Isr)(void);
	void             (*Bsr)(uint32_t privData);
    FAPI_SYS_HandleT (*Open)(const FAPI_ICC_OpenParamsT* paramsPtr,
                             int32_t* errorCodePtr);
    int32_t          (*Close)(FAPI_SYS_HandleT handle);
    int32_t (*ColdReset)( FAPI_SYS_HandleT handle,
                          const FAPI_ICC_ColdResetParamsT* paramsPtr,
                          const FAPI_ICC_FinishCallbackT finishCallback,
                          void* optData);
    int32_t (*WarmReset)( FAPI_SYS_HandleT handle,
                          const FAPI_ICC_WarmResetParamsT* paramsPtr,
                          const FAPI_ICC_FinishCallbackT finishCallback,
                          void* optData);

    int32_t (*GetAtrParams)       ( const FAPI_SYS_HandleT handle, FAPI_ICC_AtrParamsT* paramsPtr );
    int32_t (*GetBaudRate)        ( const FAPI_SYS_HandleT handle, uint32_t* baudRatePtr );
    int32_t (*GetClockFrequency)  ( const FAPI_SYS_HandleT handle, uint32_t *khzPtr );
    int32_t (*GetProtocol)        ( const FAPI_SYS_HandleT handle, uint32_t *protocolPtr );
    int32_t (*GetSlotStatus)      ( const FAPI_SYS_HandleT handle );
    int32_t (*SetClkDivider)      ( const uint32_t blockIndex, const uint32_t clkDivider );
    int32_t (*SetResetInverted)   ( const uint32_t blockIndex, const uint32_t invert );
    int32_t (*SetVccInverted)     ( const uint32_t blockIndex, const uint32_t invert );
    int32_t (*SetProtocol)        ( const FAPI_ICC_OperationModeT* operationModePtr );
    int32_t (*SetBaudRate)        ( FAPI_SYS_HandleT handle, const uint32_t baudRate );
    int32_t (*SetCardNotifier)    ( FAPI_SYS_HandleT handle,
                                    const FAPI_ICC_CardNotifyFuncT callback, void* optData );
    int32_t (*SetClockFrequency)  ( FAPI_SYS_HandleT handle, const uint32_t khz );
    int32_t (*SetGuardTime)       ( const uint32_t blockIndex, const uint32_t guardTime );
    int32_t (*GetGuardTime)       ( const uint32_t blockIndex, uint32_t* guardTimePtr );
    int32_t (*WriteBytes)( FAPI_SYS_HandleT handle,
                                 uint8_t* sendBufferPtr,
                           const uint32_t sendSize,
                           const uint32_t timeOut );
    int32_t (*ReadByte)( FAPI_SYS_HandleT handle, const uint32_t timeOut );
    int32_t (*GetData)( FAPI_SYS_HandleT handle,
                              uint8_t* sendBufferPtr,
                        const uint32_t sendSize,
                              uint8_t* receiveBufferPtr,
                        const uint32_t receiveSize,
                              uint32_t* receivedBytesPtr );

    int32_t (*T0Init)(void);

    FAPI_SYS_HandleT (*T0Open)( const FAPI_ICC_T0_OpenParamsT* paramsPtr,
                                 int32_t* errorCodePtr);

    int32_t (*T0GetIccHandle)(FAPI_SYS_HandleT  toHandle,
                              FAPI_SYS_HandleT* iccHandlePtr);

    int32_t (*T0Mode)(const FAPI_SYS_HandleT handle,
                      uint32_t* readDataPtr,
                      void* optDataPtr);

    int32_t (*T0SendCmd)( const FAPI_SYS_HandleT      handle,
                          FAPI_ICC_TOcommandT*  cmd_ptr,
                          FAPI_ICC_TOresponseT* resp_ptr );

    int32_t (*T0SetPps)( FAPI_SYS_HandleT toHandle, const uint32_t usePppsEnable, FAPI_ICC_PpsParamsT* ppsParamsPtr );
    int32_t (*T1Init)(void);
    int32_t (*T1Mode)( const FAPI_SYS_HandleT handle,
                       uint32_t* readDataPtr,
                       void* optDataPtr);
} FAPEX_ICC_FuncStrT;

#endif /* _FAPEX_ICC_FUNC_H */
