/*!
*************************************************************************
**
** \file    fapex_sync_func.h
**
** \brief   function pointer definiton header file 
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_SYNC_FUNC_H
#define _FAPEX_SYNC_FUNC_H

#include "fapi/drv_sync.h"

typedef struct {
    int32_t           (*Init)(void);
    void              (*Exit)(void);
    FAPI_SYS_HandleT  (*Open)(const FAPI_SYNC_OpenParamsT* pParams,
                              int32_t* errorCodePtr);
    int32_t           (*Close)(FAPI_SYS_HandleT handle);
    void              (*DpllFaster)(uint32_t regFactor);
    void              (*DpllSlower)(uint32_t regFactor);
    void              (*DpllRegulation)( FAPI_SYS_HandleT handle, uint32_t stc,
                                         uint32_t ref, uint64_t timeStamp);  
    int64_t           (*GetDecoderOffset)(void);
    int32_t           (*GetDpllControlFlags)(FAPI_SYS_HandleT handle,uint32_t *dpllControlFlags);
    uint32_t          (*GetEnvironment)(void);
    int32_t           (*GetPcrStcDifference)(uint64_t* pcrStcDifferencePtr);
    int32_t           (*GetPcr)(uint64_t* pcrPtr);
    uint32_t          (*GetPcrPid)(void);
    FAPI_SYNC_StatusT (*GetStatus)(void);
    int32_t           (*GetStc)(uint64_t* stcPtr);
    int32_t           (*GetStcLower)(uint32_t* stcPtr);
    uint32_t          (*GetStcRegulation)(void);
    void              (*PcrNotifier)(FAPI_SYS_HandleT handle, uint64_t pcr);
    void              (*ResetRegulation)(void);
    void              (*ResetStatus)(void);
    int32_t           (*SetDecoderOffset)(int64_t decoderOffset, uint32_t syncEnvironment);
    int32_t           (*SetDpllEnable)(uint32_t enableDpll);
    int32_t           (*SetEnvironment)(uint32_t syncEnvironment);
    int32_t           (*SetPcrPid)(uint32_t pid);
    int32_t           (*SetPllFreq)(uint32_t pllFreq);
    int32_t           (*SetStatus)(FAPI_SYS_HandleT handle, FAPI_SYNC_StatusT status);
    int32_t           (*SetStcLower)(uint32_t stc);
    int32_t           (*SetStcRegulation)(uint32_t enableStcRegulation);
    FAPI_SYS_HandleT  (*GetHandleForTimeSync)(void);
    void              (*SetCurrentStcPcr)(uint32_t stc,uint32_t pcr);
    int32_t           (*SetStreamSegmentMode)(uint32_t strSegMode);
    int32_t           (*SetSyncToHdEncoder)(FAPI_SYS_HandleT viencHdHandle);
} FAPEX_SYNC_FuncStrT;

#endif /* _FAPEX_SYNC_FUNC_H */
