/*!
*************************************************************************
**
** \file    fapex_tsd_func.h
**
** \brief   function pointer definiton header file 
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_TSD_FUNC_H
#define _FAPEX_TSD_FUNC_H

#include "fapi/drv_tsd.h"

typedef struct {
    int32_t          (*Init)(void);
    void             (*Exit)(void);
    uint32_t         (*Isr)(void);
    void             (*Bsr)(uint32_t privData);
    FAPI_SYS_HandleT (*Open)(const FAPI_TSD_OpenParamsT* paramsPtr,
                             int32_t* errorCodePtr);
    int32_t          (*Close)(FAPI_SYS_HandleT handle);
    int32_t          (*SetChannelPid)(FAPI_SYS_HandleT handle,
                                      const uint32_t pid,
                                      const uint32_t enableContinuityCheck,
                                      const uint32_t removeTsHeader);
    int32_t          (*GetChannelPid)(FAPI_SYS_HandleT handle);
    int32_t          (*StartChannel)(FAPI_SYS_HandleT handle);
    int32_t          (*StopChannel)(FAPI_SYS_HandleT handle);
    int32_t          (*SetPesParser)(FAPI_SYS_HandleT handle,
                                     uint32_t removeHeader,
                                     uint32_t enableNewPtsIrq,
                                     uint32_t enablePesParser);
    int32_t          (*Configure)(const uint32_t blockIndex, 
                                  FAPI_TSD_TsInputSettingsT* tsInputSettingsPtr,
                                  const FAPI_TSD_TsOutputSettingsT* tsOutputSettingsPtr);
    
    int32_t          (*SetSyncChannel)(const uint32_t blockIndex );
    int32_t          (*SetBuffer)(FAPI_SYS_HandleT handle,
                                  const FAPI_BM_InfoT bufferInfo);
    int32_t          (*GetBuffer)(FAPI_SYS_HandleT handle,
                                  FAPI_BM_InfoT* bufferInfoPtr);
    int32_t          (*SetSectionFilter)(const FAPI_SYS_HandleT handle,
                                         const FAPI_TSD_NumberOfTablesT numberOfTables,
                                         const FAPI_TSD_FilterDataT* filterDataPtr);
    int32_t          (*GetSectionFilter)(const FAPI_SYS_HandleT handle,
                                         FAPI_TSD_NumberOfTablesT* numberOfTablesPtr,
                                         FAPI_TSD_FilterDataT* filterDataPtr);
    int32_t          (*ClearSectionFilter)(const FAPI_SYS_HandleT handle);
    int32_t          (*SetSectionFilterMode)(const FAPI_SYS_HandleT handle,
                                             const uint32_t targetIndex,
                                             const uint32_t oneShotFiltering);
    int32_t          (*GetSectionFilterMode)(const FAPI_SYS_HandleT handle,
                                             const uint32_t targetIndex);
    int32_t          (*SetPacketEndNotifier)(const FAPI_SYS_HandleT handle,
                                             FAPI_TSD_GeneralNotifyT notifier,
                                             void* optDataPtr,
                                             const uint32_t enableNotifier);
    int32_t          (*SetScrambledPacketNotifier)(const FAPI_SYS_HandleT handle,
                                                   FAPI_TSD_ScrambledNotifyT notifier,
                                                   void* optDataPtr,
                                                   const uint32_t enableNotifier,
                                                   const uint32_t enableOneShot);
    int32_t          (*SetPacketTimeoutNotifier)(const uint32_t blockIndex,
                                                 FAPI_TSD_TimeoutNotifyT notifier,
                                                 void* optDataPtr,
                                                 const uint32_t enableNotifier,
                                                 const uint32_t timeout);
    int32_t          (*SetValidPacketTimeoutNotifier)(const uint32_t blockIndex,
                                                      FAPI_TSD_TimeoutNotifyT notifier,
                                                      void* optDataPtr,
                                                      const uint32_t enableNotifier,
                                                      const uint32_t timeout);
    int32_t          (*SetPtsNotifier)(const FAPI_SYS_HandleT handle,
                                       FAPI_TSD_PtsNotifyT notifier,
                                       void* optDataPtr,
                                       const uint32_t enableNotifier);
    int32_t          (*SetPtsEvaluator)(const FAPI_SYS_HandleT handle,
                                        FAPI_TSD_PtsEvaluateT evaluator,
                                        void* optDataPtr,
                                        const uint32_t enableEvaluator);
    uint32_t         (*GetPacketControlAddress)(uint32_t blockIndex);
    uint32_t         (*GetPacketRegisterAddress)(uint32_t blockIndex,
                                                 uint32_t index);
    uint32_t         (*GetLinearPacketRegisterAddress)(uint32_t blockIndex,
                                                       uint32_t index);
    uint32_t         (*GetLinearPacketAutoRegisterAddress)(uint32_t blockIndex,
                                                           uint32_t index);
    int32_t          (*SetFreeChannelPid)(FAPI_SYS_HandleT handle,
                                          const uint32_t pid,
                                          const uint32_t enableContinuityCheck,
                                          const uint32_t removeTsHeader);
    int32_t          (*ClearChannelPid)(FAPI_SYS_HandleT handle);
    int32_t          (*GetChannelId)(FAPI_SYS_HandleT handle);
    int32_t          (*GetBlockId)(FAPI_SYS_HandleT handle);
    int32_t          (*GetPesParserId)(FAPI_SYS_HandleT handle);
    FAPI_SYS_HandleT (*GetHandleByPid)(const uint32_t blockIndex,
                                             uint32_t pid,
                                             int32_t* errorCodePtr );
    

} FAPEX_TSD_FuncStrT;

#endif /* _FAPEX_TSD_FUNC_H */
