/*!
*************************************************************************
**
** \file    fapex_videc_func.h
**
** \brief   function pointer definiton header file 
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/
#ifndef _FAPEX_VIDEC_FUNC_H
#define _FAPEX_VIDEC_FUNC_H

#include "fapi/drv_videc.h"

typedef struct {
    void               (*SetBoardMemory)(const uint32_t ddr1StartAddress,
                                         const uint32_t ddr1Size,
                                         const uint32_t ddr2StartAddress,
                                         const uint32_t ddr2Size,
                                        const uint32_t ddr2FrameStartAddress,
                                        const uint32_t ddr2FrameSize
                                         );
    void               (*PreInit)(const FAPI_VIDEC_DecoderNumberEnumT
                                                      piMemoryForDecoderNumber,
                                      const uint32_t piVdMemoryAllocationType,
                                      const uint32_t piFrMemoryAllocationType);
    int32_t            (*GetPreInitError)(
                                          const FAPI_VIDEC_DecoderNumberEnumT
                                                     piMemoryForDecoderNumber);
    int32_t            (*GetFirmwareVersion)(
                                     const FAPI_VIDEC_DecoderEnumT decoderType,
                                                    uint32_t* firmwareVersion);
    int32_t            (*Init)(void);
    void               (*Exit)(void);
    uint32_t           (*IsrDecode)(void);
    void               (*BsrDecode)(uint32_t arg);
    uint32_t           (*IsrDisplay)(void);
    void               (*BsrDisplay)(uint32_t arg);
    int32_t            (*ReportMemory)(void);
    FAPI_SYS_HandleT   (*Open)(const FAPI_VIDEC_OpenParamsT* paramsPtr,
                                                        int32_t* errorCodePtr);
    int32_t            (*Close)(FAPI_SYS_HandleT handle);
    int32_t            (*Reset)(FAPI_SYS_HandleT handle, 
                                              FAPI_VIDEC_ResetEnumT resetType);
    int32_t            (*FreezeContinue)(FAPI_SYS_HandleT handle,
                             FAPI_VIDEC_FreezeContinueStateEnumT freezeSwitch);
    int32_t            (*Dump)(FAPI_SYS_HandleT handle,
                               uint32_t option);
    int32_t            (*Start)(FAPI_SYS_HandleT handle);
    int32_t            (*Stop)(FAPI_SYS_HandleT handle);
    int32_t            (*EnableDisable)(FAPI_SYS_HandleT handle,
                              FAPI_VIDEC_EnableDisableStateEnumT enableSwitch);
    int32_t            (*SetPlaybackMode)(FAPI_SYS_HandleT handle,
                                            FAPI_VIDEC_PlaybackModeEnumT mode);
    int32_t            (*SetPlaybackEnvironment)(
                                                       FAPI_SYS_HandleT handle,
                                     FAPI_VIDEC_PlaybackEnvironmentEnumT mode);
    int32_t            (*GetVesParams)(FAPI_SYS_HandleT handle,
                                          FAPI_VIDEC_VESParamsT* vesParamsPtr,
                                                         uint32_t suspendType);
    int32_t            (*GetVesBufferPosition)(FAPI_SYS_HandleT handle,
                                                          uint32_t* bufferPtr);
    int32_t            (*GetVesBufferSize)(FAPI_SYS_HandleT handle,
                                                         uint32_t* bufferSize);
    int32_t            (*GetVesFillLevel)(FAPI_SYS_HandleT handle,
                                                          uint32_t* fillLevel);
    int32_t            (*GetVesAvailableSpace)(FAPI_SYS_HandleT handle,
                                                     uint32_t* availableSpace);
    int32_t            (*GetState)(FAPI_SYS_HandleT handle,
                                            FAPI_VIDEC_GetStateT* getStatePtr);
    int32_t            (*GetHealth)(FAPI_SYS_HandleT handle,
                                          FAPI_VIDEC_HealthStateEnumT* health);
    int32_t            (*GetDecodingLatency)(FAPI_SYS_HandleT handle,
                                               int64_t* decoderLatencyInTicks);
    int32_t            (*GetStreamSegmentMode)(FAPI_SYS_HandleT handle,
                                          FAPI_VIDEC_StreamSegmentModeEnumT* streamSegmentMode);
    void               (*AudCallback)(uint32_t searchPattern,
                                      uint32_t readPointerPosition,
                                      void* optData,
                                      uint32_t separatedByPts,
                                      uint32_t valid);
    int32_t            (*AudFeedExternal)(FAPI_SYS_HandleT handle,
                                 uint32_t audSearchPattern, uint32_t foundPtr,
                                                                 uint64_t pts);
    int32_t            (*Recover)(FAPI_SYS_HandleT handle);
    int32_t            (*DecodeSingleDirectIFrame)(
                                                       FAPI_SYS_HandleT handle,
                                      uint32_t* dataPtr, uint32_t sizeInBytes);
    int32_t            (*DisplaySingleDirectIFrame)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*DecodeSequenceOfDirectIFrame)(
                                                       FAPI_SYS_HandleT handle,
                                      uint32_t* dataPtr, uint32_t sizeInBytes);
    int32_t            (*DisplaySequenceOfDirectIFrame)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*SetEntropyDecInfoCallback)(
                                                       FAPI_SYS_HandleT handle,
                                      FAPI_VIDEC_EntropyDecInfoCallbackT 
                                                entropyDecInfoCallbackFunction,
                                                         void* entropyOptData);
    int32_t            (*SetVideoDecInfoCallback)(FAPI_SYS_HandleT handle,
                                      FAPI_VIDEC_VideoDecInfoCallbackT
                                                    vldDecInfoCallbackFunction,
                                                             void* vldOptData);
    int32_t            (*SetAutoScaleCallback)(FAPI_SYS_HandleT handle,
                                      FAPI_VIDEC_AutoScaleCallbackT
                                                     autoScaleCallbackFunction,
                                                           void* scalerHandle);
    int32_t            (*SetDecoderToHdEncoder)(
                                                       FAPI_SYS_HandleT handle,
                                               FAPI_SYS_HandleT viencHdHandle);
    int32_t            (*SetPesParserToDecoder)(
                                                       FAPI_SYS_HandleT handle,
                                       FAPI_VIDEC_PesParserEnumT pesParserNum);
    int32_t            (*SetFrameDecodedCallback)(
                                                       FAPI_SYS_HandleT handle,
                                      FAPI_VIDEC_FrameDecodedCallbackT
                                                  frameDecodedCallbackFunction,
                                                    void* frameDecodedOptData);
    int32_t            (*SetFrameAvailableCallback)(
                                                       FAPI_SYS_HandleT handle,
                                      FAPI_VIDEC_FrameAvailableCallbackT 
                                                frameAvailableCallbackFunction,
                                                  void* frameAvailableOptData);
    int32_t            (*SetVesOverflowCallback)(
                                                       FAPI_SYS_HandleT handle,
                                      FAPI_VIDEC_VesOverflowCallbackT 
                                                   vesOverflowCallbackFunction,
                                                     void* vesOverflowOptData);
    int32_t            (*SetDirectIFrameMode)(FAPI_SYS_HandleT handle,
                                        FAPI_VIDEC_DirectIFrameModeEnumT mode);
    int32_t            (*SetChannelId)(FAPI_SYS_HandleT handle, 
                                                           uint32_t blockIndex,
                                                           uint32_t channelId);
    int32_t            (*SetParamChangeCallback)(FAPI_SYS_HandleT handle,
                                         FAPI_VIDEC_ParamChangeCallbackT
                                                   paramChangeCallbackFunction,
                                                     void* paramChangeOptData);
    int32_t            (*SetAfdState)(FAPI_SYS_HandleT handle,
                                            FAPI_VIDEC_AfdStateEnumT afdState);
    int32_t            (*SetOutputDelay)(FAPI_SYS_HandleT handle,
                                         int32_t outputDelayMsec);
    int32_t            (*SetEnableDelay)(FAPI_SYS_HandleT handle,
                                         uint32_t enableDelayInFrameNumber);
    int32_t            (*SetSyncBase)(FAPI_SYS_HandleT handle,
                                            FAPI_VIDEC_SyncBaseEnumT syncBase);
    int32_t            (*SetFrameRate)(FAPI_SYS_HandleT handle, uint32_t frameRate);
    int32_t            (*SetStreamSegmentMode)(FAPI_SYS_HandleT handle,
                                               FAPI_VIDEC_StreamSegmentModeEnumT streamSegmentMode);
    int32_t            (*SetAutoRecover)(FAPI_SYS_HandleT handle,
                                               FAPI_VIDEC_AutoRecoverEnumT autoRecover,
                                              uint64_t timeInterval);
    int32_t            (*ClearEntropyDecInfoCallback)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearVideoDecInfoCallback)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearAutoScaleCallback)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearDecoderToHdEncoder)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearPesParserToDecoder)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearFrameDecodedCallback)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearFrameAvailableCallback)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearVesOverflowCallback)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearDirectIFrameMode)(
                                                      FAPI_SYS_HandleT handle);
    int32_t            (*ClearChannelId)(FAPI_SYS_HandleT handle);
    int32_t            (*ClearParamChangeCallback)(FAPI_SYS_HandleT handle);
    int32_t            (*ClearOutputDelay)(FAPI_SYS_HandleT handle);
    int32_t            (*ClearEnableDelay)(FAPI_SYS_HandleT handle);
    int32_t            (*ClearSyncBase)(FAPI_SYS_HandleT handle);
    int32_t            (*ClearFrameRate)(FAPI_SYS_HandleT handle);
    int32_t            (*ClearStreamSegmentMode)(FAPI_SYS_HandleT handle);
    int32_t            (*ClearAutoRecover)(FAPI_SYS_HandleT handle);

    uint32_t           firmware_type;   //!< 0:external boot, 1:embedded boot
} FAPEX_VIDEC_FuncStrT;

#endif /* _FAPEX_VIDEC_FUNC_H */
